@@ -15,12 +15,19 @@
 
 #include "fc_types.h"
 
-struct ai_choice;
+#include "ai.h" /* incident_type */
+
 struct Treaty;
 struct Clause;
 struct ai_data;
 
-void ai_diplomacy_calculate(struct player *pplayer, struct ai_data *ai);
+enum war_reason {
+  WAR_REASON_BEHAVIOUR, WAR_REASON_SPACE, WAR_REASON_EXCUSE,
+  WAR_REASON_HATRED, WAR_REASON_ALLIANCE, WAR_REASON_NONE
+};
+
+void ai_diplomacy_begin_new_phase(struct player *pplayer,
+				  struct ai_data *ai);
 void ai_diplomacy_actions(struct player *pplayer);
 
 void ai_treaty_evaluate(struct player *pplayer, struct player *aplayer,
@@ -28,4 +35,12 @@ void ai_treaty_evaluate(struct player *pplayer, struct player *aplayer,
 void ai_treaty_accepted(struct player *pplayer, struct player *aplayer, 
                         struct Treaty *ptreaty);
 
+void ai_incident(enum incident_type type, struct player *violator,
+                 struct player *victim);
+
+bool ai_on_war_footing(struct player *pplayer);
+
+void ai_diplomacy_first_contact(struct player *pplayer,
+                                struct player *aplayer);
+
 #endif
