@@ -63,13 +63,13 @@ static void ai_data_city_impr_calc(struct player *pplayer, struct ai_data *ai)
 
   memset(count, 0, sizeof(count));
 
-  impr_type_iterate(id) {
-    struct req_source source = {
-      .type = REQ_BUILDING,
-      .value = {.building = id}
+  improvement_iterate(pimprove) {
+    struct universal source = {
+      .kind = VUT_IMPROVEMENT,
+      .value = {.building = pimprove}
     };
 
-    ai->impr_calc[id] = AI_IMPR_ESTIMATE;
+    ai->impr_calc[improvement_index(pimprove)] = AI_IMPR_ESTIMATE;
 
     /* Find largest extension */
     effect_list_iterate(get_req_source_effects(&source), peffect) {
@@ -92,13 +92,13 @@ static void ai_data_city_impr_calc(struct player *pplayer, struct ai_data *ai)
       case EFT_OUTPUT_WASTE_PCT:
       case EFT_UPKEEP_FREE:
 	requirement_list_iterate(peffect->reqs, preq) {
-	  if (preq->source.type == REQ_BUILDING
-	      && preq->source.value.building == id) {
-            if (ai->impr_calc[id] != AI_IMPR_CALCULATE_FULL) {
-	      ai->impr_calc[id] = AI_IMPR_CALCULATE;
+	  if (VUT_IMPROVEMENT == preq->source.kind
+	      && preq->source.value.building == pimprove) {
+            if (ai->impr_calc[improvement_index(pimprove)] != AI_IMPR_CALCULATE_FULL) {
+	      ai->impr_calc[improvement_index(pimprove)] = AI_IMPR_CALCULATE;
             }
-	    if (preq->range > ai->impr_range[id]) {
-	      ai->impr_range[id] = preq->range;
+	    if (preq->range > ai->impr_range[improvement_index(pimprove)]) {
+	      ai->impr_range[improvement_index(pimprove)] = preq->range;
 	    }
 	  }
 	} requirement_list_iterate_end;
@@ -107,11 +107,11 @@ static void ai_data_city_impr_calc(struct player *pplayer, struct ai_data *ai)
       case EFT_OUTPUT_PER_TILE:
       case EFT_OUTPUT_INC_TILE:
 	requirement_list_iterate(peffect->reqs, preq) {
-	  if (preq->source.type == REQ_BUILDING
-	      && preq->source.value.building == id) {
-	    ai->impr_calc[id] = AI_IMPR_CALCULATE_FULL;
-	    if (preq->range > ai->impr_range[id]) {
-	      ai->impr_range[id] = preq->range;
+	  if (VUT_IMPROVEMENT == preq->source.kind
+	      && preq->source.value.building == pimprove) {
+	    ai->impr_calc[improvement_index(pimprove)] = AI_IMPR_CALCULATE_FULL;
+	    if (preq->range > ai->impr_range[improvement_index(pimprove)]) {
+	      ai->impr_range[improvement_index(pimprove)] = preq->range;
 	    }
 	  }
 	} requirement_list_iterate_end;
@@ -121,7 +121,7 @@ static void ai_data_city_impr_calc(struct player *pplayer, struct ai_data *ai)
       break;
       }
     } effect_list_iterate_end;
-  } impr_type_iterate_end;
+  } improvement_iterate_end;
 }
 
 /**************************************************************************
@@ -137,11 +137,11 @@ static bool player_has_really_useful_tech_parasite(struct player* pplayer)
     return FALSE;
   }
   
-  tech_type_iterate(tech) {
+  advance_index_iterate(A_FIRST, tech) {
     int players_having;
 
-    if (get_invention(pplayer, tech) == TECH_KNOWN
-        || !tech_is_available(pplayer, tech)) {
+    if (!player_invention_reachable(pplayer, tech)
+        || TECH_KNOWN == player_invention_state(pplayer, tech)) {
       continue;
     }
 
@@ -152,7 +152,7 @@ static bool player_has_really_useful_tech_parasite(struct player* pplayer)
         continue;
       }
 
-      if (get_invention(aplayer, tech) == TECH_KNOWN
+      if (TECH_KNOWN == player_invention_state(aplayer, tech)
           || get_player_research(aplayer)->researching == tech) {
 	players_having++;
 	if (players_having >= players_needed) {
@@ -160,12 +160,12 @@ static bool player_has_really_useful_tech_parasite(struct player* pplayer)
 	}
       }
     } players_iterate_end;
-  } tech_type_iterate_end;
+  } advance_index_iterate_end;
   return FALSE;
 }
 
 /**************************************************************************
-  Analyze rulesets. Must be run after rulesets after loaded, unlike
+  Analyze rulesets. Must be run after rulesets are loaded, unlike
   _init, which must be run before savegames are loaded, which is usually
   before rulesets.
 **************************************************************************/
@@ -186,23 +186,24 @@ static void count_my_units(struct player *pplayer)
   memset(&ai->stats.units, 0, sizeof(ai->stats.units));
 
   unit_list_iterate(pplayer->units, punit) {
-    switch (unit_type(punit)->move_type) {
-    case LAND_MOVING:
+    struct unit_class *pclass = unit_class(punit);
+
+    if (pclass->ai.land_move != MOVE_NONE
+        && pclass->ai.sea_move != MOVE_NONE) {
+      /* Can move both land and ocean */
+      ai->stats.units.amphibious++;
+    } else if (pclass->ai.land_move != MOVE_NONE) {
+      /* Can move only at land */
       ai->stats.units.land++;
-      break;
-    case SEA_MOVING:
+    } else if (pclass->ai.sea_move != MOVE_NONE) {
+      /* Can move only at sea */
       ai->stats.units.sea++;
-      break;
-    case HELI_MOVING:
-    case AIR_MOVING:
-      ai->stats.units.air++;
-      break;
     }
 
     if (unit_has_type_flag(punit, F_TRIREME)) {
       ai->stats.units.triremes++;
     }
-    if (unit_has_type_flag(punit, F_MISSILE)) {
+    if (uclass_has_flag(unit_class(punit), UCF_MISSILE)) {
       ai->stats.units.missiles++;
     }
     if (unit_has_type_flag(punit, F_PARATROOPERS)) {
@@ -228,7 +229,7 @@ static void count_my_units(struct player *pplayer)
 void ai_data_phase_init(struct player *pplayer, bool is_new_phase)
 {
   struct ai_data *ai = &aidata[player_index(pplayer)];
-  int i;
+  int i, j, k;
   int nuke_units = num_role_units(F_NUCLEAR);
   bool danger_of_nukes = FALSE;
 
@@ -240,7 +241,6 @@ void ai_data_phase_init(struct player *pplayer, bool is_new_phase)
   ai->num_oceans        = map.num_oceans;
   ai->threats.continent = fc_calloc(ai->num_continents + 1, sizeof(bool));
   ai->threats.invasions = FALSE;
-  ai->threats.air       = FALSE;
   ai->threats.nuclear   = 0; /* none */
   ai->threats.ocean     = fc_calloc(ai->num_oceans + 1, sizeof(bool));
   ai->threats.igwall    = FALSE;
@@ -255,8 +255,10 @@ void ai_data_phase_init(struct player *pplayer, bool is_new_phase)
      * enough to warrant city walls. Concentrate instead on 
      * coastal fortresses and hunting down enemy transports. */
     city_list_iterate(aplayer->cities, acity) {
-      Continent_id continent = tile_get_continent(acity->tile);
-      ai->threats.continent[continent] = TRUE;
+      Continent_id continent = tile_continent(acity->tile);
+      if (continent >= 0) {
+        ai->threats.continent[continent] = TRUE;
+      }
     } city_list_iterate_end;
 
     unit_list_iterate(aplayer->units, punit) {
@@ -267,20 +269,27 @@ void ai_data_phase_init(struct player *pplayer, bool is_new_phase)
       if (is_sailing_unit(punit)) {
         /* If the enemy has not started sailing yet, or we have total
          * control over the seas, don't worry, keep attacking. */
-        if (is_ground_units_transport(punit)) {
-          ai->threats.invasions = TRUE;
+        if (get_transporter_capacity(punit) > 0) {
+          unit_class_iterate(punitclass) {
+            if (punitclass->move_type == LAND_MOVING
+                && can_unit_type_transport(unit_type(punit), punitclass)) {
+              /* Enemy can transport some land units! */
+              ai->threats.invasions = TRUE;
+              break;
+            }
+          } unit_class_iterate_end;
         }
 
         /* The idea is that while our enemies don't have any offensive
          * seaborne units, we don't have to worry. Go on the offensive! */
         if (unit_type(punit)->attack_strength > 1) {
-	  if (is_ocean(tile_get_terrain(punit->tile))) {
-	    Continent_id continent = tile_get_continent(punit->tile);
+	  if (is_ocean_tile(punit->tile)) {
+	    Continent_id continent = tile_continent(punit->tile);
 	    ai->threats.ocean[-continent] = TRUE;
 	  } else {
 	    adjc_iterate(punit->tile, tile2) {
-	      if (is_ocean(tile_get_terrain(tile2))) {
-	        Continent_id continent = tile_get_continent(tile2);
+	      if (is_ocean_tile(tile2)) {
+	        Continent_id continent = tile_continent(tile2);
 	        ai->threats.ocean[-continent] = TRUE;
 	      }
 	    } adjc_iterate_end;
@@ -289,15 +298,8 @@ void ai_data_phase_init(struct player *pplayer, bool is_new_phase)
         continue;
       }
 
-      /* The next idea is that if our enemies don't have any offensive
-       * airborne units, we don't have to worry. Go on the offensive! */
-      if ((is_air_unit(punit) || is_heli_unit(punit))
-           && unit_type(punit)->attack_strength > 1) {
-        ai->threats.air = TRUE;
-      }
-
       /* If our enemy builds missiles, worry about missile defence. */
-      if (unit_has_type_flag(punit, F_MISSILE)
+      if (uclass_has_flag(unit_class(punit), UCF_MISSILE)
           && unit_type(punit)->attack_strength > 1) {
         ai->threats.missile = TRUE;
       }
@@ -321,6 +323,53 @@ void ai_data_phase_init(struct player *pplayer, bool is_new_phase)
   /* Increase from fear to terror if opponent actually has nukes */
   if (danger_of_nukes) ai->threats.nuclear++; /* sum of both fears */
 
+  /*** Channels ***/
+
+  /* Ways to cross from one ocean to another through a city. */
+  ai->channels = fc_calloc((ai->num_oceans + 1) * (ai->num_oceans + 1), sizeof(int));
+  players_iterate(aplayer) {
+    if (pplayers_allied(pplayer, aplayer)) {
+      city_list_iterate(aplayer->cities, pcity) {
+        adjc_iterate(pcity->tile, tile1) {
+          if (is_ocean_tile(tile1)) {
+            adjc_iterate(pcity->tile, tile2) {
+              if (is_ocean_tile(tile2) 
+                  && tile_continent(tile1) != tile_continent(tile2)) {
+                ai->channels[(-tile_continent(tile1)) * ai->num_oceans
+                             + (-tile_continent(tile2))] = TRUE;
+                ai->channels[(-tile_continent(tile2)) * ai->num_oceans
+                             + (-tile_continent(tile1))] = TRUE;
+              }
+            } adjc_iterate_end;
+          }
+        } adjc_iterate_end;
+      } city_list_iterate_end;
+    }
+  } players_iterate_end;
+
+  /* If we can go i -> j and j -> k, we can also go i -> k. */
+  for(i = 1; i <= ai->num_oceans; i++) {
+    for(j = 1; j <= ai->num_oceans; j++) {
+      if (ai->channels[i * ai->num_oceans + j]) {
+        for(k = 1; k <= ai->num_oceans; k++) {
+          ai->channels[i * ai->num_oceans + k] |= 
+            ai->channels[j * ai->num_oceans + k];
+        }
+      }
+    }
+  }
+
+  if (game.server.debug[DEBUG_FERRIES]) {
+    for(i = 1; i <= ai->num_oceans; i++) {
+      for(j = 1; j <= ai->num_oceans; j++) {
+        if (ai->channels[i * ai->num_oceans + j]) {
+          freelog(LOG_TEST, "%s: oceans %d and %d are connected",
+                  player_name(pplayer), i, j);
+       }
+      }
+    }
+  }
+
   /*** Exploration ***/
 
   ai->explore.land_done = TRUE;
@@ -328,9 +377,9 @@ void ai_data_phase_init(struct player *pplayer, bool is_new_phase)
   ai->explore.continent = fc_calloc(ai->num_continents + 1, sizeof(bool));
   ai->explore.ocean = fc_calloc(ai->num_oceans + 1, sizeof(bool));
   whole_map_iterate(ptile) {
-    Continent_id continent = tile_get_continent(ptile);
+    Continent_id continent = tile_continent(ptile);
 
-    if (is_ocean(ptile->terrain)) {
+    if (is_ocean_tile(ptile)) {
       if (ai->explore.sea_done && ai_handicap(pplayer, H_TARGETS) 
           && !map_is_known(ptile, pplayer)) {
 	/* We're not done there. */
@@ -340,7 +389,7 @@ void ai_data_phase_init(struct player *pplayer, bool is_new_phase)
       /* skip rest, which is land only */
       continue;
     }
-    if (ai->explore.continent[ptile->continent]) {
+    if (ai->explore.continent[tile_continent(ptile)]) {
       /* we don't need more explaining, we got the point */
       continue;
     }
@@ -364,7 +413,10 @@ void ai_data_phase_init(struct player *pplayer, bool is_new_phase)
   ai->stats.cities = fc_calloc(ai->num_continents + 1, sizeof(int));
   ai->stats.average_production = 0;
   city_list_iterate(pplayer->cities, pcity) {
-    ai->stats.cities[(int)tile_get_continent(pcity->tile)]++;
+    Continent_id continent = tile_continent(pcity->tile);
+    if (continent >= 0) {
+      ai->stats.cities[continent]++;
+    }
     ai->stats.average_production += pcity->surplus[O_SHIELD];
   } city_list_iterate_end;
   ai->stats.average_production /= MAX(1, city_list_size(pplayer->cities));
@@ -372,12 +424,12 @@ void ai_data_phase_init(struct player *pplayer, bool is_new_phase)
   unit_list_iterate(pplayer->units, punit) {
     struct tile *ptile = punit->tile;
 
-    if (!is_ocean(ptile->terrain) && unit_has_type_flag(punit, F_SETTLERS)) {
-      ai->stats.workers[(int)tile_get_continent(punit->tile)]++;
+    if (!is_ocean_tile(ptile) && unit_has_type_flag(punit, F_SETTLERS)) {
+      ai->stats.workers[(int)tile_continent(punit->tile)]++;
     }
     if (unit_has_type_flag(punit, F_DIPLOMAT) && punit->ai.ai_role == AIUNIT_ATTACK) {
       /* Heading somewhere on a mission, reserve target. */
-      struct city *pcity = tile_get_city(punit->goto_tile);
+      struct city *pcity = tile_city(punit->goto_tile);
 
       if (pcity) {
         BV_SET(ai->stats.diplomat_reservations, pcity->id);
@@ -388,7 +440,7 @@ void ai_data_phase_init(struct player *pplayer, bool is_new_phase)
 
   /*** Diplomacy ***/
 
-  if (pplayer->ai.control && !is_barbarian(pplayer) && is_new_phase) {
+  if (pplayer->ai_data.control && !is_barbarian(pplayer) && is_new_phase) {
     ai_diplomacy_begin_new_phase(pplayer, ai);
   }
 
@@ -496,7 +548,8 @@ void ai_data_phase_init(struct player *pplayer, bool is_new_phase)
     bool found_human = FALSE;
     ai->max_num_cities = 3;
     players_iterate(aplayer) {
-      if (aplayer == pplayer || aplayer->ai.control || !aplayer->is_alive) {
+      if (aplayer == pplayer || aplayer->ai_data.control
+          || !aplayer->is_alive) {
         continue;
       }
       ai->max_num_cities = MAX(ai->max_num_cities,
@@ -545,8 +598,11 @@ void ai_data_phase_done(struct player *pplayer)
   free(ai->stats.cities);
   ai->stats.cities = NULL;
 
-  ai->num_continents    = 0;
-  ai->num_oceans        = 0;
+  free(ai->channels);
+  ai->channels = NULL;
+
+  ai->num_continents = 0;
+  ai->num_oceans     = 0;
 }
 
 /**************************************************************************
@@ -586,11 +642,12 @@ void ai_data_init(struct player *pplayer)
 
   ai->govt_reeval = 0;
   ai->government_want = fc_realloc(ai->government_want,
-				   ((game.control.government_count + 1)
+				   ((government_count() + 1)
 				    * sizeof(*ai->government_want)));
   memset(ai->government_want, 0,
-	 (game.control.government_count + 1) * sizeof(*ai->government_want));
+	 (government_count() + 1) * sizeof(*ai->government_want));
 
+  ai->channels = NULL;
   ai->wonder_city = 0;
   ai->diplomacy.strategy = WIN_OPEN;
   ai->diplomacy.timer = 0;
@@ -605,7 +662,7 @@ void ai_data_init(struct player *pplayer)
     ai->diplomacy.player_intel[i].war_reason = WAR_REASON_NONE;
     ai->diplomacy.player_intel[i].distance = 1;
     ai->diplomacy.player_intel[i].ally_patience = 0;
-    pplayer->ai.love[i] = 1;
+    pplayer->ai_data.love[i] = 1;
     ai->diplomacy.player_intel[i].asked_about_peace = 0;
     ai->diplomacy.player_intel[i].asked_about_alliance = 0;
     ai->diplomacy.player_intel[i].asked_about_ceasefire = 0;
@@ -614,3 +671,17 @@ void ai_data_init(struct player *pplayer)
   ai->wants_no_science = FALSE;
   ai->max_num_cities = 10000;
 }
+
+/**************************************************************************
+  Is there a channel going from ocean c1 to ocean c2?
+  Returns FALSE if either is not an ocean.
+**************************************************************************/
+bool ai_channel(struct player *pplayer, Continent_id c1, Continent_id c2)
+{
+  struct ai_data *ai = ai_data_get(pplayer);
+
+  if (c1 >= 0 || c2 >= 0) {
+    return FALSE;
+  }
+  return (c1 == c2 || ai->channels[(-c1) * ai->num_oceans + (-c2)]);
+}
