@@ -14,7 +14,7 @@
 #define FC__AIDATA_H
 
 /* max size of a short */
-#define MAX_NUM_ID 32767
+#define MAX_NUM_ID (1+MAX_UINT16)
 
 #include "shared.h"		/* bool type */
 
@@ -49,15 +49,15 @@ struct ai_dip_intel {
   struct player *at_war_with_ally;
   struct player *is_allied_with_ally;
 
-  char spam;      /* timer to avoid spamming a player with chat */
+  signed char spam;      /* timer to avoid spamming a player with chat */
   int distance;   /* average distance to that player's cities */
   int countdown;  /* we're on a countdown to war declaration */
   enum war_reason war_reason; /* why we declare war */
-  char ally_patience; /* we EXPECT our allies to help us! */
-  char asked_about_peace;     /* don't ask again */
-  char asked_about_alliance;  /* don't nag! */
-  char asked_about_ceasefire; /* don't ... you get the point */
-  char warned_about_space;
+  signed char ally_patience; /* we EXPECT our allies to help us! */
+  signed char asked_about_peace;     /* don't ask again */
+  signed char asked_about_alliance;  /* don't nag! */
+  signed char asked_about_ceasefire; /* don't ... you get the point */
+  signed char warned_about_space;
 };
 
 BV_DEFINE(bv_id, MAX_NUM_ID);
@@ -87,7 +87,6 @@ struct ai_data {
     bool invasions;   /* check if we need to consider invasions */
     bool *continent;  /* non-allied cities on continent? */
     bool *ocean;      /* non-allied offensive ships in ocean? */
-    bool air;         /* check for non-allied offensive aircraft */
     bool missile;     /* check for non-allied missiles */
     int nuclear;      /* nuke check: 0=no, 1=capability, 2=built */
     bool igwall;      /* enemies have igwall units */
@@ -101,6 +100,9 @@ struct ai_data {
     bool sea_done;    /* nothing more to explore at sea */
   } explore;
 
+  /* Keep track of available ocean channels */
+  bool *channels;
+
   /* This struct is used for statistical unit building, eg to ensure
    * that we don't build too few or too many units of a given type. */
   struct {
@@ -109,8 +111,8 @@ struct ai_data {
       /* Unit-flag counts. */
       int triremes, missiles;
 
-      /* Move-type counts (air includes helicoptor here). */
-      int land, sea, air;
+      /* Move-type counts */
+      int land, sea, amphibious;
 
       /* Upgradeable units */
       int upgradeable;
@@ -165,11 +167,12 @@ void ai_data_init(struct player *pplayer);
 void ai_data_phase_init(struct player *pplayer, bool is_new_phase);
 void ai_data_phase_done(struct player *pplayer);
 
-void ai_data_init(struct player *pplayer);
 void ai_data_analyze_rulesets(struct player *pplayer);
 
 struct ai_data *ai_data_get(struct player *pplayer);
 const struct ai_dip_intel *ai_diplomacy_get(const struct player *pplayer,
 					    const struct player *aplayer);
 
+bool ai_channel(struct player *pplayer, Continent_id c1, Continent_id c2);
+
 #endif
