@@ -30,6 +30,11 @@ void aiferry_init_stats(struct player *pplayer);
  */
 int aiferry_find_boat(struct unit *punit, int cap, struct pf_path **path);
 
+/* 
+ * Initializes aiferry stats for a new unit
+ */
+void aiferry_init_ferry(struct unit *ferry);
+
 /*
  * Release the boat reserved in punit's ai.ferryboat field.
  */
@@ -55,6 +60,7 @@ bool ai_amphibious_goto_constrained(struct unit *ferry,
 				    struct pft_amphibious *parameter);
 
 bool is_boat_free(struct unit *boat, struct unit *punit, int cap);
+bool is_boss_of_boat(struct unit *punit);
 
 /*
  * Main boat managing function.  Gets units on board to where they want to
