@@ -27,6 +27,7 @@
 #include "game.h"
 #include "log.h"
 #include "mem.h"
+#include "specialist.h"
 #include "support.h"
 
 #include "agents.h"
@@ -284,20 +285,14 @@ static const char *get_prod_complete_string(struct city *pcity, int surplus)
     return buffer;
   }
 
-  stock = pcity->shield_stock;
-  if (pcity->production.is_unit) {
-    cost = unit_build_shield_cost(utype_by_number(pcity->production.value));
-  } else {
-    if (!pcity->production.is_unit
-	&& improvement_has_flag(pcity->production.value, IF_GOLD)) {
-      my_snprintf(buffer, sizeof(buffer),
-		  improvement_name_translation(pcity->production.value));
-      return buffer;
-    }
-    cost = impr_build_shield_cost(pcity->production.value);
+  if (city_production_has_flag(pcity, IF_GOLD)) {
+    mystrlcpy(buffer,
+	      improvement_name_translation(pcity->production.value.building),
+	      sizeof(buffer));
+    return buffer;
   }
-
-  stock += surplus;
+  stock = pcity->shield_stock + surplus;
+  cost = city_production_build_shield_cost(pcity);
 
   if (stock >= cost) {
     turns = 1;
@@ -337,7 +332,7 @@ const char *cmafec_get_result_descr(struct city *pcity,
     } output_type_iterate_end;
 
     my_snprintf(buf[6], BUFFER_SIZE, "%d/%s%s",
-		pcity->size - cm_count_specialist(pcity, result),
+		pcity->size - cm_result_specialists(result),
 		specialists_string(result->specialists),
 		result->happy ? _(" happy") : "");
 
