@@ -17,20 +17,24 @@
 
 #include <assert.h>
 
+/* utility */
 #include "fcintl.h"
 #include "log.h"
 #include "support.h"
 
+/* common */
 #include "city.h"
+#include "game.h"
+#include "specialist.h"
 #include "unitlist.h"
 
 #include "citydlg_g.h"
 #include "mapview_g.h"
 
+/* client */
 #include "citydlg_common.h"
-#include "civclient.h"		/* for can_client_issue_orders() */
+#include "client_main.h"		/* for can_client_issue_orders() */
 #include "climap.h"
-#include "clinet.h"
 #include "control.h"
 #include "mapview_common.h"
 #include "options.h"		/* for concise_city_production */
@@ -183,7 +187,7 @@ void city_dialog_redraw_map(struct city *pcity,
     citydlg_iterate(pcity, ptile, pedge, pcorner, canvas_x, canvas_y) {
       struct unit *punit
 	= ptile ? get_drawable_unit(tileset, ptile, pcity) : NULL;
-      struct city *pcity_draw = ptile ? ptile->city : NULL;
+      struct city *pcity_draw = ptile ? tile_city(ptile) : NULL;
 
       put_one_element(pcanvas, layer, ptile, pedge, pcorner,
 		      punit, pcity_draw, canvas_x, canvas_y, pcity);
@@ -200,6 +204,7 @@ void city_dialog_redraw_map(struct city *pcity,
 void get_city_dialog_production(struct city *pcity,
 				char *buffer, size_t buffer_len)
 {
+  char time[50];
   int turns, cost, stock;
 
   if (pcity == NULL) {
@@ -217,39 +222,31 @@ void get_city_dialog_production(struct city *pcity,
     return;
   }
 
-  turns = city_turns_to_build(pcity, pcity->production, TRUE);
+  if (city_production_has_flag(pcity, IF_GOLD)) {
+    my_snprintf(buffer, buffer_len, _("%3d gold per turn"),
+                MAX(0, pcity->surplus[O_SHIELD]));
+    return;
+  }
+  turns = city_production_turns_to_build(pcity, TRUE);
   stock = pcity->shield_stock;
+  cost = city_production_build_shield_cost(pcity);
 
-  if (pcity->production.is_unit) {
-    cost = unit_build_shield_cost(utype_by_number(pcity->production.value));
+  if (turns < FC_INFINITY) {
+    if (concise_city_production) {
+      my_snprintf(time, sizeof(time), "%3d", turns);
+    } else {
+      my_snprintf(time, sizeof(time),
+                  PL_("%3d turn", "%3d turns", turns), turns);
+    }
   } else {
-    cost = impr_build_shield_cost(pcity->production.value);
+    my_snprintf(time, sizeof(time), "%s",
+                concise_city_production ? "-" : _("never"));
   }
 
-  if (!pcity->production.is_unit
-      && improvement_has_flag(pcity->production.value, IF_GOLD)) {
-    my_snprintf(buffer, buffer_len, _("%3d gold per turn"),
-		MAX(0, pcity->surplus[O_SHIELD]));
+  if (concise_city_production) {
+    my_snprintf(buffer, buffer_len, _("%3d/%3d:%s"), stock, cost, time);
   } else {
-    char time[50];
-
-    if (turns < FC_INFINITY) {
-      if (concise_city_production) {
-	my_snprintf(time, sizeof(time), "%3d", turns);
-      } else {
-	my_snprintf(time, sizeof(time),
-		    PL_("%3d turn", "%3d turns", turns), turns);
-      }
-    } else {
-      my_snprintf(time, sizeof(time), "%s",
-		  concise_city_production ? "-" : _("never"));
-    }
-
-    if (concise_city_production) {
-      my_snprintf(buffer, buffer_len, _("%3d/%3d:%s"), stock, cost, time);
-    } else {
-      my_snprintf(buffer, buffer_len, _("%3d/%3d %s"), stock, cost, time);
-    }
+    my_snprintf(buffer, buffer_len, _("%3d/%3d %s"), stock, cost, time);
   }
 }
 
@@ -265,33 +262,37 @@ void get_city_dialog_production(struct city *pcity,
  less flexibility.
 **************************************************************************/
 void get_city_dialog_production_full(char *buffer, size_t buffer_len,
-				     struct city_production target,
+				     struct universal target,
 				     struct city *pcity)
 {
-  if (!target.is_unit && improvement_has_flag(target.value, IF_GOLD)) {
-    my_snprintf(buffer, buffer_len, _("%s (XX) %d/turn"),
-		get_impr_name_ex(pcity, target.value),
-		MAX(0, pcity->surplus[O_SHIELD]));
+  int turns = city_turns_to_build(pcity, target, TRUE);
+  int cost = universal_build_shield_cost(&target);
+
+  switch (target.kind) {
+  case VUT_IMPROVEMENT:
+    mystrlcpy(buffer,
+	      city_improvement_name_translation(pcity, target.value.building),
+	      buffer_len);
+
+    if (improvement_has_flag(target.value.building, IF_GOLD)) {
+      cat_snprintf(buffer, buffer_len, " (--) ");
+      cat_snprintf(buffer, buffer_len, _("%d/turn"),
+		   MAX(0, pcity->surplus[O_SHIELD]));
+      return;
+    }
+    break;
+  default:
+    universal_name_translation(&target, buffer, buffer_len);
+    break;
+  };
+  cat_snprintf(buffer, buffer_len, " (%d) ", cost);
+
+  if (turns < FC_INFINITY) {
+    cat_snprintf(buffer, buffer_len,
+		 PL_("%d turn", "%d turns", turns),
+		 turns);
   } else {
-    int turns = city_turns_to_build(pcity, target, TRUE);
-    const char *name;
-    int cost;
-
-    if (target.is_unit) {
-      name = utype_values_translation(utype_by_number(target.value));
-      cost = unit_build_shield_cost(utype_by_number(target.value));
-    } else {
-      name = get_impr_name_ex(pcity, target.value);
-      cost = impr_build_shield_cost(target.value);
-    }
-
-    if (turns < FC_INFINITY) {
-      my_snprintf(buffer, buffer_len,
-		  PL_("%s (%d) %d turn", "%s (%d) %d turns", turns),
-		  name, cost, turns);
-    } else {
-      my_snprintf(buffer, buffer_len, "%s (%d) never", name, cost);
-    }
+    cat_snprintf(buffer, buffer_len, _("never"));
   }
 }
 
@@ -301,63 +302,79 @@ void get_city_dialog_production_full(char *buffer, size_t buffer_len,
  column_size bytes.  City may be NULL.
 **************************************************************************/
 void get_city_dialog_production_row(char *buf[], size_t column_size,
-				    struct city_production target,
+				    struct universal target,
 				    struct city *pcity)
 {
-  if (target.is_unit) {
-    struct unit_type *ptype = utype_by_number(target.value);
+  universal_name_translation(&target, buf[0], column_size);
+
+  switch (target.kind) {
+  case VUT_UTYPE:
+  {
+    struct unit_type *ptype = target.value.utype;
 
-    mystrlcpy(buf[0], utype_name_translation(ptype), column_size);
     mystrlcpy(buf[1], utype_values_string(ptype), column_size);
-    my_snprintf(buf[2], column_size, "(%d)", unit_build_shield_cost(ptype));
-  } else {
-    struct player *pplayer = pcity ? city_owner(pcity) : game.player_ptr;
+    my_snprintf(buf[2], column_size, "(%d)", utype_build_shield_cost(ptype));
+    break;
+  }
+  case VUT_IMPROVEMENT:
+  {
+    struct player *pplayer = pcity ? city_owner(pcity) : client.conn.playing;
+    struct impr_type *pimprove = target.value.building;
 
     /* Total & turns left meaningless on capitalization */
-    if (improvement_has_flag(target.value, IF_GOLD)) {
-      mystrlcpy(buf[0], improvement_name_translation(target.value),
-		column_size);
+    if (improvement_has_flag(pimprove, IF_GOLD)) {
       buf[1][0] = '\0';
       my_snprintf(buf[2], column_size, "---");
     } else {
-      mystrlcpy(buf[0], improvement_name_translation(target.value),
-		column_size);
-
-      /* from city.c get_impr_name_ex() */
-      if (pcity && is_building_replaced(pcity, target.value, RPT_CERTAIN)) {
-	my_snprintf(buf[1], column_size, "*");
+      int upkeep = pcity ? city_improvement_upkeep(pcity, pimprove)
+                         : pimprove->upkeep;
+      /* from city.c city_improvement_name_translation() */
+      if (pcity && is_building_replaced(pcity, pimprove, RPT_CERTAIN)) {
+	my_snprintf(buf[1], column_size, "%d*", upkeep);
       } else {
-	const char *state = "";
+	const char *state = NULL;
 
-	if (is_great_wonder(target.value)) {
-          if (improvement_obsolete(pplayer, target.value)) {
+	if (is_great_wonder(pimprove)) {
+          if (improvement_obsolete(pplayer, pimprove)) {
             state = _("Obsolete");
-          } else if (great_wonder_was_built(target.value)) {
+          } else if (great_wonder_is_built(pimprove)) {
             state = _("Built");
+          } else if (great_wonder_is_destroyed(pimprove)) {
+            state = _("Destroyed");
           } else {
             state = _("Great Wonder");
           }
-	}
-	if (is_small_wonder(target.value)) {
-	  state = _("Small Wonder");
-	  if (find_city_from_small_wonder(pplayer, target.value)) {
-	    state = _("Built");
-	  }
-	  if (improvement_obsolete(pplayer, target.value)) {
+	} else if (is_small_wonder(pimprove)) {
+	  if (improvement_obsolete(pplayer, pimprove)) {
 	    state = _("Obsolete");
-	  }
+	  } else if (find_city_from_small_wonder(pplayer, target.value.building)) {
+	    state = _("Built");
+          } else {
+            state = _("Small Wonder");
+          }
 	}
-	my_snprintf(buf[1], column_size, "%s", state);
+        if (state) {
+          mystrlcpy(buf[1], state, column_size);
+        } else {
+          my_snprintf(buf[1], column_size, "%d", upkeep);
+        }
       }
 
       my_snprintf(buf[2], column_size, "%d",
-		  impr_build_shield_cost(target.value));
+		  impr_build_shield_cost(pimprove));
     }
+    break;
   }
+  default:
+    buf[1][0] = '\0';
+    buf[2][0] = '\0';
+    break;
+  };
 
   /* Add the turns-to-build entry in the 4th position */
   if (pcity) {
-    if (!target.is_unit && improvement_has_flag(target.value, IF_GOLD)) {
+    if (VUT_IMPROVEMENT == target.kind
+     && improvement_has_flag(target.value.building, IF_GOLD)) {
       my_snprintf(buf[3], column_size, _("%d/turn"),
 		  MAX(0, pcity->surplus[O_SHIELD]));
     } else {
@@ -366,7 +383,7 @@ void get_city_dialog_production_row(char *buf[], size_t column_size,
       if (turns < FC_INFINITY) {
 	my_snprintf(buf[3], column_size, "%d", turns);
       } else {
-	my_snprintf(buf[3], column_size, "%s", _("never"));
+	my_snprintf(buf[3], column_size, _("never"));
       }
     }
   } else {
@@ -383,7 +400,7 @@ void get_city_dialog_output_text(const struct city *pcity,
 {
   int total = 0;
   int priority;
-  int tax[O_COUNT];
+  int tax[O_LAST];
   struct output_type *output = &output_types[otype];
 
   buf[0] = '\0';
@@ -393,7 +410,7 @@ void get_city_dialog_output_text(const struct city *pcity,
   total += pcity->citizen_base[otype];
 
   /* Hack to get around the ugliness of add_tax_income. */
-  memset(tax, 0, O_COUNT * sizeof(*tax));
+  memset(tax, 0, O_LAST * sizeof(*tax));
   add_tax_income(city_owner(pcity), pcity->prod[O_TRADE], tax);
   if (tax[otype] != 0) {
     cat_snprintf(buf, bufsz, _("%+4d : Taxed from trade\n"), tax[otype]);
@@ -405,19 +422,22 @@ void get_city_dialog_output_text(const struct city *pcity,
   if (otype == O_TRADE) {
     int i;
 
-    for (i = 0; i < NUM_TRADEROUTES; i++) {
-      if (pcity->trade[i] != 0 && pcity->trade_value[i] != 0) {
-	/* There have been bugs causing the trade city to not be sent
-	 * properly to the client.  If this happens we trust the
-	 * trade_value[] array and simply don't give the name of the
-	 * city. */
-	struct city *trade_city = game_find_city_by_number(pcity->trade[i]);
-	/* TRANS: "unknown" location */
-	const char *name = trade_city ? city_name(trade_city) : _("(unknown)");
+    for (i = 0; i < NUM_TRADE_ROUTES; i++) {
+      if (pcity->trade[i] != 0) {
+        /* There have been bugs causing the trade city to not be sent
+         * properly to the client.  If this happens we trust the
+         * trade_value[] array and simply don't give the name of the
+         * city.
+         *
+         * NB: (pcity->trade_value[i] == 0) is valid case.  The trade route
+         * is established but doesn't give trade surplus. */
+        struct city *trade_city = game_find_city_by_number(pcity->trade[i]);
+        /* TRANS: "unknown" location */
+        const char *name = trade_city ? city_name(trade_city) : _("(unknown)");
 
-	cat_snprintf(buf, bufsz, _("%+4d : Trade route with %s\n"),
-		     pcity->trade_value[i], name);
-	total += pcity->trade_value[i];
+        cat_snprintf(buf, bufsz, _("%+4d : Trade route with %s\n"),
+                     pcity->trade_value[i], name);
+        total += pcity->trade_value[i];
       }
     }
   } else if (otype == O_GOLD) {
@@ -452,14 +472,22 @@ void get_city_dialog_output_text(const struct city *pcity,
 		     peffect->value);
 	total = new_total;
       } effect_list_iterate_end;
-      effect_list_unlink_all(plist);
       effect_list_free(plist);
     }
   }
 
   if (pcity->waste[otype] != 0) {
-    cat_snprintf(buf, bufsz,
-		 _("%+4d : Waste\n"), -pcity->waste[otype]);
+    char *fmt;
+    switch (otype) {
+      case O_SHIELD:
+      default: /* FIXME other output types? */
+        fmt = _("%+4d : Waste\n");
+        break;
+      case O_TRADE:
+        fmt = _("%+4d : Corruption\n");
+        break;
+    }
+    cat_snprintf(buf, bufsz, fmt, -pcity->waste[otype]);
     total -= pcity->waste[otype];
   }
 
@@ -481,6 +509,52 @@ void get_city_dialog_output_text(const struct city *pcity,
 	       _("%4d : Total surplus"), pcity->surplus[otype]);
 }
 
+/**************************************************************************
+  Return text describing the chance for a plague.
+**************************************************************************/
+void get_city_dialog_illness_text(const struct city *pcity,
+                                  char *buf, size_t bufsz)
+{
+  int illness, ill_base, ill_size, ill_trade, ill_pollution;
+  struct effect_list *plist;
+
+  buf[0] = '\0';
+
+  if (!game.info.illness_on) {
+    cat_snprintf(buf, bufsz, _("Illness deactivated in ruleset."));
+    return;
+  }
+
+  illness = city_illness_calc(pcity, &ill_base, &ill_size, &ill_trade,
+                              &ill_pollution);
+
+  cat_snprintf(buf, bufsz, _("%+2.1f : Risk from overcrowding\n"),
+               ((float)(ill_size) / 10.0));
+  cat_snprintf(buf, bufsz, _("%+2.1f : Risk from trade\n"),
+               ((float)(ill_trade) / 10.0));
+  cat_snprintf(buf, bufsz, _("%+2.1f : Risk from pollution\n"),
+               ((float)(ill_pollution) / 10.0));
+
+  plist = effect_list_new();
+
+  (void) get_city_bonus_effects(plist, pcity, NULL, EFT_HEALTH_PCT);
+
+  effect_list_iterate(plist, peffect) {
+    char buf2[512];
+
+    get_effect_req_text(peffect, buf2, sizeof(buf2));
+
+    cat_snprintf(buf, bufsz,
+                 _("%+2.1f : Bonus from %s\n"),
+                 -(0.1 * ill_base * peffect->value / 100), buf2);
+  } effect_list_iterate_end;
+  effect_list_free(plist);
+
+  cat_snprintf(buf, bufsz, _("==== : Adds up to\n"));
+  cat_snprintf(buf, bufsz, _("%2.1f : Total chance for a plague"),
+               ((float)(illness) / 10.0));
+}
+
 /**************************************************************************
   Return text describing the pollution output.
 **************************************************************************/
@@ -494,15 +568,15 @@ void get_city_dialog_pollution_text(const struct city *pcity,
   buf[0] = '\0';
 
   cat_snprintf(buf, bufsz,
-	       "%+4d : Pollution from shields\n", prod);
+	       _("%+4d : Pollution from shields\n"), prod);
   cat_snprintf(buf, bufsz,
-	       "%+4d : Pollution from citizens\n", pop);
+	       _("%+4d : Pollution from citizens\n"), pop);
   cat_snprintf(buf, bufsz,
-	       "%+4d : Pollution modifier\n", mod);
+	       _("%+4d : Pollution modifier\n"), mod);
   cat_snprintf(buf, bufsz,
-	       "==== : Adds up to\n");
+	       _("==== : Adds up to\n"));
   cat_snprintf(buf, bufsz,
-	       "%4d : Total surplus", pollu);
+	       _("%4d : Total surplus"), pollu);
 }
 
 /**************************************************************************
@@ -564,9 +638,9 @@ void city_rotate_specialist(struct city *pcity, int citizen_index)
   /* Loop through all specialists in order until we find a usable one
    * (or run out of choices). */
   to = from;
-  assert(to >= 0 && to < SP_COUNT);
+  assert(to >= 0 && to < specialist_count());
   do {
-    to = (to + 1) % SP_COUNT;
+    to = (to + 1) % specialist_count();
   } while (to != from && !city_can_use_specialist(pcity, to));
 
   if (from != to) {
@@ -583,7 +657,7 @@ void activate_all_units(struct tile *ptile)
   struct unit *pmyunit = NULL;
 
   unit_list_iterate(punit_list, punit) {
-    if (game.player_ptr == unit_owner(punit)) {
+    if (unit_owner(punit) == client.conn.playing) {
       /* Activate this unit. */
       pmyunit = punit;
       request_new_unit_activity(punit, ACTIVITY_IDLE);
@@ -598,10 +672,11 @@ void activate_all_units(struct tile *ptile)
 /**************************************************************************
   Change the production of a given city.  Return the request ID.
 **************************************************************************/
-int city_change_production(struct city *pcity, struct city_production target)
+int city_change_production(struct city *pcity, struct universal target)
 {
-  return dsend_packet_city_change(&aconnection, pcity->id,
-				  target.value, target.is_unit);
+  return dsend_packet_city_change(&client.conn, pcity->id,
+				  target.kind,
+				  universal_number(&target));
 }
 
 /**************************************************************************
@@ -609,16 +684,64 @@ int city_change_production(struct city *pcity, struct city_production target)
 
   Note that the worklist does NOT include the current production.
 **************************************************************************/
-int city_set_worklist(struct city *pcity, struct worklist *pworklist)
+int city_set_worklist(struct city *pcity, const struct worklist *pworklist)
 {
-  struct worklist copy;
+  return dsend_packet_city_worklist(&client.conn, pcity->id, pworklist);
+}
 
-  copy_worklist(&copy, pworklist);
 
-  /* Don't send the worklist name to the server. */
-  copy.name[0] = '\0';
+/**************************************************************************
+  Commit the changes to the worklist for the city.
+**************************************************************************/
+void city_worklist_commit(struct city *pcity, struct worklist *pwl)
+{
+  int k;
 
-  return dsend_packet_city_worklist(&aconnection, pcity->id, &copy);
+  /* Update the worklist.  Remember, though -- the current build
+     target really isn't in the worklist; don't send it to the server
+     as part of the worklist.  Of course, we have to search through
+     the current worklist to find the first _now_available_ build
+     target (to cope with players who try mean things like adding a
+     Battleship to a city worklist when the player doesn't even yet
+     have the Map Making tech).  */
+
+  for (k = 0; k < MAX_LEN_WORKLIST; k++) {
+    int same_as_current_build;
+    struct universal target;
+
+    if (!worklist_peek_ith(pwl, &target, k))
+      break;
+
+    same_as_current_build = are_universals_equal(&pcity->production, &target);
+
+    /* Very special case: If we are currently building a wonder we
+       allow the construction to continue, even if we the wonder is
+       finished elsewhere, ie unbuildable. */
+    if (k == 0
+     && VUT_IMPROVEMENT == target.kind
+     && is_wonder(target.value.building)
+     && same_as_current_build) {
+      worklist_remove(pwl, k);
+      break;
+    }
+
+    /* If it can be built... */
+    if (can_city_build_now(pcity, target)) {
+      /* ...but we're not yet building it, then switch. */
+      if (!same_as_current_build) {
+        /* Change the current target */
+	city_change_production(pcity, target);
+      }
+
+      /* This item is now (and may have always been) the current
+         build target.  Drop it out of the worklist. */
+      worklist_remove(pwl, k);
+      break;
+    }
+  }
+
+  /* Send the rest of the worklist on its way. */
+  city_set_worklist(pcity, pwl);
 }
 
 
@@ -631,17 +754,13 @@ int city_set_worklist(struct city *pcity, struct worklist *pworklist)
   Note that the queue DOES include the current production.
 **************************************************************************/
 static bool base_city_queue_insert(struct city *pcity, int position,
-				   struct city_production item)
+				   struct universal item)
 {
   if (position == 0) {
-    struct city_production old = pcity->production;
+    struct universal old = pcity->production;
 
     /* Insert as current production. */
-    if (item.is_unit
-	&& !can_build_unit_direct(pcity, utype_by_number(item.value))) {
-      return FALSE;
-    }
-    if (!item.is_unit && !can_build_improvement_direct(pcity, item.value)) {
+    if (!can_city_build_direct(pcity, item)) {
       return FALSE;
     }
 
@@ -653,12 +772,7 @@ static bool base_city_queue_insert(struct city *pcity, int position,
   } else if (position >= 1
 	     && position <= worklist_length(&pcity->worklist)) {
     /* Insert into middle. */
-    if (item.is_unit
-	&& !can_eventually_build_unit(pcity, utype_by_number(item.value))) {
-      return FALSE;
-    }
-    if (!item.is_unit
-	&& !can_eventually_build_improvement(pcity, item.value)) {
+    if (!can_city_build_later(pcity, item)) {
       return FALSE;
     }
     if (!worklist_insert(&pcity->worklist, item, position - 1)) {
@@ -666,12 +780,7 @@ static bool base_city_queue_insert(struct city *pcity, int position,
     }
   } else {
     /* Insert at end. */
-    if (item.is_unit
-	&& !can_eventually_build_unit(pcity, utype_by_number(item.value))) {
-      return FALSE;
-    }
-    if (!item.is_unit
-	&& !can_eventually_build_improvement(pcity, item.value)) {
+    if (!can_city_build_later(pcity, item)) {
       return FALSE;
     }
     if (!worklist_append(&pcity->worklist, item)) {
@@ -687,7 +796,7 @@ static bool base_city_queue_insert(struct city *pcity, int position,
   Note that the queue DOES include the current production.
 **************************************************************************/
 bool city_queue_insert(struct city *pcity, int position,
-		       struct city_production item)
+		       struct universal item)
 {
   if (base_city_queue_insert(pcity, position, item)) {
     city_set_worklist(pcity, &pcity->worklist);
@@ -704,7 +813,7 @@ bool city_queue_insert(struct city *pcity, int position,
 **************************************************************************/
 bool city_queue_clear(struct city *pcity)
 {
-  init_worklist(&pcity->worklist);
+  worklist_init(&pcity->worklist);
 
   return TRUE;
 }
@@ -715,7 +824,7 @@ bool city_queue_clear(struct city *pcity)
   Note that the queue DOES include the current production.
 **************************************************************************/
 bool city_queue_insert_worklist(struct city *pcity, int position,
-				struct worklist *worklist)
+				const struct worklist *worklist)
 {
   bool success = FALSE;
 
@@ -746,7 +855,7 @@ bool city_queue_insert_worklist(struct city *pcity, int position,
 **************************************************************************/
 void city_get_queue(struct city *pcity, struct worklist *pqueue)
 {
-  copy_worklist(pqueue, &pcity->worklist);
+  worklist_copy(pqueue, &pcity->worklist);
 
   /* The GUI wants current production to be in the task list, but the
      worklist API wants it out for reasons unknown. Perhaps someone enjoyed
@@ -761,12 +870,12 @@ void city_get_queue(struct city *pcity, struct worklist *pqueue)
 /**************************************************************************
   Set the city current production and the worklist, like it should be.
 **************************************************************************/
-bool city_set_queue(struct city *pcity, struct worklist *pqueue)
+bool city_set_queue(struct city *pcity, const struct worklist *pqueue)
 {
   struct worklist copy;
-  struct city_production target;
+  struct universal target;
 
-  copy_worklist(&copy, pqueue);
+  worklist_copy(&copy, pqueue);
 
   /* The GUI wants current production to be in the task list, but the
      worklist API wants it out for reasons unknown. Perhaps someone enjoyed
@@ -774,8 +883,7 @@ bool city_set_queue(struct city *pcity, struct worklist *pqueue)
   if (worklist_peek(&copy, &target)) {
 
     if (!city_can_change_build(pcity)
-        && (target.value != pcity->production.value
-            || target.is_unit != pcity->production.is_unit)) {
+        && !are_universals_equal(&pcity->production, &target)) {
       /* We cannot change production to one from worklist.
        * Do not replace old worklist with new one. */
       return FALSE;
@@ -806,13 +914,15 @@ bool city_can_buy(const struct city *pcity)
    * doesn't allow for error messages.  It doesn't check the cost of
    * buying; that's handled separately (and with an error message). */
   return (can_client_issue_orders()
-	  && pcity
-	  && city_owner(pcity) == game.player_ptr
+	  && NULL != pcity
+	  && city_owner(pcity) == client.conn.playing
 	  && pcity->turn_founded != game.info.turn
 	  && !pcity->did_buy
-	  && (pcity->production.is_unit || !improvement_has_flag(pcity->production.value, IF_GOLD))
-	  && !(pcity->production.is_unit && pcity->anarchy != 0)
-	  && city_buy_cost(pcity) > 0);
+	  && (VUT_UTYPE == pcity->production.kind
+	     || !improvement_has_flag(pcity->production.value.building, IF_GOLD))
+	  && !(VUT_UTYPE == pcity->production.kind 
+	      && pcity->anarchy != 0)
+	  && city_production_buy_gold_cost(pcity) > 0);
 }
 
 /**************************************************************************
@@ -820,7 +930,7 @@ bool city_can_buy(const struct city *pcity)
 **************************************************************************/
 int city_sell_improvement(struct city *pcity, Impr_type_id sell_id)
 {
-  return dsend_packet_city_sell(&aconnection, pcity->id, sell_id);
+  return dsend_packet_city_sell(&client.conn, pcity->id, sell_id);
 }
 
 /**************************************************************************
@@ -828,7 +938,7 @@ int city_sell_improvement(struct city *pcity, Impr_type_id sell_id)
 **************************************************************************/
 int city_buy_production(struct city *pcity)
 {
-  return dsend_packet_city_buy(&aconnection, pcity->id);
+  return dsend_packet_city_buy(&client.conn, pcity->id);
 }
 
 /**************************************************************************
@@ -837,7 +947,7 @@ int city_buy_production(struct city *pcity)
 int city_change_specialist(struct city *pcity, Specialist_type_id from,
 			   Specialist_type_id to)
 {
-  return dsend_packet_city_change_specialist(&aconnection, pcity->id, from,
+  return dsend_packet_city_change_specialist(&client.conn, pcity->id, from,
 					     to);
 }
 
@@ -847,13 +957,18 @@ int city_change_specialist(struct city *pcity, Specialist_type_id from,
 **************************************************************************/
 int city_toggle_worker(struct city *pcity, int city_x, int city_y)
 {
-  assert(is_valid_city_coords(city_x, city_y));
+  struct tile *ptile = city_map_to_tile(city_tile(pcity), city_x, city_y);
 
-  if (pcity->city_map[city_x][city_y] == C_TILE_WORKER) {
-    return dsend_packet_city_make_specialist(&aconnection, pcity->id, city_x,
+  assert(is_valid_city_coords(city_x, city_y));
+  if (NULL == ptile) {
+    return 0;
+  }
+
+  if (NULL != tile_worked(ptile) && tile_worked(ptile) == pcity) {
+    return dsend_packet_city_make_specialist(&client.conn, pcity->id, city_x,
 					     city_y);
-  } else if (pcity->city_map[city_x][city_y] == C_TILE_EMPTY) {
-    return dsend_packet_city_make_worker(&aconnection, pcity->id, city_x,
+  } else if (city_can_work_tile(pcity, ptile)) {
+    return dsend_packet_city_make_worker(&client.conn, pcity->id, city_x,
 					 city_y);
   } else {
     return 0;
@@ -865,5 +980,5 @@ int city_toggle_worker(struct city *pcity, int city_x, int city_y)
 **************************************************************************/
 int city_rename(struct city *pcity, const char *name)
 {
-  return dsend_packet_city_rename(&aconnection, pcity->id, name);
+  return dsend_packet_city_rename(&client.conn, pcity->id, name);
 }
