@@ -16,18 +16,27 @@
 #endif
 
 #include <assert.h>
+#include <math.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
+/* utility */
 #include "fcintl.h"
+#include "log.h"
 #include "support.h"
 
+/* common */
 #include "city.h"
 #include "game.h"
 #include "map.h"
+#include "specialist.h"
 #include "unitlist.h"
 
+/* agents */
 #include "cma_fec.h"
+
+/* client */
 #include "options.h"
 
 #include "cityrepdata.h"
@@ -60,8 +69,8 @@ static const char *cr_entry_hstate_concise(const struct city *pcity,
 					   const void *data)
 {
   static char buf[4];
-  my_snprintf(buf, sizeof(buf), "%s", (city_celebrating(pcity) ? "+" :
-				       (city_unhappy(pcity) ? "-" : " ")));
+  my_snprintf(buf, sizeof(buf), "%s", (city_celebrating(pcity) ? "*" :
+				       (city_unhappy(pcity) ? "X" : " ")));
   return buf;
 }
 
@@ -70,7 +79,7 @@ static const char *cr_entry_hstate_verbose(const struct city *pcity,
 {
   static char buf[16];
   my_snprintf(buf, sizeof(buf), "%s",
-	      (city_celebrating(pcity) ? Q_("?city_state:Rapture") :
+	      (city_celebrating(pcity) ? Q_("?city_state:Celebrating") :
 	       (city_unhappy(pcity) ? Q_("?city_state:Disorder") :
 		Q_("?city_state:Peace"))));
   return buf;
@@ -132,7 +141,7 @@ static const char *cr_entry_specialist(const struct city *pcity,
   static char buf[8];
   const struct specialist *sp = data;
 
-  my_snprintf(buf, sizeof(buf), "%2d", pcity->specialists[sp->index]);
+  my_snprintf(buf, sizeof(buf), "%2d", pcity->specialists[specialist_index(sp)]);
   return buf;
 }
 
@@ -357,25 +366,15 @@ static const char *cr_entry_building(const struct city *pcity,
     worklist_is_empty(&pcity->worklist) ? "" :
     concise_city_production ? "+" : _("(worklist)");
 	
-  if (!pcity->production.is_unit
-      && improvement_has_flag(pcity->production.value, IF_GOLD)) {
+  if (city_production_has_flag(pcity, IF_GOLD)) {
     my_snprintf(buf, sizeof(buf), "%s (%d)%s",
-		get_impr_name_ex(pcity, pcity->production.value),
+		city_production_name_translation(pcity),
 		MAX(0, pcity->surplus[O_SHIELD]), from_worklist);
   } else {
-    const char *name;
-    int cost;
-
-    if(pcity->production.is_unit) {
-      name = utype_name_translation(utype_by_number(pcity->production.value));
-      cost = unit_build_shield_cost(utype_by_number(pcity->production.value));
-    } else {
-      name = get_impr_name_ex(pcity, pcity->production.value);
-      cost = impr_build_shield_cost(pcity->production.value);
-    }
-
-    my_snprintf(buf, sizeof(buf), "%s (%d/%d)%s", name,
-		pcity->shield_stock, cost,
+    my_snprintf(buf, sizeof(buf), "%s (%d/%d)%s",
+		city_production_name_translation(pcity),
+		pcity->shield_stock,
+		city_production_build_shield_cost(pcity),
 		from_worklist);
   }
 
@@ -385,17 +384,18 @@ static const char *cr_entry_building(const struct city *pcity,
 static const char *cr_entry_build_cost(const struct city *pcity,
 				  const void *data)
 {
-  int price = city_buy_cost(pcity);
-  int turns = city_turns_to_build(pcity, pcity->production, TRUE);
   char bufone[8];
   char buftwo[8];
   static char buf[32];
+  int price;
+  int turns;
 
-  if (!pcity->production.is_unit
-      && improvement_has_flag(pcity->production.value, IF_GOLD)) {
+  if (city_production_has_flag(pcity, IF_GOLD)) {
     my_snprintf(buf, sizeof(buf), "*");
     return buf;
   }
+  price = city_production_buy_gold_cost(pcity);
+  turns = city_production_turns_to_build(pcity, TRUE);
 
   if (price > 99999) {
     my_snprintf(bufone, sizeof(bufone), "---");
@@ -427,6 +427,19 @@ static const char *cr_entry_waste(const struct city *pcity,
   return buf;
 }
 
+static const char *cr_entry_plague_risk(const struct city *pcity,
+                                        const void *data)
+{
+  static char buf[8];
+  if (!game.info.illness_on) {
+    my_snprintf(buf, sizeof(buf), " -.-");
+  } else {
+    my_snprintf(buf, sizeof(buf), "%4.1f",
+                (float)city_illness_calc(pcity, NULL, NULL, NULL, NULL)/10.0);
+  }
+  return buf;
+}
+
 static const char *cr_entry_cma(const struct city *pcity,
 				const void *data)
 {
@@ -443,17 +456,21 @@ static const char *cr_entry_cma(const struct city *pcity,
 #define FUNC_TAG(var)  cr_entry_##var, #var 
 
 static const struct city_report_spec base_city_report_specs[] = {
+  { TRUE, -15, 0, NULL,  N_("?city:Name"), N_("City Name"),
+    NULL, FUNC_TAG(cityname) },
+  { TRUE,   2, 1, NULL,  N_("?size [short]:Sz"), N_("Size"),
+    NULL, FUNC_TAG(size) },
+  { TRUE,  -8, 1, NULL,  N_("State"),   N_("Celebrating/Peace/Disorder"),
+    NULL, FUNC_TAG(hstate_verbose) },
+  { FALSE,  1, 1, NULL,  NULL,          N_("Concise *=Celebrating, X=Disorder"),
+    NULL, FUNC_TAG(hstate_concise) },
+
   /* Specialists grouped with init_city_report_game_data specialists */ 
   { FALSE,  7, 1, N_("Special"),
     N_("?entertainers/scientists/taxmen:E/S/T"),
     N_("Entertainers, Scientists, Taxmen"),
     NULL, FUNC_TAG(specialists) },
 
-  { FALSE,  1, 1, NULL,  NULL,          N_("Concise + Rapture, - Disorder"),
-    NULL, FUNC_TAG(hstate_concise) },
-  { TRUE,  -8, 1, NULL,  N_("State"),   N_("Rapture/Peace/Disorder"),
-    NULL, FUNC_TAG(hstate_verbose) },
-
   { FALSE, 2, 1, NULL, N_("?Happy workers:H"), N_("Workers: Happy"),
     NULL, FUNC_TAG(happy) },
   { FALSE, 2, 1, NULL, N_("?Content workers:C"), N_("Workers: Content"),
@@ -462,7 +479,7 @@ static const struct city_report_spec base_city_report_specs[] = {
     NULL, FUNC_TAG(unhappy) },
   { FALSE, 2, 1, NULL, N_("?Angry workers:A"), N_("Workers: Angry"),
     NULL, FUNC_TAG(angry) },
-  { TRUE, 10, 1, N_("Workers"),
+  { TRUE, 10, 1, N_("?city:Workers"),
     N_("?happy/content/unhappy/angry:H/C/U/A"),
     N_("Workers: Happy, Content, Unhappy, Angry"),
     NULL, FUNC_TAG(workers) },
@@ -485,12 +502,6 @@ static const struct city_report_spec base_city_report_specs[] = {
     N_("?Stock/Target:(Have/Need)"),
     N_("Turns until growth/famine"),
     NULL, FUNC_TAG(growturns) },
-  { TRUE,   2, 1, NULL,  N_("?size [short]:Sz"), N_("Size"),
-    NULL, FUNC_TAG(size) },
-
-  /* city name closer to center, try to keep within scroll window */
-  { TRUE, -15, 0, NULL,  N_("?city:Name"), N_("City Name"),
-    NULL, FUNC_TAG(cityname) },
 
   { TRUE,  10, 1, N_("Surplus"), N_("?food/production/trade:F/P/T"),
                                  N_("Surplus: Food, Production, Trade"),
@@ -520,10 +531,13 @@ static const struct city_report_spec base_city_report_specs[] = {
     NULL, FUNC_TAG(num_trade) },
   { FALSE,  3, 1, NULL, N_("?pollution [short]:Pol"), N_("Pollution"),
     NULL, FUNC_TAG(pollution) },
+  { FALSE,  4, 1, N_("?plague risk [short]:Pla"), N_("(%)"), N_("Plague risk"),
+    NULL, FUNC_TAG(plague_risk) },
   { FALSE, 15, 1, NULL, N_("?cma:Governor"), N_("Citizen Governor"),
     NULL, FUNC_TAG(cma) },
 
   { TRUE,   9, 1, N_("Production"), N_("Turns/Buy"),
+  /*N_("Turns or gold to complete production"), future menu needs translation */
     N_("Production"),
     NULL, FUNC_TAG(build_cost) },
   { TRUE,   0, 1, N_("Currently Building"),
@@ -562,24 +576,26 @@ void init_city_report_game_data(void)
   struct city_report_spec *p;
   int i;
 
-  num_creport_cols = ARRAY_SIZE(base_city_report_specs) + SP_COUNT;
+  num_creport_cols = ARRAY_SIZE(base_city_report_specs) + specialist_count();
   city_report_specs
     = fc_realloc(city_report_specs,
 		 num_creport_cols * sizeof(*city_report_specs));
   p = &city_report_specs[0];
 
-  specialist_type_iterate(sp) {
+  specialist_type_iterate(i) {
+    struct specialist *s = specialist_by_number(i);
     p->show = FALSE;
     p->width = 2;
     p->space = 1;
     p->title1 = Q_("?specialist:S");
-    p->title2 = Q_(get_specialist(sp)->short_name);
-    my_snprintf(explanation[sp], sizeof(explanation[sp]), "%s: %s",
-		_("Specialists"), _(get_specialist(sp)->name));
-    p->explanation = explanation[sp];
-    p->data = get_specialist(sp);
+    p->title2 = specialist_abbreviation_translation(s);
+    my_snprintf(explanation[i], sizeof(explanation[i]),
+		_("Specialists: %s"),
+		specialist_name_translation(s));
+    p->explanation = explanation[i];
+    p->data = s;
     p->func = cr_entry_specialist;
-    p->tagname = get_specialist(sp)->name;
+    p->tagname = specialist_rule_name(s);
     p++;
   } specialist_type_iterate_end;
 
@@ -597,7 +613,7 @@ void init_city_report_game_data(void)
     p++;
   }
 
-  assert(NUM_CREPORT_COLS == ARRAY_SIZE(base_city_report_specs) + SP_COUNT);
+  assert(NUM_CREPORT_COLS == ARRAY_SIZE(base_city_report_specs) + specialist_count());
 }
 
 /**********************************************************************
@@ -606,8 +622,8 @@ void init_city_report_game_data(void)
   it's better than sorting alphabetically.
 
   The GUI gives us two values to compare (as strings).  We try to split
-  them into an array of integer and string fields, then we compare
-  lexicographically.  Two integer fields are compared in the obvious
+  them into an array of numeric and string fields, then we compare
+  lexicographically.  Two numeric fields are compared in the obvious
   way, two character fields are compared alphabetically.  Arbitrarily, a
   numeric field is sorted before a character field (for "justification"
   note that numbers are before letters in the ASCII table).
@@ -617,7 +633,7 @@ void init_city_report_game_data(void)
    A datum_vector represents a long string of alternating strings and
    numbers. */
 struct datum {
-  long numeric_value;
+  float numeric_value;
   char *string_value;
   bool is_numeric;
 };
@@ -640,10 +656,10 @@ static void init_datum_string(struct datum *dat, const char *left,
 }
 
 /**********************************************************************
-  Init a datum from a number (a long because we happen to use
-  strtol).
+  Init a datum from a number (a float because we happen to use
+  strtof).
 **********************************************************************/
-static void init_datum_number(struct datum *dat, long val)
+static void init_datum_number(struct datum *dat, float val)
 {
   dat->is_numeric = TRUE;
   dat->numeric_value = val;
@@ -670,7 +686,15 @@ static int datum_compare(const struct datum *a, const struct datum *b)
 {
   if(a->is_numeric == b->is_numeric) {
     if(a->is_numeric) {
-      return a->numeric_value - b->numeric_value;
+      if (a->numeric_value == b->numeric_value) {
+        return 0;
+      } else if (a->numeric_value < b->numeric_value) {
+        return -1;
+      } else if (a->numeric_value > b->numeric_value) {
+        return +1;
+      } else {
+        return 0; /* shrug */
+      }
     } else {
       return strcmp(a->string_value, b->string_value);
     }
@@ -718,15 +742,15 @@ static void split_string(struct datum_vector *data, const char *str)
   string_start = str;
   while(*str) {
     char *endptr;
-    long value;
+    float value;
 
-    value = strtol(str, &endptr, 0);
-    if(endptr == str) {
-      /* that wasn't an integer; go on */
+    value = strtof(str, &endptr);
+    if(endptr == str || !isfinite(value)) {
+      /* that wasn't a sensible number; go on */
       str++;
     } else {
-      /* that was an integer, so stop the string we were parsing, add 
-         it (unless it's empty), then add the integer we just parsed */
+      /* that was a number, so stop the string we were parsing, add 
+         it (unless it's empty), then add the number we just parsed */
       struct datum d;
 
       if(str != string_start) {
