@@ -15,26 +15,24 @@
 
 #include "shared.h"		/* bool type */
 
-struct city;
-
-/* abbreviate long city names to this length in the city report: */
-#define REPORT_CITYNAME_ABBREV 15
+#include "fc_types.h"
 
 /* Number of city report columns: have to set this manually now... */
-#define NUM_CREPORT_COLS 14
+#define NUM_CREPORT_COLS (num_city_report_spec())
 
 struct city_report_spec {
   bool show;			/* modify this to customize */
   int width;			/* 0 means variable; rightmost only */
   int space;			/* number of leading spaces (see below) */
-  char *title1;			/* already translated or NULL */
-  char *title2;			/* already translated or NULL */
-  char *explanation;		/* already translated */ 
-  char *(*func)(struct city*);
-  char *tagname;		/* for save_options */
+  const char *title1;		/* already translated or NULL */
+  const char *title2;		/* already translated or NULL */
+  const char *explanation;	/* already translated */ 
+  void *data;
+  const char *(*func)(const struct city * pcity, const void *data);
+  const char *tagname;		/* for save_options */
 };
 
-extern struct city_report_spec city_report_specs[];
+extern struct city_report_spec *city_report_specs;
 
 /* Use tagname rather than index for load/save, because later
    additions won't necessarily be at the end.
@@ -59,8 +57,10 @@ extern struct city_report_spec city_report_specs[];
 */
 int num_city_report_spec(void);
 bool *city_report_spec_show_ptr(int i);
-char *city_report_spec_tagname(int i);
+const char *city_report_spec_tagname(int i);
 
-void init_city_report_data(void);
+void init_city_report_game_data(void);
+
+int cityrepfield_compare(const char *field1, const char *field2);
 
 #endif  /* FC__CITYREPDATA_H */
