@@ -14,6 +14,7 @@
 #define FC__CONTROL_H
 
 #include "packets.h"
+#include "unitlist.h"
 
 enum cursor_hover_state {
   HOVER_NONE = 0,
@@ -24,31 +25,19 @@ enum cursor_hover_state {
   HOVER_PATROL
 };
 
-enum cursor_action_state {
-  CURSOR_ACTION_DEFAULT,
-  CURSOR_ACTION_GOTO,
-  CURSOR_ACTION_SELECT,
-  CURSOR_ACTION_INVALID,
-  CURSOR_ACTION_ATTACK,
-  CURSOR_ACTION_NUKE,
-  CURSOR_ACTION_PARATROOPER,
-  CURSOR_ACTION_WAIT
-};
-
 /* Selecting unit from a stack without popup. */
 enum quickselect_type {
   SELECT_POPUP = 0, SELECT_SEA, SELECT_LAND, SELECT_APPEND
 };
 
 void control_init(void);
-void control_done(void);
+void control_free(void);
 void control_unit_killed(struct unit *punit);
 
 void unit_change_battlegroup(struct unit *punit, int battlegroup);
 void unit_register_battlegroup(struct unit *punit);
 
 extern enum cursor_hover_state hover_state;
-extern enum cursor_action_state action_state;
 extern enum unit_activity connect_activity;
 extern enum unit_orders goto_last_order;
 extern bool non_ai_unit_focus;
@@ -73,7 +62,10 @@ void request_move_unit_direction(struct unit *punit, int dir);
 void request_new_unit_activity(struct unit *punit, enum unit_activity act);
 void request_new_unit_activity_targeted(struct unit *punit,
 					enum unit_activity act,
-					enum tile_special_type tgt);
+					enum tile_special_type tgt,
+                                        Base_type_id base);
+void request_new_unit_activity_base(struct unit *punit,
+				    const struct base_type *pbase);
 void request_unit_load(struct unit *pcargo, struct unit *ptransporter);
 void request_unit_unload(struct unit *pcargo);
 void request_unit_autosettlers(const struct unit *punit);
@@ -90,7 +82,7 @@ void request_unit_paradrop(struct unit_list *punits);
 void request_unit_patrol(void);
 void request_unit_pillage(struct unit *punit);
 void request_unit_sentry(struct unit *punit);
-void request_unit_unload_all(struct unit *punit);
+struct unit *request_unit_unload_all(struct unit *punit);
 void request_unit_airlift(struct unit *punit, struct city *pcity);
 void request_unit_return(struct unit *punit);
 void request_unit_upgrade(struct unit *punit);
@@ -120,13 +112,15 @@ void request_diplomat_action(enum diplomat_actions action, int dipl_id,
 void request_diplomat_answer(enum diplomat_actions action, int dipl_id,
 			     int target_id, int value);
 void request_toggle_city_outlines(void);
+void request_toggle_city_output(void);
 void request_toggle_map_grid(void);
 void request_toggle_map_borders(void);
+void request_toggle_city_full_bar(void);
 void request_toggle_city_names(void);
 void request_toggle_city_growth(void);
 void request_toggle_city_productions(void);
 void request_toggle_city_buycost(void);
-void request_toggle_city_traderoutes(void);
+void request_toggle_city_trade_routes(void);
 void request_toggle_terrain(void);
 void request_toggle_coastline(void);
 void request_toggle_roads_rails(void);
@@ -137,6 +131,8 @@ void request_toggle_specials(void);
 void request_toggle_pollution(void);
 void request_toggle_cities(void);
 void request_toggle_units(void);
+void request_toggle_unit_solid_bg(void);
+void request_toggle_unit_shields(void);
 void request_toggle_focus_unit(void);
 void request_toggle_fog_of_war(void);
 
@@ -169,11 +165,12 @@ void process_diplomat_arrival(struct unit *pdiplomat, int victim_id);
 
 void key_cancel_action(void);
 void key_center_capital(void);
+void key_city_full_bar_toggle(void);
 void key_city_names_toggle(void);
 void key_city_growth_toggle(void);
 void key_city_productions_toggle(void);
 void key_city_buycost_toggle(void);
-void key_city_traderoutes_toggle(void);
+void key_city_trade_routes_toggle(void);
 void key_terrain_toggle(void);
 void key_coastline_toggle(void);
 void key_roads_rails_toggle(void);
@@ -184,10 +181,13 @@ void key_specials_toggle(void);
 void key_pollution_toggle(void);
 void key_cities_toggle(void);
 void key_units_toggle(void);
+void key_unit_solid_bg_toggle(void);
+void key_unit_shields_toggle(void);
 void key_focus_unit_toggle(void);
 void key_fog_of_war_toggle(void);
 void key_end_turn(void);
 void key_city_outlines_toggle(void);
+void key_city_output_toggle(void);
 void key_map_grid_toggle(void);
 void key_map_borders_toggle(void);
 void key_recall_previous_focus_unit(void);
@@ -215,7 +215,7 @@ void key_unit_pillage(void);
 void key_unit_pollution(void);
 void key_unit_road(void);
 void key_unit_sentry(void);
-void key_unit_traderoute(void);
+void key_unit_trade_route(void);
 void key_unit_transform(void);
 void key_unit_unload_all(void);
 void key_unit_wait(void);
@@ -223,6 +223,10 @@ void key_unit_wakeup_others(void);
 void key_unit_assign_battlegroup(int battlegroup, bool append);
 void key_unit_select_battlegroup(int battlegroup, bool append);
 
+void key_editor_toggle(void);
+void key_editor_recalculate_borders(void);
+void key_editor_toggle_fogofwar(void);
+
 /* don't change this unless you also put more entries in data/Freeciv */
 #define MAX_NUM_UNITS_BELOW 4
 
