@@ -15,9 +15,11 @@
 #include <config.h>
 #endif
 
+/* common */
 #include "game.h"
 
-#include "civclient.h"
+/* client */
+#include "client_main.h"
 #include "control.h"
 
 #include "gotodlg.h"
@@ -27,10 +29,10 @@
 **************************************************************************/
 void popup_goto_dialog(void)
 {
-  if (get_client_state() != CLIENT_GAME_RUNNING_STATE) {
+  if (C_S_RUNNING != client_state()) {
     return;
   }
-  if (!get_unit_in_focus()) {
+  if (0 == get_num_units_in_focus()) {
     return;
   }
   /* PORTME */
