@@ -17,6 +17,7 @@
 
 #include <stdio.h>
 
+/* common & utility */
 #include "fcintl.h"
 #include "game.h"
 #include "log.h"
@@ -26,7 +27,8 @@
 #include "support.h"
 #include "unitlist.h"
 
-#include "civclient.h"
+/* client */
+#include "client_main.h"
 #include "climap.h"
 #include "climisc.h"
 #include "colors.h"
@@ -145,6 +147,10 @@ static struct {
 
 struct ct_string *text_templates[FONT_COUNT];
 
+static bool all_dirty = FALSE;
+
+struct region_list *region_list;
+
 /****************************************************************************
   Return the dimensions of the area (container widget; maximum size) for
   the overview.
@@ -298,7 +304,7 @@ void refresh_overview_canvas(void)
 
   struct ct_rect rect;
 
-  freelog(LOG_NORMAL, "refresh_overview_canvas()");
+  freelog(LOG_DEBUG, "refresh_overview_canvas()");
   whole_map_iterate(x, y) {
     overview_update_tile0(x, y);
   } whole_map_iterate_end;
@@ -365,7 +371,7 @@ void show_city_desc(struct canvas *pcanvas, int canvas_x, int canvas_y,
 
 #if 0
   /* try to trace that hard-to-find assert that we sometimes get */
-  freelog(LOG_NORMAL, "show_city_desc(%s) pcx=%d->%d (%d) pcy=%d->%d (%d)", city_name(pcity),
+  freelog(LOG_TEST, "show_city_desc(%s) pcx=%d->%d (%d) pcy=%d->%d (%d)", city_name(pcity),
           canvas_x, canvas_x+tileset_tile_width(tileset) / 2, all_rect.width,
           canvas_y, canvas_y+tileset_tile_height(tileset), all_rect.height);
 #endif
@@ -383,7 +389,7 @@ void show_city_desc(struct canvas *pcanvas, int canvas_x, int canvas_y,
   line1_b = ct_string_clone4(style->growth_template, buffer2,
 			    enum_color_to_be_color(color));
 
-  if (draw_city_productions && (city_owner(pcity) == game.player_idx)) {
+  if (draw_city_productions && (city_owner(pcity) == client.conn.playing)) {
     get_city_mapview_production(pcity, buffer, sizeof(buffer));
     line2 =
 	ct_string_clone4(style->prod_template, buffer,
@@ -450,8 +456,7 @@ void flush_mapcanvas(int canvas_x, int canvas_y,
   freelog(LOG_DEBUG, "flush_mapcanvas=%s", ct_rect_to_string(&rect));
   be_copy_osda_to_osda(sw_window_get_canvas_background(mapview_window),
 		       mapview.store->osda, &size, &pos, &pos);
-  sw_window_canvas_background_region_needs_repaint(mapview_window,
-						   &rect);
+  sw_window_canvas_background_region_needs_repaint(mapview_window, &rect);
 }
 
 /**************************************************************************
@@ -461,10 +466,14 @@ void flush_mapcanvas(int canvas_x, int canvas_y,
 void dirty_rect(int canvas_x, int canvas_y,
 		int pixel_width, int pixel_height)
 {
-  struct ct_rect rect = { canvas_x, canvas_y, pixel_width, pixel_height };
-
-  //freelog(LOG_NORMAL, "dirty_rect(...)");
-  sw_window_canvas_background_region_needs_repaint(mapview_window, &rect);
+  if (!all_dirty) {
+    struct ct_rect *rect = fc_malloc(sizeof(*rect));
+  
+    *rect = (struct ct_rect){ canvas_x, canvas_y, pixel_width, pixel_height };
+  
+    //freelog(LOG_TEST, "dirty_rect(...)");
+    region_list_append(region_list, rect);
+  }
 }
 
 /**************************************************************************
@@ -473,11 +482,7 @@ void dirty_rect(int canvas_x, int canvas_y,
 **************************************************************************/
 void dirty_all(void)
 {
-  struct ct_rect rect;
-
-  sw_widget_get_bounds(mapview_window, &rect);
-  sw_window_canvas_background_region_needs_repaint(mapview_window, 
-                                                   &rect);
+  all_dirty = TRUE;
 }
 
 /**************************************************************************
@@ -487,7 +492,22 @@ void dirty_all(void)
 **************************************************************************/
 void flush_dirty(void)
 {
-  flush_mapcanvas(0, 0, mapview.width, mapview.height);
+  if (all_dirty) {
+    region_list_iterate(region_list, region) {
+      free(region);
+    } region_list_iterate_end;
+    region_list_clear(region_list);
+    
+    flush_mapcanvas(0, 0, mapview.width, mapview.height);
+    all_dirty = FALSE;
+  } else {
+    region_list_iterate(region_list, region) {
+      flush_mapcanvas(region->x, region->y, region->width, region->height);
+      free(region);
+    } region_list_iterate_end;
+    region_list_clear(region_list);
+  }
+  sw_paint_all();
 }
 
 /**************************************************************************
@@ -653,7 +673,7 @@ static void update_focus_tile_list2(void)
 
   ptile = get_focus_tile();
   unit_list_iterate(ptile->units, aunit) {
-    if (game.info.player_idx == unit_owner(aunit)->player_no) {
+    if (unit_owner(aunit) == client.conn.playing) {
       set_unit_focus(aunit);
       break;
     }
@@ -681,8 +701,8 @@ static void update_focus_tile_list2(void)
     item->info_text=mystrdup(popup_info_text(ptile));
   }
 
-  if (tile_get_city(ptile)) {
-    struct city *pcity=tile_get_city(ptile);
+  if (tile_city(ptile)) {
+    struct city *pcity=tile_city(ptile);
     struct tile_list2_item *item = &tile_list2.item[tile_list2.items];
 
     tile_list2.items++;
@@ -854,7 +874,64 @@ static void action_button_callback(struct sw_widget *widget, void *data)
 {
   char *action = (char *) data;
 
-  freelog(LOG_NORMAL, "action '%s' requested", action);
+  freelog(LOG_VERBOSE, "action_button_callback(): action '%s' requested", action);  
+  
+  if (strcmp(action, "unit_fortifying") == 0) {
+    key_unit_fortify();
+  } else if (strcmp(action, "unit_add_to_city") == 0) {
+    key_unit_build_city();
+  } else if (strcmp(action, "unit_build_city") == 0) {
+    key_unit_build_city();
+  } else if (strcmp(action, "unit_auto_explore") == 0) {
+    key_unit_auto_explore();
+  } else if (strcmp(action, "unit_irrigate") == 0) {
+    key_unit_irrigate();
+  } else if (strcmp(action, "unit_sentry") == 0) {
+    key_unit_sentry();
+  } else if (strcmp(action, "unit_return_nearest") == 0) {
+/*    key_? */
+  } else if (strcmp(action, "unit_disband") == 0) {
+    key_unit_disband();
+  } else if (strcmp(action, "unit_mine") == 0) {
+    key_unit_mine();
+  } else if (strcmp(action, "unit_road") == 0) {
+    key_unit_road();
+  } else if (strcmp(action, "unit_auto_settler") == 0) {
+    key_unit_auto_settle();
+  } else if (strcmp(action, "unit_connect") == 0) {
+/*    key_unit_connect(enum unit_activity activity);*/
+  } else if (strcmp(action, "unit_auto_attack") == 0) {
+/*    key_? */    
+  } else if (strcmp(action, "unit_homecity") == 0) {
+    key_unit_homecity();
+  } else if (strcmp(action, "unit_fortress") == 0) {
+    key_unit_fortress();
+  } else if (strcmp(action, "unit_pillage") == 0) {
+    key_unit_pillage();
+  } else if (strcmp(action, "unit_airbase") == 0) {
+    key_unit_airbase();
+  } else if (strcmp(action, "unit_transform") == 0) {
+    key_unit_transform();
+  } else if (strcmp(action, "unit_unload") == 0) {
+    key_unit_unload_all();
+  } else if (strcmp(action, "unit_goto") == 0) {
+    key_unit_goto();
+  } else if (strcmp(action, "unit_goto_city") == 0) {
+/*    key_? */
+  } else if (strcmp(action, "unit_airlift") == 0) {
+/*    key_? */
+  } else if (strcmp(action, "unit_patrol") == 0) {
+    key_unit_patrol();
+  } else if (strcmp(action, "unit_wait") == 0) {
+    key_unit_wait();
+  } else if (strcmp(action, "unit_railroad") == 0) {
+/*    key_? */
+  } else if (strcmp(action, "unit_done") == 0) {
+    key_unit_done();
+  } else {
+    freelog(LOG_VERBOSE,
+      "action_button_callback(): action request '%s' not handled", action);    
+  }
 }
 
 /**************************************************************************
@@ -993,7 +1070,7 @@ static void action_callback(const char *action)
       redraw_selection_rectangle();
     }
   } else {
-    freelog(LOG_NORMAL, "action '%s' requested", action);
+    freelog(LOG_VERBOSE, "action_callback() action '%s' requested", action);
   }
 }
 
@@ -1121,23 +1198,24 @@ static const char *info_get_value(const char *id)
     return textyear(game.info.year);
   } else if (strcmp(id, "gold") == 0) {
     my_snprintf(buffer, sizeof(buffer),
-		"%d", game.player_ptr->economic.gold);
+		"%d", client.conn.playing->economic.gold);
     return buffer;
   } else if (strcmp(id, "nation_name") == 0) {
-      return nation_plural_for_player(game.player_ptr);
+      return nation_plural_for_player(client.conn.playing);
   } else if (strcmp(id, "population") == 0) {
-      return population_to_text(civ_population(game.player_ptr));
+      return population_to_text(civ_population(client.conn.playing));
   } else if (strcmp(id, "general") == 0) {
       my_snprintf(buffer, sizeof(buffer),
 		  _("Population: %s\n"
 		"Year: %s\n"
 		"Gold %d\n"
 		"Tax: %d Lux: %d Sci: %d"),
-	      population_to_text(civ_population(game.player_ptr)),
-	      textyear(game.info.year), game.player_ptr->economic.gold,
-	      game.player_ptr->economic.tax,
-	      game.player_ptr->economic.luxury,
-	      game.player_ptr->economic.science);
+	      population_to_text(civ_population(client.conn.playing)),
+	      textyear(game.info.year),
+	      client.conn.playing->economic.gold,
+	      client.conn.playing->economic.tax,
+	      client.conn.playing->economic.luxury,
+	      client.conn.playing->economic.science);
       return buffer;
   } else if (strcmp(id, "focus_item") == 0) {
       return tile_list2.item[tile_list2.selected].info_text;
@@ -1147,11 +1225,12 @@ static const char *info_get_value(const char *id)
 		"Year: %s "
 		"Gold %d "
 		"Tax: %d Lux: %d Sci: %d"),
-	      population_to_text(civ_population(game.player_ptr)),
-	      textyear(game.info.year), game.player_ptr->economic.gold,
-	      game.player_ptr->economic.tax,
-	      game.player_ptr->economic.luxury,
-	      game.player_ptr->economic.science);
+	      population_to_text(civ_population(client.conn.playing)),
+	      textyear(game.info.year),
+	      client.conn.playing->economic.gold,
+	      client.conn.playing->economic.tax,
+	      client.conn.playing->economic.luxury,
+	      client.conn.playing->economic.science);
       return buffer;
 #endif
   } else {
@@ -1169,6 +1248,8 @@ void popup_mapcanvas(void)
   struct ct_rect rect;
   struct ct_size screen_size;
 
+  region_list = region_list_new();
+  
   be_screen_get_size(&screen_size);
 
   env.info_get_value = info_get_value;
@@ -1211,6 +1292,11 @@ void popup_mapcanvas(void)
 **************************************************************************/
 void popdown_mapcanvas(void)
 {
+  region_list_iterate(region_list, region) {
+    free(region);
+  } region_list_iterate_end;
+  region_list_free(region_list);
+  
   te_destroy_screen(screen);
 }
 
@@ -1227,9 +1313,13 @@ void draw_selection_rectangle(int canvas_x, int canvas_y, int w, int h)
 **************************************************************************/
 static void unshow_actions(void)
 {
+  struct ct_rect rect;
   int i;
 
   for (i = 0; i < actions_shown.actions; i++) {
+    sw_widget_get_bounds(actions_shown.action[i].widget, &rect);    
+    be_draw_region(sw_widget_get_osda(screen->window), &rect,
+                                       be_get_color(0, 0, 0, MIN_OPACITY));
     sw_window_remove(actions_shown.action[i].widget);
     actions_shown.action[i].widget = NULL;
     free(actions_shown.action[i].name);
@@ -1319,12 +1409,12 @@ static void fill_actions(void)
     if (unit_can_help_build_wonder_here(punit)) {
       ADD("unit_help_wonder");
     }
-    if (unit_can_est_traderoute_here(punit)) {
-      ADD("unit_traderoute");
+    if (unit_can_est_trade_route_here(punit)) {
+      ADD("unit_trade_route");
     }
 
     if (can_unit_add_or_build_city(punit)) {
-      if (tile_get_city(punit->tile)) {
+      if (tile_city(punit->tile)) {
 	ADD("unit_add_to_city");
       } else {
 	ADD("unit_build_city");
@@ -1348,7 +1438,7 @@ static void fill_actions(void)
     if (can_unit_do_connect(punit, ACTIVITY_IDLE)) {
       ADD("unit_connect");
     }
-    if (!(is_air_unit(punit) || is_heli_unit(punit))) {
+    if (!utype_fuel(unit_type(punit))) {
       ADD("unit_return_nearest");
     }
     if (!unit_has_type_flag(punit, F_UNDISBANDABLE)) {
@@ -1374,7 +1464,7 @@ static void fill_actions(void)
     struct city *pcity = item->pcity;
     int i;
 
-    if (game.player_ptr->economic.gold >= city_buy_cost(pcity)) {
+    if (city_production_buy_gold_cost(pcity) <= client.conn.playing->economic.gold) {
       ADD("city_buy");
     } else {
       ADD_DIS("city_buy");
@@ -1529,3 +1619,11 @@ void set_city_names_font_sizes(int my_city_names_font_size,
   freelog(LOG_ERROR, "Ignore set_city_names_font_sizes call.");
   /* PORTME */
 }
+
+/**************************************************************************
+  This function will change the current mouse cursor.
+**************************************************************************/
+void update_mouse_cursor(enum cursor_type new_cursor_type)
+{
+  /* PORT ME */
+}
