@@ -19,13 +19,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <math.h> /* sqrt */
 
 #include <gtk/gtk.h>
 #include "gtkpixcomm.h"
 
+/* common & utility */
 #include "city.h"
 #include "fcintl.h"
-#include "game.h"
 #include "government.h"
 #include "mem.h"
 #include "shared.h"
@@ -35,6 +36,8 @@
 #include "support.h"
 #include "version.h"
 
+/* client */
+#include "client_main.h"
 #include "climisc.h"
 #include "colors.h"
 #include "graphics.h"
@@ -60,7 +63,7 @@ static GtkWidget *help_frame;
 static GtkTextBuffer *help_text;
 static GtkWidget *help_text_sw;
 static GtkWidget *help_vbox;
-static GtkWidget *unit_tile;
+static GtkWidget *help_tile;
 static GtkWidget *help_box;
 static GtkWidget *help_itable;
 static GtkWidget *help_wtable;
@@ -107,9 +110,8 @@ static const char *help_tlabel_name[4][5] =
     { N_("Mine Rslt/Time:"),	NULL, NULL, N_("Trans. Rslt/Time:"),	NULL }
 };
 
-#define REQ_NONE _("None")
-#define REQ_NEVER _("(Never)")
-
+#define REQ_LABEL_NONE _("None")
+#define REQ_LABEL_NEVER _("(Never)")
 
 static void create_help_dialog(void);
 static void help_update_dialog(const struct help_item *pitem);
@@ -186,8 +188,9 @@ static void create_tech_tree(int tech, int levels, GtkTreeIter *parent)
   GtkTreeIter   l;
   GValue        value = { 0, };
 
-  if (advances[tech].req[0] == A_LAST && advances[tech].req[1] == A_LAST) {
-    bg = COLOR_REQTREE_UNREACHABLE;
+  if (advance_required(tech, AR_ONE) == A_LAST
+   && advance_required(tech, AR_TWO) == A_LAST) {
+    bg = COLOR_REQTREE_UNKNOWN;
 
     gtk_tree_store_append(tstore, &l, parent);
     help_advances[tech] = TRUE;
@@ -206,18 +209,18 @@ static void create_tech_tree(int tech, int levels, GtkTreeIter *parent)
   }
 
   bg = COLOR_REQTREE_BACKGROUND;
-  switch (get_invention(game.player_ptr, tech)) {
+  switch (player_invention_state(client.conn.playing, tech)) {
   case TECH_UNKNOWN:
-    bg = COLOR_REQTREE_UNREACHABLE;
+    bg = COLOR_REQTREE_UNKNOWN;
     break;
   case TECH_KNOWN:
     bg = COLOR_REQTREE_KNOWN;
     break;
-  case TECH_REACHABLE:
-    bg = COLOR_REQTREE_REACHABLE;
+  case TECH_PREREQS_KNOWN:
+    bg = COLOR_REQTREE_PREREQS_KNOWN;
     break;
   }
-  turns_to_tech = num_unknown_techs_for_goal(game.player_ptr, tech);
+  turns_to_tech = num_unknown_techs_for_goal(client.conn.playing, tech);
 
   /* l is the original in the tree. */
   original = !help_advances[tech];
@@ -226,7 +229,7 @@ static void create_tech_tree(int tech, int levels, GtkTreeIter *parent)
   help_advances[tech] = TRUE;
 
   g_value_init(&value, G_TYPE_STRING);
-  g_value_set_static_string(&value, advance_name_for_player(game.player_ptr, tech));
+  g_value_set_static_string(&value, advance_name_for_player(client.conn.playing, tech));
   gtk_tree_store_set_value(tstore, &l, 0, &value);
   g_value_unset(&value);
 
@@ -241,10 +244,10 @@ static void create_tech_tree(int tech, int levels, GtkTreeIter *parent)
 
   if (original) {
     /* only add children to orginals */
-    if (advances[tech].req[0] != A_NONE)
-      create_tech_tree(advances[tech].req[0], levels, &l);
-    if (advances[tech].req[1] != A_NONE)
-      create_tech_tree(advances[tech].req[1], levels, &l);
+    if (advance_required(tech, AR_ONE) != A_NONE)
+      create_tech_tree(advance_required(tech, AR_ONE), levels, &l);
+    if (advance_required(tech, AR_TWO) != A_NONE)
+      create_tech_tree(advance_required(tech, AR_TWO), levels, &l);
   }
   return;
 }
@@ -262,7 +265,7 @@ static void help_tech_tree_activated_callback(GtkTreeView *view,
 
   gtk_tree_model_get_iter(GTK_TREE_MODEL(tstore), &it, path);
   gtk_tree_model_get(GTK_TREE_MODEL(tstore), &it, 2, &tech, -1);
-  select_help_item_string(advance_name_for_player(game.player_ptr, tech), HELP_TECH);
+  select_help_item_string(advance_name_for_player(client.conn.playing, tech), HELP_TECH);
 }
 
 /**************************************************************************
@@ -295,9 +298,9 @@ static void help_hyperlink_callback(GtkWidget *w)
   /* FIXME: May be able to skip, or may need to modify, advances[A_NONE]
      below, depending on which i18n is done elsewhere.
   */
-  if (strcmp(s, REQ_NEVER) != 0
-      && strcmp(s, REQ_NONE) != 0
-      && strcmp(s, advance_name_translation(A_NONE)) != 0)
+  if (strcmp(s, REQ_LABEL_NEVER) != 0
+      && strcmp(s, REQ_LABEL_NONE) != 0
+      && strcmp(s, advance_name_translation(advance_by_number(A_NONE))) != 0)
     select_help_item_string(s, type);
 }
 
@@ -311,7 +314,7 @@ static GtkWidget *help_hyperlink_new(GtkWidget *label, enum help_page_type type)
   button = gtk_button_new();
   gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
   gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
-  gtk_widget_set_name(label, "help link");
+  gtk_widget_set_name(label, "help_link");
   gtk_container_add(GTK_CONTAINER(button), label);
   gtk_widget_show(button);
   g_signal_connect_swapped(button, "clicked",
@@ -340,14 +343,14 @@ static void help_box_hide(void)
 {
   gtk_widget_hide(help_box);
 
-  gtk_widget_hide(unit_tile);
+  gtk_widget_hide(help_tile);
 
   gtk_widget_hide(help_itable);
   gtk_widget_hide(help_wtable);
   gtk_widget_hide(help_utable);
   gtk_widget_hide(help_ttable);
   
-  gtk_widget_hide(unit_tile);
+  gtk_widget_hide(help_tile); /* FIXME: twice? */
 
   gtk_widget_hide(help_vbox);
   gtk_widget_hide(help_text_sw);
@@ -509,8 +512,8 @@ static void create_help_dialog(void)
   help_box = gtk_vbox_new(FALSE, 5);
   gtk_container_add(GTK_CONTAINER(help_frame), help_box);
 
-  unit_tile = gtk_pixcomm_new(tileset_full_tile_width(tileset), tileset_full_tile_height(tileset));
-  gtk_box_pack_start(GTK_BOX(help_box), unit_tile, FALSE, FALSE, 0);
+  help_tile = gtk_pixcomm_new(tileset_full_tile_width(tileset), tileset_full_tile_height(tileset));
+  gtk_box_pack_start(GTK_BOX(help_box), help_tile, FALSE, FALSE, 0);
 
   help_itable = gtk_table_new(1, 6, FALSE);
   gtk_box_pack_start(GTK_BOX(help_box), help_itable, FALSE, FALSE, 0);
@@ -525,7 +528,7 @@ static void create_help_dialog(void)
     } else {
       gtk_table_attach_defaults(GTK_TABLE(help_itable),
 			        help_ilabel[i], i, i+1, 0, 1);
-      gtk_widget_set_name(help_ilabel[i], "help label");
+      gtk_widget_set_name(help_ilabel[i], "help_label");
     }
     gtk_widget_show(help_ilabel[i]);
   }
@@ -543,7 +546,7 @@ static void create_help_dialog(void)
     } else {
       gtk_table_attach_defaults(GTK_TABLE(help_wtable),
 			        help_wlabel[i], i, i+1, 0, 1);
-      gtk_widget_set_name(help_wlabel[i], "help label");
+      gtk_widget_set_name(help_wlabel[i], "help_label");
     }
     gtk_widget_show(help_wlabel[i]);
   }
@@ -568,7 +571,7 @@ static void create_help_dialog(void)
       } else {
         gtk_table_attach_defaults(GTK_TABLE(help_utable),
 			          help_ulabel[j][i], i, i+1, j, j+1);
-        gtk_widget_set_name(help_ulabel[j][i], "help label");
+        gtk_widget_set_name(help_ulabel[j][i], "help_label");
       }
       gtk_widget_show(help_ulabel[j][i]);
     }
@@ -577,15 +580,24 @@ static void create_help_dialog(void)
   help_ttable = gtk_table_new(5, 5, FALSE);
   gtk_box_pack_start(GTK_BOX(help_box), help_ttable, FALSE, FALSE, 0);
 
-  for (i=0; i<5; i++) {
-    for (j=0; j<4; j++) {
+  for (j=0; j<4; j++) {
+    for (i=0; i<5; i++) {
       help_tlabel[j][i] =
 	  gtk_label_new(help_tlabel_name[j][i] ? _(help_tlabel_name[j][i]) : "");
-      gtk_widget_set_name(help_tlabel[j][i], "help label");
+      gtk_widget_set_name(help_tlabel[j][i], "help_label");
 
-      gtk_table_attach_defaults(GTK_TABLE(help_ttable),
-					  help_tlabel[j][i], i, i+1, j, j+1);
-      gtk_widget_show(help_tlabel[j][i]);
+      /* Ugly (but these numbers are hardcoded in help_update_terrain() too) */
+      if (j==1 && i==1) {
+          /* Extra wide cell for terrain specials */
+          gtk_table_attach_defaults(GTK_TABLE(help_ttable),
+                                    help_tlabel[j][i], i, i+4, j, j+1);
+          gtk_widget_show(help_tlabel[j][i]);
+          break; /* skip rest of row */
+      } else {
+          gtk_table_attach_defaults(GTK_TABLE(help_ttable),
+                                    help_tlabel[j][i], i, i+1, j, j+1);
+          gtk_widget_show(help_tlabel[j][i]);
+      }
     }
   }
 
@@ -599,7 +611,7 @@ static void create_help_dialog(void)
   gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
   gtk_container_set_border_width(GTK_CONTAINER(text), 5);
   gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD);
-  gtk_widget_set_name(text, "help text");
+  gtk_widget_set_name(text, "help_text");
   help_text = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
   gtk_widget_show(text);
 
@@ -683,18 +695,18 @@ static void create_help_page(enum help_page_type type)
 ...
 **************************************************************************/
 static void help_update_improvement(const struct help_item *pitem,
-				    char *title, int which)
+				    char *title)
 {
-  char buf[64000];
-  
+  char buf[8192];
+  struct impr_type *imp = find_improvement_by_translated_name(title);
+
   create_help_page(HELP_IMPROVEMENT);
-  
-  if (which<game.control.num_impr_types) {
-    struct impr_type *imp = improvement_by_number(which);
-    char *req = REQ_NONE;
+
+  if (imp  &&  !is_great_wonder(imp)) {
+    const char *req = REQ_LABEL_NONE;
     char req_buf[512];
 
-    sprintf(buf, "%d", impr_build_shield_cost(which));
+    sprintf(buf, "%d", impr_build_shield_cost(imp));
     gtk_label_set_text(GTK_LABEL(help_ilabel[1]), buf);
     sprintf(buf, "%d", imp->upkeep);
     gtk_label_set_text(GTK_LABEL(help_ilabel[3]), buf);
@@ -704,7 +716,7 @@ static void help_update_improvement(const struct help_item *pitem,
      * to be extended.  Remember MAX_NUM_REQS is a compile-time
      * definition. */
     requirement_vector_iterate(&imp->reqs, preq) {
-      req = get_req_source_text(&preq->source, req_buf, sizeof(req_buf));
+      req = universal_name_translation(&preq->source, req_buf, sizeof(req_buf));
       break;
     } requirement_vector_iterate_end;
     gtk_label_set_text(GTK_LABEL(help_ilabel[5]), req);
@@ -713,12 +725,12 @@ static void help_update_improvement(const struct help_item *pitem,
   else {
     gtk_label_set_text(GTK_LABEL(help_ilabel[1]), "0");
     gtk_label_set_text(GTK_LABEL(help_ilabel[3]), "0");
-    gtk_label_set_text(GTK_LABEL(help_ilabel[5]), REQ_NEVER);
-/*    create_tech_tree(help_improvement_tree, 0, game.control.num_tech_types, 3);*/
+    gtk_label_set_text(GTK_LABEL(help_ilabel[5]), REQ_LABEL_NEVER);
+/*    create_tech_tree(help_improvement_tree, 0, advance_count(), 3);*/
   }
   gtk_widget_show(help_itable);
 
-  helptext_building(buf, sizeof(buf), which, pitem->text);
+  helptext_building(buf, sizeof(buf), client.conn.playing, pitem->text, imp);
   gtk_text_buffer_set_text(help_text, buf, -1);
   gtk_widget_show(help_text_sw);
 }
@@ -727,18 +739,18 @@ static void help_update_improvement(const struct help_item *pitem,
 ...
 **************************************************************************/
 static void help_update_wonder(const struct help_item *pitem,
-			       char *title, int which)
+			       char *title)
 {
-  char buf[64000];
+  char buf[8192];
+  struct impr_type *imp = find_improvement_by_translated_name(title);
 
   create_help_page(HELP_WONDER);
 
-  if (which<game.control.num_impr_types) {
-    struct impr_type *imp = improvement_by_number(which);
+  if (imp  &&  is_great_wonder(imp)) {
     int i;
     char req_buf[512];
 
-    sprintf(buf, "%d", impr_build_shield_cost(which));
+    sprintf(buf, "%d", impr_build_shield_cost(imp));
     gtk_label_set_text(GTK_LABEL(help_wlabel[1]), buf);
 
     /* FIXME: this should show ranges and all the MAX_NUM_REQS reqs. 
@@ -748,29 +760,29 @@ static void help_update_wonder(const struct help_item *pitem,
     i = 0;
     requirement_vector_iterate(&imp->reqs, preq) {
       gtk_label_set_text(GTK_LABEL(help_wlabel[3 + i]),
-			 get_req_source_text(&preq->source,
+			 universal_name_translation(&preq->source,
 					     req_buf, sizeof(req_buf)));
       i++;
       break;
     } requirement_vector_iterate_end;
-    if (tech_exists(imp->obsolete_by)) {
+    if (valid_advance(imp->obsolete_by)) {
       gtk_label_set_text(GTK_LABEL(help_wlabel[5]),
-			 advance_name_for_player(game.player_ptr, imp->obsolete_by));
+			 advance_name_for_player(client.conn.playing, advance_number(imp->obsolete_by)));
     } else {
-      gtk_label_set_text(GTK_LABEL(help_wlabel[5]), REQ_NEVER);
+      gtk_label_set_text(GTK_LABEL(help_wlabel[5]), REQ_LABEL_NEVER);
     }
 /*    create_tech_tree(help_improvement_tree, 0, imp->tech_req, 3);*/
   }
   else {
     /* can't find wonder */
     gtk_label_set_text(GTK_LABEL(help_wlabel[1]), "0");
-    gtk_label_set_text(GTK_LABEL(help_wlabel[3]), REQ_NEVER);
-    gtk_label_set_text(GTK_LABEL(help_wlabel[5]), REQ_NONE);
-/*    create_tech_tree(help_improvement_tree, 0, game.control.num_tech_types, 3); */
+    gtk_label_set_text(GTK_LABEL(help_wlabel[3]), REQ_LABEL_NEVER);
+    gtk_label_set_text(GTK_LABEL(help_wlabel[5]), REQ_LABEL_NONE);
+/*    create_tech_tree(help_improvement_tree, 0, advance_count(), 3); */
   }
   gtk_widget_show(help_wtable);
 
-  helptext_building(buf, sizeof(buf), which, pitem->text);
+  helptext_building(buf, sizeof(buf), client.conn.playing, pitem->text, imp);
   gtk_text_buffer_set_text(help_text, buf, -1);
   gtk_widget_show(help_text_sw);
 }
@@ -779,14 +791,15 @@ static void help_update_wonder(const struct help_item *pitem,
 ...
 **************************************************************************/
 static void help_update_unit_type(const struct help_item *pitem,
-				  char *title, struct unit_type *utype)
+				  char *title)
 {
-  char buf[64000];
+  char buf[8192];
+  struct unit_type *utype = find_unit_type_by_translated_name(title);
 
   create_help_page(HELP_UNIT);
 
   if (utype) {
-    sprintf(buf, "%d", unit_build_shield_cost(utype));
+    sprintf(buf, "%d", utype_build_shield_cost(utype));
     gtk_label_set_text(GTK_LABEL(help_ulabel[0][1]), buf);
     sprintf(buf, "%d", utype->attack_strength);
     gtk_label_set_text(GTK_LABEL(help_ulabel[0][4]), buf);
@@ -802,36 +815,36 @@ static void help_update_unit_type(const struct help_item *pitem,
 		       helptext_unit_upkeep_str(utype));
     sprintf(buf, "%d", (int)sqrt((double)utype->vision_radius_sq));
     gtk_label_set_text(GTK_LABEL(help_ulabel[3][4]), buf);
-    if(utype->tech_requirement==A_LAST) {
-      gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]), REQ_NEVER);
+    if (A_NEVER == utype->require_advance) {
+      gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]), REQ_LABEL_NEVER);
     } else {
       gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]),
-			 advance_name_for_player(game.player_ptr,
-				       utype->tech_requirement));
+			 advance_name_for_player(client.conn.playing,
+				       advance_number(utype->require_advance)));
     }
-/*    create_tech_tree(help_improvement_tree, 0, utype->tech_requirement, 3);*/
-    if (utype->obsoleted_by == U_NOT_OBSOLETED) {
-      gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]), REQ_NONE);
+/*    create_tech_tree(help_improvement_tree, 0, advance_number(utype->require_advance), 3);*/
+    if (U_NOT_OBSOLETED == utype->obsoleted_by) {
+      gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]), REQ_LABEL_NONE);
     } else {
       gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]),
 			 utype_name_translation(utype->obsoleted_by));
     }
 
-    helptext_unit(buf, sizeof(buf), utype, pitem->text);
+    helptext_unit(buf, sizeof(buf), client.conn.playing, pitem->text, utype);
 
     gtk_text_buffer_set_text(help_text, buf, -1);
     gtk_widget_show(help_text_sw);
 
-    gtk_pixcomm_freeze(GTK_PIXCOMM(unit_tile));
+    gtk_pixcomm_freeze(GTK_PIXCOMM(help_tile));
     {
       struct canvas store;
 
       store.type = CANVAS_PIXCOMM;
-      store.v.pixcomm = GTK_PIXCOMM(unit_tile);
+      store.v.pixcomm = GTK_PIXCOMM(help_tile);
       create_overlay_unit(&store, utype);
     }
-    gtk_pixcomm_thaw(GTK_PIXCOMM(unit_tile));
-    gtk_widget_show(unit_tile);
+    gtk_pixcomm_thaw(GTK_PIXCOMM(help_tile));
+    gtk_widget_show(help_tile);
   }
   else {
     gtk_label_set_text(GTK_LABEL(help_ulabel[0][1]), "0");
@@ -843,9 +856,9 @@ static void help_update_unit_type(const struct help_item *pitem,
     gtk_label_set_text(GTK_LABEL(help_ulabel[3][1]), "0");
     gtk_label_set_text(GTK_LABEL(help_ulabel[3][4]), "0");
 
-    gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]), REQ_NEVER);
+    gtk_label_set_text(GTK_LABEL(help_ulabel[4][1]), REQ_LABEL_NEVER);
 /*    create_tech_tree(help_improvement_tree, 0, A_LAST, 3);*/
-    gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]), REQ_NONE);
+    gtk_label_set_text(GTK_LABEL(help_ulabel[4][4]), REQ_LABEL_NONE);
 
     gtk_text_buffer_set_text(help_text, buf, -1);
     gtk_widget_show(help_text_sw);
@@ -875,15 +888,16 @@ static char *my_chomp(char *str, size_t len)
 /**************************************************************************
 ...
 **************************************************************************/
-static void help_update_tech(const struct help_item *pitem, char *title, int i)
+static void help_update_tech(const struct help_item *pitem, char *title)
 {
-  int j;
+  int i, j;
   GtkWidget *w, *hbox;
-  char buf[4096];
+  char buf[8192];
+  struct advance *padvance = find_advance_by_translated_name(title);
 
   create_help_page(HELP_TECH);
 
-  if (!is_future_tech(i)) {
+  if (padvance  &&  !is_future_tech(i = advance_number(padvance))) {
     GtkTextBuffer *txt;
     size_t len;
 
@@ -897,7 +911,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
     gtk_widget_show(help_tree_sw);
     gtk_widget_show(help_tree_buttons_hbox);
 
-    helptext_tech(buf, sizeof(buf), i, pitem->text);
+    helptext_advance(buf, sizeof(buf), client.conn.playing, pitem->text, i);
     len = strlen(buf);
     my_chomp(buf, len);
 
@@ -905,7 +919,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
       GdkPixbuf *pixbuf = sprite_get_pixbuf(get_tech_sprite(tileset, i));
 
       w = gtk_image_new_from_pixbuf(pixbuf);
-      gtk_widget_set_name(w, "help tech image");
+      gtk_widget_set_name(w, "help_tech_image");
       gtk_box_pack_start(GTK_BOX(help_vbox), w, FALSE, FALSE, 0);
       gtk_widget_show(w);
     }
@@ -913,7 +927,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
     w = gtk_text_view_new();
     gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(w), FALSE);
     gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(w), GTK_WRAP_WORD);
-    gtk_widget_set_name(w, "help text");
+    gtk_widget_set_name(w, "help_text");
     gtk_container_set_border_width(GTK_CONTAINER(w), 5);
     gtk_text_view_set_editable(GTK_TEXT_VIEW(w), FALSE);
     gtk_box_pack_start(GTK_BOX(help_vbox), w, TRUE, TRUE, 0);
@@ -928,37 +942,40 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
     gtk_box_pack_start(GTK_BOX(help_vbox), w, TRUE, TRUE, 5);
     gtk_widget_show(w);
 
-    impr_type_iterate(j) {
+    improvement_iterate(pimprove) {
       /* FIXME: need a more general mechanism for this, since this
        * helptext needs to be shown in all possible req source types. */
-      requirement_vector_iterate(&improvement_by_number(j)->reqs, preq) {
-	if (preq->source.type == REQ_TECH
-	    && preq->source.value.tech == i) {
+      requirement_vector_iterate(&pimprove->reqs, preq) {
+	if (VUT_ADVANCE == preq->source.kind
+	    && preq->source.value.advance == padvance) {
 	  hbox = gtk_hbox_new(FALSE, 0);
 	  gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
 	  w = gtk_label_new(_("Allows"));
 	  gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-	  w = help_slink_new(improvement_name_translation(j),
-	      is_great_wonder(j) ? HELP_WONDER
-	      : HELP_IMPROVEMENT);
+	  w = help_slink_new(improvement_name_translation(pimprove),
+			     is_great_wonder(pimprove)
+			     ? HELP_WONDER
+			     : HELP_IMPROVEMENT);
 	  gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
 	  gtk_widget_show_all(hbox);
 	}
       } requirement_vector_iterate_end;
-      if (i == improvement_by_number(j)->obsolete_by) {
+      if (padvance == pimprove->obsolete_by) {
         hbox = gtk_hbox_new(FALSE, 0);
         gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
         w = gtk_label_new(_("Obsoletes"));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = help_slink_new(improvement_name_translation(j),
-			  is_great_wonder(j) ? HELP_WONDER : HELP_IMPROVEMENT);
+        w = help_slink_new(improvement_name_translation(pimprove),
+			   is_great_wonder(pimprove)
+			   ? HELP_WONDER
+			   : HELP_IMPROVEMENT);
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
         gtk_widget_show_all(hbox);
       }
-    } impr_type_iterate_end;
+    } improvement_iterate_end;
 
     unit_type_iterate(punittype) {
-      if (i != punittype->tech_requirement) {
+      if (padvance != punittype->require_advance) {
 	continue;
       }
       hbox = gtk_hbox_new(FALSE, 0);
@@ -970,51 +987,50 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
       gtk_widget_show_all(hbox);
     } unit_type_iterate_end;
 
-    for (j = 0; j < game.control.num_tech_types; j++) {
-      if(i==advances[j].req[0]) {
-	if(advances[j].req[1]==A_NONE) {
+    advance_iterate(A_NONE, ptest) {
+      if (padvance == advance_requires(ptest, AR_ONE)) {
+	if (advance_by_number(A_NONE) == advance_requires(ptest, AR_TWO)) {
           hbox = gtk_hbox_new(FALSE, 0);
           gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
           w = gtk_label_new(_("Allows"));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-          w = help_slink_new(advance_name_for_player(game.player_ptr, j), HELP_TECH);
+          w = help_slink_new(advance_name_translation(ptest), HELP_TECH);
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           gtk_widget_show_all(hbox);
-	}
-	else {
+	} else {
           hbox = gtk_hbox_new(FALSE, 0);
           gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
           w = gtk_label_new(_("Allows"));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-          w = help_slink_new(advance_name_for_player(game.player_ptr, j), HELP_TECH);
+          w = help_slink_new(advance_name_translation(ptest), HELP_TECH);
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           w = gtk_label_new(_("with"));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-          w = help_slink_new(advance_name_for_player(game.player_ptr,
-					   advances[j].req[1]), HELP_TECH);
+          w = help_slink_new(advance_name_translation(advance_requires(ptest, AR_TWO)),
+                             HELP_TECH);
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           w = gtk_label_new(Q_("?techhelp:"));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           gtk_widget_show_all(hbox);
 	}
       }
-      if(i==advances[j].req[1]) {
+      if (padvance == advance_requires(ptest, AR_TWO)) {
         hbox = gtk_hbox_new(FALSE, 0);
         gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
         w = gtk_label_new(_("Allows"));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = help_slink_new(advance_name_for_player(game.player_ptr, j), HELP_TECH);
+        w = help_slink_new(advance_name_translation(ptest), HELP_TECH);
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
         w = gtk_label_new(_("with"));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = help_slink_new(advance_name_for_player(game.player_ptr,
-					 advances[j].req[0]), HELP_TECH);
+        w = help_slink_new(advance_name_translation(advance_requires(ptest, AR_ONE)),
+                           HELP_TECH);
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
         w = gtk_label_new(Q_("?techhelp:"));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
         gtk_widget_show_all(hbox);
       }
-    }
+    } advance_iterate_end;
     gtk_widget_show(help_vbox);
   }
 }
@@ -1023,9 +1039,10 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 ...
 **************************************************************************/
 static void help_update_terrain(const struct help_item *pitem,
-				char *title, struct terrain *pterrain)
+				char *title)
 {
-  char buf[4096];
+  char buf[8192];
+  struct terrain *pterrain = find_terrain_by_translated_name(title);
 
   create_help_page(HELP_TERRAIN);
 
@@ -1047,7 +1064,12 @@ static void help_update_terrain(const struct help_item *pitem,
       struct resource **r;
 
       for (r = pterrain->resources; *r; r++) {
-	sprintf (buf + strlen (buf), " %s,", resource_name_translation(*r));
+        /* TRANS: " Whales (2/1/2)," */
+        sprintf (buf + strlen (buf), " %s (%d/%d/%d),",
+                 resource_name_translation(*r),
+                 pterrain->output[O_FOOD]   + (*r)->output[O_FOOD],
+                 pterrain->output[O_SHIELD] + (*r)->output[O_SHIELD],
+                 pterrain->output[O_TRADE]  + (*r)->output[O_TRADE]);
       }
       buf[strlen (buf) - 1] = '.';
     } else {
@@ -1106,8 +1128,8 @@ static void help_update_terrain(const struct help_item *pitem,
     gtk_label_set_text(GTK_LABEL(help_tlabel[3][4]), buf);
   }
 
-  helptext_terrain(buf, sizeof(buf), pterrain, pitem->text);
-  
+  helptext_terrain(buf, sizeof(buf), client.conn.playing, pitem->text, pterrain);
+
   gtk_text_buffer_set_text(help_text, buf, -1);
   gtk_widget_show(help_text_sw);
 
@@ -1118,14 +1140,15 @@ static void help_update_terrain(const struct help_item *pitem,
   This is currently just a text page, with special text:
 **************************************************************************/
 static void help_update_government(const struct help_item *pitem,
-				   char *title, struct government *gov)
+				   char *title)
 {
-  char buf[4096];
+  char buf[8192];
+  struct government *gov = find_government_by_translated_name(title);
 
   if (!gov) {
     strcat(buf, pitem->text);
   } else {
-    helptext_government(buf, sizeof(buf), gov, pitem->text);
+    helptext_government(buf, sizeof(buf), client.conn.playing, pitem->text, gov);
   }
   create_help_page(HELP_TEXT);
   gtk_text_buffer_set_text(help_text, buf, -1);
@@ -1137,7 +1160,6 @@ static void help_update_government(const struct help_item *pitem,
 **************************************************************************/
 static void help_update_dialog(const struct help_item *pitem)
 {
-  int i;
   char *top;
 
   /* figure out what kind of item is required for pitem ingo */
@@ -1151,30 +1173,22 @@ static void help_update_dialog(const struct help_item *pitem)
 
   switch(pitem->type) {
   case HELP_IMPROVEMENT:
-    i = find_improvement_by_translated_name(top);
-    if (i != B_LAST && is_great_wonder(i)) {
-      i = B_LAST;
-    }
-    help_update_improvement(pitem, top, i);
+    help_update_improvement(pitem, top);
     break;
   case HELP_WONDER:
-    i = find_improvement_by_translated_name(top);
-    if (i != B_LAST && !is_great_wonder(i)) {
-      i = B_LAST;
-    }
-    help_update_wonder(pitem, top, i);
+    help_update_wonder(pitem, top);
     break;
   case HELP_UNIT:
-    help_update_unit_type(pitem, top, find_unit_type_by_translated_name(top));
+    help_update_unit_type(pitem, top);
     break;
   case HELP_TECH:
-    help_update_tech(pitem, top, find_advance_by_translated_name(top));
+    help_update_tech(pitem, top);
     break;
   case HELP_TERRAIN:
-    help_update_terrain(pitem, top, find_terrain_by_translated_name(top));
+    help_update_terrain(pitem, top);
     break;
   case HELP_GOVERNMENT:
-    help_update_government(pitem, top, find_government_by_translated_name(top));
+    help_update_government(pitem, top);
     break;
   case HELP_TEXT:
   default:
