@@ -18,6 +18,7 @@
 #include <assert.h>
 #include <gtk/gtk.h>
 
+/* common & utility */
 #include "combat.h"
 #include "fcintl.h"
 #include "game.h"
@@ -26,15 +27,19 @@
 #include "support.h"
 #include "unit.h"
 
+#include "overview_common.h"
+
+/* client */
 #include "chatline.h"
 #include "citydlg.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climap.h"
-#include "clinet.h"
 #include "climisc.h"
 #include "colors.h"
 #include "control.h"
 #include "dialogs.h"
+#include "editgui.h"
+#include "editor.h"
 #include "graphics.h"
 #include "gui_main.h"
 #include "inputdlg.h"
@@ -46,10 +51,8 @@
 
 #include "mapctrl.h"
 
-/* Color to use to display the workers */
-int city_workers_color=COLOR_STD_WHITE;
-
 struct tmousepos { int x, y; };
+extern gint cur_x, cur_y;
 
 /**************************************************************************
 ...
@@ -83,18 +86,18 @@ static void popupinfo_positioning_callback(GtkWidget *w, GtkAllocation *alloc,
     maxx = minx + map_canvas->allocation.width;
     maxy = miny + map_canvas->allocation.height;
 
-    if (x > mapview_canvas.width/2) {
+    if (x > mapview.width/2) {
       /* right part of the map */
       x += minx;
-      y += miny + (NORMAL_TILE_HEIGHT - alloc->height)/2;
+      y += miny + (tileset_tile_height(tileset) - alloc->height)/2;
 
       y = CLIP(miny, y, maxy - alloc->height);
 
       gtk_window_move(GTK_WINDOW(w), x - alloc->width, y);
     } else {
       /* left part of the map */
-      x += minx + NORMAL_TILE_WIDTH;
-      y += miny + (NORMAL_TILE_HEIGHT - alloc->height)/2;
+      x += minx + tileset_tile_width(tileset);
+      y += miny + (tileset_tile_height(tileset) - alloc->height)/2;
 
       y = CLIP(miny, y, maxy - alloc->height);
 
@@ -110,14 +113,10 @@ static void popupinfo_positioning_callback(GtkWidget *w, GtkAllocation *alloc,
 static void popit(GdkEventButton *event, struct tile *ptile)
 {
   GtkWidget *p;
-  static struct tile *cross_list[2 + 1];
-  struct tile **cross_head = cross_list;
-  int i;
   static struct tmousepos mousepos;
   struct unit *punit;
-  bool is_orders;
 
-  if (tile_get_known(ptile) >= TILE_KNOWN_FOGGED) {
+  if (TILE_UNKNOWN != client_tile_get_known(ptile)) {
     p=gtk_window_new(GTK_WINDOW_POPUP);
     gtk_widget_set_app_paintable(p, TRUE);
     gtk_container_set_border_width(GTK_CONTAINER(p), 4);
@@ -125,22 +124,16 @@ static void popit(GdkEventButton *event, struct tile *ptile)
 
     punit = find_visible_unit(ptile);
 
-    is_orders = show_unit_orders(punit);
-
-    if (punit && punit->goto_tile) {
-      *cross_head = punit->goto_tile;
-      cross_head++;
+    if (punit) {
+      mapdeco_set_gotoroute(punit);
+      if (punit->goto_tile) {
+        mapdeco_set_crosshair(punit->goto_tile, TRUE);
+      }
     }
-    *cross_head = ptile;
-    cross_head++;
+    mapdeco_set_crosshair(ptile, TRUE);
 
-    *cross_head = NULL;
-    for (i = 0; cross_list[i]; i++) {
-      put_cross_overlay_tile(cross_list[i]);
-    }
     g_signal_connect(p, "destroy",
-		     G_CALLBACK(popupinfo_popdown_callback),
-		     GINT_TO_POINTER(is_orders));
+		     G_CALLBACK(popupinfo_popdown_callback), NULL);
 
     mousepos.x = event->x;
     mousepos.y = event->y;
@@ -164,13 +157,8 @@ static void popit(GdkEventButton *event, struct tile *ptile)
 **************************************************************************/
 void popupinfo_popdown_callback(GtkWidget *w, gpointer data)
 {
-  bool full = GPOINTER_TO_INT(data);
-
-  if (full) {
-    update_map_canvas_visible();
-  } else {
-    dirty_all();
-  }
+  mapdeco_clear_crosshairs();
+  mapdeco_clear_gotoroutes();
 }
 
  /**************************************************************************
@@ -178,7 +166,7 @@ void popupinfo_popdown_callback(GtkWidget *w, gpointer data)
 **************************************************************************/
 static void name_new_city_callback(GtkWidget * w, gpointer data)
 {
-  dsend_packet_unit_build_city(&aconnection, GPOINTER_TO_INT(data),
+  dsend_packet_unit_build_city(&client.conn, GPOINTER_TO_INT(data),
 			       input_dialog_get_input(w));
   input_dialog_destroy(w);
 }
@@ -214,6 +202,10 @@ void set_turn_done_button_state(bool state)
 **************************************************************************/
 gboolean butt_release_mapcanvas(GtkWidget *w, GdkEventButton *ev, gpointer data)
 {
+  if (editor_is_active()) {
+    return handle_edit_mouse_button_release(ev);
+  }
+
   if (ev->button == 1 || ev->button == 3) {
     release_goto_button(ev->x, ev->y);
   }
@@ -233,13 +225,17 @@ gboolean butt_down_mapcanvas(GtkWidget *w, GdkEventButton *ev, gpointer data)
   struct city *pcity = NULL;
   struct tile *ptile = NULL;
 
+  if (editor_is_active()) {
+    return handle_edit_mouse_button_press(ev);
+  }
+
   if (!can_client_change_view()) {
     return TRUE;
   }
 
   gtk_widget_grab_focus(map_canvas);
   ptile = canvas_pos_to_tile(ev->x, ev->y);
-  pcity = ptile ? ptile->city : NULL;
+  pcity = ptile ? tile_city(ptile) : NULL;
 
   switch (ev->button) {
 
@@ -253,11 +249,13 @@ gboolean butt_down_mapcanvas(GtkWidget *w, GdkEventButton *ev, gpointer data)
     else if (ev->state & GDK_CONTROL_MASK) {
       action_button_pressed(ev->x, ev->y, SELECT_SEA);
     }
-    /* <SHIFT> + LMB: Copy Production. */
+    /* <SHIFT> + LMB: Append focus unit Production. */
     else if (ptile && (ev->state & GDK_SHIFT_MASK)) {
-      clipboard_copy_production(ptile);
+      action_button_pressed(ev->x, ev->y, SELECT_APPEND);
     }
     /* <ALT> + LMB: popit (same as middle-click) */
+    /* FIXME: we need a general mechanism for letting freeciv work with
+     * 1- or 2-button mice. */
     else if (ptile && (ev->state & GDK_MOD1_MASK)) {
       popit(ev, ptile);
     }
@@ -287,15 +285,30 @@ gboolean butt_down_mapcanvas(GtkWidget *w, GdkEventButton *ev, gpointer data)
 
   case 3: /* RIGHT mouse button */
 
-    /* <CONTROL> + RMB : Quickselect a land unit. */
-    if (ev->state & GDK_CONTROL_MASK) {
-      action_button_pressed(ev->x, ev->y, SELECT_LAND);
+    /* <CONTROL> + <ALT> + RMB : insert city or tile chat link. */
+    /* <CONTROL> + <ALT> + <SHIFT> + RMB : insert unit chat link. */
+    if (ptile && (ev->state & GDK_MOD1_MASK)
+        && (ev->state & GDK_CONTROL_MASK)) {
+      inputline_make_chat_link(ptile, (ev->state & GDK_SHIFT_MASK) != 0);
     }
-    /* <SHIFT> + RMB: Paste Production. */
-    else if ((ev->state & GDK_SHIFT_MASK) && pcity) {
+    /* <SHIFT> + <ALT> + RMB : Show/hide workers. */
+    else if ((ev->state & GDK_SHIFT_MASK) && (ev->state & GDK_MOD1_MASK)) {
+      overlay_workers_at_city();
+    }
+    /* <SHIFT + CONTROL> + RMB: Paste Production. */
+    else if ((ev->state & GDK_SHIFT_MASK) && (ev->state & GDK_CONTROL_MASK)
+             && pcity != NULL) {
       clipboard_paste_production(pcity);
       cancel_tile_hiliting();
     }
+    /* <SHIFT> + RMB: Copy Production. */
+    else if (ev->state & GDK_SHIFT_MASK) {
+      clipboard_copy_production(ptile);
+    }
+    /* <CONTROL> + RMB : Quickselect a land unit. */
+    else if (ev->state & GDK_CONTROL_MASK) {
+      action_button_pressed(ev->x, ev->y, SELECT_LAND);
+    }
     /* Plain RMB click. */
     else {
       /*  A foolproof user will depress button on canvas,
@@ -308,7 +321,8 @@ gboolean butt_down_mapcanvas(GtkWidget *w, GdkEventButton *ev, gpointer data)
       }
       cancel_tile_hiliting();
       if (hover_state == HOVER_NONE) {
-        anchor_selection_rectangle(ev->x, ev->y);
+        anchor_selection_rectangle(ev->x, ev->y,
+				   (ev->state & GDK_SHIFT_MASK) != 0);
         rbutton_down = TRUE; /* causes rectangle updates */
       }
     }
@@ -331,7 +345,7 @@ void create_line_at_mouse_pos(void)
 
   gdk_window_get_pointer(map_canvas->window, &x, &y, 0);
   if (x >= 0 && y >= 0
-      && x < mapview_canvas.width && y < mapview_canvas.width) {
+      && x < mapview.width && y < mapview.width) {
     update_line(x, y);
   } else {
     gdk_window_get_pointer(overview_canvas->window, &x, &y, 0);
@@ -361,15 +375,58 @@ void update_rect_at_mouse_pos(void)
 }
 
 /**************************************************************************
-...
+  Triggered by the mouse moving on the mapcanvas, this function will
+  update the mouse cursor and goto lines. 
 **************************************************************************/
 gboolean move_mapcanvas(GtkWidget *w, GdkEventMotion *ev, gpointer data)
 {
+  if (gui_gtk2_mouse_over_map_focus && !GTK_WIDGET_HAS_FOCUS(map_canvas)) {
+    gtk_widget_grab_focus(map_canvas);
+  }
+
+  if (editor_is_active()) {
+    return handle_edit_mouse_move(ev);
+  }
+
+  cur_x = ev->x;
+  cur_y = ev->y;
   update_line(ev->x, ev->y);
   update_rect_at_mouse_pos();
   if (keyboardless_goto_button_down && hover_state == HOVER_NONE) {
     maybe_activate_keyboardless_goto(ev->x, ev->y);
   }
+  control_mouse_cursor(canvas_pos_to_tile(ev->x, ev->y));
+
+  return TRUE;
+}
+
+/**************************************************************************
+  This function will reset the mouse cursor if it leaves the map.
+**************************************************************************/
+gboolean leave_mapcanvas(GtkWidget *widget, GdkEventCrossing *event)
+{
+  int canvas_x, canvas_y;
+
+  if (gtk_notebook_get_current_page(GTK_NOTEBOOK(top_notebook))
+      != gtk_notebook_page_num(GTK_NOTEBOOK(top_notebook), map_widget)) {
+    /* Map is not currently topmost tab. Do not use tile specific cursors. */
+    update_mouse_cursor(CURSOR_DEFAULT);
+    return TRUE;
+  }
+
+  /* Bizarrely, this function can be called even when we don't "leave"
+   * the map canvas, for instance, it gets called any time the mouse is
+   * clicked. */
+  gdk_window_get_pointer(map_canvas->window, &canvas_x, &canvas_y, NULL);
+  if (map_exists()
+      && canvas_x >= 0 && canvas_y >= 0
+      && canvas_x < mapview.width && canvas_y < mapview.height) {
+    control_mouse_cursor(canvas_pos_to_tile(canvas_x, canvas_y));
+  } else {
+    update_mouse_cursor(CURSOR_DEFAULT);
+  }
+
+  update_unit_info_label(get_units_in_focus());
   return TRUE;
 }
 
@@ -397,14 +454,15 @@ gboolean butt_down_overviewcanvas(GtkWidget *w, GdkEventButton *ev, gpointer dat
   if (can_client_change_view() && (ev->button == 3)) {
     center_tile_mapcanvas(map_pos_to_tile(xtile, ytile));
   } else if (can_client_issue_orders() && (ev->button == 1)) {
-    do_map_click(map_pos_to_tile(xtile, ytile), SELECT_POPUP);
+    do_map_click(map_pos_to_tile(xtile, ytile),
+		 (ev->state & GDK_SHIFT_MASK) ? SELECT_POPUP : SELECT_APPEND);
   }
 
   return TRUE;
 }
 
 /**************************************************************************
-  Draws the on the map the tiles the given city is using
+  Best effort to center the map on the currently selected unit(s)
 **************************************************************************/
 void center_on_unit(void)
 {
@@ -412,9 +470,9 @@ void center_on_unit(void)
 }
 
 /**************************************************************************
-  Draws the on the map the tiles the given city is using
+  Shows/hides overlay on the map for the city at this location
 **************************************************************************/
-void key_city_workers(GtkWidget *w, GdkEventKey *ev)
+void overlay_workers_at_city(void)
 {
   int x, y;
   
