@@ -24,6 +24,7 @@
 
 #include <gtk/gtk.h>
 
+/* utility */
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
@@ -31,31 +32,38 @@
 #include "support.h"
 #include "timing.h"
 
-#include "game.h"
+/* common */
 #include "government.h"		/* government_graphic() */
 #include "map.h"
 #include "player.h"
 
-#include "civclient.h"
+/* client */
+#include "client_main.h"
 #include "climap.h"
 #include "climisc.h"
 #include "colors.h"
 #include "control.h" /* get_unit_in_focus() */
-#include "graphics.h"
-#include "gui_main.h"
-#include "gui_stuff.h"
-#include "mapctrl.h"
+#include "editor.h"
 #include "options.h"
 #include "overview_common.h"
 #include "tilespec.h"
 #include "text.h"
+
+/* client/gui-gtk-2.0 */
+#include "citydlg.h" /* For reset_city_dialogs() */
+#include "editgui.h"
+#include "graphics.h"
+#include "gui_main.h"
+#include "gui_stuff.h"
+#include "mapctrl.h"
+#include "repodlgs.h"
 #include "wldlg.h"
 
-#include "citydlg.h" /* For reset_city_dialogs() */
 #include "mapview.h"
 
 static GtkObject *map_hadj, *map_vadj;
 static int cursor_timer_id = 0, cursor_type = -1, cursor_frame = 0;
+static int mapview_frozen_level = 0;
 
 /**************************************************************************
   If do_restore is FALSE it will invert the turn done button style. If
@@ -95,25 +103,35 @@ void update_timeout_label(void)
 /**************************************************************************
 ...
 **************************************************************************/
-void update_info_label( void )
+void update_info_label(void)
 {
   GtkWidget *label;
-  const struct player *pplayer = game.player_ptr;
+  const struct player *pplayer = client.conn.playing;
 
   label = gtk_frame_get_label_widget(GTK_FRAME(main_frame_civ_name));
   if (pplayer != NULL) {
-    char nation[MAX_LEN_NAME];
+    const gchar *name;
+    gunichar c;
 
     /* Capitalize the first character of the translated nation
-     * plural name so that the frame label looks good. I assume
-     * that in the case that capitalization does not make sense
-     * (e.g. multi-byte characters or no "upper case" form in
-     * the translation language) my_toupper() will just return
-     * the same value as was passed into it. */
-    sz_strlcpy(nation, nation_plural_for_player(pplayer));
-    nation[0] = my_toupper(nation[0]);
+     * plural name so that the frame label looks good. */
+    name = nation_plural_for_player(pplayer);
+    c = g_utf8_get_char_validated(name, -1);
+    if ((gunichar) -1 != c && (gunichar) -2 != c) {
+      gchar nation[MAX_LEN_NAME];
+      gchar *next;
+      gint len;
 
-    gtk_label_set_text(GTK_LABEL(label), nation);
+      len = g_unichar_to_utf8(g_unichar_toupper(c), nation);
+      nation[len] = '\0';
+      next = g_utf8_find_next_char(name, NULL);
+      if (NULL != next) {
+        sz_strlcat(nation, next);
+      }
+      gtk_label_set_text(GTK_LABEL(label), nation);
+    } else {
+      gtk_label_set_text(GTK_LABEL(label), name);
+    }
   } else {
     gtk_label_set_text(GTK_LABEL(label), "-");
   }
@@ -125,18 +143,18 @@ void update_info_label( void )
 		      client_cooling_sprite(),
 		      client_government_sprite());
 
-  if (game.player_ptr) {
+  if (NULL != client.conn.playing) {
     int d = 0;
 
-    for (; d < game.player_ptr->economic.luxury /10; d++) {
+    for (; d < client.conn.playing->economic.luxury /10; d++) {
       struct sprite *sprite = get_tax_sprite(tileset, O_LUXURY);
 
       gtk_image_set_from_pixbuf(GTK_IMAGE(econ_label[d]),
 				sprite_get_pixbuf(sprite));
     }
  
-    for (; d < (game.player_ptr->economic.science
-		+ game.player_ptr->economic.luxury) / 10; d++) {
+    for (; d < (client.conn.playing->economic.science
+		+ client.conn.playing->economic.luxury) / 10; d++) {
       struct sprite *sprite = get_tax_sprite(tileset, O_SCIENCE);
 
       gtk_image_set_from_pixbuf(GTK_IMAGE(econ_label[d]),
@@ -196,7 +214,7 @@ static gint anim_cursor_cb(gpointer data)
 /**************************************************************************
   This function will change the current mouse cursor.
 **************************************************************************/
-static void modify_mouse_cursor(enum cursor_type new_cursor_type)
+void update_mouse_cursor(enum cursor_type new_cursor_type)
 {
   cursor_type = new_cursor_type;
   if (!cursor_timer_id) {
@@ -216,7 +234,6 @@ static void modify_mouse_cursor(enum cursor_type new_cursor_type)
 **************************************************************************/
 void update_unit_info_label(struct unit_list *punits)
 {
-  enum cursor_type mouse_cursor_type = CURSOR_DEFAULT;
   GtkWidget *label;
 
   label = gtk_frame_get_label_widget(GTK_FRAME(unit_info_frame));
@@ -224,58 +241,7 @@ void update_unit_info_label(struct unit_list *punits)
 		     get_unit_info_label_text1(punits));
 
   gtk_label_set_text(GTK_LABEL(unit_info_label),
-		     get_unit_info_label_text2(punits));
-
-  if (action_state == CURSOR_ACTION_WAIT) {
-    mouse_cursor_type = CURSOR_WAIT;
-  } else {
-  switch (hover_state) {
-  case HOVER_NONE:
-    if (action_state == CURSOR_ACTION_SELECT) {
-      mouse_cursor_type = CURSOR_SELECT;
-    } else if (action_state == CURSOR_ACTION_PARATROOPER) {
-      mouse_cursor_type = CURSOR_PARADROP;
-    } else if (action_state == CURSOR_ACTION_NUKE) {
-      mouse_cursor_type = CURSOR_NUKE;
-    } else {
-      mouse_cursor_type = CURSOR_DEFAULT;
-    }
-    break;
-  case HOVER_PATROL:
-    if (action_state == CURSOR_ACTION_INVALID) {
-      mouse_cursor_type = CURSOR_INVALID;
-    } else {
-      mouse_cursor_type = CURSOR_PATROL;
-    }
-    break;
-  case HOVER_GOTO:
-    if (action_state == CURSOR_ACTION_GOTO) {
-      mouse_cursor_type = CURSOR_GOTO;
-    } else if (action_state == CURSOR_ACTION_DEFAULT) {
-      mouse_cursor_type = CURSOR_DEFAULT;
-    } else if (action_state == CURSOR_ACTION_ATTACK) {
-      mouse_cursor_type = CURSOR_ATTACK;
-    } else {
-      mouse_cursor_type = CURSOR_INVALID;
-    }
-    break;
-  case HOVER_CONNECT:
-    if (action_state == CURSOR_ACTION_INVALID) {
-      mouse_cursor_type = CURSOR_INVALID;
-    } else {
-      mouse_cursor_type = CURSOR_GOTO;
-    }
-    break;
-  case HOVER_NUKE:
-    mouse_cursor_type = CURSOR_NUKE;
-    break;
-  case HOVER_PARADROP:
-    mouse_cursor_type = CURSOR_PARADROP;
-    break;
-  }
-  }
-
-  modify_mouse_cursor(mouse_cursor_type);
+		     get_unit_info_label_text2(punits, 0));
 
   update_unit_pix_label(punits);
 }
@@ -357,19 +323,43 @@ gboolean overview_canvas_expose(GtkWidget *w, GdkEventExpose *ev, gpointer data)
   return TRUE;
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-static bool map_center = TRUE;
-static bool map_configure = FALSE;
-
-gboolean map_canvas_configure(GtkWidget * w, GdkEventConfigure * ev,
-			      gpointer data)
+/****************************************************************************
+  Freeze the drawing of the map.
+****************************************************************************/
+void mapview_freeze(void)
 {
-  if (map_canvas_resized(ev->width, ev->height)) {
-    map_configure = TRUE;
-  }
+  mapview_frozen_level++;
+}
 
+/****************************************************************************
+  Thaw the drawing of the map.
+****************************************************************************/
+void mapview_thaw(void)
+{
+  if (1 < mapview_frozen_level) {
+    mapview_frozen_level--;
+  } else {
+    assert(0 < mapview_frozen_level);
+    mapview_frozen_level = 0;
+    dirty_all();
+  }
+}
+
+/****************************************************************************
+  Return whether the map should be drawn or not.
+****************************************************************************/
+bool mapview_is_frozen(void)
+{
+  return (0 < mapview_frozen_level);
+}
+
+/**************************************************************************
+  Update on canvas widget size change
+**************************************************************************/
+gboolean map_canvas_configure(GtkWidget *w, GdkEventConfigure *ev,
+                              gpointer data)
+{
+  map_canvas_resized(ev->width, ev->height);
   return TRUE;
 }
 
@@ -378,42 +368,16 @@ gboolean map_canvas_configure(GtkWidget * w, GdkEventConfigure * ev,
 **************************************************************************/
 gboolean map_canvas_expose(GtkWidget *w, GdkEventExpose *ev, gpointer data)
 {
-  static bool cleared = FALSE;
-
-  if (!can_client_change_view()) {
-    if (!cleared) {
-      gtk_widget_queue_draw(w);
-      cleared = TRUE;
-    }
-    map_center = TRUE;
+  if (can_client_change_view() && map_exists() && !mapview_is_frozen()) {
+    /* First we mark the area to be updated as dirty.  Then we unqueue
+     * any pending updates, to make sure only the most up-to-date data
+     * is written (otherwise drawing bugs happen when old data is copied
+     * to screen).  Then we draw all changed areas to the screen. */
+    unqueue_mapview_updates(FALSE);
+    gdk_draw_drawable(map_canvas->window, civ_gc, mapview.store->v.pixmap,
+                      ev->area.x, ev->area.y, ev->area.x, ev->area.y,
+                      ev->area.width, ev->area.height);
   }
-  else
-  {
-    if (map_exists()) { /* do we have a map at all */
-      /* First we mark the area to be updated as dirty.  Then we unqueue
-       * any pending updates, to make sure only the most up-to-date data
-       * is written (otherwise drawing bugs happen when old data is copied
-       * to screen).  Then we draw all changed areas to the screen. */
-      unqueue_mapview_updates(FALSE);
-      gdk_draw_drawable(map_canvas->window, civ_gc, mapview.store->v.pixmap,
-			ev->area.x, ev->area.y, ev->area.x, ev->area.y,
-			ev->area.width, ev->area.height);
-      cleared = FALSE;
-    } else {
-      if (!cleared) {
-        gtk_widget_queue_draw(w);
-	cleared = TRUE;
-      }
-    }
-
-    if (!map_center) {
-      center_on_something();
-      map_center = FALSE;
-    }
-  }
-
-  map_configure = FALSE;
-
   return TRUE;
 }
 
@@ -422,11 +386,13 @@ gboolean map_canvas_expose(GtkWidget *w, GdkEventExpose *ev, gpointer data)
   screen.
 **************************************************************************/
 void flush_mapcanvas(int canvas_x, int canvas_y,
-		     int pixel_width, int pixel_height)
+                     int pixel_width, int pixel_height)
 {
-  gdk_draw_drawable(map_canvas->window, civ_gc, mapview.store->v.pixmap,
-		    canvas_x, canvas_y, canvas_x, canvas_y,
-		    pixel_width, pixel_height);
+  if (NULL != map_canvas->window && !mapview_is_frozen()) {
+    gdk_draw_drawable(map_canvas->window, civ_gc, mapview.store->v.pixmap,
+                      canvas_x, canvas_y, canvas_x, canvas_y,
+                      pixel_width, pixel_height);
+  }
 }
 
 #define MAX_DIRTY_RECTS 20
@@ -463,8 +429,11 @@ static void queue_flush(void)
   later.
 **************************************************************************/
 void dirty_rect(int canvas_x, int canvas_y,
-		int pixel_width, int pixel_height)
+                int pixel_width, int pixel_height)
 {
+  if (mapview_is_frozen()) {
+    return;
+  }
   if (num_dirty_rects < MAX_DIRTY_RECTS) {
     dirty_rects[num_dirty_rects].x = canvas_x;
     dirty_rects[num_dirty_rects].y = canvas_y;
@@ -480,6 +449,9 @@ void dirty_rect(int canvas_x, int canvas_y,
 **************************************************************************/
 void dirty_all(void)
 {
+  if (mapview_is_frozen()) {
+    return;
+  }
   num_dirty_rects = MAX_DIRTY_RECTS;
   queue_flush();
 }
@@ -491,6 +463,9 @@ void dirty_all(void)
 **************************************************************************/
 void flush_dirty(void)
 {
+  if (mapview_is_frozen()) {
+    return;
+  }
   if (num_dirty_rects == MAX_DIRTY_RECTS) {
     flush_mapcanvas(0, 0, map_canvas->allocation.width,
 		    map_canvas->allocation.height);
@@ -548,7 +523,8 @@ void put_unit_gpixmap(struct unit *punit, GtkPixcomm *p)
   unit, the proper way to do this is probably something like what Civ II does.
   (One food/shield/mask drawn N times, possibly one top of itself. -- SKi 
 **************************************************************************/
-void put_unit_gpixmap_city_overlays(struct unit *punit, GtkPixcomm *p)
+void put_unit_gpixmap_city_overlays(struct unit *punit, GtkPixcomm *p,
+                                    int *upkeep_cost, int happy_cost)
 {
   struct canvas store;
  
@@ -557,7 +533,8 @@ void put_unit_gpixmap_city_overlays(struct unit *punit, GtkPixcomm *p)
 
   gtk_pixcomm_freeze(p);
 
-  put_unit_city_overlays(punit, &store, 0, tileset_tile_height(tileset));
+  put_unit_city_overlays(punit, &store, 0, tileset_tile_height(tileset),
+                         upkeep_cost, happy_cost);
 
   gtk_pixcomm_thaw(p);
 }
@@ -692,7 +669,7 @@ void pixmap_put_overlay_tile_draw(GdkDrawable *pixmap,
     return;
   }
 
-  if (fog && better_fog
+  if (fog && gui_gtk2_better_fog
       && ((ssprite->pixmap && !ssprite->pixmap_fogged)
 	  || (!ssprite->pixmap && !ssprite->pixbuf_fogged))) {
     fog_sprite(ssprite);
@@ -700,11 +677,11 @@ void pixmap_put_overlay_tile_draw(GdkDrawable *pixmap,
 	|| (!ssprite->pixmap && !ssprite->pixbuf_fogged)) {
       freelog(LOG_NORMAL,
 	      _("Better fog will only work in truecolor.  Disabling it"));
-      better_fog = FALSE;
+      gui_gtk2_better_fog = FALSE;
     }
   }
 
-  if (fog && better_fog) {
+  if (fog && gui_gtk2_better_fog) {
     if (ssprite->pixmap) {
       if (ssprite->mask) {
 	gdk_gc_set_clip_origin(civ_gc, canvas_x, canvas_y);
@@ -868,9 +845,11 @@ void draw_selection_rectangle(int canvas_x, int canvas_y, int w, int h)
 **************************************************************************/
 void tileset_changed(void)
 {
+  science_dialog_redraw();
   reset_city_dialogs();
   reset_unit_table();
   blank_max_unit_size();
+  editgui_tileset_changed();
 
   /* keep the icon of the executable on Windows (see PR#36491) */
 #ifndef WIN32_NATIVE
