@@ -23,19 +23,20 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+/* common & utility */
 #include "diptreaty.h"
 #include "fcintl.h"
-#include "game.h"
 #include "packets.h"
 #include "nation.h"
 #include "player.h"
 #include "support.h"
 
+/* client */
 #include "chatline.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"
-#include "clinet.h"
 #include "connectdlg_common.h"
+#include "dialogs.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "inteldlg.h"
@@ -100,7 +101,7 @@ void popdown_players_dialog(void)
 /**************************************************************************
 ...
 **************************************************************************/
-static GdkPixbuf *create_player_icon(struct player *plr)
+static GdkPixbuf *create_player_icon(const struct player *plr)
 {
   int width, height;
   GdkPixbuf *tmp;
@@ -146,8 +147,8 @@ static void update_players_menu(void)
       gtk_widget_set_sensitive(players_sship_command, FALSE);
     }
 
-    if (game.player_ptr) {
-      switch (pplayer_get_diplstate(game.player_ptr,
+    if (NULL != client.conn.playing) {
+      switch (pplayer_get_diplstate(client.conn.playing,
 				    player_by_number(plrno))->type) {
       case DS_WAR:
       case DS_NO_CONTACT:
@@ -156,7 +157,7 @@ static void update_players_menu(void)
       default:
 	gtk_widget_set_sensitive(players_war_command,
 				 can_client_issue_orders()
-				 && game.info.player_idx != plrno);
+				 && player_by_number(plrno) != client.conn.playing);
       }
     } else {
       gtk_widget_set_sensitive(players_war_command, FALSE);
@@ -164,7 +165,7 @@ static void update_players_menu(void)
 
     gtk_widget_set_sensitive(players_vision_command,
 			     can_client_issue_orders()
-			     && gives_shared_vision(game.player_ptr, plr));
+			     && gives_shared_vision(client.conn.playing, plr));
 
     gtk_widget_set_sensitive(players_meet_command, can_meet_with_player(plr));
     gtk_widget_set_sensitive(players_int_command, can_intel_with_player(plr));
@@ -205,11 +206,11 @@ static gboolean button_press_callback(GtkTreeView *view, GdkEventButton *ev)
       plr = player_by_number(id);
 
       if (ev->button == 1) {
-	if (can_intel_with_player(plr)) {
-	  popup_intel_dialog(plr);
-	}
-      } else {
-	dsend_packet_diplomacy_init_meeting_req(&aconnection, id);
+        if (can_intel_with_player(plr)) {
+          popup_intel_dialog(plr);
+        }
+      } else if (can_meet_with_player(plr)) {
+        dsend_packet_diplomacy_init_meeting_req(&client.conn, id);
       }
     }
   }
@@ -230,11 +231,11 @@ static gint plrdlg_sort_func(GtkTreeModel *model,
   n = GPOINTER_TO_INT(data);
 
   gtk_tree_model_get_value(model, a, num_player_dlg_columns + 2, &value);
-  player1 = &game.players[g_value_get_int(&value)];
+  player1 = player_by_number(g_value_get_int(&value));
   g_value_unset(&value);
   
   gtk_tree_model_get_value(model, b, num_player_dlg_columns + 2, &value);
-  player2 = &game.players[g_value_get_int(&value)];
+  player2 = player_by_number(g_value_get_int(&value));
   g_value_unset(&value);
   
   return player_dlg_columns[n].sort_func(player1, player2);
@@ -400,6 +401,7 @@ void create_players_dialog(void)
   int i;
   GtkWidget *sep, *sw;
   GtkWidget *menubar, *menu, *item, *vbox;
+  enum ai_level level;
 
   gui_dialog_new(&players_dialog_shell, GTK_NOTEBOOK(top_notebook), NULL);
   /* TRANS: Nations report title */
@@ -414,7 +416,7 @@ void create_players_dialog(void)
 
   players_list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
   g_object_unref(store);
-  gtk_widget_set_name(players_list, "small font");
+  gtk_widget_set_name(players_list, "small_font");
 
   players_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(players_list));
   g_signal_connect(players_selection, "changed",
@@ -537,11 +539,16 @@ void create_players_dialog(void)
   sep = gtk_separator_menu_item_new();
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep);
 
-  for (i = 0; i < NUM_SKILL_LEVELS; i++) {
-    item = gtk_menu_item_new_with_label(_(skill_level_names[i]));
-    g_signal_connect(item, "activate",
-	G_CALLBACK(players_ai_skill_callback), GUINT_TO_POINTER(i));
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  for (level = 0; level < AI_LEVEL_LAST; level++) {
+    if (is_settable_ai_level(level)) {
+      const char *level_name = ai_level_name(level);
+
+      item = gtk_menu_item_new_with_label(level_name);
+      g_signal_connect(item, "activate",
+                       G_CALLBACK(players_ai_skill_callback),
+                       GUINT_TO_POINTER(level));
+      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    }
   }
   gtk_widget_show_all(menu);
 
@@ -602,49 +609,54 @@ GdkPixbuf *get_flag(const struct nation_type *nation)
 
 
 /**************************************************************************
- Builds the text for the cells of a row in the player report. If
- update is TRUE, only the changable entries are build.
+  Fills the player list with the information for 'pplayer' at the row
+  given by 'it'.
 **************************************************************************/
-static void build_row(GtkTreeIter *it, int i)
+static void fill_row(GtkTreeIter *it, const struct player *pplayer)
 {
-  struct player *plr = player_by_number(i);
+  struct player_dlg_column* pcol;
   GdkPixbuf *pixbuf;
-  gint style = PANGO_STYLE_NORMAL, weight = PANGO_WEIGHT_NORMAL;
+  int style = PANGO_STYLE_NORMAL, weight = PANGO_WEIGHT_NORMAL;
   int k;
-  gchar *p;
+
+  if (!it || !pplayer) {
+    return;
+  }
 
   for (k = 0; k < num_player_dlg_columns; k++) {
-    struct player_dlg_column* pcol = &player_dlg_columns[k];
-    switch(pcol->type) {
-      case COL_TEXT:
-      case COL_RIGHT_TEXT:
-        p = (gchar*)(pcol->func(plr));
-	gtk_list_store_set(store, it, k, p, -1);
-	break;
-      case COL_FLAG:
-        pixbuf = get_flag(nation_of_player(plr));
+    pcol = &player_dlg_columns[k];
+    switch (pcol->type) {
+    case COL_TEXT:
+    case COL_RIGHT_TEXT:
+      gtk_list_store_set(store, it, k, pcol->func(pplayer), -1);
+      break;
+    case COL_FLAG:
+      pixbuf = get_flag(nation_of_player(pplayer));
+      if (pixbuf != NULL) {
         gtk_list_store_set(store, it, k, pixbuf, -1);
         g_object_unref(pixbuf);
-	break;
-      case COL_COLOR:
-	pixbuf = create_player_icon(plr);
+      }
+      break;
+    case COL_COLOR:
+      pixbuf = create_player_icon(pplayer);
+      if (pixbuf != NULL) {
         gtk_list_store_set(store, it, k, pixbuf, -1);
-	g_object_unref(pixbuf);
-	break;
-      case COL_BOOLEAN:
-        gtk_list_store_set(store, it, k, (gboolean)pcol->bool_func(plr), -1);
-	break;
+        g_object_unref(pixbuf);
+      }
+      break;
+    case COL_BOOLEAN:
+      gtk_list_store_set(store, it, k, pcol->bool_func(pplayer), -1);
+      break;
     }
   }
 
   /* The playerid */
-  gtk_list_store_set(store, it,
-    ncolumns - 1, (gint)i,
-    -1);
+  gtk_list_store_set(store, it, ncolumns - 1,
+                     player_number(pplayer), -1);
 
    /* now add some eye candy ... */
-  if (game.player_ptr) {
-    switch (pplayer_get_diplstate(game.player_ptr, plr)->type) {
+  if (client_has_player()) {
+    switch (pplayer_get_diplstate(client_player(), pplayer)->type) {
     case DS_WAR:
       weight = PANGO_WEIGHT_NORMAL;
       style = PANGO_STYLE_ITALIC;
@@ -662,83 +674,49 @@ static void build_row(GtkTreeIter *it, int i)
       style = PANGO_STYLE_NORMAL;
       break;
     case DS_LAST:
+    default:
       assert(0);
       break;
     }
   }
-   gtk_list_store_set(store, it,
-       num_player_dlg_columns, style,
-       num_player_dlg_columns + 1, weight,
-       -1);
+
+  gtk_list_store_set(store, it, num_player_dlg_columns, style,
+                     num_player_dlg_columns + 1, weight, -1);
 }
 
 /**************************************************************************
-...
+  Return TRUE if the player should be shown in the player list.
 **************************************************************************/
-static bool player_should_be_shown(int plrno) {
-  return is_valid_player_id(plrno)
+static bool player_should_be_shown(const struct player *pplayer)
+{
+  return NULL != pplayer && player_slot_is_used(pplayer)
 	 && (player_dlg_show_dead_players
-	     || game.players[plrno].is_alive)
-	 && (!is_barbarian(&game.players[plrno]));
+	     || pplayer->is_alive)
+	 && (!is_barbarian(pplayer));
 }
 
 /**************************************************************************
-...
+  Clear and refill the entire player list.
 **************************************************************************/
 void update_players_dialog(void)
 {
-  if (players_dialog_shell && !is_plrdlg_frozen()) {
-    gboolean exists[game.info.nplayers];
-    gint i;
-    GtkTreeIter it, it_next;
+  GtkTreeIter iter;
 
-    for (i = 0; i < game.info.nplayers; i++) {
-      exists[i] = FALSE;
-    }
-
-    if (gtk_tree_model_get_iter_first(model, &it)) {
-      gint plrno;
-      bool more;
-
-      do {
-	it_next = it;
-	more = gtk_tree_model_iter_next(model, &it_next);
-
-	gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
-
-	/*
-	 * The nation already had a row in the player report. In that
-	 * case we just update the row. If player is dead we remove him
-	 * if necessary.
-	 */
-	if (player_should_be_shown(plrno)) {
-	  exists[plrno] = TRUE;
-	  build_row(&it, plrno);
-	} else {
-	  gtk_list_store_remove(store, &it);
-	}
-	it = it_next;
-      } while (more);
-    }
-
-    players_iterate(pplayer) {
-      /* skip barbarians */
-      if (player_should_be_shown(pplayer->player_no)) {
-	if (!exists[pplayer->player_no]) {
-	  /* 
-	   * A nation is not in the player report yet. This happens when
-	   * the report is just opened and after a split.
-	   */
-	  gtk_list_store_append(store, &it);
-
-	  build_row(&it, pplayer->player_no);
-	}
-      }
-    } players_iterate_end;
-
-    update_players_menu();
-    update_views();
+  if (!players_dialog_shell || is_plrdlg_frozen()) {
+    return;
   }
+
+  gtk_list_store_clear(store);
+  players_iterate(pplayer) {
+    if (!player_should_be_shown(pplayer)) {
+      continue;
+    }
+    gtk_list_store_append(store, &iter);
+    fill_row(&iter, pplayer);
+  } players_iterate_end;
+
+  update_players_menu();
+  update_views();
 }
 
 /**************************************************************************
@@ -755,7 +733,7 @@ void players_meet_callback(GtkMenuItem *item, gpointer data)
 
     gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
 
-    dsend_packet_diplomacy_init_meeting_req(&aconnection, plrno);
+    dsend_packet_diplomacy_init_meeting_req(&client.conn, plrno);
   }
 }
 
@@ -773,7 +751,7 @@ void players_war_callback(GtkMenuItem *item, gpointer data)
     gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
 
     /* can be any pact clause */
-    dsend_packet_diplomacy_cancel_pact(&aconnection, plrno,
+    dsend_packet_diplomacy_cancel_pact(&client.conn, plrno,
 				       CLAUSE_CEASEFIRE);
   }
 }
@@ -790,7 +768,7 @@ void players_vision_callback(GtkMenuItem *item, gpointer data)
     gint plrno;
 
     gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
-    dsend_packet_diplomacy_cancel_pact(&aconnection, plrno, CLAUSE_VISION);
+    dsend_packet_diplomacy_cancel_pact(&client.conn, plrno, CLAUSE_VISION);
   }
 }
 
@@ -807,8 +785,8 @@ void players_intel_callback(GtkMenuItem *item, gpointer data)
 
     gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
 
-    if (can_intel_with_player(&game.players[plrno])) {
-      popup_intel_dialog(&game.players[plrno]);
+    if (can_intel_with_player(player_by_number(plrno))) {
+      popup_intel_dialog(player_by_number(plrno));
     }
   }
 }
@@ -825,7 +803,7 @@ void players_sship_callback(GtkMenuItem *item, gpointer data)
     gint plrno;
 
     gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
-    popup_spaceship_dialog(&game.players[plrno]);
+    popup_spaceship_dialog(player_by_number(plrno));
   }
 }
 
@@ -839,13 +817,10 @@ static void players_ai_toggle_callback(GtkMenuItem *item, gpointer data)
 
   if (gtk_tree_selection_get_selected(players_selection, &model, &it)) {
     gint plrno;
-    char buf[512];
 
     gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
 
-    my_snprintf(buf, sizeof(buf), "/aitoggle \"%s\"",
-                player_name(player_by_number(plrno)));
-    send_chat(buf);
+    send_chat_printf("/aitoggle \"%s\"", player_name(player_by_number(plrno)));
   }
 }
 
@@ -859,14 +834,12 @@ static void players_ai_skill_callback(GtkMenuItem *item, gpointer data)
 
   if (gtk_tree_selection_get_selected(players_selection, &model, &it)) {
     gint plrno;
-    char buf[512];
 
     gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
 
-    my_snprintf(buf, sizeof(buf), "/%s %s",
-                skill_level_names[GPOINTER_TO_UINT(data)],
-                player_name(player_by_number(plrno)));
-    send_chat(buf);
+    send_chat_printf("/%s %s",
+                     ai_level_cmd(GPOINTER_TO_UINT(data)),
+                     player_name(player_by_number(plrno)));
   }
 }
 
