@@ -22,27 +22,34 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include "city.h"
-#include "citydlg_common.h"
-#include "civclient.h"
+/* utility */
 #include "fcintl.h"
-#include "game.h"
+#include "log.h"
+#include "mem.h"
+#include "support.h"
+
+/* common */
+#include "city.h"
+#include "packets.h"
+#include "worklist.h"
+
+/* client */
+#include "citydlg_common.h"
+#include "client_main.h"
+#include "climisc.h"
+#include "global_worklist.h"
+#include "options.h"
+#include "tilespec.h"
+
+/* gui-gtk-2.0 */
+#include "canvas.h"
+#include "citydlg.h"
+#include "graphics.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "helpdlg.h"
 #include "inputdlg.h"
-#include "mem.h"
-#include "packets.h"
-#include "worklist.h"
-#include "support.h"
-#include "climisc.h"
-#include "clinet.h"
-#include "options.h"
-#include "tilespec.h"
 
-#include "canvas.h"
-#include "citydlg.h"
-#include "graphics.h"
 #include "wldlg.h"
 
 static GtkWidget *worklists_shell;
@@ -59,10 +66,12 @@ static GtkListStore *worklists_store;
 
 static int max_unit_height = -1, max_unit_width = -1;
 
-static void popup_worklist(struct worklist *pwl);
-static void popdown_worklist(struct worklist *pwl);
+static void reset_global_worklist(GtkWidget *editor,
+                                  struct global_worklist *pgwl);
+static void popup_worklist(struct global_worklist *pgwl);
+static void popdown_worklist(struct global_worklist *pgwl);
 static void dst_row_callback(GtkTreeView *view, GtkTreePath *path,
-			     GtkTreeViewColumn *col, gpointer data);
+                             GtkTreeViewColumn *col, gpointer data);
 
 /****************************************************************
 ...
@@ -106,32 +115,17 @@ static void worklists_destroy_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 void update_worklist_report_dialog(void)
 {
-  struct player *plr;
-  int i;
   GtkTreeIter it;
 
-  if (!worklists_shell) {
-    return;
-  }
-
-  plr = game.player_ptr;
-
   gtk_list_store_clear(worklists_store);
+  global_worklists_iterate(pgwl) {
+    gtk_list_store_append(worklists_store, &it);
 
-  if (!game.player_ptr) {
-    return;
-  }
-
-  for (i = 0; i < MAX_NUM_WORKLISTS; i++) {
-    if (plr->worklists[i].is_valid) {
-      gtk_list_store_append(worklists_store, &it);
-      
-       gtk_list_store_set(worklists_store, &it,
-			  0, plr->worklists[i].name,
-			  1, i,
-			  -1); 
-    }
-  }
+    gtk_list_store_set(worklists_store, &it,
+                       0, global_worklist_name(pgwl),
+                       1, global_worklist_id(pgwl),
+                       -1); 
+  } global_worklists_iterate_end;
 }
 
 /****************************************************************
@@ -139,71 +133,49 @@ void update_worklist_report_dialog(void)
 *****************************************************************/
 static void worklists_response(GtkWidget *w, gint response)
 {
-  struct player *plr;
-  int i, pos;
+  struct global_worklist *pgwl;
+  int id;
   GtkTreeSelection *selection;
   GtkTreeModel *model;
   GtkTreeIter it;
 
-  plr = game.player_ptr;
-
-  if (!game.player_ptr) {
-    return;
-  }
-
-  for (i = 0; i < MAX_NUM_WORKLISTS; i++) {
-    if (!plr->worklists[i].is_valid) {
-      break;
-    }
-  }
-  
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(worklists_list));
 
   if (gtk_tree_selection_get_selected(selection, &model, &it)) {
-    gtk_tree_model_get(model, &it, 1, &pos, -1);
+    gtk_tree_model_get(model, &it, 1, &id, -1);
+    pgwl = global_worklist_by_id(id);
   } else {
-    pos = -1;
+    pgwl = NULL;
+    id = -1;
   }
 
- switch (response) {
-    case WORKLISTS_NEW:
-      /* No more worklist slots free.  (!!!Maybe we should tell the user?) */
-      if (i == MAX_NUM_WORKLISTS) {
-        return;
-      }
+  switch (response) {
+  case WORKLISTS_NEW:
+    global_worklist_new(_("new"));
+    update_worklist_report_dialog();
+    return;
 
-      /* Validate this slot. */
-      init_worklist(&plr->worklists[i]);
-      plr->worklists[i].is_valid = TRUE;
-      strcpy(plr->worklists[i].name, _("new"));
-
-      update_worklist_report_dialog();
+  case WORKLISTS_DELETE:
+    if (!pgwl) {
       return;
+    }
 
-    case WORKLISTS_DELETE:
-      if (pos == -1) {
-	return;
-      }
+    popdown_worklist(pgwl);
+    global_worklist_destroy(pgwl);
+    update_worklist_report_dialog();
+    return;
 
-      popdown_worklist(&plr->worklists[pos]);
-
-      plr->worklists[pos].is_valid = FALSE;
-      plr->worklists[pos].name[0] = '\0';
-
-      update_worklist_report_dialog();
+  case WORKLISTS_PROPERTIES:
+    if (!pgwl) {
       return;
+    }
 
-    case WORKLISTS_PROPERTIES:
-      if (pos == -1) {
-	return;
-      }
+    popup_worklist(pgwl);
+    return;
 
-      popup_worklist(&plr->worklists[pos]);
-      return;
-
-    default:
-      gtk_widget_destroy(worklists_shell);
-      return;
+  default:
+    gtk_widget_destroy(worklists_shell);
+    return;
   }
 }
 
@@ -211,23 +183,28 @@ static void worklists_response(GtkWidget *w, gint response)
 ...
 *****************************************************************/
 static void cell_edited(GtkCellRendererText *cell,
-			const gchar *spath,
-			const gchar *text, gpointer data)
+                        const gchar *spath,
+                        const gchar *text, gpointer data)
 {
   GtkTreePath *path;
   GtkTreeIter it;
-  int pos;
-  struct player *plr = game.player_ptr;
+  struct global_worklist *pgwl;
+  int id;
 
   path = gtk_tree_path_new_from_string(spath);
   gtk_tree_model_get_iter(GTK_TREE_MODEL(worklists_store), &it, path);
-  
-  gtk_tree_model_get(GTK_TREE_MODEL(worklists_store), &it, 1, &pos, -1);
-
-  sz_strlcpy(plr->worklists[pos].name, text);
-  gtk_list_store_set(worklists_store, &it, 0, text, -1);
-
   gtk_tree_path_free(path);
+
+  gtk_tree_model_get(GTK_TREE_MODEL(worklists_store), &it, 1, &id, -1);
+  pgwl = global_worklist_by_id(id);
+
+  if (!pgwl) {
+    gtk_list_store_remove(worklists_store, &it);
+    return;
+  }
+
+  global_worklist_set_name(pgwl, text);
+  gtk_list_store_set(worklists_store, &it, 0, text, -1);
 }
 
 /****************************************************************
@@ -320,7 +297,7 @@ void popup_worklists_report(void)
 ...
 *****************************************************************/
 struct worklist_data {
-  struct worklist *pwl;
+  int global_worklist_id;
   struct city *pcity;
 
   GtkWidget *editor;
@@ -365,12 +342,12 @@ void add_worklist_dnd_target(GtkWidget *w)
 /****************************************************************
 ...
 *****************************************************************/
-static GtkWidget *get_worklist(struct worklist *pwl)
+static GtkWidget *get_worklist(int global_worklist_id)
 {
   if (hash) {
     gpointer ret;
-    
-    ret = g_hash_table_lookup(hash, pwl);
+
+    ret = g_hash_table_lookup(hash, GINT_TO_POINTER(global_worklist_id));
     return ret;
   } else {
     return NULL;
@@ -380,21 +357,21 @@ static GtkWidget *get_worklist(struct worklist *pwl)
 /****************************************************************
 ...
 *****************************************************************/
-static void insert_worklist(struct worklist *pwl, GtkWidget *editor)
+static void insert_worklist(int global_worklist_id, GtkWidget *editor)
 {
   if (!hash) {
     hash = g_hash_table_new(g_direct_hash, g_direct_equal);
   }
-  g_hash_table_insert(hash, pwl, editor);
+  g_hash_table_insert(hash, GINT_TO_POINTER(global_worklist_id), editor);
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-static void delete_worklist(struct worklist *pwl)
+static void delete_worklist(int global_worklist_id)
 {
   if (hash) {
-    g_hash_table_remove(hash, pwl);
+    g_hash_table_remove(hash, GINT_TO_POINTER(global_worklist_id));
   }
 }
 
@@ -409,27 +386,27 @@ static void worklist_response(GtkWidget *shell, gint response)
 /****************************************************************
   Worklist editor window used by the global worklist report.
 *****************************************************************/
-static void popup_worklist(struct worklist *pwl)
+static void popup_worklist(struct global_worklist *pgwl)
 {
   GtkWidget *shell;
 
-  if (!(shell = get_worklist(pwl))) {
+  if (!(shell = get_worklist(global_worklist_id(pgwl)))) {
     GtkWidget *editor;
-    
-    shell = gtk_dialog_new_with_buttons(pwl->name,
-					GTK_WINDOW(worklists_shell),
-					GTK_DIALOG_DESTROY_WITH_PARENT,
-					GTK_STOCK_CLOSE,
-					GTK_RESPONSE_CLOSE,
-					NULL);
+
+    shell = gtk_dialog_new_with_buttons(global_worklist_name(pgwl),
+                                        GTK_WINDOW(worklists_shell),
+                                        GTK_DIALOG_DESTROY_WITH_PARENT,
+                                        GTK_STOCK_CLOSE,
+                                        GTK_RESPONSE_CLOSE,
+                                        NULL);
     gtk_window_set_role(GTK_WINDOW(shell), "worklist");
     gtk_window_set_position(GTK_WINDOW(shell), GTK_WIN_POS_MOUSE);
     g_signal_connect(shell, "response", G_CALLBACK(worklist_response), NULL);
     gtk_window_set_default_size(GTK_WINDOW(shell), 500, 400);
   
     editor = create_worklist();
-    reset_worklist(editor, pwl, NULL);
-    insert_worklist(pwl, editor);
+    reset_global_worklist(editor, pgwl);
+    insert_worklist(global_worklist_id(pgwl), editor);
 
     gtk_container_add(GTK_CONTAINER(GTK_DIALOG(shell)->vbox), editor);
     gtk_widget_show(editor);
@@ -443,11 +420,11 @@ static void popup_worklist(struct worklist *pwl)
 /****************************************************************
 ...
 *****************************************************************/
-static void popdown_worklist(struct worklist *pwl)
+static void popdown_worklist(struct global_worklist *pgwl)
 {
   GtkWidget *shell;
   
-  if ((shell = get_worklist(pwl))) {
+  if ((shell = get_worklist(global_worklist_id(pgwl)))) {
     GtkWidget *parent;
 
     parent = gtk_widget_get_toplevel(shell);
@@ -463,9 +440,9 @@ static void worklist_destroy(GtkWidget *editor, gpointer data)
   struct worklist_data *ptr;
 
   ptr = data;
-  
-  if (ptr->pwl) {
-    delete_worklist(ptr->pwl);
+
+  if (ptr->global_worklist_id != -1) {
+    delete_worklist(ptr->global_worklist_id);
   }
 
   free(ptr);
@@ -476,31 +453,29 @@ static void worklist_destroy(GtkWidget *editor, gpointer data)
 *****************************************************************/
 static void menu_item_callback(GtkMenuItem *item, struct worklist_data *ptr)
 {
-  struct player *plr;
-  gint pos;
-  struct worklist *pwl;
+  struct global_worklist *pgwl;
+  const struct worklist *pwl;
+  size_t i;
 
-  if (!game.player_ptr) {
+  if (NULL == client.conn.playing) {
     return;
   }
 
-  plr = game.player_ptr;
-  pos = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(item), "pos"));
+  pgwl = global_worklist_by_id(GPOINTER_TO_INT
+                               (g_object_get_data(G_OBJECT(item), "id")));
+  if (!pgwl) {
+    return;
+  }
+  pwl = global_worklist_get(pgwl);
 
-  pwl = &plr->worklists[pos];
+  for (i = 0; i < worklist_length(pwl); i++) {
+    GtkTreeIter it;
+    cid cid;
 
-  if (pwl->is_valid) {
-    int i;
+    cid = cid_encode(pwl->entries[i]);
 
-    for (i = 0; i < worklist_length(pwl); i++) {
-      GtkTreeIter it;
-      cid cid;
-
-      cid = cid_encode(pwl->entries[i]);
-
-      gtk_list_store_append(ptr->dst, &it);
-      gtk_list_store_set(ptr->dst, &it, 0, (gint) cid, -1);
-    }
+    gtk_list_store_append(ptr->dst, &it);
+    gtk_list_store_set(ptr->dst, &it, 0, (gint) cid, -1);
   }
 
   commit_worklist(ptr);
@@ -511,29 +486,21 @@ static void menu_item_callback(GtkMenuItem *item, struct worklist_data *ptr)
 *****************************************************************/
 static void popup_add_menu(GtkMenuShell *menu, gpointer data)
 {
-  struct player *plr;
-  int i;
   GtkWidget *item;
 
   gtk_container_foreach(GTK_CONTAINER(menu),
-			(GtkCallback) gtk_widget_destroy, NULL);
-  plr = game.player_ptr;
+                        (GtkCallback) gtk_widget_destroy, NULL);
 
-  if (!game.player_ptr) {
-    return;
-  }
+  global_worklists_iterate(pgwl) {
+    item = gtk_menu_item_new_with_label(global_worklist_name(pgwl));
+    g_object_set_data(G_OBJECT(item), "id",
+                      GINT_TO_POINTER(global_worklist_id(pgwl)));
+    gtk_widget_show(item);
 
-  for (i = 0; i < MAX_NUM_WORKLISTS; i++) {
-    if (plr->worklists[i].is_valid) {
-      item = gtk_menu_item_new_with_label(plr->worklists[i].name);
-      g_object_set_data(G_OBJECT(item), "pos", GINT_TO_POINTER(i));
-      gtk_widget_show(item);
-
-      gtk_container_add(GTK_CONTAINER(menu), item);
-      g_signal_connect(item, "activate",
-		       G_CALLBACK(menu_item_callback), data);
-    }
-  }
+    gtk_container_add(GTK_CONTAINER(menu), item);
+    g_signal_connect(item, "activate",
+                     G_CALLBACK(menu_item_callback), data);
+  } global_worklists_iterate_end;
 
   item = gtk_separator_menu_item_new();
   gtk_widget_show(item);
@@ -563,19 +530,19 @@ static void help_callback(GtkWidget *w, gpointer data)
 
   if (gtk_tree_selection_get_selected(selection, &model, &it)) {
     gint cid;
-    struct city_production target;
+    struct universal target;
 
     gtk_tree_model_get(model, &it, 0, &cid, -1);
     target = cid_decode(cid);
 
-    if (target.is_unit) {
-      popup_help_dialog_typed(utype_name_translation(utype_by_number(target.value)),
+    if (VUT_UTYPE == target.kind) {
+      popup_help_dialog_typed(utype_name_translation(target.value.utype),
 			      HELP_UNIT);
-    } else if (is_great_wonder(target.value)) {
-      popup_help_dialog_typed(improvement_name_translation(target.value),
+    } else if (is_great_wonder(target.value.building)) {
+      popup_help_dialog_typed(improvement_name_translation(target.value.building),
 			      HELP_WONDER);
     } else {
-      popup_help_dialog_typed(improvement_name_translation(target.value),
+      popup_help_dialog_typed(improvement_name_translation(target.value.building),
 			      HELP_IMPROVEMENT);
     }
   } else {
@@ -898,7 +865,7 @@ static void src_selection_callback(GtkTreeSelection *selection, gpointer data)
   /* update widget sensitivity. */
   if (gtk_tree_selection_get_selected(selection, NULL, NULL)) {
     if (can_client_issue_orders()
-      && (!ptr->pcity || city_owner(ptr->pcity) == game.player_ptr)) {
+      && (!ptr->pcity || city_owner(ptr->pcity) == client.conn.playing)) {
       /* if ptr->pcity is NULL, this is a global worklist */
       gtk_widget_set_sensitive(ptr->change_cmd, TRUE);
       gtk_widget_set_sensitive(ptr->prepend_cmd, TRUE);
@@ -968,7 +935,7 @@ static void cell_render_func(GtkTreeViewColumn *col, GtkCellRenderer *rend,
 			     gpointer data)
 {
   gint cid;
-  struct city_production target;
+  struct universal target;
 
   gtk_tree_model_get(model, it, 0, &cid, -1);
   target = cid_production(cid);
@@ -976,7 +943,7 @@ static void cell_render_func(GtkTreeViewColumn *col, GtkCellRenderer *rend,
   if (GTK_IS_CELL_RENDERER_PIXBUF(rend)) {
     GdkPixbuf *pix;
 
-    if (target.is_unit) {
+    if (VUT_UTYPE == target.kind) {
       struct canvas store;
 
       pix = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
@@ -984,19 +951,18 @@ static void cell_render_func(GtkTreeViewColumn *col, GtkCellRenderer *rend,
 
       store.type = CANVAS_PIXBUF;
       store.v.pixbuf = pix;
-      create_overlay_unit(&store, utype_by_number(target.value));
+      create_overlay_unit(&store, target.value.utype);
 
       g_object_set(rend, "pixbuf", pix, NULL);
       g_object_unref(pix);
     } else {
-      struct sprite *sprite = get_building_sprite(tileset, target.value);
+      struct sprite *sprite = get_building_sprite(tileset, target.value.building);
 
       pix = sprite_get_pixbuf(sprite);
       g_object_set(rend, "pixbuf", pix, NULL);
     }
   } else {
     struct city **pcity = data;
-    struct player *plr;
     gint column;
     char *row[4];
     char  buf[4][64];
@@ -1011,10 +977,12 @@ static void cell_render_func(GtkTreeViewColumn *col, GtkCellRenderer *rend,
     get_city_dialog_production_row(row, sizeof(buf[0]), target, *pcity);
     g_object_set(rend, "text", row[column], NULL);
 
-    if (!target.is_unit && *pcity) {
-      plr = city_owner(*pcity);
-      useless = improvement_obsolete(plr, target.value)
-	|| is_building_replaced(*pcity, target.value, RPT_CERTAIN);
+    if (NULL != *pcity  &&  VUT_IMPROVEMENT == target.kind) {
+      struct player *plr = city_owner(*pcity);
+
+      useless = improvement_obsolete(plr, target.value.building)
+	|| (! impr_provides_buildable_units(plr, target.value.building)
+           && is_building_replaced(*pcity, target.value.building, RPT_CERTAIN));
       /* Mark building redundant if we are really certain that there is
        * no use for it. */
       g_object_set(rend, "strikethrough", useless, NULL);
@@ -1051,14 +1019,14 @@ static void populate_view(GtkTreeView *view, struct city **ppcity,
                    i, titles[i], rend, cell_render_func, ppcity, NULL);
   col = gtk_tree_view_get_column(view, i);
 
-  if (show_task_icons) {
+  if (gui_gtk2_show_task_icons) {
     if (max_unit_width == -1 || max_unit_height == -1) {
       update_max_unit_size();
     }
   } else {
     g_object_set(col, "visible", FALSE, NULL);
   }
-  if (show_task_icons) {
+  if (gui_gtk2_show_task_icons) {
     g_object_set(rend, "height", max_unit_height, NULL);
   }
 
@@ -1081,7 +1049,7 @@ static void populate_view(GtkTreeView *view, struct city **ppcity,
     if (pos == 3) {
       *pcol = col;
     }
-    if (show_task_icons) {
+    if (gui_gtk2_show_task_icons) {
       g_object_set(rend, "height", max_unit_height, NULL);
     }
   }
@@ -1097,18 +1065,15 @@ GtkWidget *create_worklist(void)
   GtkWidget *menubar, *item, *menu, *image;
   GtkWidget *table2, *arrow, *check;
   GtkSizeGroup *group;
-
   GtkListStore *src_store, *dst_store;
-
   struct worklist_data *ptr;
 
-
   ptr = fc_malloc(sizeof(*ptr));
 
   src_store = gtk_list_store_new(1, G_TYPE_INT);
   dst_store = gtk_list_store_new(1, G_TYPE_INT);
 
-  ptr->pwl = NULL;
+  ptr->global_worklist_id = -1;
   ptr->pcity = NULL;
   ptr->src = src_store;
   ptr->dst = dst_store;
@@ -1139,7 +1104,7 @@ GtkWidget *create_worklist(void)
   src_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(src_store));
   g_object_unref(src_store);
   gtk_size_group_add_widget(group, src_view);
-  gtk_widget_set_name(src_view, "small font");
+  gtk_widget_set_name(src_view, "small_font");
 
   populate_view(GTK_TREE_VIEW(src_view), &ptr->pcity, &ptr->src_col);
   gtk_container_add(GTK_CONTAINER(sw), src_view);
@@ -1231,7 +1196,7 @@ GtkWidget *create_worklist(void)
   dst_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(dst_store));
   g_object_unref(dst_store);
   gtk_size_group_add_widget(group, dst_view);
-  gtk_widget_set_name(dst_view, "small font");
+  gtk_widget_set_name(dst_view, "small_font");
 
   populate_view(GTK_TREE_VIEW(dst_view), &ptr->pcity, &ptr->dst_col);
   gtk_container_add(GTK_CONTAINER(sw), dst_view);
@@ -1273,7 +1238,7 @@ GtkWidget *create_worklist(void)
   ptr->help_cmd = button;
   gtk_widget_set_sensitive(ptr->help_cmd, FALSE);
 
-  button = gtk_button_new_with_mnemonic(_("Chan_ge Production"));
+  button = gtk_button_new_with_mnemonic(_("Change Prod_uction"));
   gtk_container_add(GTK_CONTAINER(bbox), button);
   g_signal_connect(button, "clicked",
 		   G_CALLBACK(change_callback), ptr);
@@ -1315,65 +1280,75 @@ GtkWidget *create_worklist(void)
 
 
 /****************************************************************
-...
+  ...
 *****************************************************************/
-void reset_worklist(GtkWidget *editor, struct worklist *pwl, struct city *pcity)
+void reset_city_worklist(GtkWidget *editor, struct city *pcity)
 {
   struct worklist_data *ptr;
 
   ptr = g_object_get_data(G_OBJECT(editor), "data");
 
-  ptr->pwl = pwl;
+  ptr->global_worklist_id = -1;
   ptr->pcity = pcity;
 
   gtk_list_store_clear(ptr->src);
   gtk_list_store_clear(ptr->dst);
 
-  /* DND callbacks and object visibility settings. */
-  if (pcity) {
-    g_object_set(ptr->src_col, "visible", TRUE, NULL);
-    g_object_set(ptr->dst_col, "visible", TRUE, NULL);
+  g_object_set(ptr->src_col, "visible", TRUE, NULL);
+  g_object_set(ptr->dst_col, "visible", TRUE, NULL);
 
-    gtk_tree_view_enable_model_drag_source(GTK_TREE_VIEW(ptr->src_view),
-					   GDK_BUTTON1_MASK,
-					   wl_dnd_targets,
-					   G_N_ELEMENTS(wl_dnd_targets),
-					   GDK_ACTION_COPY);
-  } else {
-    gtk_widget_hide(ptr->change_cmd);
-    g_object_set(ptr->src_col, "visible", FALSE, NULL);
-    g_object_set(ptr->dst_col, "visible", FALSE, NULL);
-
-    gtk_tree_view_unset_rows_drag_source(GTK_TREE_VIEW(ptr->src_view));
-  }
+  gtk_tree_view_enable_model_drag_source(GTK_TREE_VIEW(ptr->src_view),
+                                         GDK_BUTTON1_MASK,
+                                         wl_dnd_targets,
+                                         G_N_ELEMENTS(wl_dnd_targets),
+                                         GDK_ACTION_COPY);
 }
 
 /****************************************************************
-...
+  ...
+*****************************************************************/
+static void reset_global_worklist(GtkWidget *editor,
+                                  struct global_worklist *pgwl)
+{
+  struct worklist_data *ptr;
+
+  ptr = g_object_get_data(G_OBJECT(editor), "data");
+
+  ptr->global_worklist_id = global_worklist_id(pgwl);
+  ptr->pcity = NULL;
+
+  gtk_list_store_clear(ptr->src);
+  gtk_list_store_clear(ptr->dst);
+
+  gtk_widget_hide(ptr->change_cmd);
+  g_object_set(ptr->src_col, "visible", FALSE, NULL);
+  g_object_set(ptr->dst_col, "visible", FALSE, NULL);
+
+  gtk_tree_view_unset_rows_drag_source(GTK_TREE_VIEW(ptr->src_view));
+}
+
+/****************************************************************
+  ...
 *****************************************************************/
 void refresh_worklist(GtkWidget *editor)
 {
   struct worklist_data *ptr;
-  struct worklist *pwl, queue;
-
-  struct city_production targets[MAX_NUM_PRODUCTION_TARGETS];
+  const struct global_worklist *pgwl = NULL;
+  struct worklist queue;
+  struct universal targets[MAX_NUM_PRODUCTION_TARGETS];
   int i, targets_used;
   struct item items[MAX_NUM_PRODUCTION_TARGETS];
-
   bool selected;
   gint id;
   GtkTreeIter it;
-
   GtkTreePath *path;
-
   GtkTreeModel *model;
   gboolean exists;
 
   ptr = g_object_get_data(G_OBJECT(editor), "data");
-  pwl = ptr->pwl;
-  
-  if (!pwl) {
-    return;
+
+  if (!ptr->pcity
+      && !(pgwl = global_worklist_by_id(ptr->global_worklist_id))) {
   }
 
   /* refresh source tasks. */
@@ -1385,7 +1360,8 @@ void refresh_worklist(GtkWidget *editor)
   }
   gtk_list_store_clear(ptr->src);
 
-  targets_used = collect_eventually_buildable_targets(targets, ptr->pcity, ptr->future);
+  targets_used = collect_eventually_buildable_targets(targets, ptr->pcity,
+                                                      ptr->future);
   name_and_sort_items(targets, targets_used, items, FALSE, ptr->pcity);
 
   path = NULL;
@@ -1411,11 +1387,12 @@ void refresh_worklist(GtkWidget *editor)
   if (ptr->pcity) {
     city_get_queue(ptr->pcity, &queue);
   } else {
-    copy_worklist(&queue, pwl);
+    assert(NULL != pgwl);
+    worklist_copy(&queue, global_worklist_get(pgwl));
   }
 
   for (i = 0; i < worklist_length(&queue); i++) {
-    struct city_production target = queue.entries[i];
+    struct universal target = queue.entries[i];
 
     if (!exists) {
       gtk_list_store_append(ptr->dst, &it);
@@ -1444,7 +1421,7 @@ void refresh_worklist(GtkWidget *editor)
   /* update widget sensitivity. */
   if (ptr->pcity) {
     if ((can_client_issue_orders() &&
-	 city_owner(ptr->pcity) == game.player_ptr)) {
+	 city_owner(ptr->pcity) == client.conn.playing)) {
       gtk_widget_set_sensitive(ptr->add_cmd, TRUE);
       gtk_widget_set_sensitive(ptr->dst_view, TRUE);
     } else {
@@ -1462,21 +1439,14 @@ void refresh_worklist(GtkWidget *editor)
 *****************************************************************/
 static void commit_worklist(struct worklist_data *ptr)
 {
-  struct worklist *pwl, queue;
+  struct worklist queue;
   GtkTreeModel *model;
   GtkTreeIter it;
-  int i;
-
-  pwl = ptr->pwl;
-  
-  if (!pwl) {
-    return;
-  }
+  size_t i;
 
   model = GTK_TREE_MODEL(ptr->dst);
-  
-  init_worklist(&queue);
-  sz_strlcpy(queue.name, pwl->name);
+
+  worklist_init(&queue);
 
   i = 0;
   if (gtk_tree_model_get_iter_first(model, &it)) {
@@ -1485,7 +1455,7 @@ static void commit_worklist(struct worklist_data *ptr)
 
       /* oops, the player has a worklist longer than what we can store. */
       if (i >= MAX_LEN_WORKLIST) {
-	break;
+        break;
       }
 
       gtk_tree_model_get(model, &it, 0, &cid, -1);
@@ -1504,6 +1474,11 @@ static void commit_worklist(struct worklist_data *ptr)
       refresh_worklist(ptr->editor);
     }
   } else {
-    copy_worklist(pwl, &queue);
+    struct global_worklist *pgwl;
+
+    pgwl = global_worklist_by_id(ptr->global_worklist_id);
+    if (pgwl) {
+      global_worklist_set(pgwl, &queue);
+    }
   }
 }
