@@ -23,6 +23,9 @@
 /* common */
 #include "game.h"
 
+/* client */
+#include "client_main.h"
+
 /* gui-sdl */
 #include "citydlg.h"
 #include "cma_fe.h"
@@ -87,7 +90,7 @@ static int popup_citydlg_from_city_report_callback(struct widget *pWidget)
 static int popup_worklist_from_city_report_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    popup_worklist_editor(pWidget->data.city, &pWidget->data.city->worklist);
+    popup_worklist_editor(pWidget->data.city, NULL);
   }
   return -1;
 }
@@ -256,7 +259,7 @@ static void real_info_city_report_dialog_update(void)
   
   pLast = pBuf;
   count = 0; 
-  city_list_iterate(game.player_ptr->cities, pCity) {
+  city_list_iterate(client.conn.playing->cities, pCity) {
     
     pStr = create_str16_from_char(city_name(pCity), adj_font(12));
     pStr->style |= TTF_STYLE_BOLD;
@@ -278,7 +281,7 @@ static void real_info_city_report_dialog_update(void)
     pBuf->action = popup_citydlg_from_city_report_callback;
     set_wstate(pBuf, FC_WS_NORMAL);
     pBuf->data.city = pCity;
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf , WF_HIDDEN);
     }
     hh = pBuf->size.h;
@@ -291,7 +294,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->style |= SF_CENTER;
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -301,7 +304,7 @@ static void real_info_city_report_dialog_update(void)
     /* ----------- */
     pBuf = create_checkbox(pWindow->dst,
 	    cma_is_city_under_agent(pCity, NULL), WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -317,7 +320,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->fgcol = *get_game_colorRGB(COLOR_OVERVIEW_LAND);	
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -331,7 +334,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->fgcol = *get_game_colorRGB(COLOR_THEME_CITYDLG_FOOD_SURPLUS);
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -359,7 +362,7 @@ static void real_info_city_report_dialog_update(void)
     }
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -373,7 +376,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->fgcol = *get_game_colorRGB(COLOR_THEME_CITYDLG_TRADE);
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -386,7 +389,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->style |= SF_CENTER;
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -400,7 +403,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->fgcol = *get_game_colorRGB(COLOR_THEME_CITYDLG_GOLD);
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -414,7 +417,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->fgcol = *get_game_colorRGB(COLOR_THEME_CITYDLG_SCIENCE);
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -428,7 +431,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->fgcol = *get_game_colorRGB(COLOR_THEME_CITYDLG_LUX);
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -443,7 +446,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->fgcol = *get_game_colorRGB(COLOR_THEME_CITYDLG_PROD);
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -456,7 +459,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->style |= SF_CENTER;
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -471,7 +474,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->fgcol = *get_game_colorRGB(COLOR_THEME_CITYDLG_SUPPORT);
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -485,7 +488,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->fgcol = *get_game_colorRGB(COLOR_THEME_CITYDLG_TRADE);
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 					WF_RESTORE_BACKGROUND);
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -493,15 +496,20 @@ static void real_info_city_report_dialog_update(void)
     add_to_gui_list(MAX_ID - pCity->id, pBuf);
 
     /* ----------- */
-    if(pCity->production.is_unit) {
-      struct unit_type *pUnitType = utype_by_number(pCity->production.value);
-      pLogo = ResizeSurface(get_unittype_surface(pUnitType), adj_size(36), adj_size(24), 1);      
-      togrow = unit_build_shield_cost(pUnitType);
+    if(VUT_UTYPE == pCity->production.kind) {
+      struct unit_type *pUnitType = pCity->production.value.utype;
+      pLogo = ResizeSurfaceBox(get_unittype_surface(pUnitType),
+                               adj_size(36), adj_size(24), 1,
+                               TRUE, TRUE);
+      togrow = utype_build_shield_cost(pUnitType);
       pName = utype_name_translation(pUnitType);
     } else {
-      pLogo = ResizeSurface(get_building_surface(pCity->production.value), adj_size(36), adj_size(24), 1);
-      togrow = impr_build_shield_cost(pCity->production.value);
-      pName = improvement_name_translation(pCity->production.value);
+      struct impr_type *pImprove = pCity->production.value.building;
+      pLogo = ResizeSurfaceBox(get_building_surface(pCity->production.value.building),
+                               adj_size(36), adj_size(24), 1,
+                               TRUE, TRUE);
+      togrow = impr_build_shield_cost(pImprove);
+      pName = improvement_name_translation(pImprove);
     }
     
     if(!worklist_is_empty(&(pCity->worklist))) {
@@ -509,9 +517,7 @@ static void real_info_city_report_dialog_update(void)
       dst.y = 0;
       alphablit(pIcons->pWorklist, NULL, pLogo, &dst);
       my_snprintf(cBuf, sizeof(cBuf), "%s\n(%d/%d)\n%s",
-      	pName, pCity->shield_stock, togrow,
-		pCity->worklist.name[0] != '\0' ?
-			      pCity->worklist.name : _("worklist"));
+      	pName, pCity->shield_stock, togrow, _("worklist"));
     } else {
       my_snprintf(cBuf, sizeof(cBuf), "%s\n(%d/%d)%s",
       	pName, pCity->shield_stock, togrow,
@@ -522,8 +528,7 @@ static void real_info_city_report_dialog_update(void)
     pStr = create_str16_from_char(cBuf, adj_font(10));
     pStr->style |= SF_CENTER;
     
-    togrow = city_turns_to_build(pCity,
-    	pCity->production, TRUE);
+    togrow = city_production_turns_to_build(pCity, TRUE);
     if(togrow == 999)
     {
       my_snprintf(cBuf, sizeof(cBuf), "%s", _("never"));
@@ -535,7 +540,7 @@ static void real_info_city_report_dialog_update(void)
     pBuf = create_icon2(pLogo, pWindow->dst,
     	(WF_WIDGET_HAS_INFO_LABEL|WF_RESTORE_BACKGROUND|WF_FREE_THEME));
     pBuf->string16 = pStr;
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -549,7 +554,7 @@ static void real_info_city_report_dialog_update(void)
     pStr->fgcol = *get_game_colorRGB(COLOR_THEME_CITYREP_TEXT);
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
 			(WF_SELLECT_WITHOUT_BAR|WF_RESTORE_BACKGROUND));
-    if(count > 9 * COL) {
+    if(count > 13 * COL) {
       set_wflag(pBuf, WF_HIDDEN);
     }
     hh = MAX(hh, pBuf->size.h);
@@ -562,7 +567,6 @@ static void real_info_city_report_dialog_update(void)
     }
     
     count += COL;
-    area.h += (hh + adj_size(2));
   } city_list_iterate_end;
   
   H = hh;
@@ -575,18 +579,20 @@ static void real_info_city_report_dialog_update(void)
   if(count) {
     pCityRep->pEndActiveWidgetList = pLast->prev;
     pCityRep->pBeginActiveWidgetList = pCityRep->pBeginWidgetList;
-    if(count > 10 * COL) {
+    if(count > 14 * COL) {
       pCityRep->pActiveWidgetList = pCityRep->pEndActiveWidgetList;
       if(pCityRep->pScroll) {
 	pCityRep->pScroll->count = count;
       }
-      ww = create_vertical_scrollbar(pCityRep, COL, 10, TRUE, TRUE);
+      ww = create_vertical_scrollbar(pCityRep, COL, 14, TRUE, TRUE);
       area.w += ww;
-      area.h = MAX(area.h, (10 * (hh + adj_size(2))));
+      area.h = 14 * (hh + adj_size(2));
+    } else {
+      area.h = (count / COL) * (hh + adj_size(2));
     }
   }
   
-  area.h += pText2->h + adj_size(40);
+  area.h += pText2->h + adj_size(6);
   area.w += adj_size(2);
   
   pLogo = theme_get_background(theme, BACKGROUND_CITYREP);
@@ -638,7 +644,7 @@ static void real_info_city_report_dialog_update(void)
   dst.x = area.x + adj_size(2);
   dst.y = area.y + 1;
   dst.w = (name_w + adj_size(6)) + (pText1->w + adj_size(8)) + adj_size(5);
-  dst.h = area.h - adj_size(33);
+  dst.h = area.h - adj_size(2);
   SDL_FillRectAlpha(pWindow->theme, &dst, &bg_color);
   
   putframe(pWindow->theme, dst.x , dst.y, dst.x + dst.w, dst.y + dst.h - 1,
@@ -670,7 +676,7 @@ static void real_info_city_report_dialog_update(void)
   dst.w = (pIcons->pBIG_Food->w + adj_size(6)) + adj_size(10) +
 	  (pIcons->pBIG_Food_Surplus->w + adj_size(6)) + adj_size(10) +
 	  			pText2->w + adj_size(6 + 2);
-  dst.h = area.h - adj_size(33);
+  dst.h = area.h - adj_size(2);
   SDL_FillRectAlpha(pWindow->theme, &dst, get_game_colorRGB(COLOR_THEME_CITYREP_FOODSTOCK));
   
   putframe(pWindow->theme, dst.x , dst.y, dst.x + dst.w, dst.y + dst.h - 1,
@@ -704,7 +710,7 @@ static void real_info_city_report_dialog_update(void)
 	  (pIcons->pBIG_Coin->w + adj_size(6)) + adj_size(10) +
 	  (pIcons->pBIG_Colb->w + adj_size(6)) + adj_size(10) +
 	  (pIcons->pBIG_Luxury->w + adj_size(6)) + adj_size(4);
-  dst.h = area.h - adj_size(33);
+  dst.h = area.h - adj_size(2);
   
   SDL_FillRectAlpha(pWindow->theme, &dst, get_game_colorRGB(COLOR_THEME_CITYREP_TRADE));
   
@@ -741,7 +747,7 @@ static void real_info_city_report_dialog_update(void)
 	  (pIcons->pBIG_Shield_Corr->w + adj_size(6)) + adj_size(10) +
 	  (pUnits_Icon->w + adj_size(6)) + adj_size(10) +
 	  (pIcons->pBIG_Shield_Surplus->w + adj_size(6)) + adj_size(4);
-  dst.h = area.h - adj_size(33);
+  dst.h = area.h - adj_size(2);
   
   SDL_FillRectAlpha(pWindow->theme, &dst, get_game_colorRGB(COLOR_THEME_CITYREP_PROD));
   
@@ -1013,16 +1019,18 @@ static struct widget * real_city_report_dialog_update_city(struct widget *pWidge
   copy_chars_to_string16(pWidget->string16, cBuf);
   
   /* change production */
-  if(pCity->production.is_unit) {
-    struct unit_type *pUnitType = utype_by_number(pCity->production.value);
-    pLogo = ResizeSurface(get_unittype_surface(utype_by_number(pCity->production.value)),
+  if(VUT_UTYPE == pCity->production.kind) {
+    struct unit_type *pUnitType = pCity->production.value.utype;
+    pLogo = ResizeSurface(get_unittype_surface(pUnitType),
               adj_size(36), adj_size(24), 1);
-    togrow = unit_build_shield_cost(pUnitType);
+    togrow = utype_build_shield_cost(pUnitType);
     pName = utype_name_translation(pUnitType);
   } else {
-    pLogo = ResizeSurface(get_building_surface(pCity->production.value), adj_size(36), adj_size(24), 1);
-    togrow = impr_build_shield_cost(pCity->production.value);
-    pName = improvement_name_translation(pCity->production.value);
+    struct impr_type *pImprove = pCity->production.value.building;
+    pLogo = ResizeSurface(get_building_surface(pCity->production.value.building),
+              adj_size(36), adj_size(24), 1);
+    togrow = impr_build_shield_cost(pImprove);
+    pName = improvement_name_translation(pImprove);
   }
     
   if(!worklist_is_empty(&(pCity->worklist))) {
@@ -1030,9 +1038,7 @@ static struct widget * real_city_report_dialog_update_city(struct widget *pWidge
     dst.y = 0;
     alphablit(pIcons->pWorklist, NULL, pLogo, &dst);
     my_snprintf(cBuf, sizeof(cBuf), "%s\n(%d/%d)\n%s",
-      	pName, pCity->shield_stock, togrow,
-		pCity->worklist.name[0] != '\0' ?
-			      pCity->worklist.name : _("worklist"));
+      	pName, pCity->shield_stock, togrow, _("worklist"));
   } else {
     my_snprintf(cBuf, sizeof(cBuf), "%s\n(%d/%d)",
       		pName, pCity->shield_stock, togrow);
@@ -1046,8 +1052,7 @@ static struct widget * real_city_report_dialog_update_city(struct widget *pWidge
   
   /* hurry productions */
   pWidget = pWidget->prev;
-  togrow = city_turns_to_build(pCity,
-    	pCity->production, TRUE);
+  togrow = city_production_turns_to_build(pCity, TRUE);
   if(togrow == 999)
   {
     my_snprintf(cBuf, sizeof(cBuf), "%s", _("never"));
@@ -1090,7 +1095,7 @@ void city_report_dialog_update(void)
   
     /* find if the lists are identical (if not then rebuild all) */
     pWidget = pCityRep->pEndActiveWidgetList;/* name of first city */
-    city_list_iterate(game.player_ptr->cities, pCity) {
+    city_list_iterate(client.conn.playing->cities, pCity) {
       if (pCity->id == MAX_ID - pWidget->ID) {
         count = COL;
         while(count) {
@@ -1110,7 +1115,7 @@ void city_report_dialog_update(void)
 
     /* update widget city list (widget list is the same that city list) */
     pWidget = pCityRep->pEndActiveWidgetList;
-    city_list_iterate(game.player_ptr->cities, pCity) {
+    city_list_iterate(client.conn.playing->cities, pCity) {
       pWidget = real_city_report_dialog_update_city(pWidget, pCity);  
     } city_list_iterate_end;
 
