@@ -23,12 +23,10 @@
 
 /* common */
 #include "diptreaty.h"
-#include "game.h"
 
 /* client */
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"
-#include "clinet.h"
 #include "packhand.h"
 
 /* gui-sdl */
@@ -95,7 +93,8 @@ void diplomacy_dialog_done()
 *****************************************************************/
 static struct diplomacy_dialog *get_diplomacy_dialog(int other_player_id)
 {
-  struct player *plr0 = game.player_ptr, *plr1 = player_by_number(other_player_id);
+  struct player *plr0 = client.conn.playing;
+  struct player *plr1 = player_by_number(other_player_id);
 
   dialog_list_iterate(dialog_list, pdialog) {
     if ((pdialog->treaty.plr0 == plr0 && pdialog->treaty.plr1 == plr1) ||
@@ -146,7 +145,7 @@ static int remove_clause_callback(struct widget *pWidget)
       pdialog = get_diplomacy_dialog(pWidget->data.cont->id0);
     }
     
-    dsend_packet_diplomacy_remove_clause_req(&aconnection,
+    dsend_packet_diplomacy_remove_clause_req(&client.conn,
                                              player_number(pdialog->treaty.plr1),
                                              pWidget->data.cont->id0,
                                              (enum clause_type) ((pWidget->data.
@@ -211,7 +210,7 @@ void handle_diplomacy_remove_clause(int counterpart, int giver,
 static int cancel_meeting_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    dsend_packet_diplomacy_cancel_meeting_req(&aconnection,
+    dsend_packet_diplomacy_cancel_meeting_req(&client.conn,
 					    pWidget->data.cont->id1);
   }
   return -1;
@@ -220,7 +219,7 @@ static int cancel_meeting_callback(struct widget *pWidget)
 static int accept_treaty_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    dsend_packet_diplomacy_accept_treaty_req(&aconnection,
+    dsend_packet_diplomacy_accept_treaty_req(&client.conn,
 					   pWidget->data.cont->id1);
   }
   return -1;
@@ -251,7 +250,7 @@ static int pact_callback(struct widget *pWidget)
       break;
     }
     
-    dsend_packet_diplomacy_create_clause_req(&aconnection,
+    dsend_packet_diplomacy_create_clause_req(&client.conn,
                                              player_number(pdialog->treaty.plr1),
                                              pWidget->data.cont->id0,
                                              clause_type, 0);
@@ -268,7 +267,7 @@ static int vision_callback(struct widget *pWidget)
       pdialog = get_diplomacy_dialog(pWidget->data.cont->id0);
     }
   
-    dsend_packet_diplomacy_create_clause_req(&aconnection,
+    dsend_packet_diplomacy_create_clause_req(&client.conn,
                                              player_number(pdialog->treaty.plr1),
                                              pWidget->data.cont->id0,
                                              CLAUSE_VISION, 0);
@@ -285,7 +284,7 @@ static int embassy_callback(struct widget *pWidget)
       pdialog = get_diplomacy_dialog(pWidget->data.cont->id0);
     }
   
-    dsend_packet_diplomacy_create_clause_req(&aconnection,
+    dsend_packet_diplomacy_create_clause_req(&client.conn,
                                              player_number(pdialog->treaty.plr1),
                                              pWidget->data.cont->id0,
                                              CLAUSE_EMBASSY, 0);
@@ -313,7 +312,7 @@ static int maps_callback(struct widget *pWidget)
       break;
     }
   
-    dsend_packet_diplomacy_create_clause_req(&aconnection,
+    dsend_packet_diplomacy_create_clause_req(&client.conn,
                                              player_number(pdialog->treaty.plr1),
                                              pWidget->data.cont->id0,
                                              clause_type, 0);
@@ -330,7 +329,7 @@ static int techs_callback(struct widget *pWidget)
       pdialog = get_diplomacy_dialog(pWidget->data.cont->id0);
     }
     
-    dsend_packet_diplomacy_create_clause_req(&aconnection,
+    dsend_packet_diplomacy_create_clause_req(&client.conn,
                                              player_number(pdialog->treaty.plr1),
                                              pWidget->data.cont->id0,
                                              CLAUSE_ADVANCE,
@@ -366,13 +365,14 @@ static int gold_callback(struct widget *pWidget)
     }
     
     if (amount > 0) {
-      dsend_packet_diplomacy_create_clause_req(&aconnection,
+      dsend_packet_diplomacy_create_clause_req(&client.conn,
                                                player_number(pdialog->treaty.plr1),
                                                pWidget->data.cont->id0,
                                                CLAUSE_GOLD, amount);
       
     } else {
-      append_output_window(_("Invalid amount of gold specified."));
+      output_window_append(ftc_client,
+                           _("Invalid amount of gold specified."));
     }
     
     if(amount || !pWidget->string16->text) {
@@ -394,7 +394,7 @@ static int cities_callback(struct widget *pWidget)
       pdialog = get_diplomacy_dialog(pWidget->data.cont->id0);
     }
     
-    dsend_packet_diplomacy_create_clause_req(&aconnection,
+    dsend_packet_diplomacy_create_clause_req(&client.conn,
                                              player_number(pdialog->treaty.plr1),  
                                              pWidget->data.cont->id0,
                                              CLAUSE_CITY,
@@ -447,7 +447,7 @@ static struct ADVANCED_DLG * popup_diplomatic_objects(struct player *pPlayer0,
   height = 0;
   
   /* Pacts. */
-  if (game.player_ptr == pPlayer0 && type != DS_ALLIANCE) {
+  if (pPlayer0 == client.conn.playing && DS_ALLIANCE != type) {
     
     pBuf = create_iconlabel_from_chars(NULL, pWindow->dst,
 			  _("Pacts"), adj_font(12), WF_RESTORE_BACKGROUND);
@@ -558,7 +558,8 @@ static struct ADVANCED_DLG * popup_diplomatic_objects(struct player *pPlayer0,
     count++;
   }
   
-  if (!player_has_embassy(pPlayer1, pPlayer0)) {  
+  /* Don't take in account the embassy effects. */
+  if (!player_has_real_embassy(pPlayer1, pPlayer0)) {  
     pBuf = create_iconlabel_from_chars(NULL, pWindow->dst,
         _("Give embassy"), adj_font(12),
                 (WF_RESTORE_BACKGROUND|WF_DRAW_TEXT_LABEL_WITH_SPACE));
@@ -600,14 +601,13 @@ static struct ADVANCED_DLG * popup_diplomatic_objects(struct player *pPlayer0,
   
   /* Advances */
   {
-    bool flag = FALSE;
-    int i;
+    int flag = A_NONE;
     
-    for (i = 1; i < game.control.num_tech_types; i++) {
-      if (get_invention(pPlayer0, i) == TECH_KNOWN &&
-         tech_is_available(pPlayer1, i) &&
-	(get_invention(pPlayer1, i) == TECH_UNKNOWN || 
-	 get_invention(pPlayer1, i) == TECH_REACHABLE)) {
+    advance_index_iterate(A_FIRST, i) {
+      if (player_invention_state(pPlayer0, i) == TECH_KNOWN &&
+         player_invention_reachable(pPlayer1, i) &&
+	(player_invention_state(pPlayer1, i) == TECH_UNKNOWN || 
+	 player_invention_state(pPlayer1, i) == TECH_PREREQS_KNOWN)) {
 	     
 	     pBuf = create_iconlabel_from_chars(NULL, pWindow->dst,
 		_("Advances"), adj_font(12), WF_RESTORE_BACKGROUND);
@@ -617,7 +617,7 @@ static struct ADVANCED_DLG * popup_diplomatic_objects(struct player *pPlayer0,
              add_to_gui_list(ID_LABEL, pBuf);
 	     count++;
 	     
-	     my_snprintf(cBuf, sizeof(cBuf), "  %s", advance_name_translation(i));
+	     my_snprintf(cBuf, sizeof(cBuf), "  %s", advance_name_translation(advance_by_number(i)));
   
              pBuf = create_iconlabel_from_chars(NULL, pWindow->dst, cBuf, adj_font(12),
 	         (WF_RESTORE_BACKGROUND|WF_DRAW_TEXT_LABEL_WITH_SPACE));
@@ -629,20 +629,19 @@ static struct ADVANCED_DLG * popup_diplomatic_objects(struct player *pPlayer0,
 	     pBuf->data.cont = pCont;
              add_to_gui_list(MAX_ID - i, pBuf);
 	     count++;	
-	     flag = TRUE;
-	     i++;
+	     flag = ++i;
 	     break;
       }
-    }
+    } advance_index_iterate_end;
     
-    if(flag) {
-      for (; i < game.control.num_tech_types; i++) {
-	if (get_invention(pPlayer0, i) == TECH_KNOWN &&
-	   tech_is_available(pPlayer1, i) &&
-	  (get_invention(pPlayer1, i) == TECH_UNKNOWN || 
-	   get_invention(pPlayer1, i) == TECH_REACHABLE)) {
+    if(flag > A_NONE) {
+      advance_index_iterate(flag, i) {
+	if (player_invention_state(pPlayer0, i) == TECH_KNOWN &&
+	   player_invention_reachable(pPlayer1, i) &&
+	  (player_invention_state(pPlayer1, i) == TECH_UNKNOWN || 
+	   player_invention_state(pPlayer1, i) == TECH_PREREQS_KNOWN)) {
 	     
-	     my_snprintf(cBuf, sizeof(cBuf), "  %s", advance_name_translation(i));
+	     my_snprintf(cBuf, sizeof(cBuf), "  %s", advance_name_translation(advance_by_number(i)));
   
              pBuf = create_iconlabel_from_chars(NULL, pWindow->dst, cBuf, adj_font(12),
 	         (WF_RESTORE_BACKGROUND|WF_DRAW_TEXT_LABEL_WITH_SPACE));
@@ -656,7 +655,7 @@ static struct ADVANCED_DLG * popup_diplomatic_objects(struct player *pPlayer0,
 	     count++;
 	}
       }
-    }
+    } advance_index_iterate_end;
     
   }  /* Advances */
   
@@ -1019,7 +1018,7 @@ static void update_acceptance_icons(struct diplomacy_dialog *pdialog)
 static void update_clauses_list(struct diplomacy_dialog *pdialog) {
   SDL_String16 *pStr;
   struct widget *pBuf, *pWindow = pdialog->pdialog->pEndWidgetList;
-  char cBuf[64];
+  char cBuf[128];
   bool redraw_all, scroll = (pdialog->pdialog->pActiveWidgetList != NULL);
   int len = pdialog->pdialog->pScroll->pUp_Left_Button->size.w;
   
@@ -1031,7 +1030,7 @@ static void update_clauses_list(struct diplomacy_dialog *pdialog) {
     pBuf = create_iconlabel(NULL, pWindow->dst, pStr,
      (WF_FREE_DATA|WF_DRAW_TEXT_LABEL_WITH_SPACE|WF_RESTORE_BACKGROUND));
         
-    if(player_number(pclause->from) != game.info.player_idx) {
+    if (pclause->from != client.conn.playing) {
        pBuf->string16->style |= SF_CENTER_RIGHT;  
     }
   
@@ -1147,12 +1146,12 @@ void handle_diplomacy_init_meeting(int counterpart, int initiated_from)
     return;
   }
 
-  if (game.player_ptr->ai.control) {
+  if (client.conn.playing->ai_data.control) {
     return;			/* Don't show if we are AI controlled. */
   }
 
   if (!(pdialog = get_diplomacy_dialog(counterpart))) {
-    pdialog = create_diplomacy_dialog(game.player_ptr,
+    pdialog = create_diplomacy_dialog(client.conn.playing,
 				player_by_number(counterpart));
   } else {
     /* bring existing dialog to front */
@@ -1238,7 +1237,7 @@ static int withdraw_vision_dlg_callback(struct widget *pWidget)
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
     popdown_sdip_dialog();
   
-    dsend_packet_diplomacy_cancel_pact(&aconnection,
+    dsend_packet_diplomacy_cancel_pact(&client.conn,
                                        player_number(pWidget->data.player),
                                        CLAUSE_VISION);
     
@@ -1252,7 +1251,7 @@ static int cancel_pact_dlg_callback(struct widget *pWidget)
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
     popdown_sdip_dialog();
   
-    dsend_packet_diplomacy_cancel_pact(&aconnection,
+    dsend_packet_diplomacy_cancel_pact(&client.conn,
                                        player_number(pWidget->data.player),
                                        CLAUSE_CEASEFIRE);
     
@@ -1265,10 +1264,13 @@ static int call_meeting_dlg_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
     popdown_sdip_dialog();
-    
-    dsend_packet_diplomacy_init_meeting_req(&aconnection,
-                                            player_number(pWidget->data.player));
-    
+
+    if (can_meet_with_player(pWidget->data.player)) {
+      dsend_packet_diplomacy_init_meeting_req(&client.conn,
+                                              player_number
+                                              (pWidget->data.player));
+    }
+
     flush_dirty();
   }
   return -1;
@@ -1404,10 +1406,10 @@ static void popup_war_dialog(struct player *pPlayer)
 void popup_diplomacy_dialog(struct player *pPlayer)
 {
   enum diplstate_type type =
-		  pplayer_get_diplstate(game.player_ptr, pPlayer)->type;
+		  pplayer_get_diplstate(client.conn.playing, pPlayer)->type;
 
-  if(!can_meet_with_player(pPlayer)) {
-    if(type == DS_WAR || pPlayer == game.player_ptr) {
+  if (!can_meet_with_player(pPlayer)) {
+    if (DS_WAR == type || pPlayer == client.conn.playing) {
       flush_dirty();
       return;
     } else {
@@ -1481,9 +1483,9 @@ void popup_diplomacy_dialog(struct player *pPlayer)
       pBuf->next->size.w = pBuf->size.w;
       button_w = MAX(button_w , pBuf->size.w);
       button_h = MAX(button_h , pBuf->size.h);
-    
-      shared = gives_shared_vision(game.player_ptr, pPlayer);
-      
+
+      shared = gives_shared_vision(client.conn.playing, pPlayer);
+
       if(shared) {
         /* shared vision */
         pBuf = create_themeicon_button_from_chars(pTheme->UNITS2_Icon, pWindow->dst,
