@@ -49,9 +49,11 @@
 #include "unitlist.h"
 
 /* client */
-#include "civclient.h"
+#include "client_main.h"
 #include "clinet.h"
+#include "editgui_g.h"
 #include "ggzclient.h"
+#include "ggz_g.h"
 #include "tilespec.h"
 
 /* gui-sdl */
@@ -89,17 +91,6 @@ const bool gui_use_transliteration = FALSE;
 
 Uint32 SDL_Client_Flags = 0;
 
-bool gui_sdl_fullscreen = FALSE;
-
-/* default screen resolution */
-#ifdef SMALL_SCREEN
-int gui_sdl_screen_width = 320;
-int gui_sdl_screen_height = 240;
-#else
-int gui_sdl_screen_width = 640;
-int gui_sdl_screen_height = 480;
-#endif
-
 Uint32 widget_info_counter = 0;
 int MOVE_STEP_X = DEFAULT_MOVE_STEP;
 int MOVE_STEP_Y = DEFAULT_MOVE_STEP;
@@ -146,29 +137,6 @@ enum USER_EVENT_ID {
   EXIT_FROM_EVENT_LOOP
 };
 
-client_option gui_options[] = {
-  GEN_BOOL_OPTION(gui_sdl_fullscreen, N_("Full Screen"), 
-                  N_("If this option is set the client will use the "
-                     "whole screen area for drawing"),
-                  COC_INTERFACE),
-  GEN_INT_OPTION(gui_sdl_screen_width, N_("Screen width"),
-                 N_("This option saves the width of the selected screen "
-                    "resolution"),
-                 COC_INTERFACE),
-  GEN_INT_OPTION(gui_sdl_screen_height, N_("Screen height"),
-                 N_("This option saves the height of the selected screen "
-                    "resolution"),
-                 COC_INTERFACE),
-  GEN_STR_OPTION(gui_sdl_theme_name, N_("Theme"),
-		 N_("By changing this option you change the active theme. "
-		    "This is the same as using the -- --theme command-line "
-		    "parameter."),
-		 COC_GRAPHICS,
-		 get_theme_list, themespec_reread_callback),
-};
-
-const int num_gui_options = ARRAY_SIZE(gui_options);
-
 struct callback {
   void (*callback)(void *data);
   void *data;
@@ -206,7 +174,8 @@ static void print_usage(const char *argv0)
 		       "other thread (only Linux and BeOS)\n"));
   fc_fprintf(stderr, _("  -t,  --theme THEME\tUse GUI theme THEME\n"));
 
-  fc_fprintf(stderr, _("Report bugs at %s.\n"), BUG_URL);
+  /* TRANS: No full stop after the URL, could cause confusion. */
+  fc_fprintf(stderr, _("Report bugs at %s\n"), BUG_URL);
 }
 
 /**************************************************************************
@@ -228,7 +197,7 @@ static void parse_options(int argc, char **argv)
       /* init events in other thread ( only linux and BeOS ) */  
       SDL_InitSubSystem(SDL_INIT_EVENTTHREAD);
     } else if ((option = get_option_malloc("--theme", argv, &i, argc))) {
-      sz_strlcpy(gui_sdl_theme_name, option);
+      sz_strlcpy(gui_sdl_default_theme_name, option);
     }
     i++;
   }
@@ -241,7 +210,7 @@ static void parse_options(int argc, char **argv)
 static Uint16 main_key_down_handler(SDL_keysym Key, void *pData)
 {
   static struct widget *pWidget;
-  if ((pWidget = MainWidgetListKeyScaner(Key)) != NULL) {
+  if ((pWidget = find_next_widget_for_key(NULL, Key)) != NULL) {
     return widget_pressed_action(pWidget);
   } else {
     if (Key.sym == SDLK_TAB) {
@@ -260,21 +229,21 @@ static Uint16 main_key_down_handler(SDL_keysym Key, void *pData)
               struct unit *pUnit;
               struct city *pCity;
               if (NULL != (pUnit = head_of_units_in_focus()) && 
-                (pCity = pUnit->tile->city) != NULL &&
-                city_owner(pCity) == game.player_ptr) {
+                (pCity = tile_city(pUnit->tile)) != NULL &&
+                city_owner(pCity) == client.conn.playing) {
                 popup_city_dialog(pCity);
               }
 	    }
           return ID_ERROR;
 
-	  case SDLK_F1:
-            popup_city_report_dialog(FALSE);
-	  return ID_ERROR;
-	    
 	  case SDLK_F2:
 	    popup_activeunits_report_dialog(FALSE);
 	  return ID_ERROR;
 	   
+	  case SDLK_F4:
+            popup_city_report_dialog(FALSE);
+	  return ID_ERROR;
+	    
 	  case SDLK_F7:
             send_report_request(REPORT_WONDERS_OF_THE_WORLD);
           return ID_ERROR;
@@ -283,7 +252,7 @@ static Uint16 main_key_down_handler(SDL_keysym Key, void *pData)
             send_report_request(REPORT_TOP_5_CITIES);
           return ID_ERROR;
 	    
-	  case SDLK_F10:
+	  case SDLK_F9:
             if(is_meswin_open()) {
               popdown_meswin_dialog();
             } else {
@@ -297,7 +266,7 @@ static Uint16 main_key_down_handler(SDL_keysym Key, void *pData)
           return ID_ERROR;
 	    
 	  case SDLK_F12:
-            popup_spaceship_dialog(game.player_ptr);
+            popup_spaceship_dialog(client.conn.playing);
           return ID_ERROR;
 	  
 	  default:
@@ -326,12 +295,16 @@ static Uint16 main_key_up_handler(SDL_keysym Key, void *pData)
 **************************************************************************/
 static Uint16 main_mouse_button_down_handler(SDL_MouseButtonEvent *pButtonEvent, void *pData)
 {
-  static struct widget *pWidget;
-  if ((pWidget = MainWidgetListScaner(pButtonEvent->x, pButtonEvent->y)) != NULL) {
+  struct widget *pWidget;
+  
+  if ((pWidget = find_next_widget_at_pos(NULL,
+                                         pButtonEvent->x,
+                                         pButtonEvent->y)) != NULL) {
     if (!(get_wstate(pWidget) == FC_WS_DISABLED)) {
       return widget_pressed_action(pWidget);
     }
   } else {
+    /* no visible widget at this position -> map click */ 
 #ifdef UNDER_CE
     if (!check_scroll_area(pButtonEvent->x, pButtonEvent->y)) {
 #endif        
@@ -353,7 +326,7 @@ static Uint16 main_mouse_button_down_handler(SDL_MouseButtonEvent *pButtonEvent,
 static Uint16 main_mouse_button_up_handler(SDL_MouseButtonEvent *pButtonEvent, void *pData)
 {
   if (button_behavior.button_down_ticks /* button wasn't pressed over a widget */
-     && !MainWidgetListScaner(pButtonEvent->x, pButtonEvent->y)) {
+     && !find_next_widget_at_pos(NULL, pButtonEvent->x, pButtonEvent->y)) {
     *button_behavior.event = *pButtonEvent;
     button_up_on_map(&button_behavior);
   }
@@ -400,7 +373,9 @@ static Uint16 main_mouse_motion_handler(SDL_MouseMotionEvent *pMotionEvent, void
   }
 #endif          
 
-  if ((pWidget = MainWidgetListScaner(pMotionEvent->x, pMotionEvent->y)) != NULL) {
+  if ((pWidget = find_next_widget_at_pos(NULL,
+                                         pMotionEvent->x,
+                                         pMotionEvent->y)) != NULL) {
     update_mouse_cursor(CURSOR_DEFAULT);
     if (!(get_wstate(pWidget) == FC_WS_DISABLED)) {
       widget_sellected_action(pWidget);
@@ -558,7 +533,7 @@ Uint16 gui_event_loop(void *pData,
       tv.tv_sec = 0;
       tv.tv_usec = 10000;/* 10ms*/
     
-      result = my_select(MAX(net_socket, ggz_socket) + 1, &civfdset, NULL, NULL, &tv);
+      result = fc_select(MAX(net_socket, ggz_socket) + 1, &civfdset, NULL, NULL, &tv);
       if (result < 0) {
         if (errno != EINTR) {
 	  break;
@@ -656,8 +631,8 @@ Uint16 gui_event_loop(void *pData,
         case SDL_KEYDOWN:
           switch(Main.event.key.keysym.sym) {
             case SDLK_PRINT:
-              freelog(LOG_NORMAL, _("Making screenshot fc_%05d.bmp"), schot_nr);
               my_snprintf(schot, sizeof(schot), "fc_%05d.bmp", schot_nr++);
+              freelog(LOG_NORMAL, _("Making screenshot %s"), schot);
               SDL_SaveBMP(Main.screen, schot);
             break;
             
@@ -815,7 +790,7 @@ void ui_init(void)
     putframe(Main.map, 0, 0, Main.map->w - 1, Main.map->h - 1,
     			SDL_MapRGB(Main.map->format, 255, 255, 255));
     FREESURFACE(pBgd);
-    SDL_WM_SetCaption("SDL Client for Freeciv", "Freeciv");
+    SDL_WM_SetCaption(_("SDL Client for Freeciv"), _("Freeciv"));
   } else {
     
 #ifndef SMALL_SCREEN
@@ -830,7 +805,7 @@ void ui_init(void)
       FREESURFACE(pBgd);
     } else {
       SDL_FillRect(Main.map, NULL, SDL_MapRGB(Main.map->format, 0, 0, 128));
-      SDL_WM_SetCaption("SDL Client for Freeciv", "Freeciv");
+      SDL_WM_SetCaption(_("SDL Client for Freeciv"), _("Freeciv"));
     }
   }
   
@@ -860,6 +835,14 @@ void ui_init(void)
   flush_all();
 }
 
+/****************************************************************************
+  Extra initializers for client options.
+****************************************************************************/
+void gui_options_extra_init(void)
+{
+  /* Nothing to do. */
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -875,6 +858,16 @@ static void clear_double_messages_call(void)
   }
 }
 
+/**************************************************************************
+  Entry point for freeciv client program. SDL has macro magic to turn
+  this in to function named SDL_main() and it provides actual main()
+  itself.
+**************************************************************************/
+int main(int argc, char **argv)
+{
+  return client_main(argc, argv);
+}
+
 /**************************************************************************
   The main loop for the UI.  This is called from main(), and when it
   exits the client will exit.
@@ -988,12 +981,12 @@ void ui_main(int argc, char *argv[])
 #endif
   }
 
-  /* SDL_WM_SetCaption("SDL Client for Freeciv", "Freeciv"); */
+  /* SDL_WM_SetCaption(_("SDL Client for Freeciv"), _("Freeciv")); */
 
   /* this need correct Main.screen size */
   init_mapcanvas_and_overview();    
   
-  set_client_state(C_S_PREPARING);
+  set_client_state(C_S_DISCONNECTED);
 
   /* Main game loop */
   gui_event_loop(NULL, NULL, main_key_down_handler, main_key_up_handler,
@@ -1019,7 +1012,6 @@ void ui_exit()
   diplomacy_dialog_done();
   intel_dialog_done();  
 
-  callback_list_unlink_all(callbacks);
   callback_list_free(callbacks);
   
   unload_cursors();
@@ -1036,6 +1028,14 @@ void ui_exit()
   quit_sdl();
 }
 
+/**************************************************************************
+  Return our GUI type
+**************************************************************************/
+enum gui_type get_gui_type(void)
+{
+  return GUI_SDL;
+}
+
 /**************************************************************************
   Make a bell noise (beep).  This provides low-level sound alerts even
   if there is no real sound support.
@@ -1119,3 +1119,60 @@ void add_idle_callback(void (callback)(void *), void *data)
 
   callback_list_prepend(callbacks, cb);
 }
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_tileset_changed(void)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_refresh(void)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_popup_properties(const struct tile_list *tiles, int objtype)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_popdown_all(void)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_notify_object_changed(int objtype, int object_id, bool remove)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_notify_object_created(int tag, int id)
+{}
+
+/****************************************************************************
+  Stub for ggz function
+****************************************************************************/
+void gui_ggz_embed_leave_table(void)
+{}
+
+/****************************************************************************
+  Stub for ggz function
+****************************************************************************/
+void gui_ggz_embed_ensure_server(void)
+{}
+
+
+/**************************************************************************
+  Updates a gui font style.
+**************************************************************************/
+void gui_update_font(const char *font_name, const char *font_value)
+{
+  /* PORTME */
+}
