@@ -29,8 +29,8 @@
 #include "fcintl.h"
 #include "log.h"
 
-/* common */
-#include "game.h"
+/* client */
+#include "client_main.h"
 
 /* gui-sdl */
 #include "colors.h"
@@ -53,7 +53,7 @@ static SDL_Surface *pFuture_Tech_Icon;
 
 #define load_GUI_surface(pSpr, pStruct, pSurf, tag)		  \
 do {								  \
-  pSpr = theme_lookup_sprite_tag_alt(theme, tag, "", TRUE, "", ""); \
+  pSpr = theme_lookup_sprite_tag_alt(theme, LOG_FATAL, tag, "", "", ""); \
   assert(pSpr != NULL);				                  \
   pStruct->pSurf = adj_surf(GET_SURF(pSpr));            	  \
   FREESURFACE(GET_SURF(pSpr));                                    \
@@ -121,8 +121,8 @@ void reload_citizens_icons(int style)
   ...
 **************************************************************************/
 void tilespec_setup_city_gfx(void) {
-  struct sprite *pSpr = theme_lookup_sprite_tag_alt(
-                                    theme, "theme.city", "", TRUE, "", "");    
+  struct sprite *pSpr =
+    theme_lookup_sprite_tag_alt(theme, LOG_FATAL, "theme.city", "", "", "");    
 
   pCity_Surf = (pSpr ? adj_surf(GET_SURF(pSpr)) : NULL);
   
@@ -347,7 +347,7 @@ void tilespec_setup_theme(void)
   load_order_theme_surface(pBuf, OTransform_Icon, "theme.order_transform");
   load_order_theme_surface(pBuf, OAddCity_Icon, "theme.order_add_to_city");
   load_order_theme_surface(pBuf, OWonder_Icon, "theme.order_carravan_wonder");
-  load_order_theme_surface(pBuf, OTrade_Icon, "theme.order_carravan_traderoute");
+  load_order_theme_surface(pBuf, OTrade_Icon, "theme.order_carravan_trade_route");
   load_order_theme_surface(pBuf, OSpy_Icon, "theme.order_spying");
   load_order_theme_surface(pBuf, OWakeUp_Icon, "theme.order_wakeup");
   load_order_theme_surface(pBuf, OReturn_Icon, "theme.order_return");
@@ -553,16 +553,16 @@ SDL_Surface * get_tech_icon(Tech_type_id tech)
 **************************************************************************/
 SDL_Color * get_tech_color(Tech_type_id tech_id)
 {
-  if (tech_is_available(game.player_ptr, tech_id))
+  if (player_invention_reachable(client.conn.playing, tech_id))
   {
-    switch (get_invention(game.player_ptr, tech_id))
+    switch (player_invention_state(client.conn.playing, tech_id))
     {
       case TECH_UNKNOWN:
-        return get_game_colorRGB(COLOR_REQTREE_UNREACHABLE);	  
+        return get_game_colorRGB(COLOR_REQTREE_UNKNOWN);
       case TECH_KNOWN:
         return get_game_colorRGB(COLOR_REQTREE_KNOWN);
-      case TECH_REACHABLE:
-        return get_game_colorRGB(COLOR_REQTREE_REACHABLE);
+      case TECH_PREREQS_KNOWN:
+        return get_game_colorRGB(COLOR_REQTREE_PREREQS_KNOWN);
       default:
         return get_game_colorRGB(COLOR_REQTREE_BACKGROUND);
     }
