@@ -21,8 +21,10 @@
 #include "astring.h"
 #include "fcintl.h"
 
+/* common */
+
 /* client */
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"
 
 /* gui-sdl */
@@ -181,9 +183,9 @@ static int toggle_draw_neutral_status_callback(struct widget *pWidget)
 
 static bool have_diplomat_info_about(struct player *pPlayer)
 {
-  return (pPlayer == game.player_ptr ||
-  	(pPlayer != game.player_ptr
-	  && player_has_embassy(game.player_ptr, pPlayer)));
+  return (pPlayer == client.conn.playing
+	  || (pPlayer != client.conn.playing
+	   && player_has_embassy(client.conn.playing, pPlayer)));
 }
 
 /**************************************************************************
@@ -306,7 +308,6 @@ void popup_players_dialog(bool raise)
   SDL_Rect dst;
   int i, n, h;
   double a, b, r;
-  struct player *pPlayer;
   SDL_Rect area;
   
   if (pPlayers_Dlg) {
@@ -314,12 +315,12 @@ void popup_players_dialog(bool raise)
   }
   
   n = 0;
-  for(i=0; i<game.info.nplayers; i++) {
-    if(is_barbarian(get_player(i))) {
+  players_iterate(pPlayer) {
+    if(is_barbarian(pPlayer)) {
       continue;
     }
     n++;
-  }
+  } players_iterate_end;
 
   if(n < 2) {
     return;
@@ -396,9 +397,7 @@ void popup_players_dialog(bool raise)
   } 
   /* ---------- */
   
-  for(i=0; i<game.info.nplayers; i++) {
-    pPlayer = get_player(i);
-      
+  players_iterate(pPlayer) {
     if(is_barbarian(pPlayer)) {
       continue;
     }
@@ -415,7 +414,7 @@ void popup_players_dialog(bool raise)
     pBuf->string16 = pStr;
       
     if(!pPlayer->is_alive) {
-      pStr = create_str16_from_char("R.I.P" , adj_font(10));
+      pStr = create_str16_from_char(_("R.I.P.") , adj_font(10));
       pStr->style |= TTF_STYLE_BOLD;
       pStr->fgcol = *get_game_colorRGB(COLOR_THEME_PLRDLG_TEXT);
       pLogo = create_text_surf_from_str16(pStr);
@@ -437,7 +436,7 @@ void popup_players_dialog(bool raise)
     
     add_to_gui_list(ID_LABEL, pBuf);
     
-  }
+  } players_iterate_end;
   
   pPlayers_Dlg->pBeginWidgetList = pBuf;
 
@@ -575,7 +574,7 @@ static int player_nation_callback(struct widget *pWidget)
       }
     break;
     default:
-      if(pPlayer->player_no != game.info.player_idx) {
+      if (pPlayer != client.conn.playing) {
         popup_diplomacy_dialog(pPlayer);
       }
     break;
@@ -593,8 +592,7 @@ void popup_players_nations_dialog(void)
   SDL_Surface *pLogo = NULL;
   SDL_String16 *pStr;
   char cBuf[128], *state;
-  int i, n = 0, w = 0, units_h = 0;
-  struct player *pPlayer;
+  int n = 0, w = 0, units_h = 0;
   const struct player_diplstate *pDS;
   SDL_Rect area;
   
@@ -631,17 +629,15 @@ void popup_players_nations_dialog(void)
   add_to_gui_list(ID_BUTTON, pBuf);
   /* ---------- */
   
-  for(i=0; i<game.info.nplayers; i++) {
-    if(i != game.info.player_idx) {
-      pPlayer = get_player(i);
-      
+  players_iterate(pPlayer) {
+    if (pPlayer != client.conn.playing) {
       if(!pPlayer->is_alive || is_barbarian(pPlayer)) {
         continue;
       }
       
-      pDS = pplayer_get_diplstate(game.player_ptr, pPlayer);
+      pDS = pplayer_get_diplstate(client.conn.playing, pPlayer);
             
-      if(pPlayer->ai.control) {
+      if(pPlayer->ai_data.control) {
 	state = _("AI");
       } else {
         if (pPlayer->is_connected) {
@@ -727,7 +723,7 @@ void popup_players_nations_dialog(void)
       
       n++;  
     }
-  }
+  } players_iterate_end;
   pShort_Players_Dlg->pBeginWidgetList = pBuf;
   pShort_Players_Dlg->pBeginActiveWidgetList = pShort_Players_Dlg->pBeginWidgetList;
   pShort_Players_Dlg->pEndActiveWidgetList = pWindow->prev->prev;
