@@ -18,15 +18,20 @@
 #include <windows.h>
 #include <windowsx.h>
 
-#include "events.h"
+/* utility */
 #include "fcintl.h"
-#include "game.h"
+#include "log.h"
 #include "mem.h"
 #include "support.h"
 
+/* common */
+#include "events.h"
+#include "game.h"
+
+/* client */
 #include "chatline_g.h"
 #include "citydlg_g.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "cma_fec.h"
 #include "messagewin_g.h"
 
@@ -52,9 +57,9 @@ struct cma_dialog {
   HWND change;
   HWND perm;
   HWND release;
-  HWND minimal_surplus[O_MAX];
+  HWND minimal_surplus[O_LAST];
   HWND happy;
-  HWND factor[O_MAX + 1];
+  HWND factor[O_LAST + 1];
   int id;
 };
 
@@ -236,7 +241,7 @@ static void set_hscales(const struct cm_parameter *const parameter,
   } output_type_iterate_end;
   Button_SetCheck(pdialog->happy,
 		  parameter->require_happy ? BST_CHECKED : BST_UNCHECKED);
-  handle_hscroll(pdialog->mainwin, pdialog->factor[O_COUNT],
+  handle_hscroll(pdialog->mainwin, pdialog->factor[O_LAST],
 		 SB_THUMBTRACK, parameter->happy_factor);
   allow_refreshes = 1;
 }
@@ -256,7 +261,7 @@ void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
   if (!pdialog)
     return;
   /* fill in result label */
-  cm_copy_result_from_city(pcity, &result);
+  cm_result_from_main_map(&result, pcity, TRUE);
   SetWindowText(pdialog->result_label,
 		(char *) cmafec_get_result_descr(pcity, &result, &param));
   /* if called from a hscale, we _don't_ want to do this */
@@ -520,7 +525,7 @@ static void hscale_changed(struct cma_dialog *pdialog)
   } output_type_iterate_end;
   
   param.require_happy = (Button_GetCheck(pdialog->happy) == BST_CHECKED) ? 1 : 0;
-  param.happy_factor = ScrollBar_GetPos(pdialog->factor[O_COUNT]);
+  param.happy_factor = ScrollBar_GetPos(pdialog->factor[O_LAST]);
   
   /* save the change */
   cmafec_set_fe_parameter(pdialog->pcity, &param);
