@@ -90,8 +90,7 @@ void load_intro_gfx(void)
   Load the cursors (mouse substitute sprites), including a goto cursor,
   an airdrop cursor, a nuke cursor, and a patrol cursor.
 **************************************************************************/
-void
-load_cursors(void)
+void load_cursors(void)
 {
   enum cursor_type cursor;
   ICONINFO ii;
@@ -310,10 +309,7 @@ static void flush_hbmp_cache()
     crect_list_iterate(free_rects, pcrect) {
       free(pcrect);
     } crect_list_iterate_end;
-
-    crect_list_unlink_all(free_rects);
-
-    free(free_rects);
+    crect_list_free(free_rects);
   }
 
   free_rects = crect_list_new();
@@ -323,10 +319,7 @@ static void flush_hbmp_cache()
     crect_list_iterate(used_rects, pcrect) {
       free(pcrect);
     } crect_list_iterate_end;
-
-    crect_list_unlink_all(used_rects);
-
-    free(used_rects);
+    crect_list_free(used_rects);
   }
 
   used_rects = crect_list_new();
@@ -712,9 +705,9 @@ BITMAP *bmp_load_png(const char *filename)
   int has_mask;
   BYTE *src, *dst;
 
-  if (!(fp = fopen(filename, "rb"))) {
+  if (!(fp = fc_fopen(filename, "rb"))) {
     MessageBox(NULL, "failed reading", filename, MB_OK);
-    freelog(LOG_FATAL, "Failed reading PNG file: %s", filename);
+    freelog(LOG_FATAL, "Failed reading PNG file: \"%s\"", filename);
     exit(EXIT_FAILURE);
   }
     
@@ -731,7 +724,7 @@ BITMAP *bmp_load_png(const char *filename)
   }
    
   if (setjmp(pngp->jmpbuf)) {
-    freelog(LOG_FATAL, "Failed while reading PNG file: %s", filename);
+    freelog(LOG_FATAL, "Failed while reading PNG file: \"%s\"", filename);
     exit(EXIT_FAILURE);
   }
 
