@@ -21,9 +21,11 @@
 #include <string.h>  
 #include <windows.h>
 
+/* common & utility */
 #include "astring.h"
 #include "capability.h"
 #include "fcintl.h"
+#include "game.h"
 #include "log.h"
 #include "government.h"
 #include "map.h"
@@ -33,9 +35,11 @@
 #include "unit.h"
 #include "unitlist.h"
  
+/* client */
 #include "chatline.h"
 #include "cityrep.h"
-#include "civclient.h"
+#include "client_main.h"
+#include "climisc.h"
 #include "clinet.h"
 #include "connectdlg_common.h"
 #include "control.h"
@@ -107,7 +111,7 @@ enum MenuID {
 
   IDM_ORDERS_MENU,
   IDM_ORDERS_BUILD_CITY,	/* shared with BUILD_WONDER */
-  IDM_ORDERS_ROAD,		/* shared with TRADEROUTE */
+  IDM_ORDERS_ROAD,		/* shared with TRADE_ROUTE */
   IDM_ORDERS_IRRIGATE,
   IDM_ORDERS_MINE,
   IDM_ORDERS_TRANSFORM,
@@ -155,7 +159,10 @@ enum MenuID {
   IDM_HELP_CHATLINE,
   IDM_HELP_WORKLIST_EDITOR,
   IDM_HELP_CMA,
+  IDM_HELP_OVERVIEW,
   IDM_HELP_PLAYING,
+  IDM_HELP_ECONOMY,
+  IDM_HELP_CITIES,
   IDM_HELP_IMPROVEMENTS,
   IDM_HELP_UNITS,
   IDM_HELP_COMBAT,
@@ -164,8 +171,9 @@ enum MenuID {
   IDM_HELP_TERRAIN,
   IDM_HELP_WONDERS,
   IDM_HELP_GOVERNMENT,
-  IDM_HELP_HAPPINESS,
+  IDM_HELP_DIPLOMACY,
   IDM_HELP_SPACE_RACE,
+  IDM_HELP_RULESET,
   IDM_HELP_COPYING,
   IDM_HELP_ABOUT,
 
@@ -318,14 +326,14 @@ static struct my_menu main_menu[] = {
   {NULL, 0},
 
 
-  {N_("Gov_ernment"),				IDM_SUBMENU},
-  {N_("_Tax Rates")		"\tShift+T",	IDM_GOVERNMENT_TAX_RATE},
+  {N_("_Edit"),					IDM_SUBMENU},
+  {N_("_Tax Rates...")		"\tCtl+T",	IDM_GOVERNMENT_TAX_RATE},
   { "", IDM_SEPARATOR},
   {N_("_Find City")		"\tCtl+F",	IDM_GOVERNMENT_FIND_CITY},
-  {N_("_Worklists")		"\tCtl+W",	IDM_GOVERNMENT_WORKLISTS},
+  {N_("Work_lists")		"\tCtl+L",	IDM_GOVERNMENT_WORKLISTS},
   { "", IDM_SEPARATOR},
-  {N_("_Change Government"),			IDM_SUBMENU},
-  {N_("_Revolution"),				IDM_GOVERNMENT_REVOLUTION},
+  {N_("_Government"),				IDM_SUBMENU},
+  {N_("_Revolution...")         "\tCtl+Shift+R",IDM_GOVERNMENT_REVOLUTION},
   {"", IDM_SEPARATOR},
   {NULL, 0},
   {NULL, 0},
@@ -336,7 +344,7 @@ static struct my_menu main_menu[] = {
   {N_("National _Borders")	"\tCtl+B", 	IDM_VIEW_NATIONAL_BORDERS},
   {N_("City _Names")		"\tCtl+N",	IDM_VIEW_CITY_NAMES},
   {N_("City G_rowth")		"\tCtl+R", 	IDM_VIEW_CITY_GROWTH},
-  {N_("City _Productions")	"\tCtl+P",	IDM_VIEW_CITY_PRODUCTIONS},
+  {N_("City _Production Levels") "\tCtl+P",	IDM_VIEW_CITY_PRODUCTIONS},
   {N_("Terrain"),				IDM_VIEW_TERRAIN},
   {N_("Coastline"),				IDM_VIEW_COASTLINE},
   {N_("Improvements"),				IDM_SUBMENU},
@@ -369,22 +377,22 @@ static struct my_menu main_menu[] = {
   {N_("_Sentry")		"\tS",		IDM_ORDERS_SENTRY},
   {N_("Pillage")		"\tShift+P",	IDM_ORDERS_PILLAGE},
   {"", IDM_SEPARATOR},
-  {N_("Make _Homecity")		"\tH",		IDM_ORDERS_HOMECITY},
+  {N_("Set _Home City")		"\tH",		IDM_ORDERS_HOMECITY},
   {N_("_Load")			"\tL",		IDM_ORDERS_LOAD},
   {N_("_Unload")		"\tU",		IDM_ORDERS_UNLOAD},
-  {N_("Wake up o_thers")	"\tShift+W",	IDM_ORDERS_WAKEUP_OTHERS},
+  {N_("Uns_entry All On Tile")	"\tShift+S",	IDM_ORDERS_WAKEUP_OTHERS},
   {"", IDM_SEPARATOR},
   {N_("Auto Settler")		"\tA",		IDM_ORDERS_AUTO_SETTLER},
   {N_("Auto E_xplore")		"\tX",		IDM_ORDERS_AUTO_EXPLORE},
-  {N_("Connect"), 				IDM_SUBMENU},
-  {N_("_Road")			"\tCtl+Shift+R",IDM_ORDERS_CONNECT_ROAD},
-  {N_("Rai_l")			"\tCtl+Shift+L",IDM_ORDERS_CONNECT_RAIL},
-  {N_("_Irrigate")		"\tCtl+Shift+I",IDM_ORDERS_CONNECT_IRRIGATE},
+  {N_("Connect With"), 				IDM_SUBMENU},
+  {N_("_Road")			"\tShift+R",IDM_ORDERS_CONNECT_ROAD},
+  {N_("Rai_l")			"\tShift+L",IDM_ORDERS_CONNECT_RAIL},
+  {N_("_Irrigate")		"\tShift+I",IDM_ORDERS_CONNECT_IRRIGATE},
   {NULL, 0},
   {N_("Patrol")			"\tQ",		IDM_ORDERS_PATROL},
-  {N_("_Go to")			"\tG",		IDM_ORDERS_GOTO},
-  {N_("Go/Airlift to City")	"\tShift+L",	IDM_ORDERS_GOTO_CITY},
-  {N_("Return to nearest city") "\tShift+G",	IDM_ORDERS_RETURN},
+  {N_("_Go to Tile")		"\tG",		IDM_ORDERS_GOTO},
+  {N_("Go _to/Airlift to City...")	"\tT",		IDM_ORDERS_GOTO_CITY},
+  {N_("Return to Nearest City") "\tShift+G",	IDM_ORDERS_RETURN},
   {"", IDM_SEPARATOR},
   {N_("Disband Unit")		"\tShift+D",	IDM_ORDERS_DISBAND},
   {N_("Diplomat/Spy Actions")	"\tD", 		IDM_ORDERS_DIPLOMAT_DLG},
@@ -396,44 +404,48 @@ static struct my_menu main_menu[] = {
 
 
   {N_("_Reports"),				IDM_SUBMENU},
-  {N_("_Cities")		"\tF1",		IDM_REPORTS_CITIES},
   {N_("_Units")			"\tF2",		IDM_REPORTS_UNITS},
   /* TRANS: Nations report action */
   {N_("_Nations")		"\tF3",		IDM_REPORTS_PLAYERS},
+  {N_("_Cities")		"\tF4",		IDM_REPORTS_CITIES},
   {N_("_Economy")		"\tF5",		IDM_REPORTS_ECONOMY},
   /* TRANS: Research report action */
   {N_("_Research")		"\tF6",		IDM_REPORTS_SCIENCE},
   {"", IDM_SEPARATOR},
   {N_("_Wonders of the World")	"\tF7",		IDM_REPORTS_WONDERS},
-  {N_("_Top Five Cities")	"\tF8",		IDM_REPORTS_TOP_CITIES},
-  {N_("_Messages")		"\tF10",	IDM_REPORTS_MESSAGES},
+  {N_("Top _Five Cities")	"\tF8",		IDM_REPORTS_TOP_CITIES},
+  {N_("_Messages")		"\tF9",		IDM_REPORTS_MESSAGES},
   {N_("_Demographics")		"\tF11",	IDM_REPORTS_DEMOGRAPHICS},
   {N_("_Spaceship")		"\tF12",	IDM_REPORTS_SPACESHIP},
   {NULL, 0},
 
 
-  {N_("_Help"),					IDM_SUBMENU},
-  {N_("Language_s"),				IDM_HELP_LANGUAGES},
-  {N_("Co_nnecting"),				IDM_HELP_CONNECTING},
-  {N_("C_ontrols"),				IDM_HELP_CONTROLS},
-  {N_("C_hatline"),				IDM_HELP_CHATLINE},
-  {N_("_Worklist Editor"),			IDM_HELP_WORKLIST_EDITOR},
-  {N_("Citizen _Management"),			IDM_HELP_CMA},
-  {N_("_Playing"),				IDM_HELP_PLAYING},
+  {N_("Help"),					IDM_SUBMENU},
+  {N_("Overview"),				IDM_HELP_OVERVIEW},
+  {N_("Strategy and Tactics"),			IDM_HELP_PLAYING},
+  {N_("Terrain"),				IDM_HELP_TERRAIN},
+  {N_("Economy"),				IDM_HELP_ECONOMY},
+  {N_("Cities"),				IDM_HELP_CITIES},
+  {N_("City Improvements"),			IDM_HELP_IMPROVEMENTS},
+  {N_("Wonders of the World"),			IDM_HELP_WONDERS},
+  {N_("Units"),					IDM_HELP_UNITS},
+  {N_("Combat"),				IDM_HELP_COMBAT},
+  {N_("Zones of Control"),			IDM_HELP_ZOC},
+  {N_("Government"),				IDM_HELP_GOVERNMENT},
+  {N_("Diplomacy"),				IDM_HELP_DIPLOMACY},
+  {N_("Technology"),				IDM_HELP_TECH},
+  {N_("Space Race"),				IDM_HELP_SPACE_RACE},
+  {N_("About Ruleset"),				IDM_HELP_RULESET},
   {"", IDM_SEPARATOR},
-  {N_("City _Improvements"),			IDM_HELP_IMPROVEMENTS},
-  {N_("_Units"),				IDM_HELP_UNITS},
-  {N_("Com_bat"),				IDM_HELP_COMBAT},
-  {N_("_ZOC"),					IDM_HELP_ZOC},
-  {N_("Techno_logy"),				IDM_HELP_TECH},
-  {N_("_Terrain"),				IDM_HELP_TERRAIN},
-  {N_("Won_ders"),				IDM_HELP_WONDERS},
-  {N_("_Government"),				IDM_HELP_GOVERNMENT},
-  {N_("Happin_ess"),				IDM_HELP_HAPPINESS},
-  {N_("Space _Race"),				IDM_HELP_SPACE_RACE},
+  {N_("Connecting"),				IDM_HELP_CONNECTING},
+  {N_("Controls"),				IDM_HELP_CONTROLS},
+  {N_("Citizen Governor"),			IDM_HELP_CMA},
+  {N_("Chatline"),				IDM_HELP_CHATLINE},
+  {N_("Worklist Editor"),			IDM_HELP_WORKLIST_EDITOR},
   {"", IDM_SEPARATOR},
-  {N_("_Copying"),				IDM_HELP_COPYING},
-  {N_("_About"),				IDM_HELP_ABOUT},
+  {N_("Languages"),				IDM_HELP_LANGUAGES},
+  {N_("Copying"),				IDM_HELP_COPYING},
+  {N_("About Freeciv"),				IDM_HELP_ABOUT},
   {NULL, 0},
   {NULL, 0}
 };
@@ -572,7 +584,7 @@ void handle_menu(int code)
       popup_messageopt_dialog();
       break;
     case IDM_GAME_SAVE_SETTINGS:
-      save_options();
+      options_save();
       break;
     case IDM_GAME_SERVER_OPTIONS:
       popup_settable_options_dialog();
@@ -604,7 +616,7 @@ void handle_menu(int code)
       popup_find_dialog();
       break;
     case IDM_GOVERNMENT_WORKLISTS:
-      popup_worklists_report(game.player_ptr);
+      popup_worklists_report();
       break;
     case IDM_GOVERNMENT_REVOLUTION:
       popup_revolution_dialog(NULL);
@@ -704,8 +716,8 @@ void handle_menu(int code)
       unit_list_iterate(get_units_in_focus(), punit) {
         /* FIXME: this can provide different actions for different units...
          * not good! */
-        if (unit_can_est_traderoute_here(punit)) {
-          key_unit_traderoute();
+        if (unit_can_est_trade_route_here(punit)) {
+          key_unit_trade_route();
         } else {
           key_unit_road();
         }
@@ -724,7 +736,9 @@ void handle_menu(int code)
       unit_list_iterate(get_units_in_focus(), punit) {
         /* FIXME: this can provide different actions for different units...
          * not good! */
-        if (can_unit_do_activity(punit, ACTIVITY_FORTRESS)) {
+        struct base_type *pbase = get_base_by_gui_type(BASE_GUI_FORTRESS,
+                                                       punit, punit->tile);
+        if (pbase) {
           key_unit_fortress();
         } else {
           key_unit_fortify();
@@ -849,7 +863,7 @@ void handle_menu(int code)
       send_report_request(REPORT_DEMOGRAPHIC);
       break;
     case IDM_REPORTS_SPACESHIP:
-      popup_spaceship_dialog(game.player_ptr);
+      popup_spaceship_dialog(client.conn.playing);
       break;
 
 
@@ -880,9 +894,18 @@ void handle_menu(int code)
     case IDM_HELP_CMA:
       popup_help_dialog_string(HELP_CMA_ITEM);
       break;
+    case IDM_HELP_OVERVIEW:
+      popup_help_dialog_string(HELP_OVERVIEW_ITEM);
+      break;
     case IDM_HELP_PLAYING:
       popup_help_dialog_string(HELP_PLAYING_ITEM);
       break;
+    case IDM_HELP_ECONOMY:
+      popup_help_dialog_string(HELP_ECONOMY_ITEM);
+      break;
+    case IDM_HELP_CITIES:
+      popup_help_dialog_string(HELP_CITIES_ITEM);
+      break;
     case IDM_HELP_IMPROVEMENTS:
       popup_help_dialog_string(HELP_IMPROVEMENTS_ITEM);
       break;
@@ -907,12 +930,15 @@ void handle_menu(int code)
     case IDM_HELP_GOVERNMENT:
       popup_help_dialog_string(HELP_GOVERNMENT_ITEM);
       break;
-    case IDM_HELP_HAPPINESS:
-      popup_help_dialog_string(HELP_HAPPINESS_ITEM);
+    case IDM_HELP_DIPLOMACY:
+      popup_help_dialog_string(HELP_DIPLOMACY_ITEM);
       break;
     case IDM_HELP_SPACE_RACE:
       popup_help_dialog_string(HELP_SPACE_RACE_ITEM);
       break;
+    case IDM_HELP_RULESET:
+      popup_help_dialog_string(HELP_RULESET_ITEM);
+      break;
     case IDM_HELP_COPYING:
       popup_help_dialog_string(HELP_COPYING_ITEM);
       break;
@@ -997,7 +1023,7 @@ static const char *get_tile_change_menu_text(struct tile *ptile,
   struct tile newtile = *ptile;
 
   tile_apply_activity(&newtile, activity);
-  return tile_get_info_text(ptile);
+  return tile_get_info_text(ptile, 0);
 }
 
 /**************************************************************************
@@ -1019,7 +1045,7 @@ update_menus(void)
   my_enable_menu(menu, IDM_GAME_SERVER_OPTIONS,
 		 C_S_RUNNING <= client_state());
   my_enable_menu(menu, IDM_GAME_DISCONNECT,
-		 aconnection.established);
+		 client.conn.established);
 
   if (!can_client_change_view()) {
 
@@ -1030,7 +1056,6 @@ update_menus(void)
     }
 
   } else {
-    struct unit *punit;
     HMENU govts;
 
     /* remove previous government entries. */
@@ -1046,11 +1071,11 @@ update_menus(void)
     id = IDM_GOVERNMENT_CHANGE_FIRST;
 
     government_iterate(g) {
-      if (g != game.government_when_anarchy) {
-	AppendMenu(govts, MF_STRING, id + g->index,
+      if (g != game.government_during_revolution) {
+	AppendMenu(govts, MF_STRING, id + government_number(g),
 		   government_name_translation(g));
-	my_enable_menu(menu, id + g->index,
-		       can_change_to_government(game.player_ptr, g)
+	my_enable_menu(menu, id + government_number(g),
+		       can_change_to_government(client.conn.playing, g)
 		       && can_client_issue_orders());
       }
     } government_iterate_end;
@@ -1075,7 +1100,7 @@ update_menus(void)
 
 
     my_enable_menu(menu, IDM_REPORTS_SPACESHIP,
-		   (game.player_ptr->spaceship.state!=SSHIP_NONE));
+		   (SSHIP_NONE != client.conn.playing->spaceship.state));
 
     my_check_menu(menu, IDM_VIEW_MAP_GRID, draw_map_grid);
     my_enable_menu(menu, IDM_VIEW_NATIONAL_BORDERS, game.info.borders > 0);
@@ -1105,13 +1130,14 @@ update_menus(void)
       return;
     }
 
-    if (NULL != (punit = head_of_units_in_focus())) {
+    if (get_num_units_in_focus() > 0) {
       const char *irrfmt = _("Change to %s");
       const char *minfmt = _("Change to %s");
       const char *transfmt = _("Transform to %s");
       char irrtext[128], mintext[128], transtext[128];
       char *roadtext;
       struct terrain *pterrain;
+      struct unit_list *punits = get_units_in_focus();
 
       sz_strlcpy(irrtext, N_("Build Irrigation") "\tI");
       sz_strlcpy(mintext, N_("Build Mine") "\tM");
@@ -1128,139 +1154,160 @@ update_menus(void)
       /* Enable the button for adding to a city in all cases, so we
 	 get an eventual error message from the server if we try. */
       my_enable_menu(menu, IDM_ORDERS_BUILD_CITY,
-		     can_unit_add_or_build_city(punit)
-		     || unit_can_help_build_wonder_here(punit));
+		     (can_units_do(punits, can_unit_add_or_build_city)
+                      || can_units_do(punits, unit_can_help_build_wonder_here)));
       my_enable_menu(menu, IDM_ORDERS_ROAD,
-                     can_unit_do_activity(punit, ACTIVITY_ROAD)
-		     || can_unit_do_activity(punit, ACTIVITY_RAILROAD)
-		     || unit_can_est_traderoute_here(punit));
+                     (can_units_do_activity(punits, ACTIVITY_ROAD)
+                      || can_units_do_activity(punits, ACTIVITY_RAILROAD)
+                      || can_units_do(punits,
+                                      unit_can_est_trade_route_here)));
       my_enable_menu(menu, IDM_ORDERS_IRRIGATE,
-		     can_unit_do_activity(punit, ACTIVITY_IRRIGATE));
+		     can_units_do_activity(punits, ACTIVITY_IRRIGATE));
       my_enable_menu(menu, IDM_ORDERS_MINE,
-		     can_unit_do_activity(punit, ACTIVITY_MINE));
+		     can_units_do_activity(punits, ACTIVITY_MINE));
       my_enable_menu(menu, IDM_ORDERS_TRANSFORM,
-		     can_unit_do_activity(punit, ACTIVITY_TRANSFORM));
+		     can_units_do_activity(punits, ACTIVITY_TRANSFORM));
       my_enable_menu(menu, IDM_ORDERS_FORTRESS,
-		     can_unit_do_activity(punit, ACTIVITY_FORTRESS)
-		     || can_unit_do_activity(punit, ACTIVITY_FORTIFYING));
+                     can_units_do_base_gui(punits, BASE_GUI_FORTRESS)
+		     || can_units_do_activity(punits, ACTIVITY_FORTIFYING));
       my_enable_menu(menu, IDM_ORDERS_AIRBASE,
-		     can_unit_do_activity(punit, ACTIVITY_AIRBASE));
+                     can_units_do_base_gui(punits, BASE_GUI_AIRBASE));
       my_enable_menu(menu, IDM_ORDERS_POLLUTION,
-		     can_unit_do_activity(punit, ACTIVITY_POLLUTION)
-		     || can_unit_paradrop(punit));
+		     (can_units_do_activity(punits, ACTIVITY_POLLUTION)
+                      || can_units_do(punits, can_unit_paradrop)));
       my_enable_menu(menu, IDM_ORDERS_FALLOUT,
-		     can_unit_do_activity(punit, ACTIVITY_FALLOUT));
+		     can_units_do_activity(punits, ACTIVITY_FALLOUT));
       my_enable_menu(menu, IDM_ORDERS_SENTRY,
-		     can_unit_do_activity(punit, ACTIVITY_SENTRY));
+		     can_units_do_activity(punits, ACTIVITY_SENTRY));
       my_enable_menu(menu, IDM_ORDERS_PILLAGE,
-		     can_unit_do_activity(punit, ACTIVITY_PILLAGE));
+		     can_units_do_activity(punits, ACTIVITY_PILLAGE));
       my_enable_menu(menu, IDM_ORDERS_DISBAND,
-		     !unit_has_type_flag(punit, F_UNDISBANDABLE));
+		     units_have_flag(punits, F_UNDISBANDABLE, FALSE));
+      /* FIXME: Upgrade item missing */
       my_enable_menu(menu, IDM_ORDERS_HOMECITY,
-		     can_unit_change_homecity(punit));
+		     can_units_do(punits, can_unit_change_homecity));
       my_enable_menu(menu, IDM_ORDERS_LOAD,
-	can_unit_load(punit, find_transporter_for_unit(punit,
-						       punit->tile)));
+                     units_can_load(punits));
       my_enable_menu(menu, IDM_ORDERS_UNLOAD,
-	(can_unit_unload(punit, game_find_unit_by_number(punit->transported_by))
-	 && can_unit_exist_at_tile(punit, punit->tile)) 
-	|| get_transporter_occupancy(punit) > 0);
+                     units_are_occupied(punits));
       my_enable_menu(menu, IDM_ORDERS_WAKEUP_OTHERS,
-		     is_unit_activity_on_tile(ACTIVITY_SENTRY,
-					      punit->tile));
+                     units_have_activity_on_tile(punits,
+                                                 ACTIVITY_SENTRY));
       my_enable_menu(menu, IDM_ORDERS_AUTO_SETTLER,
-		     can_unit_do_autosettlers(punit));
+                     can_units_do(punits, can_unit_do_autosettlers));
       my_enable_menu(menu, IDM_ORDERS_AUTO_EXPLORE,
-		     can_unit_do_activity(punit, ACTIVITY_EXPLORE));
+                     can_units_do_activity(punits, ACTIVITY_EXPLORE));
       my_enable_menu(menu, IDM_ORDERS_CONNECT_ROAD,
-		     can_unit_do_connect(punit, ACTIVITY_ROAD));
+                     can_units_do_connect(punits, ACTIVITY_ROAD));
       my_enable_menu(menu, IDM_ORDERS_CONNECT_RAIL,
-		     can_unit_do_connect(punit, ACTIVITY_RAILROAD));
+                     can_units_do_connect(punits, ACTIVITY_RAILROAD));
       my_enable_menu(menu, IDM_ORDERS_CONNECT_IRRIGATE,
-		     can_unit_do_connect(punit, ACTIVITY_IRRIGATE));
-      my_enable_menu(menu, IDM_ORDERS_RETURN,
-		     !(is_air_unit(punit) || is_heli_unit(punit)));
+                     can_units_do_connect(punits, ACTIVITY_IRRIGATE));
+      my_enable_menu(menu, IDM_ORDERS_RETURN, TRUE);
       my_enable_menu(menu, IDM_ORDERS_DIPLOMAT_DLG,
-		     is_diplomat_unit(punit)
-		     && diplomat_can_do_action(punit, DIPLOMAT_ANY_ACTION,
-					       punit->tile));
+                     can_units_do_diplomat_action(punits, DIPLOMAT_ANY_ACTION));
       my_enable_menu(menu, IDM_ORDERS_NUKE,
-		     unit_has_type_flag(punit, F_NUCLEAR));
-      if (unit_has_type_flag(punit, F_HELP_WONDER)) {
+                     units_have_flag(punits, F_NUCLEAR, TRUE));
+      if (units_have_flag(punits, F_HELP_WONDER, TRUE)) {
 	my_rename_menu(menu, IDM_ORDERS_BUILD_CITY, N_("Help Build Wonder")
 		       "\tB");
-      } else if (unit_has_type_flag(punit, F_CITIES)) {
-	if (tile_get_city(punit->tile)) {
-	  my_rename_menu(menu, IDM_ORDERS_BUILD_CITY, N_("Add to City")
-			 "\tB");
-	} else {
+      } else {
+        bool city_on_tile = FALSE;
+
+        /* FIXME: this overloading doesn't work well with multiple focus
+         * units. */
+        unit_list_iterate(punits, punit) {
+          if (tile_city(punit->tile)) {
+            city_on_tile = TRUE;
+            break;
+          }
+        } unit_list_iterate_end;
+
+        if (city_on_tile && units_have_flag(punits, F_ADD_TO_CITY, TRUE)) {
+          my_rename_menu(menu, IDM_ORDERS_BUILD_CITY, N_("Add to City")
+                         "\tB");
+        } else {
 	  my_rename_menu(menu, IDM_ORDERS_BUILD_CITY, N_("Build City")
 			 "\tB");
-	}
-      } else {
-	my_rename_menu(menu, IDM_ORDERS_BUILD_CITY, N_("Build City") "\tB");
+        }
       }
  
-      if (unit_has_type_flag(punit, F_TRADE_ROUTE)) {
-	my_rename_menu(menu, IDM_ORDERS_ROAD, N_("Make Trade Route") "\tR");
-      } else if (unit_has_type_flag(punit, F_SETTLERS)) {
-	if (tile_has_special(punit->tile, S_ROAD)) {
-	  roadtext = N_("Build Railroad") "\tR";
-	  road_activity = ACTIVITY_RAILROAD;  
+      if (units_have_flag(punits, F_TRADE_ROUTE, TRUE)) {
+        my_rename_menu(menu, IDM_ORDERS_ROAD, N_("Establish Trade Route") "\tR");
+      } else if (units_have_flag(punits, F_SETTLERS, TRUE)) {
+	bool has_road = FALSE;
+
+	/* FIXME: this overloading doesn't work well with multiple focus
+	 * units. */
+	unit_list_iterate(punits, punit) {
+	  if (tile_has_special(punit->tile, S_ROAD)) {
+	    has_road = TRUE;
+	    break;
+	  }
+	} unit_list_iterate_end;
+
+	if (has_road) {
+          roadtext = N_("Build Railroad") "\tR";
+	  road_activity=ACTIVITY_RAILROAD;
 	} else {
-	  roadtext = N_("Build Road") "\tR";
-	  road_activity = ACTIVITY_ROAD;  
+          roadtext = N_("Build Road") "\tR";
+	  road_activity=ACTIVITY_ROAD;
 	}
+
 	my_rename_menu(menu, IDM_ORDERS_ROAD, roadtext);
       } else {
 	my_rename_menu(menu, IDM_ORDERS_ROAD, N_("Build Road") "\tR");
       }
 
-      pterrain = punit->tile->terrain;
-      if (pterrain->irrigation_result != T_NONE
-	  && pterrain->irrigation_result != pterrain) {
-	my_snprintf(irrtext, sizeof(irrtext), irrfmt,
-		    get_tile_change_menu_text(punit->tile,
-					      ACTIVITY_IRRIGATE));
- 	sz_strlcat(irrtext, "\tI");
-      } else if (tile_has_special(punit->tile, S_IRRIGATION)
-		 && player_knows_techs_with_flag(game.player_ptr,
-						 TF_FARMLAND)) {
-	sz_strlcpy(irrtext, N_("Build Farmland") "\tI");
-      }
-      if (pterrain->mining_result != T_NONE
-	  && pterrain->mining_result != pterrain) {
-	my_snprintf(mintext, sizeof(mintext), minfmt,
-		    get_tile_change_menu_text(punit->tile, ACTIVITY_MINE));
- 	sz_strlcat(mintext, "\tM");
-      }
-      if (pterrain->transform_result != T_NONE
-	  && pterrain->transform_result != pterrain) {
-	my_snprintf(transtext, sizeof(transtext), transfmt,
-		    get_tile_change_menu_text(punit->tile,
-					      ACTIVITY_TRANSFORM));
- 	sz_strlcat(transtext, "\tO");
+      if (unit_list_size(punits) == 1) {
+	struct unit *punit = unit_list_get(punits, 0);
+
+        pterrain = tile_terrain(punit->tile);
+        if (pterrain->irrigation_result != T_NONE
+            && pterrain->irrigation_result != pterrain) {
+          my_snprintf(irrtext, sizeof(irrtext), irrfmt,
+                      get_tile_change_menu_text(punit->tile,
+                                                ACTIVITY_IRRIGATE));
+          sz_strlcat(irrtext, "\tI");
+        } else if (tile_has_special(punit->tile, S_IRRIGATION)
+                   && player_knows_techs_with_flag(client.conn.playing,
+                                                   TF_FARMLAND)) {
+          sz_strlcpy(irrtext, N_("Build Farmland") "\tI");
+        }
+        if (pterrain->mining_result != T_NONE
+            && pterrain->mining_result != pterrain) {
+          my_snprintf(mintext, sizeof(mintext), minfmt,
+                      get_tile_change_menu_text(punit->tile, ACTIVITY_MINE));
+          sz_strlcat(mintext, "\tM");
+        }
+        if (pterrain->transform_result != T_NONE
+            && pterrain->transform_result != pterrain) {
+          my_snprintf(transtext, sizeof(transtext), transfmt,
+                      get_tile_change_menu_text(punit->tile,
+                                                ACTIVITY_TRANSFORM));
+          sz_strlcat(transtext, "\tO");
+        }
       }
 
       my_rename_menu(menu, IDM_ORDERS_IRRIGATE, irrtext);
       my_rename_menu(menu, IDM_ORDERS_MINE, mintext);
       my_rename_menu(menu, IDM_ORDERS_TRANSFORM, transtext);
 
-      if (can_unit_do_activity(punit, ACTIVITY_FORTIFYING)) {
+      if (can_units_do_activity(punits, ACTIVITY_FORTIFYING)) {
 	my_rename_menu(menu, IDM_ORDERS_FORTRESS, N_("Fortify") "\tF");
       } else {
 	my_rename_menu(menu, IDM_ORDERS_FORTRESS, N_("Build Fortress")
 		       "\tF");
       }
 
-      if (unit_has_type_flag(punit, F_PARATROOPERS)) {
-	my_rename_menu(menu, IDM_ORDERS_POLLUTION, N_("Paradrop") "\tP");
+      if (units_have_flag(punits, F_PARATROOPERS, TRUE)) {
+	my_rename_menu(menu, IDM_ORDERS_POLLUTION, N_("Drop Paratrooper") "\tP");
       } else {
 	my_rename_menu(menu, IDM_ORDERS_POLLUTION, N_("Clean Pollution")
 		       "\tP");
       }
 
-      if (!unit_has_type_flag(punit, F_SETTLERS)) {
+      if (units_have_flag(punits, F_SETTLERS, FALSE)) {
 	my_rename_menu(menu, IDM_ORDERS_AUTO_SETTLER, N_("Auto Attack")
 		       "\tA");
       } else {
