@@ -18,7 +18,7 @@ typedef void (*WorklistOkCallback) (struct worklist * pwl, void *data);
 typedef void (*WorklistCancelCallback) (void *data);
 
 struct worklist_window_init {
-  struct worklist *pwl;
+  int global_worklist_id;
   struct city *pcity;
   HWND parent;
   void *user_data;
@@ -28,7 +28,7 @@ struct worklist_window_init {
 
 void popup_worklist(struct worklist_window_init *init);
 
-void popup_worklists_report(struct player *pplr);
+void popup_worklists_report(void);
                                         /* The global worklist view */ 
 void update_worklist_report_dialog(void);
 void update_worklist_editor_win(HWND win);
