@@ -29,15 +29,16 @@
 #include <X11/Xaw/List.h>
 #include <X11/Xaw/Viewport.h>
 
-
+/* common & utility */
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"     /* mystrdup() */
 #include "support.h"
 #include "version.h"
 
-#include "civclient.h"
-#include "clinet.h"
+/* client */
+#include "client_main.h"
+#include "clinet.h"		/* connect_to_server() */
 #include "packhand.h"
 #include "servers.h"
 
@@ -160,7 +161,7 @@ void handle_authentication_req(enum authentication_type type, char *message)
       struct packet_authentication_reply reply;
 
       sz_strlcpy(reply.password, password);
-      send_packet_authentication_reply(&aconnection, &reply);
+      send_packet_authentication_reply(&client.conn, &reply);
       return;
     } else {
       dialog_config = ENTER_PASSWORD_TYPE;
@@ -303,7 +304,7 @@ void connect_callback(Widget w, XtPointer client_data,
       popup_start_page();
       return;
     } else {
-      append_output_window(errbuf);
+      output_window_append(ftc_client, errbuf);
     }
   case NEW_PASSWORD_TYPE:
     XtVaGetValues(iinput, XtNstring, &dp, NULL);
@@ -319,7 +320,7 @@ void connect_callback(Widget w, XtPointer client_data,
       XtSetSensitive(connw, FALSE);
       memset(password, 0, MAX_LEN_NAME);
       password[0] = '\0';
-      send_packet_authentication_reply(&aconnection, &reply);
+      send_packet_authentication_reply(&client.conn, &reply);
     } else {
       XtVaSetValues(iinput, XtNstring, "", NULL);
       XtVaSetValues(imsg, XtNlabel, 
@@ -332,7 +333,7 @@ void connect_callback(Widget w, XtPointer client_data,
     XtSetSensitive(connw, FALSE);
     XtVaGetValues(iinput, XtNstring, &dp, NULL);
     sz_strlcpy(reply.password, (char*)dp);
-    send_packet_authentication_reply(&aconnection, &reply);
+    send_packet_authentication_reply(&client.conn, &reply);
     break;
   default:
     assert(0);
@@ -345,7 +346,7 @@ void connect_callback(Widget w, XtPointer client_data,
 static void server_scan_error(struct server_scan *scan,
 			      const char *message)
 {
-  append_output_window(message);
+  output_window_append(ftc_client, message);
   freelog(LOG_NORMAL, "%s", message);
   switch (server_scan_get_type(scan)) {
   case SERVER_SCAN_LOCAL:
@@ -411,7 +412,7 @@ static void server_list_timer(XtPointer meta_list, XtIntervalId * id)
   if (get_server_list(server_list, errbuf, sizeof(errbuf))!=-1)  {
     XawListChange(meta_list, server_list, 0, 0, True);
   } else if (!lan_mode) {
-    append_output_window(errbuf);
+    output_window_append(ftc_client, errbuf);
   }
   num_lanservers_timer++;
 
@@ -533,11 +534,12 @@ void meta_list_destroy(Widget w, XtPointer client_data, XtPointer call_data)
 static int get_server_list(char **list, char *errbuf, int n_errbuf)
 {
   char line[256];
-  struct server_list *server_list = NULL;
+  const struct server_list *server_list = NULL;
 
   if (lan_mode) {
     if (lan) {
-      server_list = server_scan_get_servers(lan);
+      server_scan_poll(lan);
+      server_list = server_scan_get_list(lan);
       if (server_list == NULL) {
 	if (num_lanservers_timer == 0) {
 	  *list = mystrdup(" ");;
@@ -549,7 +551,8 @@ static int get_server_list(char **list, char *errbuf, int n_errbuf)
     }
   } else {
     if (meta) {
-      server_list = server_scan_get_servers(meta); 
+      server_scan_poll(meta);
+      server_list = server_scan_get_list(meta);
       if (!server_list) {
 	return -1;
       }
@@ -558,7 +561,7 @@ static int get_server_list(char **list, char *errbuf, int n_errbuf)
 
   server_list_iterate(server_list,pserver) {
     if (pserver == NULL) continue;
-    my_snprintf(line, sizeof(line), "%-35s %-5s %-11s %-11s %2s   %s",
+    my_snprintf(line, sizeof(line), "%-35s %-5d %-11s %-11s %2d   %s",
 		pserver->host, pserver->port, pserver->version,
 		_(pserver->state), pserver->nplayers, pserver->message);
     if (*list) free(*list);
