@@ -26,8 +26,8 @@
 
 #include <png.h>
 
+/* common & utility */
 #include "fcintl.h"
-#include "game.h"
 #include "log.h"
 #include "mem.h"
 #include "movement.h"
@@ -36,6 +36,8 @@
 #include "unit.h"
 #include "version.h"
 
+/* client */
+#include "client_main.h"
 #include "climisc.h"
 #include "colors.h"
 #include "gui_main.h"
@@ -338,26 +340,26 @@ struct sprite *load_gfxfile(const char *filename)
   png_byte alpha;
   bool pixel, reported;
 
-  fp = fopen(filename, "rb");
+  fp = fc_fopen(filename, "rb");
   if (!fp) {
-    freelog(LOG_FATAL, _("Failed reading PNG file: %s"), filename);
+    freelog(LOG_FATAL, "Failed reading PNG file: \"%s\"", filename);
     exit(EXIT_FAILURE);
   }
 
   pngp = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
   if (!pngp) {
-    freelog(LOG_FATAL, _("Failed creating PNG struct"));
+    freelog(LOG_FATAL, "Failed creating PNG struct");
     exit(EXIT_FAILURE);
   }
 
   infop = png_create_info_struct(pngp);
   if (!infop) {
-    freelog(LOG_FATAL, _("Failed creating PNG struct"));
+    freelog(LOG_FATAL, "Failed creating PNG struct");
     exit(EXIT_FAILURE);
   }
   
   if (setjmp(pngp->jmpbuf)) {
-    freelog(LOG_FATAL, _("Failed while reading PNG file: %s"), filename);
+    freelog(LOG_FATAL, "Failed while reading PNG file: \"%s\"", filename);
     exit(EXIT_FAILURE);
   }
 
@@ -394,7 +396,7 @@ struct sprite *load_gfxfile(const char *filename)
 
       free(mycolors);
     } else {
-      freelog(LOG_FATAL, _("PNG file has no palette: %s"), filename);
+      freelog(LOG_FATAL, "PNG file has no palette: \"%s\"", filename);
       exit(EXIT_FAILURE);
     }
 
@@ -411,7 +413,7 @@ struct sprite *load_gfxfile(const char *filename)
 	  ptransarray[trans[i]] = TRUE;
 	} else if (!reported) {
 	  freelog(LOG_VERBOSE,
-		  "PNG: Trasparent array entry is out of palette. File: %s",
+		  "PNG: Transparent array entry is out of palette: \"%s\"",
 		  filename);
 	  reported = TRUE;
 	}
@@ -455,7 +457,7 @@ struct sprite *load_gfxfile(const char *filename)
       png_destroy_read_struct(&pngp, &infop, (png_infopp)NULL);
     } else {
       freelog(LOG_ERROR,
-	      "Error: PNG info struct is NULL (non-fatal). File: %s",
+	      "PNG info struct is NULL (non-fatal): \"%s\"",
 	      filename);
       png_destroy_read_struct(&pngp, (png_infopp)NULL, (png_infopp)NULL);
     }
@@ -579,12 +581,23 @@ Pixmap create_overlay_unit(const struct unit_type *punittype)
 
   /* Give tile a background color, based on the type of unit */
   /* Should there be colors like COLOR_MAPVIEW_LAND etc? -ev */
-  switch (punittype->move_type) {
-    case LAND_MOVING: bg_color = COLOR_OVERVIEW_LAND; break;
-    case SEA_MOVING:  bg_color = COLOR_OVERVIEW_OCEAN; break;
-    case HELI_MOVING: bg_color = COLOR_OVERVIEW_MY_UNIT; break;
-    case AIR_MOVING:  bg_color = COLOR_OVERVIEW_ENEMY_CITY; break;
-    default:          bg_color = COLOR_OVERVIEW_UNKNOWN; break;
+  switch (unit_color_type(punittype)) {
+    case UNIT_BG_LAND:
+      bg_color = COLOR_OVERVIEW_LAND;
+      break;
+    case UNIT_BG_SEA:
+      bg_color = COLOR_OVERVIEW_OCEAN;
+      break;
+    case UNIT_BG_HP_LOSS:
+    case UNIT_BG_AMPHIBIOUS:
+      bg_color = COLOR_OVERVIEW_MY_UNIT;
+      break;
+    case UNIT_BG_FLYING:
+      bg_color = COLOR_OVERVIEW_ENEMY_CITY;
+      break;
+    default:
+      bg_color = COLOR_OVERVIEW_UNKNOWN;
+      break;
   }
   XSetForeground(display, fill_bg_gc,
 		 get_color(tileset, bg_color)->color.pixel);
@@ -593,7 +606,7 @@ Pixmap create_overlay_unit(const struct unit_type *punittype)
 
   /* If we're using flags, put one on the tile */
   if(!solid_color_behind_units)  {
-    struct sprite *flag=get_nation_flag_sprite(tileset,nation_of_player(game.player_ptr));
+    struct sprite *flag = get_nation_flag_sprite(tileset, nation_of_player(client.conn.playing));
 
     XSetClipOrigin(display, civ_gc, 0,0);
     XSetClipMask(display, civ_gc, flag->mask);
@@ -603,7 +616,7 @@ Pixmap create_overlay_unit(const struct unit_type *punittype)
   }
 
   /* Finally, put a picture of the unit in the tile */
-/*  if(i<game.control.num_unit_types) { */
+/*  if(i<utype_count()) */ {
     struct sprite *s = get_unittype_sprite(tileset, punittype);
 
     XSetClipOrigin(display,civ_gc,0,0);
@@ -611,8 +624,7 @@ Pixmap create_overlay_unit(const struct unit_type *punittype)
     XCopyArea(display, s->pixmap, pm, civ_gc,
 	      0,0, s->width,s->height, 0,0 );
     XSetClipMask(display,civ_gc,None);
-/*  } */
-
+  }
   return(pm);
 }
 
