@@ -26,18 +26,22 @@
 #include "canvas.h"
 #include "pixcomm.h"
 
+/* utility */
 #include "fcintl.h"
-#include "game.h"
-#include "government.h"		/* government_graphic() */
-#include "map.h"
 #include "mem.h"
-#include "player.h"
 #include "rand.h"
 #include "support.h"
 #include "timing.h"
-#include "unit.h"
 
-#include "civclient.h"
+/* common */
+#include "government.h"		/* government_graphic() */
+#include "map.h"
+#include "player.h"
+#include "unit.h"
+#include "unitlist.h"
+
+/* client */
+#include "client_main.h"
 #include "climap.h"
 #include "climisc.h"
 #include "colors.h"
@@ -48,15 +52,14 @@
 #include "gui_stuff.h"
 #include "mapctrl.h"
 #include "options.h"
+#include "overview_common.h"
 #include "text.h"
 #include "tilespec.h"
 
 #include "mapview.h"
 
-#define map_canvas_store (mapview_canvas.store->pixmap)
-
 static void pixmap_put_overlay_tile(Pixmap pixmap, int x, int y,
- 				    struct Sprite *ssprite);
+ 				    struct sprite *ssprite);
 
 /* the intro picture is held in this pixmap, which is scaled to
    the screen size */
@@ -64,10 +67,20 @@ Pixmap scaled_intro_pixmap;
 int scaled_intro_pixmap_width, scaled_intro_pixmap_height;
 
 
+/****************************************************************************
+  Return the dimensions of the area (container widget; maximum size) for
+  the overview.
+****************************************************************************/
+void get_overview_area_dimensions(int *width, int *height)
+{
+  XtVaGetValues(left_column_form, XtNheight, &height, NULL);
+  XtVaGetValues(below_menu_form, XtNwidth, &width, NULL);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
-void map_size_changed(void)
+void overview_size_changed(void)
 {
   Dimension h, w;
 
@@ -154,35 +167,30 @@ void update_timeout_label(void)
 **************************************************************************/
 void update_info_label(void)
 {
-  char buffer[512]; int d;
-  
-  my_snprintf(buffer, sizeof(buffer),
-	      _("%s People\n"
-		"Year: %s\n"
-		"Gold: %d\n"
-		"Tax:%d Lux:%d Sci:%d"),
-	  population_to_text(civ_population(game.player_ptr)),
-	  textyear(game.year),
-	  game.player_ptr->economic.gold,
-	  game.player_ptr->economic.tax,
-	  game.player_ptr->economic.luxury,
-	  game.player_ptr->economic.science);
-  xaw_set_label(info_command, buffer);
+  int d = 0;
+
+  xaw_set_label(info_command, get_info_label_text());
 
   set_indicator_icons(client_research_sprite(),
 		      client_warming_sprite(),
 		      client_cooling_sprite(),
-		      game.player_ptr->government);
+		      client_government_sprite());
 
-  d=0;
-  for(;d<(game.player_ptr->economic.luxury)/10;d++)
-    xaw_set_bitmap(econ_label[d], sprites.tax_luxury->pixmap);
+  if (NULL == client.conn.playing) {
+    return;
+  }
+
+  for (; d < (client.conn.playing->economic.luxury) / 10; d++) {
+    xaw_set_bitmap(econ_label[d], get_tax_sprite(tileset, O_LUXURY)->pixmap);
+  }
  
-  for(;d<(game.player_ptr->economic.science+game.player_ptr->economic.luxury)/10;d++)
-    xaw_set_bitmap(econ_label[d], sprites.tax_science->pixmap);
+  for (; d < (client.conn.playing->economic.science + client.conn.playing->economic.luxury) / 10; d++) {
+    xaw_set_bitmap(econ_label[d], get_tax_sprite(tileset, O_SCIENCE)->pixmap);
+  }
  
-   for(;d<10;d++)
-     xaw_set_bitmap(econ_label[d], sprites.tax_gold->pixmap);
+  for(; d < 10; d++) {
+    xaw_set_bitmap(econ_label[d], get_tax_sprite(tileset, O_GOLD)->pixmap);
+  }
  
   update_timeout_label();
 }
@@ -198,34 +206,32 @@ void update_info_label(void)
   Also calls update_unit_pix_label() to update the icons for units on this
   square.
 **************************************************************************/
-void update_unit_info_label(struct unit *punit)
+void update_unit_info_label(struct unit_list *punitlist)
 {
-  if(punit) {
-    char buffer[512];
-    my_snprintf(buffer, sizeof(buffer), "%s\n%s",
-		get_unit_info_label_text1(punit),
-		get_unit_info_label_text2(punit));
-    xaw_set_label(unit_info_label, buffer);
+  char buffer[512];
 
-    if (hover_unit != punit->id)
-      set_hover_state(NULL, HOVER_NONE, ACTIVITY_LAST);
+  my_snprintf(buffer, sizeof(buffer), "%s\n%s",
+              get_unit_info_label_text1(punitlist),
+              get_unit_info_label_text2(punitlist, 0));
+  xaw_set_label(unit_info_label, buffer);
 
+  if (unit_list_size(punitlist) > 0) {
     switch (hover_state) {
     case HOVER_NONE:
       XUndefineCursor(display, XtWindow(map_canvas));
       break;
     case HOVER_PATROL:
-      XDefineCursor(display, XtWindow(map_canvas), patrol_cursor);
+      XDefineCursor(display, XtWindow(map_canvas), cursors[CURSOR_PATROL]);
       break;
     case HOVER_GOTO:
     case HOVER_CONNECT:
-      XDefineCursor(display, XtWindow(map_canvas), goto_cursor);
+      XDefineCursor(display, XtWindow(map_canvas), cursors[CURSOR_GOTO]);
       break;
     case HOVER_NUKE:
-      XDefineCursor(display, XtWindow(map_canvas), nuke_cursor);
+      XDefineCursor(display, XtWindow(map_canvas), cursors[CURSOR_NUKE]);
       break;
     case HOVER_PARADROP:
-      XDefineCursor(display, XtWindow(map_canvas), drop_cursor);
+      XDefineCursor(display, XtWindow(map_canvas), cursors[CURSOR_PARADROP]);
       break;
     }
   } else {
@@ -233,7 +239,15 @@ void update_unit_info_label(struct unit *punit)
     XUndefineCursor(display, XtWindow(map_canvas));
   }
 
-  update_unit_pix_label(punit);
+  update_unit_pix_label(punitlist);
+}
+
+/**************************************************************************
+  This function will change the current mouse cursor.
+**************************************************************************/
+void update_mouse_cursor(enum cursor_type new_cursor_type)
+{
+  /* PORT ME */
 }
 
 /**************************************************************************
@@ -241,44 +255,30 @@ void update_unit_info_label(struct unit *punit)
 **************************************************************************/
 Pixmap get_thumb_pixmap(int onoff)
 {
-  return sprites.treaty_thumb[BOOL_VAL(onoff)]->pixmap;
+  /* FIXME: what about the mask? */
+  return get_treaty_thumb_sprite(tileset, BOOL_VAL(onoff))->pixmap;
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-Pixmap get_citizen_pixmap(struct citizen_type type, int cnum,
+Pixmap get_citizen_pixmap(enum citizen_category type, int cnum,
 			  struct city *pcity)
 {
-  return get_citizen_sprite(type, cnum, pcity)->pixmap;
+  return get_citizen_sprite(tileset, type, cnum, pcity)->pixmap;
 }
 
 
 /**************************************************************************
 ...
 **************************************************************************/
-void set_indicator_icons(int bulb, int sol, int flake, int gov)
+void set_indicator_icons(struct sprite *bulb, struct sprite *sol,
+			 struct sprite *flake, struct sprite *gov)
 {
-  struct Sprite *gov_sprite;
-
-  bulb = CLIP(0, bulb, NUM_TILES_PROGRESS-1);
-  sol = CLIP(0, sol, NUM_TILES_PROGRESS-1);
-  flake = CLIP(0, flake, NUM_TILES_PROGRESS-1);
-
-  xaw_set_bitmap(bulb_label, sprites.bulb[bulb]->pixmap);
-  xaw_set_bitmap(sun_label, sprites.warming[sol]->pixmap);
-  xaw_set_bitmap(flake_label, sprites.cooling[flake]->pixmap);
-
-  if (game.government_count==0) {
-    /* HACK: the UNHAPPY citizen is used for the government
-     * when we don't know any better. */
-    struct citizen_type c = {.type = CITIZEN_UNHAPPY};
-
-    gov_sprite = get_citizen_sprite(c, 0, NULL);
-  } else {
-    gov_sprite = get_government(gov)->sprite;
-  }
-  xaw_set_bitmap(government_label, gov_sprite->pixmap);
+  xaw_set_bitmap(bulb_label, bulb->pixmap);
+  xaw_set_bitmap(sun_label, sol->pixmap);
+  xaw_set_bitmap(flake_label, flake->pixmap);
+  xaw_set_bitmap(government_label, gov->pixmap);
 }
 
 /**************************************************************************
@@ -361,7 +361,12 @@ void map_canvas_expose(Widget w, XEvent *event, Region exposed,
   }
 
   if (map_exists() && !map_resized) {
-    XCopyArea(display, map_canvas_store, XtWindow(map_canvas),
+    /* First we mark the area to be updated as dirty.  Then we unqueue
+     * any pending updates, to make sure only the most up-to-date data
+     * is written (otherwise drawing bugs happen when old data is copied
+     * to screen).  Then we draw all changed areas to the screen. */
+    unqueue_mapview_updates(FALSE);
+    XCopyArea(display, mapview.store->pixmap, XtWindow(map_canvas),
 	      civ_gc,
 	      event->xexpose.x, event->xexpose.y,
 	      event->xexpose.width, event->xexpose.height,
@@ -375,7 +380,7 @@ void map_canvas_expose(Widget w, XEvent *event, Region exposed,
 **************************************************************************/
 static void pixmap_put_sprite(Pixmap pixmap,
 			      int canvas_x, int canvas_y,
-			      struct Sprite *sprite,
+			      struct sprite *sprite,
 			      int offset_x, int offset_y,
 			      int width, int height)
 {
@@ -400,7 +405,7 @@ static void pixmap_put_sprite(Pixmap pixmap,
 **************************************************************************/
 void canvas_put_sprite(struct canvas *pcanvas,
 		       int canvas_x, int canvas_y,
-		       struct Sprite *sprite,
+		       struct sprite *sprite,
 		       int offset_x, int offset_y, int width, int height)
 {
   pixmap_put_sprite(pcanvas->pixmap, canvas_x, canvas_y,
@@ -412,7 +417,7 @@ void canvas_put_sprite(struct canvas *pcanvas,
 **************************************************************************/
 void canvas_put_sprite_full(struct canvas *pcanvas,
 			    int canvas_x, int canvas_y,
-			    struct Sprite *sprite)
+			    struct sprite *sprite)
 {
   canvas_put_sprite(pcanvas, canvas_x, canvas_y,
 		    sprite, 0, 0, sprite->width, sprite->height);
@@ -424,7 +429,7 @@ void canvas_put_sprite_full(struct canvas *pcanvas,
 ****************************************************************************/
 void canvas_put_sprite_fogged(struct canvas *pcanvas,
 			      int canvas_x, int canvas_y,
-			      struct Sprite *psprite,
+			      struct sprite *psprite,
 			      bool fog, int fog_x, int fog_y)
 {
   canvas_put_sprite_full(pcanvas, canvas_x, canvas_y, psprite);
@@ -438,10 +443,10 @@ void canvas_put_sprite_fogged(struct canvas *pcanvas,
   Draw a filled-in colored rectangle onto the mapview or citydialog canvas.
 **************************************************************************/
 void canvas_put_rectangle(struct canvas *pcanvas,
-			  enum color_std color,
+			  struct color *pcolor,
 			  int canvas_x, int canvas_y, int width, int height)
 {
-  XSetForeground(display, fill_bg_gc, colors_standard[color]);
+  XSetForeground(display, fill_bg_gc, pcolor->color.pixel);
   XFillRectangle(display, pcanvas->pixmap, fill_bg_gc,
 		 canvas_x, canvas_y, width, height);
 }
@@ -450,14 +455,15 @@ void canvas_put_rectangle(struct canvas *pcanvas,
   Fill the area covered by the sprite with the given color.
 ****************************************************************************/
 void canvas_fill_sprite_area(struct canvas *pcanvas,
-			     struct Sprite *psprite, enum color_std color,
+			     struct sprite *psprite,
+			     struct color *pcolor,
 			     int canvas_x, int canvas_y)
 {
   if (psprite->has_mask) {
     XSetClipOrigin(display, fill_tile_gc, canvas_x, canvas_y);
     XSetClipMask(display, fill_tile_gc, psprite->mask);
   }
-  XSetForeground(display, fill_tile_gc, colors_standard[color]);
+  XSetForeground(display, fill_tile_gc, pcolor->color.pixel);
 
   XFillRectangle(display, pcanvas->pixmap, fill_tile_gc,
 		 canvas_x, canvas_y, psprite->width, psprite->height);
@@ -470,7 +476,7 @@ void canvas_fill_sprite_area(struct canvas *pcanvas,
 /****************************************************************************
   Fill the area covered by the sprite with the given color.
 ****************************************************************************/
-void canvas_fog_sprite_area(struct canvas *pcanvas, struct Sprite *psprite,
+void canvas_fog_sprite_area(struct canvas *pcanvas, struct sprite *psprite,
 			    int canvas_x, int canvas_y)
 {
   if (psprite->has_mask) {
@@ -479,7 +485,8 @@ void canvas_fog_sprite_area(struct canvas *pcanvas, struct Sprite *psprite,
   }
   XSetStipple(display, fill_tile_gc, gray50);
   XSetTSOrigin(display, fill_tile_gc, canvas_x, canvas_y);
-  XSetForeground(display, fill_tile_gc, colors_standard[COLOR_STD_BLACK]);
+  XSetForeground(display, fill_tile_gc,
+		 get_color(tileset, COLOR_MAPVIEW_UNKNOWN)->color.pixel);
 
   XFillRectangle(display, pcanvas->pixmap, fill_tile_gc,
 		 canvas_x, canvas_y, psprite->width, psprite->height);
@@ -492,7 +499,7 @@ void canvas_fog_sprite_area(struct canvas *pcanvas, struct Sprite *psprite,
 /**************************************************************************
   Draw a 1-pixel-width colored line onto the mapview or citydialog canvas.
 **************************************************************************/
-void canvas_put_line(struct canvas *pcanvas, enum color_std color,
+void canvas_put_line(struct canvas *pcanvas, struct color *pcolor,
 		     enum line_type ltype, int start_x, int start_y,
 		     int dx, int dy)
 {
@@ -512,11 +519,23 @@ void canvas_put_line(struct canvas *pcanvas, enum color_std color,
     break;
   }
 
-  XSetForeground(display, gc, colors_standard[color]);
+  XSetForeground(display, gc, pcolor->color.pixel);
   XDrawLine(display, pcanvas->pixmap, gc,
 	    start_x, start_y, start_x + dx, start_y + dy);
 }
 
+/**************************************************************************
+  Draw a 1-pixel-width curved colored line onto the mapview or 
+  citydialog canvas.
+**************************************************************************/
+void canvas_put_curved_line(struct canvas *pcanvas, struct color *pcolor,
+                            enum line_type ltype, int start_x, int start_y,
+                            int dx, int dy)
+{
+  /* FIXME: Implement curved line drawing. */
+  canvas_put_line(pcanvas, pcolor, ltype, start_x, start_y, dx, dy);
+}
+
 /**************************************************************************
   Flush the given part of the canvas buffer (if there is one) to the
   screen.
@@ -524,7 +543,7 @@ void canvas_put_line(struct canvas *pcanvas, enum color_std color,
 void flush_mapcanvas(int canvas_x, int canvas_y,
 		     int pixel_width, int pixel_height)
 {
-  XCopyArea(display, map_canvas_store, XtWindow(map_canvas), 
+  XCopyArea(display, mapview.store->pixmap, XtWindow(map_canvas), 
 	    civ_gc, 
 	    canvas_x, canvas_y, pixel_width, pixel_height,
 	    canvas_x, canvas_y);
@@ -656,124 +675,90 @@ void update_city_descriptions(void)
   update_map_canvas_visible();
 }
 
-/**************************************************************************
-  If necessary, clear the city descriptions out of the buffer.
-**************************************************************************/
-void prepare_show_city_descriptions(void)
-{
-  /* Nothing to do */
-}
-
 /**************************************************************************
 Draw at x = left of string, y = top of string.
 **************************************************************************/
 static void draw_shadowed_string(struct canvas *pcanvas,
 				 XFontSet fontset, GC font_gc,
-				 enum color_std foreground,
-				 enum color_std shadow,
+				 struct color *foreground,
+				 struct color *shadow,
 				 int x, int y, const char *string)
 {
   size_t len = strlen(string);
 
   y -= XExtentsOfFontSet(fontset)->max_logical_extent.y;
 
-  XSetForeground(display, font_gc, colors_standard[shadow]);
+  XSetForeground(display, font_gc, shadow->color.pixel);
   XmbDrawString(display, pcanvas->pixmap, fontset, font_gc,
       x + 1, y + 1, string, len);
 
-  XSetForeground(display, font_gc, colors_standard[foreground]);
+  XSetForeground(display, font_gc, foreground->color.pixel);
   XmbDrawString(display, pcanvas->pixmap, fontset, font_gc,
       x, y, string, len);
 }
 
+static XFontSet *fonts[FONT_COUNT] = {&main_font_set, &prod_font_set};
+static GC *font_gcs[FONT_COUNT] = {&font_gc, &prod_font_gc};
+
 /****************************************************************************
-  Draw a description for the given city.  This description may include the
-  name, turns-to-grow, production, and city turns-to-build (depending on
-  client options).
-
-  (canvas_x, canvas_y) gives the location on the given canvas at which to
-  draw the description.  This is the location of the city itself so the
-  text must be drawn underneath it.  pcity gives the city to be drawn,
-  while (*width, *height) should be set by show_ctiy_desc to contain the
-  width and height of the text block (centered directly underneath the
-  city's tile).
+  Return the size of the given text in the given font.  This size should
+  include the ascent and descent of the text.  Either of width or height
+  may be NULL in which case those values simply shouldn't be filled out.
 ****************************************************************************/
-void show_city_desc(struct canvas *pcanvas, int canvas_x, int canvas_y,
-		    struct city *pcity, int *width, int *height)
+void get_text_size(int *width, int *height,
+		   enum client_font font, const char *text)
 {
-  char buffer[512], buffer2[512];
-  enum color_std color;
-  int w, w2;
-  XFontSetExtents *main_exts = XExtentsOfFontSet(main_font_set);
-  XFontSetExtents *prod_exts = XExtentsOfFontSet(prod_font_set);
-
-  canvas_x += NORMAL_TILE_WIDTH / 2;
-  canvas_y += NORMAL_TILE_HEIGHT;
-
-  get_city_mapview_name_and_growth(pcity, buffer, sizeof(buffer),
-				   buffer2, sizeof(buffer2), &color);
-
-  w = XmbTextEscapement(main_font_set, buffer, strlen(buffer));
-  if (buffer2[0] != '\0') {
-    /* HACK: put a character's worth of space between the two strings. */
-    w += XmbTextEscapement(main_font_set, "M", 1);
+  if (width) {
+    *width = XmbTextEscapement(*fonts[font], text, strlen(text));
   }
-  w2 = XmbTextEscapement(main_font_set, buffer2, strlen(buffer2));
-
-  draw_shadowed_string(pcanvas, main_font_set, font_gc,
-		       COLOR_STD_WHITE, COLOR_STD_BLACK,
-		       canvas_x - (w + w2) / 2,
-		       canvas_y, buffer);
-
-  draw_shadowed_string(pcanvas, prod_font_set, prod_font_gc, color,
-		       COLOR_STD_BLACK,
-		       canvas_x - (w + w2) / 2 + w,
-		       canvas_y, buffer2);
-
-  *width = w + w2;
-  *height = main_exts->max_logical_extent.height;
-
-  if (draw_city_productions && (pcity->owner == game.player_idx)) {
-    if (draw_city_names) {
-      canvas_y += main_exts->max_logical_extent.height;
-    }
-
-    get_city_mapview_production(pcity, buffer, sizeof(buffer));
-    w = XmbTextEscapement(prod_font_set, buffer, strlen(buffer));
-
-    draw_shadowed_string(pcanvas, prod_font_set, prod_font_gc,
-			 COLOR_STD_WHITE, COLOR_STD_BLACK,
-			 canvas_x - w / 2,
-			 canvas_y, buffer);
-
-    *width = MAX(*width, w);
-    *height += prod_exts->max_logical_extent.height;
+  if (height) {
+    /* ??? */
+    *height = XExtentsOfFontSet(*fonts[font])->max_logical_extent.height;
   }
 }
 
+/****************************************************************************
+  Draw the text onto the canvas in the given color and font.  The canvas
+  position does not account for the ascent of the text; this function must
+  take care of this manually.  The text will not be NULL but may be empty.
+****************************************************************************/
+void canvas_put_text(struct canvas *pcanvas, int canvas_x, int canvas_y,
+		     enum client_font font, struct color *pcolor,
+		     const char *text)
+{
+  draw_shadowed_string(pcanvas, *fonts[font], *font_gcs[font],
+		       pcolor, get_color(tileset, COLOR_MAPVIEW_UNKNOWN),
+		       canvas_x, canvas_y, text);
+}
+
 /**************************************************************************
   FIXME: 
   For now only two food, one shield and two masks can be drawn per unit,
   the proper way to do this is probably something like what Civ II does.
   (One food/shield/mask drawn N times, possibly one top of itself. -- SKi 
 **************************************************************************/
-void put_unit_pixmap_city_overlays(struct unit *punit, Pixmap pm)
+void put_unit_pixmap_city_overlays(struct unit *punit, Pixmap pm,
+                                   int *upkeep_cost, int happy_cost)
 {
   struct canvas store = {pm};
  
   /* wipe the slate clean */
-  XSetForeground(display, fill_bg_gc, colors_standard[COLOR_STD_WHITE]);
-  XFillRectangle(display, pm, fill_bg_gc, 0, NORMAL_TILE_WIDTH, 
-		 NORMAL_TILE_HEIGHT, NORMAL_TILE_HEIGHT+SMALL_TILE_HEIGHT);
+  XSetForeground(display, fill_bg_gc,
+		 get_color(tileset, COLOR_MAPVIEW_CITYTEXT)->color.pixel);
+  XFillRectangle(display, pm, fill_bg_gc, 0, tileset_tile_width(tileset), 
+		 tileset_tile_height(tileset),
+		 tileset_tile_height(tileset)
+		 + tileset_small_sprite_height(tileset));
 
-  put_unit_city_overlays(punit, &store, 0, NORMAL_TILE_HEIGHT);
+  put_unit_city_overlays(punit, &store, 0, tileset_tile_height(tileset),
+                         upkeep_cost, happy_cost);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
 static void pixmap_put_overlay_tile(Pixmap pixmap, int canvas_x, int canvas_y,
- 				    struct Sprite *ssprite)
+ 				    struct sprite *ssprite)
 {
   if (!ssprite) return;
 
@@ -790,34 +775,10 @@ void put_cross_overlay_tile(struct tile *ptile)
 
   if (tile_to_canvas_pos(&canvas_x, &canvas_y, ptile)) {
     pixmap_put_overlay_tile(XtWindow(map_canvas), canvas_x, canvas_y,
-			    sprites.user.attention);
+			    get_attention_crosshair_sprite(tileset));
   }
 }
 
-/****************************************************************************
-  Draw a single tile of the citymap onto the mapview.  The tile is drawn
-  as the given color with the given worker on it.  The exact method of
-  drawing is left up to the GUI.
-****************************************************************************/
-void put_city_worker(struct canvas *pcanvas,
-		     enum color_std color, enum city_tile_type worker,
-		     int canvas_x, int canvas_y)
-{
-  if (worker == C_TILE_EMPTY) {
-    XSetStipple(display, fill_tile_gc, gray25);
-  } else if (worker == C_TILE_WORKER) {
-    XSetStipple(display, fill_tile_gc, gray50);
-  } else {
-    return;
-  }
-
-  XSetTSOrigin(display, fill_tile_gc, canvas_x, canvas_y);
-  XSetForeground(display, fill_tile_gc, colors_standard[color]);
-  XFillRectangle(display, pcanvas->pixmap, fill_tile_gc,
-		 canvas_x, canvas_y,
-		 NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
-}
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -897,7 +858,7 @@ void tileset_changed(void)
   /* Here you should do any necessary redraws (for instance, the city
    * dialogs usually need to be resized).
    */
-   reset_econ_label_pixmaps();
-   update_info_label();
-   reset_unit_below_pixmaps();
+  reset_econ_label_pixmaps();
+  update_info_label();
+  reset_unit_below_pixmaps();
 }
