@@ -25,19 +25,24 @@
 #include <X11/Xaw/SmeBSB.h>
 #include <X11/Xaw/SmeLine.h>
 
+/* utility */
 #include "fcintl.h"
+#include "log.h"
 #include "mem.h"
 #include "movement.h"
 #include "support.h"
 
+/* common */
+#include "game.h"
 #include "government.h"
 #include "map.h"
 #include "unit.h"
 #include "unitlist.h"
 
+/* client */
 #include "chatline.h"
 #include "cityrep.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"	/* can_units_do_connect */
 #include "clinet.h"
 #include "control.h" /* request_xxx and get_unit_in_focus */
@@ -118,17 +123,17 @@ static struct MenuEntry game_menu_entries[]={
     { { N_("Clear Log"), 0            },      "", MENU_GAME_CLEAR_OUTPUT, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Disconnect"), 0           },      "", MENU_GAME_DISCONNECT, 0 },
-    { { N_("Quit"), 0                 },      "", MENU_GAME_QUIT, 0 },
+    { { N_("Quit"), 0                 },      "Ctl-Q", MENU_GAME_QUIT, 0 },
     { { 0,                            },       0, MENU_END_OF_LIST, 0 }
 };
 
 static struct MenuEntry government_menu_entries[]={
-    { { N_("Tax Rates"), 0            },     "T", MENU_GOVERNMENT_RATES, 0 },
+    { { N_("Tax Rates..."), 0         },     "Ctl-T", MENU_GOVERNMENT_RATES, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
-    { { N_("Find City"), 0            },     "F", MENU_GOVERNMENT_FIND_CITY, 0 },
-    { { N_("Worklists"), 0            },     "Ctl-W", MENU_GOVERNMENT_WORKLISTS, 0 },
+    { { N_("Find City"), 0            },     "Ctl-F", MENU_GOVERNMENT_FIND_CITY, 0 },
+    { { N_("Worklists"), 0            },     "Ctl-L", MENU_GOVERNMENT_WORKLISTS, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
-    { { N_("Revolution"), 0           },     "R", MENU_GOVERNMENT_REVOLUTION, 0 },
+    { { N_("Revolution..."), 0        },     "Ctl-Shift-R", MENU_GOVERNMENT_REVOLUTION, 0 },
     { { 0,                            },       0, MENU_END_OF_LIST, 0 }
 };
 
@@ -143,14 +148,14 @@ static struct MenuEntry view_menu_entries[]={
     { { N_("City Names"), 0           }, "ctl-n", MENU_VIEW_SHOW_CITY_NAMES, 0 },
     { { N_("City Growth"), 0          }, "ctl-r",
       MENU_VIEW_SHOW_CITY_GROWTH, 0 },
-    { { N_("City Productions"), 0     }, "ctl-p", MENU_VIEW_SHOW_CITY_PRODUCTIONS, 0 },
+    { { N_("City Production Levels"), 0 }, "ctl-p", MENU_VIEW_SHOW_CITY_PRODUCTIONS, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Terrain"), 0              },      "", MENU_VIEW_SHOW_TERRAIN, 0 },
     { { N_("Coastline"), 0            },      "", MENU_VIEW_SHOW_COASTLINE, 0 },
-    { { N_("Roads and Rails"), 0      },      "", MENU_VIEW_SHOW_ROADS_RAILS, 0 },
+    { { N_("Roads & Rails"), 0        },      "", MENU_VIEW_SHOW_ROADS_RAILS, 0 },
     { { N_("Irrigation"), 0           },      "", MENU_VIEW_SHOW_IRRIGATION, 0 },
     { { N_("Mines"), 0                },      "", MENU_VIEW_SHOW_MINES, 0 },
-    { { N_("Fortress and Airbase"), 0 },      "", MENU_VIEW_SHOW_FORTRESS_AIRBASE, 0 },
+    { { N_("Fortress & Airbase"), 0   },      "", MENU_VIEW_SHOW_FORTRESS_AIRBASE, 0 },
     { { N_("Specials"), 0             },      "", MENU_VIEW_SHOW_SPECIALS, 0 },
     { { N_("Pollution & Fallout"), 0  },      "", MENU_VIEW_SHOW_POLLUTION, 0 },
     { { N_("Cities"), 0               },      "", MENU_VIEW_SHOW_CITIES, 0 },
@@ -177,80 +182,92 @@ static struct MenuEntry order_menu_entries[]={
     { { N_("Build Fortress"), 0       },     "f", MENU_ORDER_FORTRESS, 0 },
     { { N_("Build Airbase"), 0        },     "e", MENU_ORDER_AIRBASE, 0 },
     { { N_("Clean Pollution"),
-	N_("Paradrop"), 0             },     "p", MENU_ORDER_POLLUTION, 0 },
+	N_("Drop Paratrooper"), 0             },     "p", MENU_ORDER_POLLUTION, 0 },
     { { N_("Clean Nuclear Fallout"), 0},     "n", MENU_ORDER_FALLOUT, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
-    { { N_("Fortify"), 0              },     "f", MENU_ORDER_FORTIFY, 0 },
-    { { N_("Sentry"), 0               },     "s", MENU_ORDER_SENTRY, 0 },
+    { { N_("Fortify Unit"), 0         },     "f", MENU_ORDER_FORTIFY, 0 },
+    { { N_("Sentry Unit"), 0          },     "s", MENU_ORDER_SENTRY, 0 },
     { { N_("Pillage"), 0              },     "P", MENU_ORDER_PILLAGE, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
-    { { N_("Make Homecity"), 0        },     "h", MENU_ORDER_HOMECITY, 0 },
-    { { N_("Unload Transporter"), 0   },     "U",
+    { { N_("Set Home City"), 0        },     "h", MENU_ORDER_HOMECITY, 0 },
+    { { N_("Unload All From Transporter"), 0 }, "T",
       MENU_ORDER_UNLOAD_TRANSPORTER, 0 },
-    { { N_("Load"), 0                 },     "l", MENU_ORDER_LOAD, 0 },
-    { { N_("Unload"), 0               },     "u", MENU_ORDER_UNLOAD, 0 },
-    { { N_("Wake up others"), 0       },     "W", MENU_ORDER_WAKEUP_OTHERS, 0 },
+    { { N_("Load Unit"), 0            },     "l", MENU_ORDER_LOAD, 0 },
+    { { N_("Unload Unit"), 0          },     "u", MENU_ORDER_UNLOAD, 0 },
+    { { N_("Unsentry All On Tile"), 0 },     "S", MENU_ORDER_WAKEUP_OTHERS, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Auto Settler"), 0         },     "a", MENU_ORDER_AUTO_SETTLER, 0 },
     { { N_("Auto Attack"), 0          },     "a", MENU_ORDER_AUTO_ATTACK, 0 },
     { { N_("Auto Explore"), 0         },     "x", MENU_ORDER_AUTO_EXPLORE, 0 },
-    { { N_("Connect/Road"), 0         }, "ctl-R", MENU_ORDER_CONNECT_ROAD, 0 },
-    { { N_("Connect/Rail"), 0         }, "ctl-L", MENU_ORDER_CONNECT_RAIL, 0 },
-    { { N_("Connect/Irrigation"), 0   }, "ctl-I", MENU_ORDER_CONNECT_IRRIGATE, 0 },
+    { { N_("Connect With Road"), 0    },     "R", MENU_ORDER_CONNECT_ROAD, 0 },
+    { { N_("Connect With Rail"), 0    },     "L", MENU_ORDER_CONNECT_RAIL, 0 },
+    { { N_("Connect With Irrigation"), 0 },  "I", MENU_ORDER_CONNECT_IRRIGATE, 0 },
     { { N_("Patrol"), 0               },     "q", MENU_ORDER_PATROL, 0 },
-    { { N_("Go to"), 0                },     "g", MENU_ORDER_GOTO, 0 },
-    { { N_("Go/Airlift to City"), 0   },     "L", MENU_ORDER_GOTO_CITY, 0 },
-    { { N_("Return to nearest city"), 0 },    "", MENU_ORDER_RETURN, 0 },
+    { { N_("Go to Tile"), 0           },     "g", MENU_ORDER_GOTO, 0 },
+    { { N_("Go to/Airlift to City..."), 0 }, "t", MENU_ORDER_GOTO_CITY, 0 },
+    { { N_("Return to Nearest City"), 0 },   "G", MENU_ORDER_RETURN, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Disband Unit"), 0         },     "D", MENU_ORDER_DISBAND, 0 },
     { { N_("Help Build Wonder"), 0    },     "b", MENU_ORDER_BUILD_WONDER, 0 },
-    { { N_("Make Trade Route"), 0     },     "r", MENU_ORDER_TRADEROUTE, 0 },
+    { { N_("Establish Trade Route"), 0 },    "r", MENU_ORDER_TRADE_ROUTE, 0 },
     { { N_("Diplomat/Spy Actions"), 0 },     "d", MENU_ORDER_DIPLOMAT_DLG, 0},
     { { N_("Explode Nuclear"), 0      },     "N", MENU_ORDER_NUKE, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
-    { { N_("Select Same Type"), 0     },      "", MENU_ORDER_SELECT_SAME_TYPE, 0 },
+    { { N_("Select Same Type Everywhere"), 0 }, "X", MENU_ORDER_SELECT_SAME_TYPE, 0 },
     { { N_("Wait"), 0                 },     "w", MENU_ORDER_WAIT, 0 },
     { { N_("Done"), 0                 },   "spc", MENU_ORDER_DONE, 0 },
     { { 0,                            },       0, MENU_END_OF_LIST, 0 }
 };
 
 static struct MenuEntry reports_menu_entries[]={
-    { { N_("Cities"), 0               },    "F1", MENU_REPORT_CITIES, 0 },
     { { N_("Units"), 0                },    "F2", MENU_REPORT_UNITS, 0 },
     /* TRANS: Nations report action */
     { { N_("Nations"), 0              },    "F3", MENU_REPORT_PLAYERS, 0 },
+    { { N_("Cities"), 0               },    "F4", MENU_REPORT_CITIES, 0 },
     { { N_("Economy"), 0              },    "F5", MENU_REPORT_ECONOMY, 0 },
     /* TRANS: Research report action */
     { { N_("Research"), 0             },    "F6", MENU_REPORT_SCIENCE, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Wonders of the World"), 0 },    "F7", MENU_REPORT_WOW, 0 },
     { { N_("Top Five Cities"), 0      },    "F8", MENU_REPORT_TOP_CITIES, 0 },
-    { { N_("Messages"), 0             },   "F10", MENU_REPORT_MESSAGES, 0 },
+    { { N_("Messages"), 0             },    "F9", MENU_REPORT_MESSAGES, 0 },
     { { N_("Demographics"), 0         },   "F11", MENU_REPORT_DEMOGRAPHIC, 0 },
     { { N_("Spaceship"), 0            },   "F12", MENU_REPORT_SPACESHIP, 0 },
     { { 0,                            },       0, MENU_END_OF_LIST, 0 }
 };
 
+static struct MenuEntry editor_menu_entries[] = {
+    { { N_("Editing Mode"), 0         },      "", MENU_EDITOR_TOGGLE, 0 },
+    { { N_("Tools"), 0                },      "", MENU_EDITOR_TOOLS, 0 },
+    { { 0,                            },       0, MENU_END_OF_LIST, 0 }
+};
+
 static struct MenuEntry help_menu_entries[]={
-    { { N_("Languages"), 0            },      "", MENU_HELP_LANGUAGES, 0 },
-    { { N_("Connecting"), 0           },      "", MENU_HELP_CONNECTING, 0 },
-    { { N_("Controls"), 0             },      "", MENU_HELP_CONTROLS, 0 },
-    { { N_("Chatline"), 0             },      "", MENU_HELP_CHATLINE, 0 },
-    { { N_("Playing"), 0              },      "", MENU_HELP_PLAYING, 0 },
-    { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
-    { { N_("Improvements"), 0         },      "", MENU_HELP_IMPROVEMENTS, 0 },
+    { { N_("Overview"), 0             },      "", MENU_HELP_OVERVIEW, 0 },
+    { { N_("Strategy and Tactics"), 0 },      "", MENU_HELP_PLAYING, 0 },
+    { { N_("Terrain"), 0              },      "", MENU_HELP_TERRAIN, 0 },
+    { { N_("Economy"), 0              },      "", MENU_HELP_ECONOMY, 0 },
+    { { N_("Cities"), 0               },      "", MENU_HELP_CITIES, 0 },
+    { { N_("City Improvements"), 0    },      "", MENU_HELP_IMPROVEMENTS, 0 },
+    { { N_("Wonders of the World"), 0 },      "", MENU_HELP_WONDERS, 0 },
     { { N_("Units"), 0                },      "", MENU_HELP_UNITS, 0 },
     { { N_("Combat"), 0               },      "", MENU_HELP_COMBAT, 0 },
     { { N_("ZOC"), 0                  },      "", MENU_HELP_ZOC, 0 },
-    { { N_("Technology"), 0           },      "", MENU_HELP_TECH, 0 },
-    { { N_("Terrain"), 0              },      "", MENU_HELP_TERRAIN, 0 },
-    { { N_("Wonders"), 0              },      "", MENU_HELP_WONDERS, 0 },
     { { N_("Government"), 0           },      "", MENU_HELP_GOVERNMENT, 0 },
-    { { N_("Happiness"), 0            },      "", MENU_HELP_HAPPINESS, 0 },
+    { { N_("Diplomacy"), 0            },      "", MENU_HELP_DIPLOMACY, 0 },
+    { { N_("Technology"), 0           },      "", MENU_HELP_TECH, 0 },
     { { N_("Space Race"), 0           },      "", MENU_HELP_SPACE_RACE, 0 },
+    { { N_("About Ruleset"), 0        },      "", MENU_HELP_RULESET, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
+    { { N_("Connecting"), 0           },      "", MENU_HELP_CONNECTING, 0 },
+    { { N_("Controls"), 0             },      "", MENU_HELP_CONTROLS, 0 },
+    { { N_("Citizen Governor"), 0     },      "", MENU_HELP_CMA, 0 },
+    { { N_("Chatline"), 0             },      "", MENU_HELP_CHATLINE, 0 },
+    { { N_("Worklist Editor"), 0      },      "", MENU_HELP_WORKLIST_EDITOR, 0 },
+    { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
+    { { N_("Languages"), 0            },      "", MENU_HELP_LANGUAGES, 0 },
     { { N_("Copying"), 0              },      "", MENU_HELP_COPYING, 0 },
-    { { N_("About"), 0                },      "", MENU_HELP_ABOUT, 0 },
+    { { N_("About Freeciv"), 0        },      "", MENU_HELP_ABOUT, 0 },
     { { 0,                            },       0, MENU_END_OF_LIST, 0 }
 };
 
@@ -288,12 +305,12 @@ void update_menus(void)
     int i;
     int any_cities = FALSE;
 
-    for(i=0; i<game.info.nplayers; i++) {
-      if (city_list_size(game.players[i].cities)) {
+    players_iterate(pplayer) {
+      if (city_list_size(pplayer->cities)) {
 	any_cities = TRUE;
 	break;
       }
-    }
+    } players_iterate_end;
 
     XtSetSensitive(menus[MENU_REPORT]->button, True);
     XtSetSensitive(menus[MENU_VIEW]->button, True);
@@ -316,7 +333,7 @@ void update_menus(void)
     government_iterate(pgovernment) {
       Widget w;
 
-      if (pgovernment == game.government_when_anarchy) {
+      if (pgovernment == game.government_during_revolution) {
 	continue;
       }
 
@@ -324,7 +341,7 @@ void update_menus(void)
 				smeBSBObjectClass,
 				menus[MENU_GOVERNMENT]->shell, NULL, 0);
       XtAddCallback(w, XtNcallback, revolution_menu_callback, pgovernment);
-      XtSetSensitive(w, can_change_to_government(game.player_ptr,
+      XtSetSensitive(w, can_change_to_government(client.conn.playing,
 						 pgovernment));
 
       government_widgets[i] = w;
@@ -355,15 +372,14 @@ void update_menus(void)
     menu_entry_sensitive(MENU_GAME, MENU_GAME_CLEAR_OUTPUT, 1);
     menu_entry_sensitive(MENU_GAME, MENU_GAME_DISCONNECT, 1);
 
-    if (game.player_ptr) {
+    if (NULL != client.conn.playing) {
       menu_entry_sensitive(MENU_REPORT, MENU_REPORT_SPACESHIP,
-			   (game.player_ptr->spaceship.state!=SSHIP_NONE));
+			   (SSHIP_NONE != client.conn.playing->spaceship.state));
     } else {
       menu_entry_sensitive(MENU_REPORT, MENU_REPORT_SPACESHIP, 0);
     }
 
     if ((get_num_units_in_focus() > 0) && can_client_issue_orders()) {
-      Terrain_type_id ttype;
       struct terrain *tinfo;
       struct tile *ptile = NULL;
       bool can_build;
@@ -386,9 +402,9 @@ void update_menus(void)
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_TRANSFORM, 
 			   can_units_do_activity(punits, ACTIVITY_TRANSFORM));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_FORTRESS, 
-			   can_units_do_activity(punits, ACTIVITY_FORTRESS));
+			   can_units_do_base_gui(punits, BASE_GUI_FORTRESS));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_AIRBASE,
-			   can_units_do_activity(punits, ACTIVITY_AIRBASE));
+			   can_units_do_base_gui(punits, BASE_GUI_AIRBASE));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_POLLUTION, 
 			   can_units_do_activity(punits, ACTIVITY_POLLUTION)
 			   || can_units_do(punits, can_unit_paradrop));
@@ -429,23 +445,23 @@ void update_menus(void)
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_BUILD_WONDER,
 			   can_units_do(punits,
 				        unit_can_help_build_wonder_here));
-      menu_entry_sensitive(MENU_ORDER, MENU_ORDER_TRADEROUTE,
+      menu_entry_sensitive(MENU_ORDER, MENU_ORDER_TRADE_ROUTE,
 			   can_units_do(punits,
-				        unit_can_est_traderoute_here));
+				        unit_can_est_trade_route_here));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_DIPLOMAT_DLG,
 			   can_units_do_diplomat_action(punits,
 							DIPLOMAT_ANY_ACTION));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_NUKE,
                            units_have_flag(punits, F_NUCLEAR, TRUE));
 
+      /* FiXME: very odd, iterating for the first entry! */
       unit_list_iterate(punits, punit) {
 	ptile = punit->tile;
 	break;
       } unit_list_iterate_end;
 
-      tinfo = ptile->terrain;
-      ttype = tinfo->index;
-      can_build = !(ptile->city);
+      tinfo = tile_terrain(ptile);
+      can_build = !(tile_city(ptile));
 
       if (units_have_flag(punits, F_CITIES, TRUE)) {
 	if (!can_build) {
@@ -458,13 +474,13 @@ void update_menus(void)
       }
 
       if ((tinfo->irrigation_result != T_NONE)
-	  && (tinfo->irrigation_result->index != ttype)) {
+	  && (tinfo->irrigation_result != tinfo)) {
 	menu_entry_rename(MENU_ORDER, MENU_ORDER_IRRIGATE,
 			  TEXT_ORDER_IRRIGATE_CHANGE_TO,
 			  terrain_name_translation(tinfo->irrigation_result));
       }
       else if (tile_has_special(ptile, S_IRRIGATION) &&
-	       player_knows_techs_with_flag(game.player_ptr, TF_FARMLAND)) {
+	       player_knows_techs_with_flag(client.conn.playing, TF_FARMLAND)) {
 	menu_entry_rename(MENU_ORDER, MENU_ORDER_IRRIGATE,
 			  TEXT_ORDER_IRRIGATE_FARMLAND, NULL);
       } else {
@@ -473,7 +489,7 @@ void update_menus(void)
       }
 
       if ((tinfo->mining_result != T_NONE)
-	  && (tinfo->mining_result->index != ttype)) {
+	  && (tinfo->mining_result != tinfo)) {
 	menu_entry_rename(MENU_ORDER, MENU_ORDER_MINE,
 			  TEXT_ORDER_MINE_CHANGE_TO,
 			  terrain_name_translation(tinfo->mining_result));
@@ -483,7 +499,7 @@ void update_menus(void)
       }
 
       if ((tinfo->transform_result != T_NONE)
-	  && (tinfo->transform_result->index != ttype)) {
+	  && (tinfo->transform_result != tinfo)) {
 	menu_entry_rename(MENU_ORDER, MENU_ORDER_TRANSFORM,
 			  TEXT_ORDER_TRANSFORM_TRANSFORM_TO,
 			  terrain_name_translation(tinfo->transform_result));
@@ -527,7 +543,7 @@ static void game_menu_callback(Widget w, XtPointer client_data,
     popup_messageopt_dialog();
     break;
   case MENU_GAME_SAVE_SETTINGS:
-    save_options();
+    options_save();
     break;
   case MENU_GAME_SERVER_OPTIONS:
     popup_settable_options_dialog();
@@ -563,7 +579,7 @@ static void government_menu_callback(Widget w, XtPointer client_data,
     popup_rates_dialog();
     break;
   case MENU_GOVERNMENT_WORKLISTS:
-    popup_worklists_dialog(game.player_ptr);
+    popup_worklists_dialog(client.conn.playing);
     break;
   case MENU_GOVERNMENT_REVOLUTION:
     popup_revolution_dialog(NULL);
@@ -582,7 +598,7 @@ static void revolution_menu_callback(Widget w, XtPointer client_data,
   if (!can_client_issue_orders()) {
     return;
   }
-  if (game.player_ptr->revolution_finishes == -1) {
+  if (-1 == client.conn.playing->revolution_finishes) {
     popup_revolution_dialog(pgovernment);
   } else {
     /* Player already has a revolution and should just choose a government */
@@ -766,8 +782,8 @@ static void orders_menu_callback(Widget w, XtPointer client_data,
   case MENU_ORDER_BUILD_WONDER:
     key_unit_build_wonder();
     break;
-  case MENU_ORDER_TRADEROUTE:
-    key_unit_traderoute();
+  case MENU_ORDER_TRADE_ROUTE:
+    key_unit_trade_route();
     break;
   case MENU_ORDER_DIPLOMAT_DLG:
     key_unit_diplomat_actions();
@@ -776,7 +792,7 @@ static void orders_menu_callback(Widget w, XtPointer client_data,
     key_unit_nuke();
     break;
   case MENU_ORDER_SELECT_SAME_TYPE:
-    request_unit_select_same_type(get_units_in_focus());
+    request_unit_select(get_units_in_focus(), SELTYPE_SAME, SELLOC_ALL);
     break;
   case MENU_ORDER_WAIT:
     key_unit_wait();
@@ -824,13 +840,30 @@ static void reports_menu_callback(Widget w, XtPointer client_data,
     send_report_request(REPORT_DEMOGRAPHIC);
     break;
    case MENU_REPORT_SPACESHIP:
-    if (game.player_ptr) {
-      popup_spaceship_dialog(game.player_ptr);
+    if (NULL != client.conn.playing) {
+      popup_spaceship_dialog(client.conn.playing);
     }
     break;
   }
 }
 
+/****************************************************************
+  Callback for Editor menu entries.
+*****************************************************************/
+static void editor_menu_callback(Widget w, XtPointer client_data,
+				 XtPointer garbage)
+{
+  size_t pane_num = (size_t)client_data;
+
+  switch(pane_num) {
+  case MENU_EDITOR_TOGGLE:
+    key_editor_toggle();
+    break;
+  case MENU_EDITOR_TOOLS:
+    break;
+  }
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -849,12 +882,27 @@ static void help_menu_callback(Widget w, XtPointer client_data,
   case MENU_HELP_CONTROLS:
     popup_help_dialog_string(HELP_CONTROLS_ITEM);
     break;
+  case MENU_HELP_CMA:
+    popup_help_dialog_string(HELP_CMA_ITEM);
+    break;
   case MENU_HELP_CHATLINE:
     popup_help_dialog_string(HELP_CHATLINE_ITEM);
     break;
+  case MENU_HELP_WORKLIST_EDITOR:
+    popup_help_dialog_string(HELP_WORKLIST_EDITOR_ITEM);
+    break;
+  case MENU_HELP_OVERVIEW:
+    popup_help_dialog_string(HELP_OVERVIEW_ITEM);
+    break;
   case MENU_HELP_PLAYING:
     popup_help_dialog_string(HELP_PLAYING_ITEM);
     break;
+  case MENU_HELP_ECONOMY:
+    popup_help_dialog_string(HELP_ECONOMY_ITEM);
+    break;
+  case MENU_HELP_CITIES:
+    popup_help_dialog_string(HELP_CITIES_ITEM);
+    break;
   case MENU_HELP_IMPROVEMENTS:
     popup_help_dialog_string(HELP_IMPROVEMENTS_ITEM);
     break;
@@ -879,12 +927,15 @@ static void help_menu_callback(Widget w, XtPointer client_data,
   case MENU_HELP_GOVERNMENT:
     popup_help_dialog_string(HELP_GOVERNMENT_ITEM);
     break;
-  case MENU_HELP_HAPPINESS:
-    popup_help_dialog_string(HELP_HAPPINESS_ITEM);
+  case MENU_HELP_DIPLOMACY:
+    popup_help_dialog_string(HELP_DIPLOMACY_ITEM);
     break;
   case MENU_HELP_SPACE_RACE:
     popup_help_dialog_string(HELP_SPACE_RACE_ITEM);
     break;
+  case MENU_HELP_RULESET:
+    popup_help_dialog_string(HELP_RULESET_ITEM);
+    break;
   case MENU_HELP_COPYING:
     popup_help_dialog_string(HELP_COPYING_ITEM);
     break;
@@ -914,6 +965,9 @@ void setup_menus(Widget parent_form)
   create_menu(MENU_REPORT, "reportsmenu", 
 	      reports_menu_entries, reports_menu_callback, 
 	      parent_form);
+  create_menu(MENU_EDITOR, "editormenu",
+	      editor_menu_entries, editor_menu_callback,
+	      parent_form);
   create_menu(MENU_HELP, "helpmenu", 
 	      help_menu_entries, help_menu_callback, 
 	      parent_form);
