@@ -33,17 +33,19 @@
 #include <X11/Xaw/Toggle.h>     
 #include <X11/Xaw/Viewport.h>
 
+/* utility */
 #include "fcintl.h"
+#include "log.h"
 #include "mem.h"
 
+/* common */
 #include "city.h"
-#include "game.h"
 #include "packets.h"
 #include "worklist.h"
 
-#include "civclient.h"
+/* client */
+#include "client_main.h"
 #include "climisc.h"
-#include "clinet.h"
 
 #include "gui_main.h"
 #include "gui_stuff.h"
@@ -65,7 +67,7 @@ struct worklist_report_dialog {
   struct player *pplr;
   char worklist_names[MAX_NUM_WORKLISTS][MAX_LEN_NAME];
   char *worklist_names_ptrs[MAX_NUM_WORKLISTS+1];
-  struct worklist *worklist_ptr[MAX_NUM_WORKLISTS];
+  struct global_worklist *worklist_ptr[MAX_NUM_WORKLISTS];
   int wl_idx;
 };
 
@@ -82,7 +84,7 @@ struct worklist_dialog {
   Widget shell;
 
   struct city *pcity;
-  struct worklist *pwl;
+  int global_worklist_id;
 
   void *parent_data;
   WorklistOkCallback ok_callback;
@@ -104,10 +106,10 @@ static Widget worklist_report_shell = NULL;
 static struct worklist_report_dialog *report_dialog;
 
 
-static int uni_id(struct worklist *pwl, int wlinx);
+static int uni_id(const struct worklist *pwl, int wlinx);
 
 static void worklist_id_to_name(char buf[],
-				struct city_production production,
+				struct universal production,
 				struct city *pcity);
 
 static void rename_worklist_callback(Widget w, XtPointer client_data, 
@@ -157,7 +159,7 @@ static void worklist_avail_help_callback(Widget w, XtPointer client_data,
 					  XtPointer call_data);
 static void worklist_show_advanced_callback(Widget w, XtPointer client_data, 
 					  XtPointer call_data);
-static void worklist_help(struct city_production production);
+static void worklist_help(struct universal production);
 static void worklist_populate_worklist(struct worklist_dialog *pdialog);
 static void worklist_populate_targets(struct worklist_dialog *pdialog);
 
@@ -306,7 +308,7 @@ void popup_worklists_dialog(struct player *pplr)
   that can be made.  Otherwise, just list everything that technology
   will allow.
 *************************************************************************/
-Widget popup_worklist(struct worklist *pwl, struct city *pcity,
+Widget popup_worklist(struct city *pcity, struct global_worklist *pgwl,
 		      Widget parent, void *parent_data,
 		      WorklistOkCallback ok_cb,
 		      WorklistCancelCallback cancel_cb)
@@ -337,7 +339,7 @@ Widget popup_worklist(struct worklist *pwl, struct city *pcity,
   pdialog = fc_malloc(sizeof(struct worklist_dialog));
 
   pdialog->pcity = pcity;
-  pdialog->pwl = pwl;
+  pdialog->global_worklist_id = pgwl ? global_worklist_id(pgwl) : -1;
   pdialog->shell = parent;
   pdialog->parent_data = parent_data;
   pdialog->ok_callback = ok_cb;
@@ -567,14 +569,14 @@ void update_worklist_report_dialog(void)
 /****************************************************************
   Returns an unique id for element in units + buildings sequence
 *****************************************************************/
-int uni_id(struct worklist *pwl, int inx)
+int uni_id(const struct worklist *pwl, int inx)
 {
   if ((inx < 0) || (inx >= worklist_length(pwl))) {
     return WORKLIST_END;
-  } else if (pwl->entries[inx].is_unit) {
-    return pwl->entries[inx].value + B_LAST;
+  } else if (VUT_UTYPE == pwl->entries[inx].kind) {
+    return utype_number(pwl->entries[inx].value.utype) + B_LAST;
   } else {
-    return pwl->entries[inx].value;
+    return improvement_number(pwl->entries[inx].value.building);
   }
 }
 
@@ -582,21 +584,21 @@ int uni_id(struct worklist *pwl, int inx)
 
 *****************************************************************/
 void worklist_id_to_name(char buf[],
-			 struct city_production production,
+			 struct universal production,
 			 struct city *pcity)
 {
-  if (production.is_unit)
+  if (VUT_UTYPE == production.kind)
     sprintf(buf, "%s (%d)",
-	    utype_values_translation(utype_by_number(production.value)),
-	    unit_build_shield_cost(utype_by_number(production.value)));
+	    utype_values_translation(production.value.utype),
+	    utype_build_shield_cost(production.value.utype));
   else if (pcity)
     sprintf(buf, "%s (%d)",
-	    get_impr_name_ex(pcity, production.value),
-	    impr_build_shield_cost(production.value));
+	    city_improvement_name_translation(pcity, production.value.building),
+	    impr_build_shield_cost(production.value.building));
   else
     sprintf(buf, "%s (%d)",
-	    improvement_name_translation(production.value),
-	    impr_build_shield_cost(production.value));
+	    improvement_name_translation(production.value.building),
+	    impr_build_shield_cost(production.value.building));
 }
 
 
@@ -613,19 +615,20 @@ void rename_worklist_callback(Widget w, XtPointer client_data,
   pdialog = (struct worklist_report_dialog *)client_data;
   retList = XawListShowCurrent(pdialog->list);
 
-  if (retList->list_index == XAW_LIST_NONE)
+  if (retList->list_index == XAW_LIST_NONE) {
     return;
+  }
 
   pdialog->wl_idx = retList->list_index;
 
   input_dialog_create(worklist_report_shell,
-		      "renameworklist",
-		      _("What should the new name be?"),
-		      pdialog->pplr->worklists[pdialog->wl_idx].name,
-		      rename_worklist_sub_callback,
-		      (XtPointer)pdialog,
-		      rename_worklist_sub_callback,
-		      (XtPointer)NULL);
+                      "renameworklist",
+                      _("What should the new name be?"),
+                      global_worklist_name(pdialog->worklist_ptr[pdialog->wl_idx]),
+                      rename_worklist_sub_callback,
+                      (XtPointer)pdialog,
+                      rename_worklist_sub_callback,
+                      (XtPointer)NULL);
 }
 
 /****************************************************************
@@ -639,13 +642,11 @@ void rename_worklist_sub_callback(Widget w, XtPointer client_data,
   pdialog = (struct worklist_report_dialog *)client_data;
 
   if (pdialog) {
-    strncpy(pdialog->pplr->worklists[pdialog->wl_idx].name,
-            input_dialog_get_input(w), MAX_LEN_NAME);
-    pdialog->pplr->worklists[pdialog->wl_idx].name[MAX_LEN_NAME - 1] = '\0';
-
-    update_worklist_report_dialog();
+      global_worklist_set_name(pdialog->worklist_ptr[pdialog->wl_idx],
+                               input_dialog_get_input(w));
+      update_worklist_report_dialog();
   }
-  
+
   input_dialog_destroy(w);
 }
 
@@ -655,26 +656,7 @@ void rename_worklist_sub_callback(Widget w, XtPointer client_data,
 void insert_worklist_callback(Widget w, XtPointer client_data, 
 			      XtPointer call_data)
 {
-  struct worklist_report_dialog *pdialog;
-  int j;
-
-  pdialog = (struct worklist_report_dialog *)client_data;
-
-  /* Find the next free worklist for this player */
-
-  for (j = 0; j < MAX_NUM_WORKLISTS; j++)
-    if (!pdialog->pplr->worklists[j].is_valid)
-      break;
-
-  /* No more worklist slots free.  (!!!Maybe we should tell the user?) */
-  if (j == MAX_NUM_WORKLISTS)
-    return;
-
-  /* Validate this slot. */
-  init_worklist(&pdialog->pplr->worklists[j]);
-  pdialog->pplr->worklists[j].is_valid = TRUE;
-  strcpy(pdialog->pplr->worklists[j].name, _("empty worklist"));
-
+  (void) global_worklist_new(_("empty worklist"));
   update_worklist_report_dialog();
 }
 
@@ -687,29 +669,15 @@ void delete_worklist_callback(Widget w, XtPointer client_data,
 {
   struct worklist_report_dialog *pdialog;
   XawListReturnStruct *retList;
-  int i, j;
 
   pdialog = (struct worklist_report_dialog *)client_data;
   retList = XawListShowCurrent(pdialog->list);
 
-  if (retList->list_index == XAW_LIST_NONE)
+  if (retList->list_index == XAW_LIST_NONE) {
     return;
-
-  /* Look for the last free worklist */
-  for (i = 0; i < MAX_NUM_WORKLISTS; i++)
-    if (!pdialog->pplr->worklists[i].is_valid)
-      break;
-
-  for (j = retList->list_index; j < i-1; j++) {
-    copy_worklist(&pdialog->pplr->worklists[j], 
-                  &pdialog->pplr->worklists[j+1]);
   }
 
-  /* The last worklist in the set is no longer valid -- it's been slid up
-   * one slot. */
-  pdialog->pplr->worklists[i-1].is_valid = FALSE;
-  strcpy(pdialog->pplr->worklists[i-1].name, "\0");
-  
+  global_worklist_destroy(pdialog->worklist_ptr[pdialog->wl_idx]);
   update_worklist_report_dialog();
 }
 
@@ -725,14 +693,15 @@ void edit_worklist_callback(Widget w, XtPointer client_data,
   pdialog = (struct worklist_report_dialog *)client_data;
   retList = XawListShowCurrent(pdialog->list);
 
-  if (retList->list_index == XAW_LIST_NONE)
+  if (retList->list_index == XAW_LIST_NONE) {
     return;
+  }
 
   pdialog->wl_idx = retList->list_index;
 
-  popup_worklist(pdialog->worklist_ptr[pdialog->wl_idx], NULL, 
-		 worklist_report_shell,
-		 pdialog, commit_player_worklist, NULL);
+  popup_worklist(NULL, pdialog->worklist_ptr[pdialog->wl_idx],
+                 worklist_report_shell,
+                 pdialog, commit_player_worklist, NULL);
 }
 
 /****************************************************************
@@ -744,7 +713,7 @@ void commit_player_worklist(struct worklist *pwl, void *data)
 
   pdialog = (struct worklist_report_dialog *)data;
 
-  copy_worklist(&pdialog->pplr->worklists[pdialog->wl_idx], pwl);
+  global_worklist_set(pdialog->worklist_ptr[pdialog->wl_idx], pwl);
 }
 
 /****************************************************************
@@ -764,17 +733,18 @@ void close_worklistreport_callback(Widget w, XtPointer client_data,
 *****************************************************************/
 void populate_worklist_report_list(struct worklist_report_dialog *pdialog)
 {
-  int i, n;
+  int n = 0;
 
-  for (i = 0, n = 0; i < MAX_NUM_WORKLISTS; i++) {
-    if (pdialog->pplr->worklists[i].is_valid) {
-      strcpy(pdialog->worklist_names[n], pdialog->pplr->worklists[i].name);
-      pdialog->worklist_names_ptrs[n] = pdialog->worklist_names[n];
-      pdialog->worklist_ptr[n] = &pdialog->pplr->worklists[i];
-      n++;
+  global_worklists_iterate(pgwl) {
+    if (n >= MAX_NUM_WORKLISTS) {
+      break;
     }
-  }
-  
+    strcpy(pdialog->worklist_names[n], global_worklist_name(pgwl));
+    pdialog->worklist_names_ptrs[n] = pdialog->worklist_names[n];
+    pdialog->worklist_ptr[n] = pgwl;
+    n++;
+  } global_worklists_iterate_end;
+
   /* Terminators */
   pdialog->worklist_names_ptrs[n] = NULL;
 }
@@ -834,19 +804,14 @@ void insert_into_worklist(struct worklist_dialog *pdialog,
 			  int before, cid cid)
 {
   int i, first_free;
-  struct city_production target = cid_decode(cid);
+  struct universal target = cid_decode(cid);
 
   /* If this worklist is a city worklist, double check that the city
      really can (eventually) build the target.  We've made sure that
      the list of available targets is okay for this city, but a global
      worklist may try to insert an odd-ball unit or target. */
   if (pdialog->pcity
-      && ((target.is_unit
-	   && !can_eventually_build_unit(pdialog->pcity,
-					 utype_by_number(target.value)))
-	  || (!target.is_unit
-	      && !can_eventually_build_improvement(pdialog->pcity,
-						   target.value)))) {
+      && !can_city_build_later(pdialog->pcity, target)) {
     /* Nope, this city can't build this target, ever.  Don't put it into
        the worklist. */
     return;
@@ -936,7 +901,7 @@ void worklist_insert_common_callback(struct worklist_dialog *pdialog,
 {
   int target;
   int i, len;
-  struct city_production production;
+  struct universal production;
 
   /* Is there anything selected to insert? */
   if (retAvail->list_index == XAW_LIST_NONE)
@@ -947,9 +912,10 @@ void worklist_insert_common_callback(struct worklist_dialog *pdialog,
 
   if (retAvail->list_index >= pdialog->worklist_avail_num_targets) {
     /* target is a global worklist id */
-    struct player *pplr = city_owner(pdialog->pcity);
+    /* struct player *pplr = city_owner(pdialog->pcity); */
     int wl_idx = pdialog->worklist_avail_ids[retAvail->list_index];
-    struct worklist *pwl = &pplr->worklists[wl_idx];
+    const struct global_worklist *pgwl = global_worklist_by_id(wl_idx);
+    const struct worklist *pwl = global_worklist_get(pgwl);
 
     for (i = 0; i < MAX_LEN_WORKLIST && uni_id(pwl, i) != WORKLIST_END; i++) {
       insert_into_worklist(pdialog, where, uni_id(pwl, i));
@@ -959,14 +925,14 @@ void worklist_insert_common_callback(struct worklist_dialog *pdialog,
   } else if (retAvail->list_index >= 
 	     pdialog->worklist_avail_num_improvements) {
     /* target is a unit */
-    production.is_unit = true;
-    production.value = target;
+    production.kind = VUT_UTYPE;
+    production.value.utype = utype_by_number(target);
     insert_into_worklist(pdialog, where, cid_encode(production));
     where++;
   } else {
     /* target is an improvement or wonder */
-    production.is_unit = false;
-    production.value = target;
+    production.kind = VUT_IMPROVEMENT;
+    production.value.building = improvement_by_number(target);
     insert_into_worklist(pdialog, where, cid_encode(production));
     where++;
   }
@@ -1100,36 +1066,35 @@ void worklist_ok_callback(Widget w, XtPointer client_data, XtPointer call_data)
 {
   struct worklist_dialog *pdialog;
   struct worklist wl;
-  struct city_production production;
+  struct universal production;
   int i;
   
   pdialog=(struct worklist_dialog *)client_data;
   
   /* Fill in this worklist with the parameters set in the worklist 
      dialog. */
-  init_worklist(&wl);
+  worklist_init(&wl);
   
   for (i = 0; i < MAX_LEN_WORKLIST; i++) {
     if (pdialog->worklist_ids[i] == WORKLIST_END) {
       continue;
     } else if (pdialog->worklist_ids[i] >= B_LAST) {
-      production.is_unit = true;
-      production.value = pdialog->worklist_ids[i] - B_LAST;
+      production.kind = VUT_UTYPE;
+      production.value.utype = utype_by_number(pdialog->worklist_ids[i] - B_LAST);
       worklist_append(&wl, production);
     } else if (pdialog->worklist_ids[i] >= 0) {
-      production.is_unit = false;
-      production.value = pdialog->worklist_ids[i];
+      production.kind = VUT_IMPROVEMENT;
+      production.value.building = improvement_by_number(pdialog->worklist_ids[i]);
       worklist_append(&wl, production);
     } else {
       continue;
     }
   }
-  strcpy(wl.name, pdialog->pwl->name);
-  wl.is_valid = pdialog->pwl->is_valid;
-  
+
   /* Invoke the dialog's parent-specified callback */
-  if (pdialog->ok_callback)
+  if (pdialog->ok_callback) {
     (*pdialog->ok_callback)(&wl, pdialog->parent_data);
+  }
 
   /* Cleanup. */
   XtDestroyWidget(XtParent(XtParent(w)));
@@ -1166,7 +1131,7 @@ void worklist_worklist_help_callback(Widget w, XtPointer client_data,
 {
   struct worklist_dialog *pdialog;
   XawListReturnStruct *ret;
-  struct city_production production;
+  struct universal production;
 
   pdialog=(struct worklist_dialog *)client_data;
 
@@ -1175,7 +1140,8 @@ void worklist_worklist_help_callback(Widget w, XtPointer client_data,
     cid cid = pdialog->worklist_ids[ret->list_index];
     production = cid_decode(cid);
   } else {
-    production.value = -1;
+    production.kind = VUT_NONE;
+    production.value.building = NULL;
   }
 
   worklist_help(production);
@@ -1186,7 +1152,7 @@ void worklist_avail_help_callback(Widget w, XtPointer client_data,
 {
   struct worklist_dialog *pdialog;
   XawListReturnStruct *ret;
-  struct city_production production;
+  struct universal production;
 
   pdialog=(struct worklist_dialog *)client_data;
 
@@ -1194,31 +1160,34 @@ void worklist_avail_help_callback(Widget w, XtPointer client_data,
   if(ret->list_index!=XAW_LIST_NONE) {
     if (ret->list_index >= pdialog->worklist_avail_num_targets) {
       /* target is a global worklist id */
-      production.value = -1;
+      production.kind = VUT_NONE;
+      production.value.building = NULL;
     } else {
-      production.value = pdialog->worklist_avail_ids[ret->list_index];
-      production.is_unit =
-        ret->list_index >= pdialog->worklist_avail_num_improvements;
+      production =
+        universal_by_number((ret->list_index >= pdialog->worklist_avail_num_improvements)
+                               ? VUT_UTYPE : VUT_IMPROVEMENT,
+                               pdialog->worklist_avail_ids[ret->list_index]);
     }
   } else {
-    production.value = -1;
+    production.kind = VUT_NONE;
+    production.value.building = NULL;
   }
 
   worklist_help(production);
 }
 
 
-void worklist_help(struct city_production production)
+void worklist_help(struct universal production)
 {
-  if (production.value >= 0) {
-    if (production.is_unit) {
-      popup_help_dialog_typed(utype_name_translation(utype_by_number(production.value)),
+  if (production.value.building) {
+    if (VUT_UTYPE == production.kind) {
+      popup_help_dialog_typed(utype_name_translation(production.value.utype),
 					    HELP_UNIT);
-    } else if (is_great_wonder(production.value)) {
-      popup_help_dialog_typed(improvement_name_translation(production.value),
+    } else if (is_great_wonder(production.value.building)) {
+      popup_help_dialog_typed(improvement_name_translation(production.value.building),
 						   HELP_WONDER);
     } else {
-      popup_help_dialog_typed(improvement_name_translation(production.value),
+      popup_help_dialog_typed(improvement_name_translation(production.value.building),
 						   HELP_IMPROVEMENT);
     }
   }
@@ -1256,28 +1225,37 @@ void worklist_populate_worklist(struct worklist_dialog *pdialog)
 {
   int i, n;
   int id;
-  struct city_production target;
+  struct universal target;
+  const struct worklist *pwl;
 
   n = 0;
   if (pdialog->pcity) {
     /* First element is the current build target of the city. */
-    id = pdialog->pcity->production.value;
+    id = universal_number(&pdialog->pcity->production);
 
     worklist_id_to_name(pdialog->worklist_names[n],
 			pdialog->pcity->production, pdialog->pcity);
 
-    if (pdialog->pcity->production.is_unit)
+    if (VUT_UTYPE == pdialog->pcity->production.kind)
       id += B_LAST;
     pdialog->worklist_names_ptrs[n] = pdialog->worklist_names[n];
     pdialog->worklist_ids[n] = id;
     n++;
+    pwl = &pdialog->pcity->worklist;
+  } else {
+    const struct global_worklist *pgwl =
+      global_worklist_by_id(pdialog->global_worklist_id);
+
+    assert(NULL != pgwl);
+
+    pwl = global_worklist_get(pgwl);
   }
 
   /* Fill in the rest of the worklist list */
   for (i = 0; n < MAX_LEN_WORKLIST &&
-	 uni_id(pdialog->pwl, i) != WORKLIST_END; i++, n++) {
-    worklist_peek_ith(pdialog->pwl, &target, i);
-    id = uni_id(pdialog->pwl, i);
+	 uni_id(pwl, i) != WORKLIST_END; i++, n++) {
+    worklist_peek_ith(pwl, &target, i);
+    id = uni_id(pwl, i);
 
     worklist_id_to_name(pdialog->worklist_names[n],
 			target, pdialog->pcity);
@@ -1297,12 +1275,11 @@ void worklist_populate_worklist(struct worklist_dialog *pdialog)
 *****************************************************************/
 void worklist_populate_targets(struct worklist_dialog *pdialog)
 {
-  int i, n;
-  struct player *pplr = game.player_ptr;
+  int n;
   Boolean b;
   int advanced_tech;
   int can_build, can_eventually_build;
-  struct city_production production;
+  struct universal production;
 
   if (!can_client_issue_orders()) {
     return;
@@ -1319,51 +1296,51 @@ void worklist_populate_targets(struct worklist_dialog *pdialog)
     advanced_tech = False;
  
   /*     + First, improvements and Wonders. */
-  impr_type_iterate(i) {
+  improvement_iterate(pimprove) {
     /* Can the player (eventually) build this improvement? */
-    can_build = can_player_build_improvement(pplr,i);
-    can_eventually_build = can_player_eventually_build_improvement(pplr,i);
+    can_build = can_player_build_improvement_now(client.conn.playing, pimprove);
+    can_eventually_build = can_player_build_improvement_later(client.conn.playing, pimprove);
 
     /* If there's a city, can the city build the improvement? */
     if (pdialog->pcity) {
-      can_build = can_build && can_build_improvement(pdialog->pcity, i);
+      can_build = can_build && can_city_build_improvement_now(pdialog->pcity, pimprove);
       can_eventually_build = can_eventually_build &&
-	can_eventually_build_improvement(pdialog->pcity, i);
+	can_city_build_improvement_later(pdialog->pcity, pimprove);
     }
     
     if (( advanced_tech && can_eventually_build) ||
 	(!advanced_tech && can_build)) {
-      production.is_unit = false;
-      production.value = i;
+      production.kind = VUT_IMPROVEMENT;
+      production.value.building = pimprove;
       worklist_id_to_name(pdialog->worklist_avail_names[n],
 			  production, pdialog->pcity);
       pdialog->worklist_avail_names_ptrs[n]=pdialog->worklist_avail_names[n];
-      pdialog->worklist_avail_ids[n++]=i;
+      pdialog->worklist_avail_ids[n++] = improvement_number(pimprove);
     }
-  } impr_type_iterate_end;
+  } improvement_iterate_end;
   pdialog->worklist_avail_num_improvements=n;
 
   /*     + Second, units. */
-  unit_type_iterate(i) {
+  unit_type_iterate(punittype) {
     /* Can the player (eventually) build this improvement? */
-    can_build = can_player_build_unit(pplr,i);
-    can_eventually_build = can_player_eventually_build_unit(pplr,i);
+    can_build = can_player_build_unit_now(client.conn.playing, punittype);
+    can_eventually_build = can_player_build_unit_later(client.conn.playing, punittype);
 
     /* If there's a city, can the city build the improvement? */
     if (pdialog->pcity) {
-      can_build = can_build && can_build_unit(pdialog->pcity, i);
+      can_build = can_build && can_city_build_unit_now(pdialog->pcity, punittype);
       can_eventually_build = can_eventually_build &&
-	can_eventually_build_unit(pdialog->pcity, i);
+	can_city_build_unit_later(pdialog->pcity, punittype);
     }
 
     if (( advanced_tech && can_eventually_build) ||
 	(!advanced_tech && can_build)) {
-      production.is_unit = true;
-      production.value = i->index;
+      production.kind = VUT_UTYPE;
+      production.value.utype = punittype;
       worklist_id_to_name(pdialog->worklist_avail_names[n],
 			  production, pdialog->pcity);
       pdialog->worklist_avail_names_ptrs[n]=pdialog->worklist_avail_names[n];
-      pdialog->worklist_avail_ids[n++]=i->index;
+      pdialog->worklist_avail_ids[n++] = utype_number(punittype);
     }
   } unit_type_iterate_end;
 
@@ -1372,13 +1349,14 @@ void worklist_populate_targets(struct worklist_dialog *pdialog)
   /*     + Finally, the global worklists. */
   if (pdialog->pcity) {
     /* Now fill in the global worklists. */
-    for (i = 0; i < MAX_NUM_WORKLISTS; i++)
-      if (pplr->worklists[i].is_valid) {
-	strcpy(pdialog->worklist_avail_names[n], pplr->worklists[i].name);
-	pdialog->worklist_avail_names_ptrs[n]=pdialog->worklist_avail_names[n];
-	pdialog->worklist_avail_ids[n++]=i;
-      }
+    global_worklists_iterate(pgwl) {
+      sz_strlcpy(pdialog->worklist_avail_names[n],
+                 global_worklist_name(pgwl));
+      pdialog->worklist_avail_names_ptrs[n] =
+        pdialog->worklist_avail_names[n];
+      pdialog->worklist_avail_ids[n++] = global_worklist_id(pgwl);
+    } global_worklists_iterate_end;
   }
 
-  pdialog->worklist_avail_names_ptrs[n]=0;
+  pdialog->worklist_avail_names_ptrs[n] = NULL;
 }
