@@ -21,8 +21,11 @@
 
 #include "mapview_common.h"
 
+#include "unitlist.h"
+
 void update_info_label(void);
 void update_unit_info_label(struct unit_list *punitlist);
+void update_mouse_cursor(enum cursor_type new_cursor_type);
 void update_timeout_label(void);
 void update_turn_done_button(bool do_restore);
 void update_city_descriptions(void);
