@@ -14,26 +14,21 @@
 #ifndef FC__MAPVIEW_COMMON_H
 #define FC__MAPVIEW_COMMON_H
 
-#include "shared.h"		/* bool type */
+/* utility */
+#include "shared.h"             /* bool type */
 
+/* common */
 #include "fc_types.h"
+#include "featured_text.h"      /* enum text_link_type type */
 #include "map.h"
 
+/* include */
 #include "colors_g.h"
 
 #include "tilespec.h"
 
 struct canvas_store;		/* opaque type, real type is gui-dep */
 
-struct mapview_decoration {
-  /* For client Area Selection */
-  enum tile_hilite {
-    HILITE_NONE, HILITE_CITY
-  } hilite;
-
-  int crosshair; /* A refcount */
-};
-
 struct view {
   int gui_x0, gui_y0;
   int width, height;		/* Size in pixels. */
@@ -43,7 +38,24 @@ struct view {
   struct canvas *store, *tmp_store;
 };
 
-extern struct mapview_decoration *map_deco;
+void mapdeco_init(void);
+void mapdeco_free(void);
+void mapdeco_set_highlight(const struct tile *ptile, bool highlight);
+bool mapdeco_is_highlight_set(const struct tile *ptile);
+void mapdeco_clear_highlights(void);
+void mapdeco_set_crosshair(const struct tile *ptile, bool crosshair);
+bool mapdeco_is_crosshair_set(const struct tile *ptile);
+void mapdeco_clear_crosshairs(void);
+void mapdeco_set_gotoroute(const struct unit *punit);
+void mapdeco_add_gotoline(const struct tile *ptile,
+                          enum direction8 dir);
+void mapdeco_remove_gotoline(const struct tile *ptile,
+                             enum direction8 dir);
+bool mapdeco_is_gotoline_set(const struct tile *ptile,
+                             enum direction8 dir);
+void mapdeco_clear_gotoroutes(void);
+
+
 extern struct view mapview;
 
 /* HACK: Callers can set this to FALSE to disable sliding.  It should be
@@ -249,8 +261,9 @@ void put_terrain(struct tile *ptile,
 		 struct canvas *pcanvas, int canvas_x, int canvas_y);
 
 void put_unit_city_overlays(struct unit *punit,
-			    struct canvas *pcanvas,
-			    int canvas_x, int canvas_y);
+                            struct canvas *pcanvas,
+                            int canvas_x, int canvas_y, int *upkeep_cost,
+                            int happy_cost);
 void toggle_city_color(struct city *pcity);
 void toggle_unit_color(struct unit *punit);
 
@@ -264,6 +277,11 @@ void put_one_element(struct canvas *pcanvas, enum mapview_layer layer,
 		     int canvas_x, int canvas_y,
 		     const struct city *citymode);
 
+void put_drawn_sprites(struct canvas *pcanvas,
+                       int canvas_x, int canvas_y,
+                       int count, struct drawn_sprite *pdrawn,
+                       bool fog);
+
 void update_map_canvas(int canvas_x, int canvas_y, int width, int height);
 void update_map_canvas_visible(void);
 void update_city_description(struct city *pcity);
@@ -292,8 +310,11 @@ void get_city_mapview_name_and_growth(struct city *pcity,
 				      char *growth_buffer,
 				      size_t growth_buffer_len,
 				      enum color_std *grwoth_color);
+void get_city_mapview_trade_routes(struct city *pcity,
+                                   char *trade_routes_buffer,
+                                   size_t trade_routes_buffer_len,
+                                   enum color_std *trade_routes_color);
 
-void init_mapview_decorations(void);
 bool map_canvas_resized(int width, int height);
 void init_mapcanvas_and_overview(void);
 
@@ -301,4 +322,14 @@ void get_spaceship_dimensions(int *width, int *height);
 void put_spaceship(struct canvas *pcanvas, int canvas_x, int canvas_y,
 		   const struct player *pplayer);
 
+void link_marks_init(void);
+void link_marks_free(void);
+
+void link_marks_draw_all(void);
+void link_marks_clear_all(void);
+void link_marks_decrease_turn_counters(void);
+
+void link_mark_add_new(enum text_link_type type, int id);
+void link_mark_restore(enum text_link_type type, int id);
+
 #endif /* FC__MAPVIEW_COMMON_H */
