@@ -17,11 +17,17 @@
 
 #include <assert.h>
 
-#include "connection.h"
+/* utility */
 #include "fcintl.h"
-#include "game.h"
+#include "log.h"
 #include "support.h"
 
+/* common */
+#include "connection.h"
+#include "game.h"
+
+/* client */
+#include "client_main.h"
 #include "climisc.h"
 #include "text.h"
 
@@ -97,7 +103,7 @@ static const char *col_nation(const struct player *player)
 *******************************************************************/
 static const char *col_team(const struct player *player)
 {
-  return team_get_name(player->team);
+  return team_name_translation(player->team);
 }
 
 /******************************************************************
@@ -105,7 +111,7 @@ static const char *col_team(const struct player *player)
 *******************************************************************/
 static bool col_ai(const struct player *plr)
 {
-  return plr->ai.control;
+  return plr->ai_data.control;
 }
 
 /******************************************************************
@@ -114,7 +120,7 @@ static bool col_ai(const struct player *plr)
 *******************************************************************/
 static const char *col_embassy(const struct player *player)
 {
-  return get_embassy_status(game.player_ptr, player);
+  return get_embassy_status(client.conn.playing, player);
 }
 
 /******************************************************************
@@ -126,10 +132,10 @@ static const char *col_diplstate(const struct player *player)
   static char buf[100];
   const struct player_diplstate *pds;
 
-  if (!game.player_ptr || player == game.player_ptr) {
+  if (NULL == client.conn.playing || player == client.conn.playing) {
     return "-";
   } else {
-    pds = pplayer_get_diplstate(game.player_ptr, player);
+    pds = pplayer_get_diplstate(client.conn.playing, player);
     if (pds->type == DS_CEASEFIRE || pds->type == DS_ARMISTICE) {
       my_snprintf(buf, sizeof(buf), "%s (%d)",
 		  diplstate_text(pds->type), pds->turns_left);
@@ -145,10 +151,11 @@ static const char *col_diplstate(const struct player *player)
 *******************************************************************/
 static const char *col_love(const struct player *player)
 {
-  if (!game.player_ptr || player == game.player_ptr || !player->ai.control) {
+  if (NULL == client.conn.playing || player == client.conn.playing
+      || !player->ai_data.control) {
     return "-";
   } else {
-    return love_text(player->ai.love[player_index(game.player_ptr)]);
+    return love_text(player->ai_data.love[player_index(client.conn.playing)]);
   }
 }
 
@@ -156,24 +163,24 @@ static const char *col_love(const struct player *player)
   Compares ai's attitude toward the player
 ******************************************************************/
 static int cmp_love(const struct player *player1,
-                          const struct player *player2)
+                    const struct player *player2)
 {
   int love1, love2;
 
-  if (!game.player_ptr) {
+  if (NULL == client.conn.playing) {
     return player_number(player1) - player_number(player2);
   }
 
-  if (player1 == game.player_ptr || !player1->ai.control) {
+  if (player1 == client.conn.playing || !player1->ai_data.control) {
     love1 = MAX_AI_LOVE + 999;
   } else {
-    love1 = player1->ai.love[player_index(game.player_ptr)];
+    love1 = player1->ai_data.love[player_index(client.conn.playing)];
   }
 
-  if (player2 == game.player_ptr || !player2->ai.control) {
+  if (player2 == client.conn.playing || !player2->ai_data.control) {
     love2 = MAX_AI_LOVE + 999;
   } else {
-    love2 = player2->ai.love[player_index(game.player_ptr)];
+    love2 = player2->ai_data.love[player_index(client.conn.playing)];
   }
   
   return love1 - love2;
@@ -184,7 +191,7 @@ static int cmp_love(const struct player *player1,
 *******************************************************************/
 static const char *col_vision(const struct player *player)
 {
-  return get_vision_status(game.player_ptr, player);
+  return get_vision_status(client.conn.playing, player);
 }
 
 /******************************************************************
@@ -192,10 +199,11 @@ static const char *col_vision(const struct player *player)
 
   FIXME: These terms aren't very intuitive for new players.
 *******************************************************************/
-static const char *col_state(const struct player *plr)
+const char *plrdlg_col_state(const struct player *plr)
 {
   if (!plr->is_alive) {
-    return _("R.I.P");
+    /* TRANS: Dead -- Rest In Peace -- Reqia In Pace */
+    return _("R.I.P.");
   } else if (!plr->is_connected) {
     return "";
   } else if (!is_player_phase(plr, game.info.phase)) {
@@ -258,7 +266,7 @@ struct player_dlg_column player_dlg_columns[] = {
   {TRUE, COL_TEXT, N_("Embassy"), col_embassy, NULL, NULL,  "embassy"},
   {TRUE, COL_TEXT, N_("Dipl.State"), col_diplstate, NULL, NULL,  "diplstate"},
   {TRUE, COL_TEXT, N_("Vision"), col_vision, NULL, NULL,  "vision"},
-  {TRUE, COL_TEXT, N_("State"), col_state, NULL, NULL,  "state"},
+  {TRUE, COL_TEXT, N_("State"), plrdlg_col_state, NULL, NULL,  "state"},
   {FALSE, COL_TEXT, N_("?Player_dlg:Host"), col_host, NULL, NULL,  "host"},
   {FALSE, COL_RIGHT_TEXT, N_("?Player_dlg:Idle"), col_idle, NULL, NULL,  "idle"},
   {FALSE, COL_RIGHT_TEXT, N_("Ping"), get_ping_time_text, NULL, NULL,  "ping"}
