@@ -20,11 +20,13 @@
 #include <stdarg.h>
 #include <string.h>
 
-#include "game.h"
+/* common */
 #include "government.h"
 #include "improvement.h"
 #include "tech.h"
 
+/* client */
+#include "client_main.h"
 #include "tilespec.h"
 
 #include "colors_g.h"
@@ -164,7 +166,7 @@ static void node_rectangle_minimum_size(struct tree_node *node,
     *width = *height = 1;
   } else {
     get_text_size(width, height, FONT_REQTREE_TEXT,
-		  advance_name_for_player(game.player_ptr, node->tech));
+		  advance_name_for_player(client.conn.playing, node->tech));
     *width += 2;
     *height += 8;
     
@@ -174,7 +176,7 @@ static void node_rectangle_minimum_size(struct tree_node *node,
     if (reqtree_show_icons) {
       /* units */
       unit_type_iterate(unit) {
-        if (unit->tech_requirement != node->tech) {
+        if (advance_number(unit->require_advance) != node->tech) {
           continue;
         }
         sprite = get_unittype_sprite(tileset, unit);
@@ -184,12 +186,11 @@ static void node_rectangle_minimum_size(struct tree_node *node,
       } unit_type_iterate_end;
     
       /* buildings */
-      impr_type_iterate(impr_type) {
-        struct impr_type* impr = improvement_by_number(impr_type);
-        requirement_vector_iterate(&(impr->reqs), preq) {
-          if (preq->source.type == REQ_TECH
-	      && preq->source.value.tech == node->tech) {
-	    sprite = get_building_sprite(tileset, impr_type);
+      improvement_iterate(pimprove) {
+        requirement_vector_iterate(&(pimprove->reqs), preq) {
+          if (VUT_ADVANCE == preq->source.kind
+	   && advance_number(preq->source.value.advance) == node->tech) {
+	    sprite = get_building_sprite(tileset, pimprove);
             /* Improvement icons are not guaranteed to exist */
             if (sprite) {
               get_sprite_dimensions(sprite, &swidth, &sheight);
@@ -198,13 +199,13 @@ static void node_rectangle_minimum_size(struct tree_node *node,
             }
 	  }
         } requirement_vector_iterate_end;
-      } impr_type_iterate_end;
+      } improvement_iterate_end;
     
       /* governments */
       government_iterate(gov) {
         requirement_vector_iterate(&(gov->reqs), preq) {
-          if (preq->source.type == REQ_TECH
-	      && preq->source.value.tech == node->tech) {
+          if (VUT_ADVANCE == preq->source.kind
+	   && advance_number(preq->source.value.advance) == node->tech) {
             sprite = get_government_sprite(tileset, gov);
 	    get_sprite_dimensions(sprite, &swidth, &sheight);
             max_icon_height = MAX(max_icon_height, sheight);
@@ -384,16 +385,16 @@ static void calculate_diagram_layout(struct reqtree *tree)
 static struct reqtree *create_dummy_reqtree(struct player *pplayer)
 {
   struct reqtree *tree = fc_malloc(sizeof(*tree));
-  struct advance *advance;
   int j;
-  struct tree_node *nodes[game.control.num_tech_types];
+  struct tree_node *nodes[advance_count()];
 
-  tech_type_iterate(tech) {
-    if (!tech_exists(tech) || tech == A_NONE) {
+  nodes[A_NONE] = NULL;
+  advance_index_iterate(A_FIRST, tech) {
+    if (!valid_advance_by_number(tech)) {
       nodes[tech] = NULL;
       continue;
     }
-    if (pplayer && !tech_is_available(pplayer, tech)) {
+    if (pplayer && !player_invention_reachable(pplayer, tech)) {
       /* Reqtree requested for particular player and this tech is
        * unreachable to him/her. */
       nodes[tech] = NULL;
@@ -402,48 +403,42 @@ static struct reqtree *create_dummy_reqtree(struct player *pplayer)
     nodes[tech] = new_tree_node();
     nodes[tech]->is_dummy = FALSE;
     nodes[tech]->tech = tech;
-  } tech_type_iterate_end;
+  } advance_index_iterate_end;
 
-  tech_type_iterate(tech) {
-    if (!tech_exists(tech)) {
+  advance_index_iterate(A_FIRST, tech) {
+    struct advance *padvance = valid_advance_by_number(tech);
+
+    if (!padvance) {
       continue;
     }
     if (nodes[tech] == NULL) {
       continue;
     }
 
-    advance = &advances[tech];
-    /* Don't include redundant edges */
-    if (advance->req[0] != A_NONE && advance->req[1] != A_LAST) {
-      if ((advance->req[1] != A_NONE
-          && !is_tech_a_req_for_goal(game.player_ptr, advance->req[0],
-                                     advance->req[1]))
-         || advance->req[1] == A_NONE) {
-
-	add_requirement(nodes[tech], nodes[advance->req[0]]);
-      }
-
-      if (advance->req[1] != A_NONE) {
-	if (!is_tech_a_req_for_goal(game.player_ptr, advance->req[1],
-				    advance->req[0])) {
-	  add_requirement(nodes[tech], nodes[advance->req[1]]);
-	}
+    /* Formerly, we used to remove the redundant requirement nodes (the
+     * technologies already included in the requirements of the other
+     * requirement).  However, it doesn't look like a good idea, because
+     * a player can steal any technology independently of the technology
+     * tree. */
+    if (A_NONE != advance_required(tech, AR_ONE)
+        && A_LAST != advance_required(tech, AR_TWO)) {
+      add_requirement(nodes[tech], nodes[advance_required(tech, AR_ONE)]);
+      if (A_NONE != advance_required(tech, AR_TWO)) {
+        add_requirement(nodes[tech], nodes[advance_required(tech, AR_TWO)]);
       }
     }
-  } tech_type_iterate_end;
+  } advance_index_iterate_end;
 
   /* Copy nodes from local array to dynamically allocated one. 
    * Skip non-existing entries */
-  tree->nodes = fc_malloc(game.control.num_tech_types * sizeof(*tree->nodes));
+  tree->nodes = fc_calloc(advance_count(), sizeof(*tree->nodes));
   j = 0;
-  tech_type_iterate(tech) {
+  advance_index_iterate(A_FIRST, tech) {
     if (nodes[tech]) {
-      tree->nodes[j] = nodes[tech];
-      assert(tech_exists(tree->nodes[j]->tech));
-      j++;
+      assert(valid_advance_by_number(nodes[tech]->tech));
+      tree->nodes[j++] = nodes[tech];
     }
-  }
-  tech_type_iterate_end;
+  } advance_index_iterate_end;
   tree->num_nodes = j;
   tree->layers = NULL;
 
@@ -864,35 +859,41 @@ void get_reqtree_dimensions(struct reqtree *reqtree,
 static enum color_std node_color(struct tree_node *node)
 {
   if (!node->is_dummy) {
-    struct player_research* research = get_player_research(game.player_ptr);
+    struct player_research* research = get_player_research(client.conn.playing);
 
-    if (!game.player_ptr || !research) {
+    if (!research) {
       return COLOR_REQTREE_KNOWN;
     }
 
+    if (!player_invention_reachable(client.conn.playing, node->tech)) {
+      return COLOR_REQTREE_UNREACHABLE;
+    }
+
     if (research->researching == node->tech) {
       return COLOR_REQTREE_RESEARCHING;
     }
     
-    if (get_invention(game.player_ptr, node->tech) == TECH_KNOWN) {
+    if (TECH_KNOWN == player_invention_state(client.conn.playing, node->tech)) {
       return COLOR_REQTREE_KNOWN;
     }
 
-    if (is_tech_a_req_for_goal(game.player_ptr, node->tech,
-			       research->tech_goal)
+    if (is_tech_a_req_for_goal(client.conn.playing, node->tech,
+                               research->tech_goal)
 	|| node->tech == research->tech_goal) {
-      if (get_invention(game.player_ptr, node->tech) == TECH_REACHABLE) {
-	return COLOR_REQTREE_REACHABLE_GOAL;
+      if (TECH_PREREQS_KNOWN ==
+            player_invention_state(client.conn.playing, node->tech)) {
+	return COLOR_REQTREE_GOAL_PREREQS_KNOWN;
       } else {
-	return COLOR_REQTREE_UNREACHABLE_GOAL;
+	return COLOR_REQTREE_GOAL_UNKNOWN;
       }
     }
 
-    if (get_invention(game.player_ptr, node->tech) == TECH_REACHABLE) {
-      return COLOR_REQTREE_REACHABLE;
+    if (TECH_PREREQS_KNOWN ==
+          player_invention_state(client.conn.playing, node->tech)) {
+      return COLOR_REQTREE_PREREQS_KNOWN;
     }
 
-    return COLOR_REQTREE_UNREACHABLE;
+    return COLOR_REQTREE_UNKNOWN;
   } else {
     return COLOR_REQTREE_BACKGROUND;
   }
@@ -907,7 +908,7 @@ static enum color_std node_color(struct tree_node *node)
 static enum reqtree_edge_type get_edge_type(struct tree_node *node, 
                                             struct tree_node *dest_node)
 {
-  struct player_research* research;
+  struct player_research *research = get_player_research(client.conn.playing);
 
   if (dest_node == NULL) {
     /* assume node is a dummy */
@@ -945,24 +946,23 @@ static enum reqtree_edge_type get_edge_type(struct tree_node *node,
     return sum_type;
   }
 
-  research = get_player_research(game.player_ptr);
-
-  if (!game.player_ptr || !research) {
-    return REQTREE_KNOWN_EDGE; /* Global observer case */
+  if (!research) {
+    /* Global observer case */
+    return REQTREE_KNOWN_EDGE;
   }
   
   if (research->researching == dest_node->tech) {
     return REQTREE_ACTIVE_EDGE;
   }
 
-  if (is_tech_a_req_for_goal(game.player_ptr, dest_node->tech,
+  if (is_tech_a_req_for_goal(client.conn.playing, dest_node->tech,
                              research->tech_goal)
-      || research->tech_goal == dest_node->tech) {
+      || dest_node->tech == research->tech_goal) {
     return REQTREE_GOAL_EDGE;
   }
 
-  if (get_invention(game.player_ptr, node->tech) == TECH_KNOWN) {
-    if (get_invention(game.player_ptr, dest_node->tech) == TECH_KNOWN) {
+  if (TECH_KNOWN == player_invention_state(client.conn.playing, node->tech)) {
+    if (TECH_KNOWN == player_invention_state(client.conn.playing, dest_node->tech)) {
       return REQTREE_KNOWN_EDGE;
     } else {
       return REQTREE_READY_EDGE;
@@ -984,12 +984,12 @@ static enum color_std edge_color(struct tree_node *node,
   case REQTREE_ACTIVE_EDGE:
     return COLOR_REQTREE_RESEARCHING;
   case REQTREE_GOAL_EDGE:
-    return COLOR_REQTREE_UNREACHABLE_GOAL;
+    return COLOR_REQTREE_GOAL_UNKNOWN;
   case REQTREE_KNOWN_EDGE:
     /* using "text" black instead of "known" white/ground/green */
     return COLOR_REQTREE_TEXT;
   case REQTREE_READY_EDGE:
-    return COLOR_REQTREE_REACHABLE;
+    return COLOR_REQTREE_PREREQS_KNOWN;
   default:
     return COLOR_REQTREE_EDGE;
   };
@@ -1008,6 +1008,7 @@ void draw_reqtree(struct reqtree *tree, struct canvas *pcanvas,
   int i, j, k;
   int swidth, sheight;
   struct sprite* sprite;
+  struct color *color;
 
   /* draw the diagram */
   for (i = 0; i < tree->num_layers; i++) {
@@ -1027,7 +1028,7 @@ void draw_reqtree(struct reqtree *tree, struct canvas *pcanvas,
 		        LINE_GOTO,
 		        startx, starty, width, 0);
       } else {
-	const char *text = advance_name_for_player(game.player_ptr, node->tech);
+	const char *text = advance_name_for_player(client.conn.playing, node->tech);
 	int text_w, text_h;
 	int icon_startx;
 	
@@ -1056,7 +1057,7 @@ void draw_reqtree(struct reqtree *tree, struct canvas *pcanvas,
 	
 	if (reqtree_show_icons) {
  	  unit_type_iterate(unit) {
-            if (unit->tech_requirement != node->tech) {
+            if (advance_number(unit->require_advance) != node->tech) {
 	      continue;
 	    }
  	    sprite = get_unittype_sprite(tileset, unit);
@@ -1069,12 +1070,11 @@ void draw_reqtree(struct reqtree *tree, struct canvas *pcanvas,
  	    icon_startx += swidth + 2;
  	  } unit_type_iterate_end;
        
-          impr_type_iterate(impr_type) {
-	    struct impr_type* impr = improvement_by_number(impr_type);
-            requirement_vector_iterate(&(impr->reqs), preq) {
-              if (preq->source.type == REQ_TECH
-	          && preq->source.value.tech == node->tech) {
- 	        sprite = get_building_sprite(tileset, impr_type);
+          improvement_iterate(pimprove) {
+            requirement_vector_iterate(&(pimprove->reqs), preq) {
+              if (VUT_ADVANCE == preq->source.kind
+	       && advance_number(preq->source.value.advance) == node->tech) {
+ 	        sprite = get_building_sprite(tileset, pimprove);
                 /* Improvement icons are not guaranteed to exist */
                 if (sprite) {
                   get_sprite_dimensions(sprite, &swidth, &sheight);
@@ -1087,12 +1087,12 @@ void draw_reqtree(struct reqtree *tree, struct canvas *pcanvas,
                 }
  	      }
  	    } requirement_vector_iterate_end;
-          } impr_type_iterate_end;
+          } improvement_iterate_end;
 
           government_iterate(gov) {
             requirement_vector_iterate(&(gov->reqs), preq) {
-              if (preq->source.type == REQ_TECH
-                && preq->source.value.tech == node->tech) {
+              if (VUT_ADVANCE == preq->source.kind
+               && advance_number(preq->source.value.advance) == node->tech) {
                 sprite = get_government_sprite(tileset, gov);
                 get_sprite_dimensions(sprite, &swidth, &sheight);
  	        canvas_put_sprite_full(pcanvas,
@@ -1112,14 +1112,20 @@ void draw_reqtree(struct reqtree *tree, struct canvas *pcanvas,
       starty = node->node_y + node->node_height / 2;
       for (k = 0; k < node->nprovide; k++) {
 	struct tree_node *dest_node = node->provide[k];
+	color = get_color(tileset, edge_color(node, dest_node));
 
 	endx = dest_node->node_x;
 	endy = dest_node->node_y + dest_node->node_height / 2;
 	
-	canvas_put_line(pcanvas,
-		        get_color(tileset, edge_color(node, dest_node)),
-		        LINE_GOTO,
-		        startx, starty, endx - startx, endy - starty);
+        if (reqtree_curved_lines) {
+          canvas_put_curved_line(pcanvas, color, LINE_GOTO,
+                                 startx, starty, endx - startx,
+                                 endy - starty);
+        } else {
+          canvas_put_line(pcanvas, color, LINE_GOTO,
+                          startx, starty, endx - startx,
+                          endy - starty);
+        }
       }
     }
   }
