@@ -12,10 +12,13 @@
 ***********************************************************************/
 #ifndef FC__THEMES_COMMON_H
 #define FC__THEMES_COMMON_H
+
 #include "options.h"
 
 void init_themes(void);
-const char** get_themes_list(void);
+struct strvec;
+const struct strvec *get_themes_list(void);
 bool load_theme(const char* theme_name);
 void theme_reread_callback(struct client_option *option);
-#endif
+
+#endif /* FC__THEMES_COMMON_H */
