@@ -18,10 +18,16 @@
 #include <assert.h>
 #include <math.h>
 
+#include "rand.h"
+
+#include "base.h"
+#include "game.h"
 #include "log.h"
 #include "map.h"
+#include "movement.h"
 #include "packets.h"
 #include "unit.h"
+#include "unitlist.h"
 
 #include "combat.h"
 
@@ -32,12 +38,13 @@
   2) the tile contains a non-enemy city or
   3) the tile contains a non-enemy unit
 ***********************************************************************/
-bool can_player_attack_tile(struct player *pplayer, const struct tile *ptile)
+bool can_player_attack_tile(const struct player *pplayer,
+			    const struct tile *ptile)
 {
-  struct city *pcity = ptile->city;
+  struct city *pcity = tile_city(ptile);
   
   /* 1. Is there anyone there at all? */
-  if (!pcity && unit_list_size(&(ptile->units)) == 0) {
+  if (!pcity && unit_list_size((ptile->units)) == 0) {
     return FALSE;
   }
 
@@ -57,6 +64,18 @@ bool can_player_attack_tile(struct player *pplayer, const struct tile *ptile)
   return TRUE;
 }
 
+/***********************************************************************
+  Can unit attack other
+***********************************************************************/
+bool is_unit_reachable_by_unit(const struct unit *defender,
+                               const struct unit *attacker)
+{
+  struct unit_class *dclass = unit_class(defender);
+  struct unit_type *atype = unit_type(attacker);
+
+  return BV_ISSET(atype->targets, uclass_index(dclass));
+}
+
 /***********************************************************************
   Checks if a unit can physically attack pdefender at the tile
   (assuming it is adjacent and at war).
@@ -74,12 +93,11 @@ bool can_player_attack_tile(struct player *pplayer, const struct tile *ptile)
   2) Adjacency
   3) Diplomatic status
 ***********************************************************************/
-bool can_unit_attack_unit_at_tile(struct unit *punit, struct unit *pdefender,
+bool can_unit_attack_unit_at_tile(const struct unit *punit,
+				  const struct unit *pdefender,
                                   const struct tile *dest_tile)
 {
-  Terrain_type_id fromtile = punit->tile->terrain;
-  Terrain_type_id totile = dest_tile->terrain;
-  struct city *pcity = dest_tile->city;
+  struct city *pcity = tile_city(dest_tile);
 
   /* 1. Can we attack _anything_ ? */
   if (!is_military_unit(punit) || !is_attack_unit(punit)) {
@@ -87,25 +105,21 @@ bool can_unit_attack_unit_at_tile(struct unit *punit, struct unit *pdefender,
   }
 
   /* 2. Only fighters can attack planes, except in city or airbase attacks */
-  if (!unit_flag(punit, F_FIGHTER) && is_air_unit(pdefender)
-      && !(pcity || map_has_special(dest_tile, S_AIRBASE))) {
+  if (!is_unit_reachable_by_unit(pdefender, punit)
+      && !(pcity || tile_has_native_base(dest_tile, unit_type(pdefender)))) {
     return FALSE;
   }
 
   /* 3. Can't attack with ground unit from ocean, except for marines */
-  if (is_ocean(fromtile)
-      && is_ground_unit(punit)
-      && !unit_flag(punit, F_MARINES)) {
+  if (!is_native_tile(unit_type(punit), punit->tile)
+      && !can_attack_from_non_native(unit_type(punit))) {
     return FALSE;
   }
 
-  /* 4. Ground units cannot attack water units */
-  if (is_ocean(totile) && is_ground_unit(punit)) {
-    return FALSE;
-  }
-
-  /* 5. Shore bombardement can be done by certain units only */
-  if (unit_flag(punit, F_NO_LAND_ATTACK) && !is_ocean(totile)) {
+  /* 4. Most units can not attack non-native terrain.
+   *    Most ships can attack land tiles (shore bombardment) */
+  if (!is_native_tile(unit_type(punit), dest_tile)
+      && !can_attack_non_native(unit_type(punit))) {
     return FALSE;
   }
 
@@ -116,7 +130,7 @@ bool can_unit_attack_unit_at_tile(struct unit *punit, struct unit *pdefender,
   To attack a stack, unit must be able to attack every unit there (not
   including transported units).
 ************************************************************************/
-bool can_unit_attack_all_at_tile(struct unit *punit,
+bool can_unit_attack_all_at_tile(const struct unit *punit,
 				 const struct tile *ptile)
 {
   unit_list_iterate(ptile->units, aunit) {
@@ -138,7 +152,8 @@ bool can_unit_attack_all_at_tile(struct unit *punit,
   Is unit (1) diplomatically allowed to attack and (2) physically able
   to do so?
 ***********************************************************************/
-bool can_unit_attack_tile(struct unit *punit, const struct tile *dest_tile)
+bool can_unit_attack_tile(const struct unit *punit,
+			  const struct tile *dest_tile)
 {
   if (!can_player_attack_tile(unit_owner(punit), dest_tile)) {
     return FALSE;
@@ -236,21 +251,30 @@ double win_chance(int as, int ahp, int afp, int ds, int dhp, int dfp)
 /**************************************************************************
 A unit's effective firepower depend on the situation.
 **************************************************************************/
-void get_modified_firepower(struct unit *attacker, struct unit *defender,
+void get_modified_firepower(const struct unit *attacker,
+			    const struct unit *defender,
 			    int *att_fp, int *def_fp)
 {
+  struct city *pcity = tile_city(defender->tile);
+
   *att_fp = unit_type(attacker)->firepower;
   *def_fp = unit_type(defender)->firepower;
 
   /* Check CityBuster flag */
-  if (unit_flag(attacker, F_CITYBUSTER)
-      && map_get_city(defender->tile)) {
+  if (unit_has_type_flag(attacker, F_CITYBUSTER) && pcity) {
     *att_fp *= 2;
   }
 
+  if (unit_has_type_flag(attacker, F_BADWALLATTACKER)
+      && get_unittype_bonus(unit_owner(defender), defender->tile, unit_type(attacker),
+			    EFT_DEFEND_BONUS) > 0) {
+    *att_fp = 1;
+  }
+
   /* pearl harbour - defender's firepower is reduced to one, 
    *                 attacker's is multiplied by two         */
-  if (is_sailing_unit(defender) && map_get_city(defender->tile)) {
+  if (unit_has_type_flag(defender, F_BADCITYDEFENDER)
+      && tile_city(defender->tile)) {
     *att_fp *= 2;
     *def_fp = 1;
   }
@@ -259,13 +283,13 @@ void get_modified_firepower(struct unit *attacker, struct unit *defender,
    * When attacked by fighters, helicopters have their firepower
    * reduced to 1.
    */
-  if (is_heli_unit(defender) && unit_flag(attacker, F_FIGHTER)) {
+  if (unit_has_type_flag(defender, F_HELICOPTER) && unit_has_type_flag(attacker, F_FIGHTER)) {
     *def_fp = 1;
   }
 
   /* In land bombardment both units have their firepower reduced to 1 */
   if (is_sailing_unit(attacker)
-      && !is_ocean(map_get_terrain(defender->tile))
+      && !is_ocean_tile(defender->tile)
       && is_ground_unit(defender)) {
     *att_fp = 1;
     *def_fp = 1;
@@ -276,7 +300,8 @@ void get_modified_firepower(struct unit *attacker, struct unit *defender,
 Returns a double in the range [0;1] indicating the attackers chance of
 winning. The calculation takes all factors into account.
 **************************************************************************/
-double unit_win_chance(struct unit *attacker, struct unit *defender)
+double unit_win_chance(const struct unit *attacker,
+		       const struct unit *defender)
 {
   int def_power = get_total_defense_power(attacker, defender);
   int att_power = get_total_attack_power(attacker, defender);
@@ -293,41 +318,17 @@ double unit_win_chance(struct unit *attacker, struct unit *defender)
 }
 
 /**************************************************************************
-  a wrapper that returns whether or not a unit ignores citywalls
+  Try defending against nuclear attack, if succed return a city which 
+  had enough luck and EFT_NUKE_PROOF.
+  If the attack was succesful return NULL.
 **************************************************************************/
-static bool unit_ignores_citywalls(struct unit *punit)
-{
-  return (unit_flag(punit, F_IGWALL));
-}
-
-/**************************************************************************
-  Takes into account unit move_type as well, and Walls variant.
-**************************************************************************/
-bool unit_really_ignores_citywalls(struct unit *punit)
-{
-  return (unit_ignores_citywalls(punit)
-	  || is_air_unit(punit)
-	  || is_sailing_unit(punit));
-}
-
-/**************************************************************************
- a wrapper function returns 1 if the unit is on a square with fortress
-**************************************************************************/
-bool unit_on_fortress(struct unit *punit)
-{
-  return map_has_special(punit->tile, S_FORTRESS);
-}
-
-/**************************************************************************
-  a wrapper function returns 1 if there is a sdi-defense close to the square
-**************************************************************************/
-struct city *sdi_defense_close(struct player *owner,
+struct city *sdi_try_defend(const struct player *owner,
 			       const struct tile *ptile)
 {
   square_iterate(ptile, 2, ptile1) {
-    struct city *pcity = map_get_city(ptile1);
+    struct city *pcity = tile_city(ptile1);
     if (pcity && (!pplayers_allied(city_owner(pcity), owner))
-	&& get_city_bonus(pcity, EFT_NUKE_PROOF) > 0) {
+	&& myrand(100) < get_city_bonus(pcity, EFT_NUKE_PROOF)) {
       return pcity;
     }
   } square_iterate_end;
@@ -338,34 +339,35 @@ struct city *sdi_defense_close(struct player *owner,
 /**************************************************************************
  Convenience wrapper for base_get_attack_power.
 **************************************************************************/
-int get_attack_power(struct unit *punit)
+int get_attack_power(const struct unit *punit)
 {
-  return base_get_attack_power(punit->type, punit->veteran,
+  return base_get_attack_power(unit_type(punit), punit->veteran,
 			       punit->moves_left);
 }
 
 /**************************************************************************
  Returns the attack power, modified by moves left, and veteran
- status. Set moves_left to SINGLE_MOVE to disable the reduction of
- power caused by tired units.
+ status.
 **************************************************************************/
-int base_get_attack_power(Unit_Type_id type, int veteran, int moves_left)
+int base_get_attack_power(const struct unit_type *punittype,
+			  int veteran, int moves_left)
 {
   int power;
 
-  power = get_unit_type(type)->attack_strength * POWER_FACTOR;
-  power *= get_unit_type(type)->veteran[veteran].power_fact;
+  power = punittype->attack_strength * POWER_FACTOR;
+  power *= punittype->veteran[veteran].power_fact;
 
-  if (!unit_type_flag(type, F_IGTIRED) && moves_left < SINGLE_MOVE) {
+  if (game.info.tired_attack && moves_left < SINGLE_MOVE) {
     power = (power * moves_left) / SINGLE_MOVE;
   }
+
   return power;
 }
 
 /**************************************************************************
   Returns the defense power, modified by veteran status.
 **************************************************************************/
-int base_get_defense_power(struct unit *punit)
+int base_get_defense_power(const struct unit *punit)
 {
   return unit_type(punit)->defense_strength * POWER_FACTOR
   	* unit_type(punit)->veteran[punit->veteran].power_fact;
@@ -374,15 +376,17 @@ int base_get_defense_power(struct unit *punit)
 /**************************************************************************
   Returns the defense power, modified by terrain and veteran status.
 **************************************************************************/
-int get_defense_power(struct unit *punit)
+int get_defense_power(const struct unit *punit)
 {
   int db, power = base_get_defense_power(punit);
 
-  db = get_tile_type(punit->tile->terrain)->defense_bonus;
-  if (map_has_special(punit->tile, S_RIVER)) {
-    db += (db * terrain_control.river_defense_bonus) / 100;
+  if (uclass_has_flag(unit_class(punit), UCF_TERRAIN_DEFENSE)) {
+    db = 10 + tile_terrain(punit->tile)->defense_bonus / 10;
+    if (tile_has_special(punit->tile, S_RIVER)) {
+      db += (db * terrain_control.river_defense_bonus) / 100;
+    }
+    power = (power * db) / 10;
   }
-  power = (power * db) / 10;
 
   return power;
 }
@@ -391,7 +395,8 @@ int get_defense_power(struct unit *punit)
  return the modified attack power of a unit.  Currently they aren't any
  modifications...
 ***************************************************************************/
-int get_total_attack_power(struct unit *attacker, struct unit *defender)
+int get_total_attack_power(const struct unit *attacker,
+			   const struct unit *defender)
 {
   int attackpower = get_attack_power(attacker);
 
@@ -408,56 +413,49 @@ int get_total_attack_power(struct unit *attacker, struct unit *defender)
 May be called with a non-existing att_type to avoid any unit type
 effects.
 **************************************************************************/
-static int defense_multiplication(Unit_Type_id att_type,
-				  Unit_Type_id def_type,
+static int defense_multiplication(const struct unit_type *att_type,
+				  const struct unit_type *def_type,
+				  const struct player *def_player,
 				  const struct tile *ptile,
 				  int defensepower, bool fortified)
 {
-  struct city *pcity = map_get_city(ptile);
+  struct city *pcity = tile_city(ptile);
   int mod;
 
-  if (unit_type_exists(att_type)) {
-    if (unit_type_flag(def_type, F_PIKEMEN)
-	&& unit_type_flag(att_type, F_HORSE)) {
+  CHECK_UNIT_TYPE(def_type);
+
+  if (att_type) {
+    CHECK_UNIT_TYPE(att_type);
+
+    if (utype_has_flag(def_type, F_PIKEMEN)
+	&& utype_has_flag(att_type, F_HORSE)) {
       defensepower *= 2;
     }
 
-    if (unit_type_flag(def_type, F_AEGIS) &&
-	(is_air_unittype(att_type) || is_heli_unittype(att_type))) {
+    if (utype_has_flag(def_type, F_AEGIS)
+        && utype_has_flag(att_type, F_AIRUNIT)) {
       defensepower *= 5;
     }
-         
-    if (is_air_unittype(att_type) && pcity) {
-      if ((mod = get_city_bonus(pcity, EFT_AIR_DEFEND)) > 0) {
-	defensepower = defensepower * (100 + mod) / 100;
-      }
-      if ((mod = get_city_bonus(pcity, EFT_MISSILE_DEFEND)) > 0
-	  && unit_type_flag(att_type, F_MISSILE)) {
-	defensepower = defensepower * (100 + mod) / 100;
-      }
-    } else if (is_water_unit(att_type) && pcity) {
-      if ((mod = get_city_bonus(pcity, EFT_SEA_DEFEND)) > 0) {
-	defensepower = defensepower * (100 + mod) / 100;
-      }
-    }
-    if (!unit_type_flag(att_type, F_IGWALL)
-	&& (is_ground_unittype(att_type) || is_heli_unittype(att_type))
-        && pcity
-        && (mod = get_city_bonus(pcity, EFT_LAND_DEFEND)) > 0) {
-      defensepower = defensepower * (100 + mod) / 100;
+
+    if (!utype_has_flag(att_type, F_IGWALL)) {
+      /* This applies even if pcity is NULL. */
+      mod = 100 + get_unittype_bonus(def_player, ptile,
+				     att_type, EFT_DEFEND_BONUS);
+      defensepower = MAX(0, defensepower * mod / 100);
     }
 
-    if (unit_type_flag(att_type, F_FIGHTER) && is_heli_unittype(def_type)) {
+    if (utype_has_flag(att_type, F_FIGHTER) && utype_has_flag(def_type, F_HELICOPTER)) {
       defensepower /= 2;
     }
   }
 
-  if (map_has_special(ptile, S_FORTRESS) && !pcity) {
+  if (!pcity) {
     defensepower +=
-	(defensepower * terrain_control.fortress_defense_bonus) / 100;
+      defensepower * tile_bases_defense_bonus(ptile, def_type) / 100;
   }
 
-  if ((pcity || fortified) && is_ground_unittype(def_type)) {
+  if ((pcity || fortified)
+      && uclass_has_flag(utype_class(def_type), UCF_CAN_FORTIFY)) {
     defensepower = (defensepower * 3) / 2;
   }
 
@@ -468,27 +466,30 @@ static int defense_multiplication(Unit_Type_id att_type,
  May be called with a non-existing att_type to avoid any effects which
  depend on the attacker.
 **************************************************************************/
-int get_virtual_defense_power(Unit_Type_id att_type, Unit_Type_id def_type,
+int get_virtual_defense_power(const struct unit_type *att_type,
+			      const struct unit_type *def_type,
+			      const struct player *def_player,
 			      const struct tile *ptile,
 			      bool fortified, int veteran)
 {
-  int defensepower = unit_types[def_type].defense_strength;
-  Terrain_type_id t = map_get_terrain(ptile);
+  int defensepower = def_type->defense_strength;
   int db;
 
-  if (unit_types[def_type].move_type == LAND_MOVING && is_ocean(t)) {
+  if (utype_move_type(def_type) == LAND_MOVING
+      && is_ocean_tile(ptile)) {
     /* Ground units on ship doesn't defend. */
     return 0;
   }
 
-  db = get_tile_type(t)->defense_bonus;
-  if (map_has_special(ptile, S_RIVER)) {
+  db = 10 + tile_terrain(ptile)->defense_bonus / 10;
+  if (tile_has_special(ptile, S_RIVER)) {
     db += (db * terrain_control.river_defense_bonus) / 100;
   }
   defensepower *= db;
-  defensepower *= get_unit_type(def_type)->veteran[veteran].power_fact;
+  defensepower *= def_type->veteran[veteran].power_fact;
 
-  return defense_multiplication(att_type, def_type, ptile, defensepower,
+  return defense_multiplication(att_type, def_type, def_player,
+				ptile, defensepower,
 				fortified);
 }
 
@@ -497,9 +498,11 @@ int get_virtual_defense_power(Unit_Type_id att_type, Unit_Type_id def_type,
  An veteran aegis cruiser in a mountain city with SAM and SDI defense 
  being attacked by a missile gets defense 288.
 ***************************************************************************/
-int get_total_defense_power(struct unit *attacker, struct unit *defender)
+int get_total_defense_power(const struct unit *attacker,
+			    const struct unit *defender)
 {
-  return defense_multiplication(attacker->type, defender->type,
+  return defense_multiplication(unit_type(attacker), unit_type(defender),
+				unit_owner(defender),
 				defender->tile,
 				get_defense_power(defender),
 				defender->activity == ACTIVITY_FORTIFIED);
@@ -510,7 +513,8 @@ A number indicating the defense strength.
 Unlike the one got from win chance this doesn't potentially get insanely
 small if the units are unevenly matched, unlike win_chance.
 **************************************************************************/
-static int get_defense_rating(struct unit *attacker, struct unit *defender)
+static int get_defense_rating(const struct unit *attacker,
+			      const struct unit *defender)
 {
   int afp, dfp;
 
@@ -530,10 +534,11 @@ static int get_defense_rating(struct unit *attacker, struct unit *defender)
   relationship of attacker and defender is ignored; the caller should check
   this.
 **************************************************************************/
-struct unit *get_defender(struct unit *attacker, const struct tile *ptile)
+struct unit *get_defender(const struct unit *attacker,
+			  const struct tile *ptile)
 {
   struct unit *bestdef = NULL;
-  int bestvalue = -1, best_cost = 0, rating_of_best = 0;
+  int bestvalue = -99, best_cost = 0, rating_of_best = 0;
 
   /* Simply call win_chance with all the possible defenders in turn, and
    * take the best one.  It currently uses build cost as a tiebreaker in
@@ -549,7 +554,7 @@ struct unit *get_defender(struct unit *attacker, const struct tile *ptile)
      * complicated and is now handled elsewhere. */
     if (unit_can_defend_here(defender)) {
       bool change = FALSE;
-      int build_cost = unit_build_shield_cost(defender->type);
+      int build_cost = unit_build_shield_cost(defender);
       int defense_rating = get_defense_rating(attacker, defender);
       /* This will make units roughly evenly good defenders look alike. */
       int unit_def 
@@ -557,6 +562,12 @@ struct unit *get_defender(struct unit *attacker, const struct tile *ptile)
 
       assert(unit_def >= 0);
 
+      if (unit_has_type_flag(defender, F_GAMELOSS)
+          && !is_stack_vulnerable(defender->tile)) {
+        unit_def = -1; // then always use leader as last defender
+        // FIXME: multiple gameloss units with varying defense value not handled
+      }
+
       if (unit_def > bestvalue) {
 	change = TRUE;
       } else if (unit_def == bestvalue) {
@@ -578,14 +589,18 @@ struct unit *get_defender(struct unit *attacker, const struct tile *ptile)
     }
   } unit_list_iterate_end;
 
-  if (unit_list_size(&ptile->units) > 0 && !bestdef) {
-    struct unit *punit = unit_list_get(&ptile->units, 0);
+  if (unit_list_size(ptile->units) > 0 && !bestdef) {
+    struct unit *punit = unit_list_get(ptile->units, 0);
 
-    freelog(LOG_ERROR, "get_defender bug: %s's %s vs %s's %s (total %d"
-            " units) on %s at (%d,%d). ", unit_owner(attacker)->name,
-            unit_type(attacker)->name, unit_owner(punit)->name,
-            unit_type(punit)->name, unit_list_size(&ptile->units), 
-            get_terrain_name(ptile->terrain), ptile->x, ptile->y);
+    freelog(LOG_ERROR, "get_defender bug: %s %s vs %s %s (total %d"
+            " units) on \"%s\" at (%d,%d). ",
+            nation_rule_name(nation_of_unit(attacker)),
+            unit_rule_name(attacker),
+            nation_rule_name(nation_of_unit(punit)),
+            unit_rule_name(punit),
+            unit_list_size(ptile->units), 
+            terrain_rule_name(tile_terrain(ptile)),
+            TILE_XY(ptile));
   }
 
   return bestdef;
@@ -597,13 +612,14 @@ get unit at (x, y) that wants to kill defender.
 Works like get_defender; see comment there.
 This function is mostly used by the AI.
 **************************************************************************/
-struct unit *get_attacker(struct unit *defender, const struct tile *ptile)
+struct unit *get_attacker(const struct unit *defender,
+			  const struct tile *ptile)
 {
   struct unit *bestatt = 0;
   int bestvalue = -1, unit_a, best_cost = 0;
 
   unit_list_iterate(ptile->units, attacker) {
-    int build_cost = unit_build_shield_cost(attacker->type);
+    int build_cost = unit_build_shield_cost(attacker);
 
     if (pplayers_allied(unit_owner(defender), unit_owner(attacker))) {
       return NULL;
@@ -625,8 +641,7 @@ struct unit *get_attacker(struct unit *defender, const struct tile *ptile)
 **************************************************************************/
 bool is_stack_vulnerable(const struct tile *ptile)
 {
-  return !(ptile->city != NULL
-           || map_has_special(ptile, S_FORTRESS)
-           || map_has_special(ptile, S_AIRBASE)
-           || !game.rgame.killstack);
+  return (game.info.killstack
+          && !tile_has_base_flag(ptile, BF_NO_STACK_DEATH)
+          && NULL == tile_city(ptile));
 }
