@@ -56,16 +56,18 @@ const char blank_addr_str[] = "---.---.---.---";
    a connection list might corrupt the list. */
 int delayed_disconnect = 0;
 
-struct connection *current_connection;
-  
+
 /**************************************************************************
   Command access levels for client-side use; at present, they are only
   used to control access to server commands typed at the client chatline.
+  NB: These must match enum cmdlevel_id in common/connection.h.
 **************************************************************************/
 static const char *levelnames[] = {
   "none",
   "info",
+  "basic",
   "ctrl",
+  "admin",
   "hack"
 };
 
@@ -161,7 +163,7 @@ int read_socket_data(int sock, struct socket_packet_buffer *buffer)
   }
 
   freelog(LOG_DEBUG, "try reading %d bytes", buffer->nsize - buffer->ndata);
-  didget = my_readsocket(sock, (char *) (buffer->data + buffer->ndata),
+  didget = fc_readsocket(sock, (char *) (buffer->data + buffer->ndata),
 			 buffer->nsize - buffer->ndata);
 
   if (didget > 0) {
@@ -212,7 +214,7 @@ static int write_socket_data(struct connection *pc,
 
     tv.tv_sec = 0; tv.tv_usec = 0;
 
-    if (my_select(pc->sock+1, NULL, &writefs, &exceptfs, &tv) <= 0) {
+    if (fc_select(pc->sock+1, NULL, &writefs, &exceptfs, &tv) <= 0) {
       if (errno != EINTR) {
 	break;
       } else {
@@ -237,7 +239,7 @@ static int write_socket_data(struct connection *pc,
     if (FD_ISSET(pc->sock, &writefs)) {
       nblock=MIN(buf->ndata-start, MAX_LEN_PACKET);
       freelog(LOG_DEBUG,"trying to write %d limit=%d",nblock,limit);
-      if((nput=my_writesocket(pc->sock, 
+      if((nput=fc_writesocket(pc->sock, 
 			      (const char *)buf->data+start, nblock)) == -1) {
 #ifdef NONBLOCKING_SOCKETS
 	if (errno == EWOULDBLOCK || errno == EAGAIN) {
@@ -499,10 +501,10 @@ static void free_socket_packet_buffer(struct socket_packet_buffer *buf)
 /**************************************************************************
   Return pointer to static string containing a description for this
   connection, based on pconn->name, pconn->addr, and (if applicable)
-  pconn->player->name.  (Also pconn->established and pconn->observer.)
+  pconn->playing->name.  (Also pconn->established and pconn->observer.)
 
-  Note that if pconn is client's aconnection (connection to server),
-  pconn->name and pconn->addr contain empty string, and pconn->player
+  Note that when pconn is client.conn (connection to server),
+  pconn->name and pconn->addr contain empty string, and pconn->playing
   is NULL: in this case return string "server".
 **************************************************************************/
 const char *conn_description(const struct connection *pconn)
@@ -521,9 +523,9 @@ const char *conn_description(const struct connection *pconn)
     sz_strlcat(buffer, _(" (connection incomplete)"));
     return buffer;
   }
-  if (pconn->player) {
+  if (NULL != pconn->playing) {
     cat_snprintf(buffer, sizeof(buffer), _(" (player %s)"),
-		 player_name(pconn->player));
+		 player_name(pconn->playing));
   }
   if (pconn->observer) {
     sz_strlcat(buffer, _(" (observer)"));
@@ -531,6 +533,22 @@ const char *conn_description(const struct connection *pconn)
   return buffer;
 }
 
+/****************************************************************************
+  Return TRUE iff the connection is currently allowed to edit.
+****************************************************************************/
+bool can_conn_edit(const struct connection *pconn)
+{
+  return can_conn_enable_editing(pconn) && game.info.is_edit_mode;
+}
+
+/****************************************************************************
+  Return TRUE iff the connection is allowed to start editing.
+****************************************************************************/
+bool can_conn_enable_editing(const struct connection *pconn)
+{
+  return pconn->access_level == ALLOW_HACK;
+}
+
 /**************************************************************************
   Get next request id. Takes wrapping of the 16 bit wide unsigned int
   into account.
@@ -540,7 +558,7 @@ int get_next_request_id(int old_request_id)
   int result = old_request_id + 1;
 
   if ((result & 0xffff) == 0) {
-    freelog(LOG_NORMAL,
+    freelog(LOG_PACKET,
 	    "INFORMATION: request_id has wrapped around; "
 	    "setting from %d to 2", result);
     result = 2;
@@ -642,7 +660,7 @@ void connection_common_close(struct connection *pconn)
   if (!pconn->used) {
     freelog(LOG_ERROR, "WARNING: Trying to close already closed connection");
   } else {
-    my_closesocket(pconn->sock);
+    fc_closesocket(pconn->sock);
     pconn->used = FALSE;
     pconn->established = FALSE;
 
@@ -689,3 +707,45 @@ void conn_clear_packet_cache(struct connection *pc)
     }
   }
 }
+
+/**************************************************************************
+  Returns TRUE if the given connection is attached to a player which it
+  also controls (i.e. not a player observer).
+**************************************************************************/
+bool conn_controls_player(const struct connection *pconn)
+{
+  return pconn && pconn->playing && !pconn->observer;
+}
+
+/**************************************************************************
+  Returns TRUE if the given connection is a global observer.
+**************************************************************************/
+bool conn_is_global_observer(const struct connection *pconn)
+{
+  return pconn && !pconn->playing && pconn->observer;
+}
+
+/**************************************************************************
+  Returns the player that this connection is attached to, or NULL. Note
+  that this will return the observed player for connections that are
+  observing players.
+**************************************************************************/
+struct player *conn_get_player(const struct connection *pconn)
+{
+  if (!pconn) {
+    return NULL;
+  }
+  return pconn->playing;
+}
+
+/**************************************************************************
+  Returns the current access level of the given connection.
+  NB: If 'pconn' is NULL, this function will return ALLOW_NONE.
+**************************************************************************/
+enum cmdlevel_id conn_get_access(const struct connection *pconn)
+{
+  if (!pconn) {
+    return ALLOW_NONE; /* Would not want to give hack on error... */
+  }
+  return pconn->access_level;
+}
