@@ -21,16 +21,20 @@
 #include <assert.h>
 #include <limits.h>
 
+/* utility */
 #include "capability.h"
-#include "dataio.h"
-#include "events.h"
 #include "fcintl.h"
 #include "hash.h"
 #include "log.h"
-#include "map.h"
 #include "mem.h"
 #include "support.h"
 
+/* commmon */
+#include "dataio.h"
+#include "game.h"
+#include "events.h"
+#include "map.h"
+
 #include "packets.h"
 
 #ifdef USE_COMPRESSION
@@ -80,7 +84,7 @@
 ***********************************************************************/
 
 /**************************************************************************
-  It returns the request id of the outgoing packet (or 0 if pc->is_server).
+  It returns the request id of the outgoing packet (or 0 if is_server()).
 **************************************************************************/
 int send_packet_data(struct connection *pc, unsigned char *data, int len)
 {
@@ -90,9 +94,9 @@ int send_packet_data(struct connection *pc, unsigned char *data, int len)
   freelog(BASIC_PACKET_LOG_LEVEL, "sending packet type=%s(%d) len=%d",
 	  get_packet_name(data[2]), data[2], len);
 
-  if (!pc->is_server) {
+  if (!is_server()) {
     pc->client.last_request_id_used =
-	get_next_request_id(pc->client.last_request_id_used);
+        get_next_request_id(pc->client.last_request_id_used);
     result = pc->client.last_request_id_used;
     freelog(BASIC_PACKET_LOG_LEVEL, "sending request %d", result);
   }
@@ -273,11 +277,11 @@ int send_packet_data(struct connection *pc, unsigned char *data, int len)
 		packets_stats[i].size / packets_stats[i].counter,
 		get_packet_name(i),i);
       }
-      freelog(LOG_NORMAL,
+      freelog(LOG_TEST,
 	      "turn=%d; transmitted %d bytes in %d packets;average size "
 	      "per packet %d bytes", game.turn, sum, packet_counter,
 	      sum / packet_counter);
-      freelog(LOG_NORMAL, "turn=%d; transmitted %d bytes", game.turn,
+      freelog(LOG_TEST, "turn=%d; transmitted %d bytes", game.turn,
 	      pc->statistics.bytes_send);
     }    
     if (clear) {
@@ -473,19 +477,19 @@ void *get_packet_from_connection(struct connection *pc,
     if (packet_counter % 100 == 0) {
       int i, sum = 0;
 
-      freelog(LOG_NORMAL, "Received packets:");
+      freelog(LOG_TEST, "Received packets:");
       for (i = 0; i < PACKET_LAST; i++) {
 	if (packets_stats[i].counter == 0)
 	  continue;
 	sum += packets_stats[i].size;
-	freelog(LOG_NORMAL,
+	freelog(LOG_TEST,
 		"  [%-25.25s %3d]: %6d packets; %8d bytes total; "
 		"%5d bytes/packet average",
 		get_packet_name(i), i, packets_stats[i].counter,
 		packets_stats[i].size,
 		packets_stats[i].size / packets_stats[i].counter);
       }
-      freelog(LOG_NORMAL,
+      freelog(LOG_TEST,
 	      "received %d bytes in %d packets;average size "
 	      "per packet %d bytes",
 	      sum, packet_counter, sum / packet_counter);
@@ -721,8 +725,8 @@ void pre_send_packet_player_attribute_chunk(struct connection *pc,
 /**************************************************************************
   ...
 **************************************************************************/
-void post_receive_packet_game_state(struct connection *pc,
-				    struct packet_game_state *packet)
+void post_receive_packet_ruleset_control(struct connection *pc,
+                                         struct packet_ruleset_control *packet)
 {
   conn_clear_packet_cache(pc);
 }
@@ -730,8 +734,8 @@ void post_receive_packet_game_state(struct connection *pc,
 /**************************************************************************
   ...
 **************************************************************************/
-void post_send_packet_game_state(struct connection *pc,
-				 const struct packet_game_state *packet)
+void post_send_packet_ruleset_control(struct connection *pc,
+                                      const struct packet_ruleset_control *packet)
 {
   conn_clear_packet_cache(pc);
 }
