@@ -65,10 +65,6 @@ struct packet_player_ready {
   bool is_ready;
 };
 
-struct packet_game_state {
-  int value;
-};
-
 struct packet_endgame_report {
   int nscores;
   int id[MAX_NUM_PLAYERS];
@@ -90,13 +86,16 @@ struct packet_endgame_report {
 struct packet_tile_info {
   int x;
   int y;
-  Terrain_type_id type;
-  int known;
-  bool special[S_LAST];
-  Resource_type_id resource;
-  int owner;
   Continent_id continent;
+  enum known_type known;
+  int owner;
+  int worked;
+  Terrain_type_id terrain;
+  Resource_type_id resource;
+  bool special[S_LAST];
+  bv_bases bases;
   char spec_sprite[MAX_LEN_NAME];
+  Nation_type_id nation_start;
 };
 
 struct packet_game_info {
@@ -105,29 +104,30 @@ struct packet_game_info {
   int skill_level;
   int aifill;
   bool is_new_game;
+  bool is_edit_mode;
   float seconds_to_phasedone;
   int timeout;
   int turn;
   int phase;
   int year;
-  int end_year;
-  bool simultaneous_phases;
+  int start_year;
+  bool year_0_hack;
+  int end_turn;
+  char positive_year_label[MAX_LEN_NAME];
+  char negative_year_label[MAX_LEN_NAME];
+  int phase_mode;
   int num_phases;
   int min_players;
   int max_players;
-  int nplayers;
-  int player_idx;
   int globalwarming;
   int heating;
   int warminglevel;
   int nuclearwinter;
   int cooling;
   int coolinglevel;
-  int cityfactor;
   int diplcost;
   int freecost;
   int conquercost;
-  int unhappysize;
   int angrycitizen;
   int techpenalty;
   int foodbox;
@@ -150,22 +150,29 @@ struct packet_game_info {
   int occupychance;
   bool autoattack;
   bool spacerace;
+  bool endspaceship;
   int aqueductloss;
   int killcitizen;
   int razechance;
   bool savepalace;
   bool natural_city_names;
+  bool migration;
+  int mgr_turninterval;
+  bool mgr_foodneeded;
+  int mgr_distance;
+  int mgr_nationchance;
+  int mgr_worldchance;
   bool turnblock;
   bool fixedlength;
   bool auto_ai_toggle;
   bool fogofwar;
   int borders;
-  int nbarbarians;
   bool happyborders;
   bool slow_invasions;
   int add_to_size_limit;
   int notradesize;
   int fulltradesize;
+  int trademindist;
   int allowed_city_names;
   Impr_type_id palace_building;
   Impr_type_id land_defend_building;
@@ -173,24 +180,34 @@ struct packet_game_info {
   int forced_science;
   int forced_luxury;
   int forced_gold;
-  int min_city_center_output[O_MAX];
+  bool vision_reveal_tiles;
+  int min_city_center_output[O_LAST];
   int min_dist_bw_cities;
   int init_vis_radius_sq;
-  int hut_overflight;
   bool pillage_select;
   int nuke_contamination;
+  int gold_upkeep_style;
   int granary_food_ini[MAX_GRANARY_INIS];
   int granary_num_inis;
   int granary_food_inc;
+  bool illness_on;
+  int illness_min_size;
+  int illness_base_factor;
+  int illness_trade_infection;
+  int illness_pollution_factor;
   int tech_cost_style;
   int tech_leakage;
-  int tech_cost_double_year;
   bool killstack;
+  bool tired_attack;
+  int border_city_radius_sq;
+  int border_size_effect;
+  bool calendar_skip_0;
+  int upgrade_veteran_loss;
   int autoupgrade_veteran_loss;
   int incite_improvement_factor;
   int incite_unit_factor;
   int incite_total_factor;
-  int government_when_anarchy_id;
+  int government_during_revolution_id;
   int revolution_length;
   int base_pollution;
   int happy_cost;
@@ -201,11 +218,12 @@ struct packet_game_info {
   int ransom_gold;
   int save_nturns;
   int save_compress_level;
+  int save_compress_type;
   char start_units[MAX_LEN_STARTUNIT];
   int num_teams;
   char team_names_orig[MAX_NUM_TEAMS][MAX_LEN_NAME];
   bool global_advances[A_LAST];
-  int great_wonders[B_LAST];
+  int great_wonder_owners[B_LAST];
 };
 
 struct packet_map_info {
@@ -231,16 +249,19 @@ struct packet_chat_msg_req {
   char message[MAX_LEN_MSG];
 };
 
+struct packet_connect_msg {
+  char message[MAX_LEN_MSG];
+};
+
 struct packet_city_remove {
   int city_id;
 };
 
 struct packet_city_info {
   int id;
-  int owner;
   int x;
   int y;
-  char name[MAX_LEN_NAME];
+  int owner;
   int size;
   int ppl_happy[5];
   int ppl_content[5];
@@ -248,52 +269,55 @@ struct packet_city_info {
   int ppl_angry[5];
   int specialists_size;
   int specialists[SP_MAX];
-  int surplus[O_MAX];
-  int waste[O_MAX];
-  int unhappy_penalty[O_MAX];
-  int prod[O_MAX];
-  int citizen_base[O_MAX];
-  int usage[O_MAX];
+  int surplus[O_LAST];
+  int waste[O_LAST];
+  int unhappy_penalty[O_LAST];
+  int prod[O_LAST];
+  int citizen_base[O_LAST];
+  int usage[O_LAST];
   int food_stock;
   int shield_stock;
-  int trade[NUM_TRADEROUTES];
-  int trade_value[NUM_TRADEROUTES];
+  int trade[NUM_TRADE_ROUTES];
+  int trade_value[NUM_TRADE_ROUTES];
   int pollution;
-  bool production_is_unit;
+  int illness;
+  int illness_trade;
+  int production_kind;
   int production_value;
+  int turn_founded;
   int turn_last_built;
-  int changed_from_id;
-  bool changed_from_is_unit;
+  float migration_score_unused;
+  int changed_from_kind;
+  int changed_from_value;
   int before_change_shields;
   int disbanded_shields;
   int caravan_shields;
   int last_turns_shield_surplus;
-  struct worklist worklist;
-  bv_imprs improvements;
-  enum city_tile_type city_map[CITY_MAP_SIZE * CITY_MAP_SIZE];
+  int airlift;
   bool did_buy;
   bool did_sell;
   bool was_happy;
-  bool airlift;
   bool diplomat_investigate;
-  bv_city_options city_options;
-  int turn_founded;
   bool walls;
+  struct worklist worklist;
+  bv_imprs improvements;
+  bv_city_options city_options;
+  char name[MAX_LEN_NAME];
 };
 
 struct packet_city_short_info {
   int id;
-  int owner;
   int x;
   int y;
-  char name[MAX_LEN_NAME];
+  int owner;
   int size;
+  int tile_trade;
+  bool occupied;
+  bool walls;
   bool happy;
   bool unhappy;
   bv_imprs improvements;
-  bool occupied;
-  bool walls;
-  int tile_trade;
+  char name[MAX_LEN_NAME];
 };
 
 struct packet_city_sell {
@@ -307,8 +331,8 @@ struct packet_city_buy {
 
 struct packet_city_change {
   int city_id;
-  int build_id;
-  bool is_build_id_unit_id;
+  int production_kind;
+  int production_value;
 };
 
 struct packet_city_worklist {
@@ -364,7 +388,7 @@ struct packet_city_sabotage_list {
 };
 
 struct packet_player_remove {
-  int player_id;
+  int playerno;
 };
 
 struct packet_player_info {
@@ -376,7 +400,7 @@ struct packet_player_info {
   bool was_created;
   int government;
   int target_government;
-  bool embassy[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
+  bool real_embassy[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
   int city_style;
   Nation_type_id nation;
   int team;
@@ -404,7 +428,7 @@ struct packet_player_info {
   unsigned int gives_shared_vision;
   char inventions[A_LAST+1];
   int love[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
-  int small_wonders[B_LAST];
+  int wonders[B_LAST];
 };
 
 struct packet_player_phase_done {
@@ -450,6 +474,7 @@ struct packet_unit_info {
   int x;
   int y;
   int homecity;
+  int upkeep[O_LAST];
   int veteran;
   bool ai;
   bool paradropped;
@@ -461,13 +486,12 @@ struct packet_unit_info {
   int hp;
   int fuel;
   int activity_count;
-  int unhappiness;
-  int upkeep[O_MAX];
   int occupy;
   int goto_dest_x;
   int goto_dest_y;
   enum unit_activity activity;
   enum tile_special_type activity_target;
+  Base_type_id activity_base;
   int battlegroup;
   bool has_orders;
   int orders_length;
@@ -477,6 +501,7 @@ struct packet_unit_info {
   enum unit_orders orders[MAX_LEN_ROUTE];
   enum direction8 orders_dirs[MAX_LEN_ROUTE];
   enum unit_activity orders_activities[MAX_LEN_ROUTE];
+  Base_type_id orders_bases[MAX_LEN_ROUTE];
 };
 
 struct packet_unit_short_info {
@@ -491,6 +516,7 @@ struct packet_unit_short_info {
   bool transported;
   int hp;
   int activity;
+  Base_type_id activity_base;
   int transported_by;
   int packet_use;
   int info_city_id;
@@ -548,6 +574,7 @@ struct packet_unit_orders {
   enum unit_orders orders[MAX_LEN_ROUTE];
   enum direction8 dir[MAX_LEN_ROUTE];
   enum unit_activity activity[MAX_LEN_ROUTE];
+  Base_type_id base[MAX_LEN_ROUTE];
   int dest_x;
   int dest_y;
 };
@@ -614,6 +641,7 @@ struct packet_unit_change_activity {
   int unit_id;
   enum unit_activity activity;
   enum tile_special_type activity_target;
+  Base_type_id activity_base;
 };
 
 struct packet_diplomacy_init_meeting_req {
@@ -712,6 +740,10 @@ struct packet_conn_pong {
   char __dummy;			/* to avoid malloc(0); */
 };
 
+struct packet_client_info {
+  enum gui_type gui;
+};
+
 struct packet_end_phase {
   char __dummy;			/* to avoid malloc(0); */
 };
@@ -780,7 +812,6 @@ struct packet_ruleset_unit {
   char sound_move_alt[MAX_LEN_NAME];
   char sound_fight[MAX_LEN_NAME];
   char sound_fight_alt[MAX_LEN_NAME];
-  int move_type;
   int unit_class_id;
   int build_cost;
   int pop_cost;
@@ -797,7 +828,7 @@ struct packet_ruleset_unit {
   int obsoleted_by;
   int fuel;
   int happy_cost;
-  int upkeep[O_MAX];
+  int upkeep[O_LAST];
   int paratroopers_range;
   int paratroopers_mr_req;
   int paratroopers_mr_sub;
@@ -805,6 +836,9 @@ struct packet_ruleset_unit {
   float power_fact[MAX_VET_LEVELS];
   int move_bonus[MAX_VET_LEVELS];
   int bombard_rate;
+  int city_size;
+  bv_unit_classes cargo;
+  bv_unit_classes targets;
   char helptext[MAX_LEN_PACKET];
   bv_flags flags;
   bv_roles roles;
@@ -813,7 +847,6 @@ struct packet_ruleset_unit {
 struct packet_ruleset_game {
   int default_specialist;
   int global_init_techs[MAX_NUM_TECH_LIST];
-  int trireme_loss_chance[MAX_VET_LEVELS];
   int work_veteran_chance[MAX_VET_LEVELS];
   int veteran_chance[MAX_VET_LEVELS];
 };
@@ -865,15 +898,15 @@ struct packet_ruleset_terrain_control {
   bool may_transform;
   int ocean_reclaim_requirement_pct;
   int land_channel_requirement_pct;
+  int lake_max_size;
   enum special_river_move river_move_mode;
   int river_defense_bonus;
   int river_trade_incr;
   char river_help_text[MAX_LEN_PACKET];
-  int fortress_defense_bonus;
   int road_superhighway_trade_bonus;
-  int rail_tile_bonus[O_MAX];
-  int pollution_tile_penalty[O_MAX];
-  int fallout_tile_penalty[O_MAX];
+  int rail_tile_bonus[O_LAST];
+  int pollution_tile_penalty[O_LAST];
+  int fallout_tile_penalty[O_LAST];
 };
 
 struct packet_ruleset_nation_groups {
@@ -898,7 +931,7 @@ struct packet_ruleset_nation {
   bool leader_sex[MAX_NUM_LEADERS];
   bool is_available;
   bool is_playable;
-  bool is_barbarian;
+  int barbarian_type;
   int ngroups;
   int groups[MAX_NUM_NATION_GROUPS];
 };
@@ -912,6 +945,8 @@ struct packet_ruleset_city {
   struct requirement reqs[MAX_NUM_REQS];
   char graphic[MAX_LEN_NAME];
   char graphic_alt[MAX_LEN_NAME];
+  char oceanic_graphic[MAX_LEN_NAME];
+  char oceanic_graphic_alt[MAX_LEN_NAME];
   int replaced_by;
 };
 
@@ -937,12 +972,13 @@ struct packet_ruleset_building {
 struct packet_ruleset_terrain {
   Terrain_type_id id;
   bv_terrain_flags flags;
+  bv_unit_classes native_to;
   char name_orig[MAX_LEN_NAME];
   char graphic_str[MAX_LEN_NAME];
   char graphic_alt[MAX_LEN_NAME];
   int movement_cost;
   int defense_bonus;
-  int output[O_MAX];
+  int output[O_LAST];
   int num_resources;
   Resource_type_id resources[MAX_NUM_RESOURCES];
   int road_trade_incr;
@@ -956,17 +992,47 @@ struct packet_ruleset_terrain {
   Terrain_type_id transform_result;
   int transform_time;
   int rail_time;
-  int airbase_time;
-  int fortress_time;
   int clean_pollution_time;
   int clean_fallout_time;
   char helptext[MAX_LEN_PACKET];
 };
 
+struct packet_ruleset_unit_class {
+  int id;
+  char name[MAX_LEN_NAME];
+  int move_type;
+  int min_speed;
+  int hp_loss_pct;
+  int hut_behavior;
+  bv_unit_class_flags flags;
+};
+
+struct packet_ruleset_base {
+  int id;
+  char name[MAX_LEN_NAME];
+  bool buildable;
+  bool pillageable;
+  char graphic_str[MAX_LEN_NAME];
+  char graphic_alt[MAX_LEN_NAME];
+  char activity_gfx[MAX_LEN_NAME];
+  int reqs_count;
+  struct requirement reqs[MAX_NUM_REQS];
+  enum base_gui_type gui_type;
+  bv_unit_classes native_to;
+  int build_time;
+  int defense_bonus;
+  int border_sq;
+  int vision_main_sq;
+  int vision_invis_sq;
+  bv_base_flags flags;
+};
+
 struct packet_ruleset_control {
+  int num_unit_classes;
   int num_unit_types;
   int num_impr_types;
   int num_tech_types;
+  int num_base_types;
   int government_count;
   int nation_count;
   int styles_count;
@@ -974,6 +1040,8 @@ struct packet_ruleset_control {
   int resource_count;
   int num_specialist_types;
   char prefered_tileset[MAX_LEN_NAME];
+  char name[MAX_LEN_NAME];
+  char description[MAX_LEN_PACKET];
 };
 
 struct packet_single_want_hack_req {
@@ -1021,6 +1089,7 @@ struct packet_options_settable {
   char strval[MAX_LEN_PACKET];
   char default_strval[MAX_LEN_PACKET];
   int scategory;
+  bool initial_setting;
 };
 
 struct packet_ruleset_effect {
@@ -1031,7 +1100,7 @@ struct packet_ruleset_effect {
 struct packet_ruleset_effect_req {
   int effect_id;
   bool neg;
-  enum req_source_type source_type;
+  enum universals_n source_type;
   int source_value;
   enum req_range range;
   bool survives;
@@ -1041,11 +1110,257 @@ struct packet_ruleset_effect_req {
 struct packet_ruleset_resource {
   Resource_type_id id;
   char name_orig[MAX_LEN_NAME];
-  int output[O_MAX];
+  int output[O_LAST];
   char graphic_str[MAX_LEN_NAME];
   char graphic_alt[MAX_LEN_NAME];
 };
 
+struct packet_scenario_info {
+  bool is_scenario;
+  char name[256];
+  char description[MAX_LEN_PACKET];
+  bool players;
+};
+
+struct packet_save_scenario {
+  char name[MAX_LEN_NAME];
+};
+
+struct packet_vote_new {
+  int vote_no;
+  char user[MAX_LEN_NAME];
+  char desc[512];
+  int percent_required;
+  int flags;
+};
+
+struct packet_vote_update {
+  int vote_no;
+  int yes;
+  int no;
+  int abstain;
+  int num_voters;
+};
+
+struct packet_vote_remove {
+  int vote_no;
+};
+
+struct packet_vote_resolve {
+  int vote_no;
+  bool passed;
+};
+
+struct packet_vote_submit {
+  int vote_no;
+  int value;
+};
+
+struct packet_edit_mode {
+  bool state;
+};
+
+struct packet_edit_recalculate_borders {
+  char __dummy;			/* to avoid malloc(0); */
+};
+
+struct packet_edit_check_tiles {
+  char __dummy;			/* to avoid malloc(0); */
+};
+
+struct packet_edit_toggle_fogofwar {
+  int player;
+};
+
+struct packet_edit_tile_terrain {
+  int x;
+  int y;
+  Terrain_type_id terrain;
+  int size;
+};
+
+struct packet_edit_tile_resource {
+  int x;
+  int y;
+  Resource_type_id resource;
+  int size;
+};
+
+struct packet_edit_tile_special {
+  int x;
+  int y;
+  enum tile_special_type special;
+  bool remove;
+  int size;
+};
+
+struct packet_edit_tile_base {
+  int x;
+  int y;
+  Base_type_id base_type_id;
+  bool remove;
+  int size;
+};
+
+struct packet_edit_startpos {
+  int x;
+  int y;
+  Nation_type_id nation;
+};
+
+struct packet_edit_tile {
+  int id;
+  bv_special specials;
+  bv_bases bases;
+  Resource_type_id resource;
+  Terrain_type_id terrain;
+  Nation_type_id startpos_nation;
+};
+
+struct packet_edit_unit_create {
+  int owner;
+  int x;
+  int y;
+  Unit_type_id type;
+  int count;
+  int tag;
+};
+
+struct packet_edit_unit_remove {
+  int owner;
+  int x;
+  int y;
+  Unit_type_id type;
+  int count;
+};
+
+struct packet_edit_unit_remove_by_id {
+  int id;
+};
+
+struct packet_edit_unit {
+  int id;
+  Unit_type_id utype;
+  int owner;
+  int homecity;
+  int moves_left;
+  int hp;
+  int veteran;
+  int fuel;
+  enum unit_activity activity;
+  int activity_count;
+  Base_type_id activity_base;
+  bool debug;
+  bool moved;
+  bool paradropped;
+  bool done_moving;
+  int transported_by;
+};
+
+struct packet_edit_city_create {
+  int owner;
+  int x;
+  int y;
+  int size;
+  int tag;
+};
+
+struct packet_edit_city_remove {
+  int id;
+};
+
+struct packet_edit_city {
+  int id;
+  char name[MAX_LEN_NAME];
+  int owner;
+  int original;
+  int size;
+  int ppl_happy[5];
+  int ppl_content[5];
+  int ppl_unhappy[5];
+  int ppl_angry[5];
+  int specialists_size;
+  int specialists[SP_MAX];
+  int trade[NUM_TRADE_ROUTES];
+  int food_stock;
+  int shield_stock;
+  bool airlift;
+  bool debug;
+  bool did_buy;
+  bool did_sell;
+  bool was_happy;
+  int anarchy;
+  int rapture;
+  int steal;
+  int turn_founded;
+  int turn_last_built;
+  int built[B_LAST];
+  int production_kind;
+  int production_value;
+  int last_turns_shield_surplus;
+  bv_city_options city_options;
+};
+
+struct packet_edit_player_create {
+  int tag;
+};
+
+struct packet_edit_player_remove {
+  int id;
+};
+
+struct packet_edit_player {
+  int id;
+  char name[MAX_LEN_NAME];
+  char username[MAX_LEN_NAME];
+  char ranked_username[MAX_LEN_NAME];
+  int user_turns;
+  bool is_male;
+  int government;
+  int target_government;
+  Nation_type_id nation;
+  int team;
+  bool phase_done;
+  int nturns_idle;
+  bool is_alive;
+  bool surrendered;
+  int revolution_finishes;
+  bool capital;
+  bv_player embassy;
+  struct player_diplstate diplstates[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
+  int city_style;
+  int gold;
+  int tax;
+  int science;
+  int luxury;
+  int future_tech;
+  int researching;
+  int bulbs_researched;
+  bool inventions[A_LAST+1];
+  bool ai;
+};
+
+struct packet_edit_player_vision {
+  int player;
+  int x;
+  int y;
+  bool known;
+  int size;
+};
+
+struct packet_edit_game {
+  int year;
+  bool scenario;
+  char scenario_name[256];
+  char scenario_desc[MAX_LEN_PACKET];
+  bool scenario_players;
+};
+
+struct packet_edit_object_created {
+  int tag;
+  int id;
+};
+
 enum packet_type {
   PACKET_PROCESSING_STARTED,             /* 0 */
   PACKET_PROCESSING_FINISHED,
@@ -1057,8 +1372,7 @@ enum packet_type {
   PACKET_AUTHENTICATION_REPLY,
   PACKET_SERVER_SHUTDOWN,
   PACKET_NATION_SELECT_REQ = 10,         /* 10 */
-  PACKET_GAME_STATE = 12,
-  PACKET_ENDGAME_REPORT,
+  PACKET_ENDGAME_REPORT = 13,
   PACKET_TILE_INFO,
   PACKET_GAME_INFO,
   PACKET_MAP_INFO,
@@ -1157,6 +1471,8 @@ enum packet_type {
   PACKET_PLAYER_READY,
   PACKET_UNIT_BATTLEGROUP,
   PACKET_RULESET_NATION_GROUPS,
+  PACKET_RULESET_UNIT_CLASS,
+  PACKET_RULESET_BASE,                   /* 120 */
   PACKET_RULESET_EFFECT = 122,
   PACKET_RULESET_EFFECT_REQ,
   PACKET_RULESET_RESOURCE,
@@ -1164,6 +1480,38 @@ enum packet_type {
   PACKET_THAW_CLIENT,
   PACKET_BEGIN_TURN,
   PACKET_END_TURN,
+  PACKET_CLIENT_INFO,
+  PACKET_SCENARIO_INFO,                  /* 140 */
+  PACKET_SAVE_SCENARIO,
+  PACKET_VOTE_NEW = 145,
+  PACKET_VOTE_UPDATE,
+  PACKET_VOTE_REMOVE,
+  PACKET_VOTE_RESOLVE,
+  PACKET_VOTE_SUBMIT,
+  PACKET_EDIT_MODE,                      /* 150 */
+  PACKET_EDIT_RECALCULATE_BORDERS,
+  PACKET_EDIT_CHECK_TILES,
+  PACKET_EDIT_TOGGLE_FOGOFWAR,
+  PACKET_EDIT_TILE_TERRAIN,
+  PACKET_EDIT_TILE_RESOURCE,
+  PACKET_EDIT_TILE_SPECIAL,
+  PACKET_EDIT_TILE_BASE,
+  PACKET_EDIT_STARTPOS = 159,
+  PACKET_EDIT_TILE,                      /* 160 */
+  PACKET_EDIT_UNIT_CREATE,
+  PACKET_EDIT_UNIT_REMOVE,
+  PACKET_EDIT_UNIT_REMOVE_BY_ID,
+  PACKET_EDIT_UNIT,
+  PACKET_EDIT_CITY_CREATE,
+  PACKET_EDIT_CITY_REMOVE,
+  PACKET_EDIT_CITY,
+  PACKET_EDIT_PLAYER_CREATE,
+  PACKET_EDIT_PLAYER_REMOVE,
+  PACKET_EDIT_PLAYER,                    /* 170 */
+  PACKET_EDIT_PLAYER_VISION,
+  PACKET_EDIT_GAME = 180,                /* 180 */
+  PACKET_EDIT_OBJECT_CREATED,
+  PACKET_CONNECT_MSG,
 
   PACKET_LAST  /* leave this last */
 };
@@ -1208,19 +1556,13 @@ struct packet_player_ready *receive_packet_player_ready(struct connection *pc, e
 int send_packet_player_ready(struct connection *pc, const struct packet_player_ready *packet);
 int dsend_packet_player_ready(struct connection *pc, int player_no, bool is_ready);
 
-struct packet_game_state *receive_packet_game_state(struct connection *pc, enum packet_type type);
-int send_packet_game_state(struct connection *pc, const struct packet_game_state *packet);
-void lsend_packet_game_state(struct conn_list *dest, const struct packet_game_state *packet);
-int dsend_packet_game_state(struct connection *pc, int value);
-void dlsend_packet_game_state(struct conn_list *dest, int value);
-
 struct packet_endgame_report *receive_packet_endgame_report(struct connection *pc, enum packet_type type);
 int send_packet_endgame_report(struct connection *pc, const struct packet_endgame_report *packet);
 void lsend_packet_endgame_report(struct conn_list *dest, const struct packet_endgame_report *packet);
 
 struct packet_tile_info *receive_packet_tile_info(struct connection *pc, enum packet_type type);
-int send_packet_tile_info(struct connection *pc, const struct packet_tile_info *packet);
-void lsend_packet_tile_info(struct conn_list *dest, const struct packet_tile_info *packet);
+int send_packet_tile_info(struct connection *pc, bool force_send, const struct packet_tile_info *packet);
+void lsend_packet_tile_info(struct conn_list *dest, bool force_send, const struct packet_tile_info *packet);
 
 struct packet_game_info *receive_packet_game_info(struct connection *pc, enum packet_type type);
 int send_packet_game_info(struct connection *pc, const struct packet_game_info *packet);
@@ -1238,13 +1580,15 @@ void dlsend_packet_nuke_tile_info(struct conn_list *dest, int x, int y);
 struct packet_chat_msg *receive_packet_chat_msg(struct connection *pc, enum packet_type type);
 int send_packet_chat_msg(struct connection *pc, const struct packet_chat_msg *packet);
 void lsend_packet_chat_msg(struct conn_list *dest, const struct packet_chat_msg *packet);
-int dsend_packet_chat_msg(struct connection *pc, const char *message, int x, int y, enum event_type event, int conn_id);
-void dlsend_packet_chat_msg(struct conn_list *dest, const char *message, int x, int y, enum event_type event, int conn_id);
 
 struct packet_chat_msg_req *receive_packet_chat_msg_req(struct connection *pc, enum packet_type type);
 int send_packet_chat_msg_req(struct connection *pc, const struct packet_chat_msg_req *packet);
 int dsend_packet_chat_msg_req(struct connection *pc, const char *message);
 
+struct packet_connect_msg *receive_packet_connect_msg(struct connection *pc, enum packet_type type);
+int send_packet_connect_msg(struct connection *pc, const struct packet_connect_msg *packet);
+int dsend_packet_connect_msg(struct connection *pc, const char *message);
+
 struct packet_city_remove *receive_packet_city_remove(struct connection *pc, enum packet_type type);
 int send_packet_city_remove(struct connection *pc, const struct packet_city_remove *packet);
 void lsend_packet_city_remove(struct conn_list *dest, const struct packet_city_remove *packet);
@@ -1269,11 +1613,11 @@ int dsend_packet_city_buy(struct connection *pc, int city_id);
 
 struct packet_city_change *receive_packet_city_change(struct connection *pc, enum packet_type type);
 int send_packet_city_change(struct connection *pc, const struct packet_city_change *packet);
-int dsend_packet_city_change(struct connection *pc, int city_id, int build_id, bool is_build_id_unit_id);
+int dsend_packet_city_change(struct connection *pc, int city_id, int production_kind, int production_value);
 
 struct packet_city_worklist *receive_packet_city_worklist(struct connection *pc, enum packet_type type);
 int send_packet_city_worklist(struct connection *pc, const struct packet_city_worklist *packet);
-int dsend_packet_city_worklist(struct connection *pc, int city_id, struct worklist *worklist);
+int dsend_packet_city_worklist(struct connection *pc, int city_id, const struct worklist *worklist);
 
 struct packet_city_make_specialist *receive_packet_city_make_specialist(struct connection *pc, enum packet_type type);
 int send_packet_city_make_specialist(struct connection *pc, const struct packet_city_make_specialist *packet);
@@ -1315,9 +1659,7 @@ void lsend_packet_city_sabotage_list(struct conn_list *dest, const struct packet
 
 struct packet_player_remove *receive_packet_player_remove(struct connection *pc, enum packet_type type);
 int send_packet_player_remove(struct connection *pc, const struct packet_player_remove *packet);
-void lsend_packet_player_remove(struct conn_list *dest, const struct packet_player_remove *packet);
-int dsend_packet_player_remove(struct connection *pc, int player_id);
-void dlsend_packet_player_remove(struct conn_list *dest, int player_id);
+int dsend_packet_player_remove(struct connection *pc, int playerno);
 
 struct packet_player_info *receive_packet_player_info(struct connection *pc, enum packet_type type);
 int send_packet_player_info(struct connection *pc, const struct packet_player_info *packet);
@@ -1445,7 +1787,7 @@ void dlsend_packet_unit_diplomat_answer(struct conn_list *dest, int diplomat_id,
 
 struct packet_unit_change_activity *receive_packet_unit_change_activity(struct connection *pc, enum packet_type type);
 int send_packet_unit_change_activity(struct connection *pc, const struct packet_unit_change_activity *packet);
-int dsend_packet_unit_change_activity(struct connection *pc, int unit_id, enum unit_activity activity, enum tile_special_type activity_target);
+int dsend_packet_unit_change_activity(struct connection *pc, int unit_id, enum unit_activity activity, enum tile_special_type activity_target, Base_type_id activity_base);
 
 struct packet_diplomacy_init_meeting_req *receive_packet_diplomacy_init_meeting_req(struct connection *pc, enum packet_type type);
 int send_packet_diplomacy_init_meeting_req(struct connection *pc, const struct packet_diplomacy_init_meeting_req *packet);
@@ -1523,6 +1865,9 @@ int send_packet_conn_ping(struct connection *pc);
 struct packet_conn_pong *receive_packet_conn_pong(struct connection *pc, enum packet_type type);
 int send_packet_conn_pong(struct connection *pc);
 
+struct packet_client_info *receive_packet_client_info(struct connection *pc, enum packet_type type);
+int send_packet_client_info(struct connection *pc, const struct packet_client_info *packet);
+
 struct packet_end_phase *receive_packet_end_phase(struct connection *pc, enum packet_type type);
 int send_packet_end_phase(struct connection *pc);
 void lsend_packet_end_phase(struct conn_list *dest);
@@ -1612,6 +1957,14 @@ struct packet_ruleset_terrain *receive_packet_ruleset_terrain(struct connection
 int send_packet_ruleset_terrain(struct connection *pc, const struct packet_ruleset_terrain *packet);
 void lsend_packet_ruleset_terrain(struct conn_list *dest, const struct packet_ruleset_terrain *packet);
 
+struct packet_ruleset_unit_class *receive_packet_ruleset_unit_class(struct connection *pc, enum packet_type type);
+int send_packet_ruleset_unit_class(struct connection *pc, const struct packet_ruleset_unit_class *packet);
+void lsend_packet_ruleset_unit_class(struct conn_list *dest, const struct packet_ruleset_unit_class *packet);
+
+struct packet_ruleset_base *receive_packet_ruleset_base(struct connection *pc, enum packet_type type);
+int send_packet_ruleset_base(struct connection *pc, const struct packet_ruleset_base *packet);
+void lsend_packet_ruleset_base(struct conn_list *dest, const struct packet_ruleset_base *packet);
+
 struct packet_ruleset_control *receive_packet_ruleset_control(struct connection *pc, enum packet_type type);
 int send_packet_ruleset_control(struct connection *pc, const struct packet_ruleset_control *packet);
 void lsend_packet_ruleset_control(struct conn_list *dest, const struct packet_ruleset_control *packet);
@@ -1650,6 +2003,114 @@ struct packet_ruleset_resource *receive_packet_ruleset_resource(struct connectio
 int send_packet_ruleset_resource(struct connection *pc, const struct packet_ruleset_resource *packet);
 void lsend_packet_ruleset_resource(struct conn_list *dest, const struct packet_ruleset_resource *packet);
 
+struct packet_scenario_info *receive_packet_scenario_info(struct connection *pc, enum packet_type type);
+int send_packet_scenario_info(struct connection *pc, const struct packet_scenario_info *packet);
+
+struct packet_save_scenario *receive_packet_save_scenario(struct connection *pc, enum packet_type type);
+int send_packet_save_scenario(struct connection *pc, const struct packet_save_scenario *packet);
+int dsend_packet_save_scenario(struct connection *pc, const char *name);
+
+struct packet_vote_new *receive_packet_vote_new(struct connection *pc, enum packet_type type);
+int send_packet_vote_new(struct connection *pc, const struct packet_vote_new *packet);
+
+struct packet_vote_update *receive_packet_vote_update(struct connection *pc, enum packet_type type);
+int send_packet_vote_update(struct connection *pc, const struct packet_vote_update *packet);
+
+struct packet_vote_remove *receive_packet_vote_remove(struct connection *pc, enum packet_type type);
+int send_packet_vote_remove(struct connection *pc, const struct packet_vote_remove *packet);
+
+struct packet_vote_resolve *receive_packet_vote_resolve(struct connection *pc, enum packet_type type);
+int send_packet_vote_resolve(struct connection *pc, const struct packet_vote_resolve *packet);
+
+struct packet_vote_submit *receive_packet_vote_submit(struct connection *pc, enum packet_type type);
+int send_packet_vote_submit(struct connection *pc, const struct packet_vote_submit *packet);
+
+struct packet_edit_mode *receive_packet_edit_mode(struct connection *pc, enum packet_type type);
+int send_packet_edit_mode(struct connection *pc, const struct packet_edit_mode *packet);
+int dsend_packet_edit_mode(struct connection *pc, bool state);
+
+struct packet_edit_recalculate_borders *receive_packet_edit_recalculate_borders(struct connection *pc, enum packet_type type);
+int send_packet_edit_recalculate_borders(struct connection *pc);
+
+struct packet_edit_check_tiles *receive_packet_edit_check_tiles(struct connection *pc, enum packet_type type);
+int send_packet_edit_check_tiles(struct connection *pc);
+
+struct packet_edit_toggle_fogofwar *receive_packet_edit_toggle_fogofwar(struct connection *pc, enum packet_type type);
+int send_packet_edit_toggle_fogofwar(struct connection *pc, const struct packet_edit_toggle_fogofwar *packet);
+int dsend_packet_edit_toggle_fogofwar(struct connection *pc, int player);
+
+struct packet_edit_tile_terrain *receive_packet_edit_tile_terrain(struct connection *pc, enum packet_type type);
+int send_packet_edit_tile_terrain(struct connection *pc, const struct packet_edit_tile_terrain *packet);
+int dsend_packet_edit_tile_terrain(struct connection *pc, int x, int y, Terrain_type_id terrain, int size);
+
+struct packet_edit_tile_resource *receive_packet_edit_tile_resource(struct connection *pc, enum packet_type type);
+int send_packet_edit_tile_resource(struct connection *pc, const struct packet_edit_tile_resource *packet);
+int dsend_packet_edit_tile_resource(struct connection *pc, int x, int y, Resource_type_id resource, int size);
+
+struct packet_edit_tile_special *receive_packet_edit_tile_special(struct connection *pc, enum packet_type type);
+int send_packet_edit_tile_special(struct connection *pc, const struct packet_edit_tile_special *packet);
+int dsend_packet_edit_tile_special(struct connection *pc, int x, int y, enum tile_special_type special, bool remove, int size);
+
+struct packet_edit_tile_base *receive_packet_edit_tile_base(struct connection *pc, enum packet_type type);
+int send_packet_edit_tile_base(struct connection *pc, const struct packet_edit_tile_base *packet);
+int dsend_packet_edit_tile_base(struct connection *pc, int x, int y, Base_type_id base_type_id, bool remove, int size);
+
+struct packet_edit_startpos *receive_packet_edit_startpos(struct connection *pc, enum packet_type type);
+int send_packet_edit_startpos(struct connection *pc, const struct packet_edit_startpos *packet);
+int dsend_packet_edit_startpos(struct connection *pc, int x, int y, Nation_type_id nation);
+
+struct packet_edit_tile *receive_packet_edit_tile(struct connection *pc, enum packet_type type);
+int send_packet_edit_tile(struct connection *pc, const struct packet_edit_tile *packet);
+
+struct packet_edit_unit_create *receive_packet_edit_unit_create(struct connection *pc, enum packet_type type);
+int send_packet_edit_unit_create(struct connection *pc, const struct packet_edit_unit_create *packet);
+int dsend_packet_edit_unit_create(struct connection *pc, int owner, int x, int y, Unit_type_id type, int count, int tag);
+
+struct packet_edit_unit_remove *receive_packet_edit_unit_remove(struct connection *pc, enum packet_type type);
+int send_packet_edit_unit_remove(struct connection *pc, const struct packet_edit_unit_remove *packet);
+int dsend_packet_edit_unit_remove(struct connection *pc, int owner, int x, int y, Unit_type_id type, int count);
+
+struct packet_edit_unit_remove_by_id *receive_packet_edit_unit_remove_by_id(struct connection *pc, enum packet_type type);
+int send_packet_edit_unit_remove_by_id(struct connection *pc, const struct packet_edit_unit_remove_by_id *packet);
+int dsend_packet_edit_unit_remove_by_id(struct connection *pc, int id);
+
+struct packet_edit_unit *receive_packet_edit_unit(struct connection *pc, enum packet_type type);
+int send_packet_edit_unit(struct connection *pc, const struct packet_edit_unit *packet);
+
+struct packet_edit_city_create *receive_packet_edit_city_create(struct connection *pc, enum packet_type type);
+int send_packet_edit_city_create(struct connection *pc, const struct packet_edit_city_create *packet);
+int dsend_packet_edit_city_create(struct connection *pc, int owner, int x, int y, int size, int tag);
+
+struct packet_edit_city_remove *receive_packet_edit_city_remove(struct connection *pc, enum packet_type type);
+int send_packet_edit_city_remove(struct connection *pc, const struct packet_edit_city_remove *packet);
+int dsend_packet_edit_city_remove(struct connection *pc, int id);
+
+struct packet_edit_city *receive_packet_edit_city(struct connection *pc, enum packet_type type);
+int send_packet_edit_city(struct connection *pc, const struct packet_edit_city *packet);
+
+struct packet_edit_player_create *receive_packet_edit_player_create(struct connection *pc, enum packet_type type);
+int send_packet_edit_player_create(struct connection *pc, const struct packet_edit_player_create *packet);
+int dsend_packet_edit_player_create(struct connection *pc, int tag);
+
+struct packet_edit_player_remove *receive_packet_edit_player_remove(struct connection *pc, enum packet_type type);
+int send_packet_edit_player_remove(struct connection *pc, const struct packet_edit_player_remove *packet);
+int dsend_packet_edit_player_remove(struct connection *pc, int id);
+
+struct packet_edit_player *receive_packet_edit_player(struct connection *pc, enum packet_type type);
+int send_packet_edit_player(struct connection *pc, const struct packet_edit_player *packet);
+void lsend_packet_edit_player(struct conn_list *dest, const struct packet_edit_player *packet);
+
+struct packet_edit_player_vision *receive_packet_edit_player_vision(struct connection *pc, enum packet_type type);
+int send_packet_edit_player_vision(struct connection *pc, const struct packet_edit_player_vision *packet);
+int dsend_packet_edit_player_vision(struct connection *pc, int player, int x, int y, bool known, int size);
+
+struct packet_edit_game *receive_packet_edit_game(struct connection *pc, enum packet_type type);
+int send_packet_edit_game(struct connection *pc, const struct packet_edit_game *packet);
+
+struct packet_edit_object_created *receive_packet_edit_object_created(struct connection *pc, enum packet_type type);
+int send_packet_edit_object_created(struct connection *pc, const struct packet_edit_object_created *packet);
+int dsend_packet_edit_object_created(struct connection *pc, int tag, int id);
+
 
 void delta_stats_report(void);
 void delta_stats_reset(void);
