@@ -22,12 +22,20 @@ struct conn_list;
 struct cm_result;
 
 void city_refresh(struct city *pcity);          /* call if city has changed */
-void global_city_refresh(struct player *pplayer); /* tax/govt changed */
+void city_refresh_for_player(struct player *pplayer); /* tax/govt changed */
+
+void city_refresh_queue_add(struct city *pcity);
+void city_refresh_queue_processing(void);
 
 void auto_arrange_workers(struct city *pcity); /* will arrange the workers */
-void apply_cmresult_to_city(struct city *pcity, struct cm_result *cmr);
+void apply_cmresult_to_city(struct city *pcity,
+			    const struct cm_result *const cmr);
+
+bool city_change_size(struct city *pcity, int new_size);
+bool city_reduce_size(struct city *pcity, int pop_loss,
+                      struct player *destroyer);
+void city_repair_size(struct city *pcity, int change);
 
-bool city_reduce_size(struct city *pcity, int pop_loss);
 void send_global_city_turn_notifications(struct conn_list *dest);
 void send_city_turn_notifications(struct conn_list *dest, struct city *pcity);
 void update_city_activities(struct player *pplayer);
@@ -38,4 +46,7 @@ void remove_obsolete_buildings(struct player *pplayer);
 void advisor_choose_build(struct player *pplayer, struct city *pcity);
 
 void nullify_prechange_production(struct city *pcity);
+
+void check_city_migrations(void);
+
 #endif  /* FC__CITYTURN_H */
