@@ -23,11 +23,17 @@
 #include <readline/readline.h>
 #endif
 
+/* utility */
 #include "fciconv.h"
 #include "fcintl.h"
 #include "log.h"
 #include "support.h"
 
+/* common */
+#include "game.h"
+
+/* server */
+#include "notify.h"
 #include "srv_main.h"
 
 #include "console.h"
@@ -45,12 +51,30 @@ static int con_dump(enum rfc_status rfc_status, const char *message, ...);
 Function to handle log messages.
 This must match the log_callback_fn typedef signature.
 ************************************************************************/
-static void con_handle_log(int level, const char *message)
+static void con_handle_log(int level, const char *message, bool file_too)
 {
-  if(console_rfcstyle) {
-    con_write(C_LOG_BASE+level, "%s", message);
-  } else {
-    con_write(C_LOG_BASE+level, "%d: %s", level, message);
+  if (LOG_ERROR == level) {
+    notify_conn(NULL, NULL, E_LOG_ERROR, ftc_warning, "%s", message);
+  } else if (LOG_FATAL >= level) {
+    /* Make sure that message is not left to buffers when server dies */
+    conn_list_iterate(game.est_connections, pconn) {
+      pconn->send_buffer->do_buffer_sends = 0;
+      pconn->compression.frozen_level = 0;
+    } conn_list_iterate_end;
+
+    notify_conn(NULL, NULL, E_LOG_FATAL, ftc_warning, "%s", message);
+    notify_conn(NULL, NULL, E_LOG_FATAL, ftc_warning,
+                _("Please report this message at %s"),
+                BUG_URL);
+  }
+
+  /* Write debug/verbose message to console only when not written to file. */
+  if (!file_too || level <= LOG_NORMAL) {
+    if (console_rfcstyle) {
+      con_write(C_LOG_BASE + level, "%s", message);
+    } else {
+      con_write(C_LOG_BASE + level, "%d: %s", level, message);
+    }
   }
 }
 
@@ -81,9 +105,7 @@ static void con_update_prompt(void)
 ************************************************************************/
 void con_log_init(const char *log_filename, int log_level)
 {
-  bool has_file = (log_filename && strlen(log_filename) > 0);
-
-  log_init(log_filename, log_level, has_file ? NULL : con_handle_log);
+  log_init(log_filename, log_level, con_handle_log);
 }
 
 #ifndef HAVE_LIBREADLINE
@@ -99,7 +121,7 @@ static int con_dump(enum rfc_status rfc_status, const char *message, ...)
   my_vsnprintf(buf, sizeof(buf), message, args);
   va_end(args);
 
-  if(console_prompt_is_showing) {
+  if (console_prompt_is_showing) {
     fc_printf("\n");
   }
   if ((console_rfcstyle) && (rfc_status >= 0)) {
@@ -117,14 +139,18 @@ Write to console and add line-break, and show prompt if required.
 ************************************************************************/
 void con_write(enum rfc_status rfc_status, const char *message, ...)
 {
-  static char buf[MAX_LEN_CONSOLE_LINE];
+  /* First buffer contains featured text tags */
+  static char buf1[(MAX_LEN_CONSOLE_LINE * 3) / 2];
+  static char buf2[MAX_LEN_CONSOLE_LINE];
   va_list args;
-  
+
   va_start(args, message);
-  my_vsnprintf(buf, sizeof(buf), message, args);
+  my_vsnprintf(buf1, sizeof(buf1), message, args);
   va_end(args);
 
-  con_puts(rfc_status, buf);
+  /* remove all format tags */
+  featured_text_to_plain_text(buf1, buf2, sizeof(buf2), NULL);
+  con_puts(rfc_status, buf2);
 }
 
 /************************************************************************
@@ -136,7 +162,7 @@ this allows con_puts(C_COMMENT,"");
 ************************************************************************/
 void con_puts(enum rfc_status rfc_status, const char *str)
 {
-  if(console_prompt_is_showing) {
+  if (console_prompt_is_showing) {
     fc_printf("\n");
   }
   if ((console_rfcstyle) && (rfc_status >= 0)) {
