@@ -18,29 +18,36 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+/* utility */
+#include "fcintl.h"
+#include "log.h"
+#include "mem.h"
+
+/* common */
 #include "diptreaty.h"
 #include "events.h"
-#include "fcintl.h"
 #include "game.h"
-#include "log.h"
 #include "map.h"
-#include "mem.h"
 #include "packets.h"
 #include "player.h"
 #include "unit.h"
 
+/* ai */
+#include "advdiplomacy.h"
+
+/* scripting */
 #include "script.h"
 
+/* server */
 #include "citytools.h"
 #include "cityturn.h"
 #include "maphand.h"
 #include "plrhand.h"
+#include "notify.h"
 #include "settlers.h"
 #include "techtools.h"
 #include "unittools.h"
 
-#include "advdiplomacy.h"
-
 #include "diplhand.h"
 
 #define SPECLIST_TAG treaty
@@ -56,6 +63,28 @@ static struct treaty_list *treaties = NULL;
 /* FIXME: Should this be put in a ruleset somewhere? */
 #define TURNS_LEFT 16
 
+/**************************************************************************
+  Calls treaty_evaluate function if such is set for AI player                           
+**************************************************************************/
+static void call_treaty_evaluate(struct player *pplayer, struct player *aplayer,
+                                 struct Treaty *ptreaty)
+{
+  if (pplayer->ai_data.control && pplayer->ai->funcs.treaty_evaluate) {
+    pplayer->ai->funcs.treaty_evaluate(pplayer, aplayer, ptreaty);
+  }
+}
+
+/**************************************************************************
+  Calls treaty_accepted function if such is set for AI player
+**************************************************************************/
+static void call_treaty_accepted(struct player *pplayer, struct player *aplayer,
+                                 struct Treaty *ptreaty)
+{
+  if (pplayer->ai_data.control && pplayer->ai->funcs.treaty_accepted) {
+    pplayer->ai->funcs.treaty_accepted(pplayer, aplayer, ptreaty);
+  }
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -86,7 +115,7 @@ void free_treaties(void)
     free(pt);
   } treaty_list_iterate_end;
 
-  treaty_list_unlink_all(treaties);
+  treaty_list_clear(treaties);
 }
 
 /**************************************************************************
@@ -115,6 +144,7 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
   struct Treaty *ptreaty;
   bool *player_accept, *other_accept;
   enum dipl_reason diplcheck;
+  bool worker_refresh_required = FALSE;
   struct player *pother = valid_player_by_number(counterpart);
 
   if (NULL == pother || pplayer == pother) {
@@ -145,7 +175,7 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
       if (pclause->from == pplayer) {
 	switch(pclause->type) {
 	case CLAUSE_EMBASSY:
-          if (player_has_embassy(pother, pplayer)) {
+          if (player_has_real_embassy(pother, pplayer)) {
             freelog(LOG_ERROR, "%s tried to give embassy to %s, who already "
                     "has an embassy",
                     player_name(pplayer),
@@ -154,25 +184,25 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
           }
           break;
 	case CLAUSE_ADVANCE:
-          if (!tech_is_available(pother, pclause->value)) {
+          if (!player_invention_reachable(pother, pclause->value)) {
 	    /* It is impossible to give a technology to a civilization that
 	     * can never possess it (the client should enforce this). */
 	    freelog(LOG_ERROR, "Treaty: %s can't have tech %s",
                     nation_rule_name(nation_of_player(pother)),
 		    advance_name_by_player(pplayer, pclause->value));
-	    notify_player(pplayer, NULL, E_DIPLOMACY,
+            notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
                           _("The %s can't accept %s."),
                           nation_plural_for_player(pother),
 			  advance_name_for_player(pplayer, pclause->value));
 	    return;
           }
-	  if (get_invention(pplayer, pclause->value) != TECH_KNOWN) {
+	  if (player_invention_state(pplayer, pclause->value) != TECH_KNOWN) {
 	    freelog(LOG_ERROR,
                     "Nation %s try to give unknown tech %s to nation %s.",
 		    nation_rule_name(nation_of_player(pplayer)),
 		    advance_name_by_player(pplayer, pclause->value),
 		    nation_rule_name(nation_of_player(pother)));
-	    notify_player(pplayer, NULL, E_DIPLOMACY,
+            notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
 			  _("You don't have tech %s, you can't accept treaty."),
 			  advance_name_for_player(pplayer, pclause->value));
 	    return;
@@ -181,22 +211,22 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
 	case CLAUSE_CITY:
 	  pcity = game_find_city_by_number(pclause->value);
 	  if (!pcity) { /* Can't find out cityname any more. */
-	    notify_player(pplayer, NULL, E_DIPLOMACY,
+            notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
 			  _("City you are trying to give no longer exists, "
 			    "you can't accept treaty."));
 	    return;
 	  }
 	  if (city_owner(pcity) != pplayer) {
-	    notify_player(pplayer, NULL, E_DIPLOMACY,
+            notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
 			  _("You are not owner of %s, you can't accept treaty."),
-			  city_name(pcity));
+			  city_link(pcity));
 	    return;
 	  }
 	  if (is_capital(pcity)) {
-	    notify_player(pplayer, NULL, E_DIPLOMACY,
+            notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
 			  _("Your capital (%s) is requested, "
 			    "you can't accept treaty."),
-			  city_name(pcity));
+			  city_link(pcity));
 	    return;
 	  }
 	  break;
@@ -214,13 +244,19 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
           break;
 	case CLAUSE_ALLIANCE:
           diplcheck = pplayer_can_make_treaty(pplayer, pother, DS_ALLIANCE);
+          if (diplcheck == DIPL_ALLIANCE_PROBLEM) {
+            notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
+                          _("You cannot form an alliance because you are "
+                            "at war with an ally of %s."),
+                          player_name(pother));
+          }
           if (diplcheck != DIPL_OK) {
             return;
           }
           break;
 	case CLAUSE_GOLD:
 	  if (pplayer->economic.gold < pclause->value) {
-	    notify_player(pplayer, NULL, E_DIPLOMACY,
+            notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
 			  _("You don't have enough gold, "
 			    "you can't accept treaty."));
 	    return;
@@ -252,16 +288,16 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
 					   player_number(pplayer),
  					   player_number(pplayer));
 
-    notify_player(pplayer, NULL, E_DIPLOMACY,
-		  PL_("A treaty containing %d clause was agreed upon.",
-		      "A treaty containing %d clauses was agreed upon.",
-		      nclauses),
-		  nclauses);
-    notify_player(pother, NULL, E_DIPLOMACY,
-		  PL_("A treaty containing %d clause was agreed upon.",
-		      "A treaty containing %d clauses was agreed upon.",
-		      nclauses),
-		  nclauses);
+    notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
+                  PL_("A treaty containing %d clause was agreed upon.",
+                      "A treaty containing %d clauses was agreed upon.",
+                      nclauses),
+                  nclauses);
+    notify_player(pother, NULL, E_DIPLOMACY, ftc_server,
+                  PL_("A treaty containing %d clause was agreed upon.",
+                      "A treaty containing %d clauses was agreed upon.",
+                      nclauses),
+                  nclauses);
 
     /* Check that one who accepted treaty earlier still have everything
        (s)he promised to give. */
@@ -273,34 +309,34 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
 	case CLAUSE_CITY:
 	  pcity = game_find_city_by_number(pclause->value);
 	  if (!pcity) { /* Can't find out cityname any more. */
-	    notify_player(pplayer, NULL, E_DIPLOMACY,
-			  _("One of the cities the %s are giving away"
-			    " is destroyed! Treaty canceled!"),
-			  nation_plural_for_player(pother));
-	    notify_player(pother, NULL, E_DIPLOMACY,
-			  _("One of the cities the %s are giving away"
-			    " is destroyed! Treaty canceled!"),
-			  nation_plural_for_player(pother));
+            notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
+                          _("One of the cities the %s are giving away"
+                            " is destroyed! Treaty canceled!"),
+                          nation_plural_for_player(pother));
+            notify_player(pother, NULL, E_DIPLOMACY, ftc_server,
+                          _("One of the cities the %s are giving away"
+                            " is destroyed! Treaty canceled!"),
+                          nation_plural_for_player(pother));
 	    goto cleanup;
 	  }
 	  if (city_owner(pcity) != pother) {
-	    notify_player(pplayer, NULL, E_DIPLOMACY,
-			  _("The %s no longer control %s! "
-			    "Treaty canceled!"),
-			  nation_plural_for_player(pother),
-			  city_name(pcity));
-	    notify_player(pother, NULL, E_DIPLOMACY,
-			  _("The %s no longer control %s! "
-			    "Treaty canceled!"),
-			  nation_plural_for_player(pother),
-			  city_name(pcity));
+            notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
+                          _("The %s no longer control %s! "
+                            "Treaty canceled!"),
+                          nation_plural_for_player(pother),
+                          city_link(pcity));
+            notify_player(pother, NULL, E_DIPLOMACY, ftc_server,
+                          _("The %s no longer control %s! "
+                            "Treaty canceled!"),
+                          nation_plural_for_player(pother),
+                          city_link(pcity));
 	    goto cleanup;
 	  }
 	  if (is_capital(pcity)) {
-	    notify_player(pother, NULL, E_DIPLOMACY,
-			  _("Your capital (%s) is requested, "
-			    "you can't accept treaty."),
-			  city_name(pcity));
+            notify_player(pother, NULL, E_DIPLOMACY, ftc_server,
+                          _("Your capital (%s) is requested, "
+                            "you can't accept treaty."),
+                          city_link(pcity));
 	    goto cleanup;
 	  }
 
@@ -327,14 +363,14 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
           break;
 	case CLAUSE_GOLD:
 	  if (pother->economic.gold < pclause->value) {
-	    notify_player(pplayer, NULL, E_DIPLOMACY,
-			  _("The %s don't have the promised amount "
-			    "of gold! Treaty canceled!"),
-			  nation_plural_for_player(pother));
-	    notify_player(pother, NULL, E_DIPLOMACY,
-			  _("The %s don't have the promised amount "
-			    "of gold! Treaty canceled!"),
-			  nation_plural_for_player(pother));
+            notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
+                          _("The %s don't have the promised amount "
+                            "of gold! Treaty canceled!"),
+                          nation_plural_for_player(pother));
+            notify_player(pother, NULL, E_DIPLOMACY, ftc_server,
+                          _("The %s don't have the promised amount "
+                            "of gold! Treaty canceled!"),
+                          nation_plural_for_player(pother));
 	    goto cleanup;
 	  }
 	  break;
@@ -344,34 +380,30 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
       }
     } clause_list_iterate_end;
 
-    if (pplayer->ai.control) {
-      ai_treaty_accepted(pplayer, pother, ptreaty);
-    }
-    if (pother->ai.control) {
-      ai_treaty_accepted(pother, pplayer, ptreaty);
-    }
+    call_treaty_accepted(pplayer, pother, ptreaty);
+    call_treaty_accepted(pother, pplayer, ptreaty);
 
     clause_list_iterate(ptreaty->clauses, pclause) {
       struct player *pgiver = pclause->from;
       struct player *pdest = (pplayer == pgiver) ? pother : pplayer;
       enum diplstate_type old_diplstate = 
-        pgiver->diplstates[pdest->player_no].type;
+        pgiver->diplstates[player_index(pdest)].type;
 
       switch (pclause->type) {
       case CLAUSE_EMBASSY:
         establish_embassy(pdest, pgiver); /* sic */
-        notify_player(pgiver, NULL, E_TREATY_SHARED_VISION,
-                         _("You gave an embassy to %s."),
-                         player_name(pdest));
-        notify_player(pdest, NULL, E_TREATY_SHARED_VISION,
-                         _("%s allowed you to create an embassy!"),
-                         player_name(pgiver));
+        notify_player(pgiver, NULL, E_TREATY_SHARED_VISION, ftc_server,
+                      _("You gave an embassy to %s."),
+                      player_name(pdest));
+        notify_player(pdest, NULL, E_TREATY_SHARED_VISION, ftc_server,
+                      _("%s allowed you to create an embassy!"),
+                      player_name(pgiver));
         break;
       case CLAUSE_ADVANCE:
         /* It is possible that two players open the diplomacy dialog
          * and try to give us the same tech at the same time. This
          * should be handled discreetly instead of giving a core dump. */
-        if (get_invention(pdest, pclause->value) == TECH_KNOWN) {
+        if (player_invention_state(pdest, pclause->value) == TECH_KNOWN) {
 	  freelog(LOG_VERBOSE,
                   "Nation %s already know tech %s, that %s want to give them.",
 		  nation_rule_name(nation_of_player(pdest)),
@@ -379,18 +411,18 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
 		  nation_rule_name(nation_of_player(pgiver)));
           break;
         }
-	notify_player(pdest, NULL, E_TECH_GAIN,
-			 _("You are taught the knowledge of %s."),
-			 advance_name_for_player(pdest, pclause->value));
+        notify_player(pdest, NULL, E_TECH_GAIN, ftc_server,
+                      _("You are taught the knowledge of %s."),
+                      advance_name_for_player(pdest, pclause->value));
 
-	notify_embassies(pdest, pgiver, NULL, E_TECH_GAIN,
-			 _("The %s have acquired %s from the %s."),
-			 nation_plural_for_player(pdest),
-			 advance_name_for_player(pdest, pclause->value),
-			 nation_plural_for_player(pgiver));
+        notify_embassies(pdest, pgiver, NULL, E_TECH_GAIN, ftc_server,
+                         _("The %s have acquired %s from the %s."),
+                         nation_plural_for_player(pdest),
+                         advance_name_for_player(pdest, pclause->value),
+                         nation_plural_for_player(pgiver));
 
 	script_signal_emit("tech_researched", 3,
-			   API_TYPE_TECH_TYPE, &advances[pclause->value],
+			   API_TYPE_TECH_TYPE, advance_by_number(pclause->value),
 			   API_TYPE_PLAYER, pdest,
 			   API_TYPE_STRING, "traded");
 	do_dipl_cost(pdest, pclause->value);
@@ -398,138 +430,135 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
 	found_new_tech(pdest, pclause->value, FALSE, TRUE);
 	break;
       case CLAUSE_GOLD:
-	notify_player(pdest, NULL, E_DIPLOMACY,
-		      _("You get %d gold."), pclause->value);
+          notify_player(pdest, NULL, E_DIPLOMACY, ftc_server,
+                        _("You get %d gold."), pclause->value);
 	pgiver->economic.gold -= pclause->value;
 	pdest->economic.gold += pclause->value * (100 - game.info.diplcost) / 100;
 	break;
       case CLAUSE_MAP:
 	give_map_from_player_to_player(pgiver, pdest);
-	notify_player(pdest, NULL, E_DIPLOMACY,
-		      /* TRANS: ... Polish worldmap. */
-		      _("You receive the %s worldmap."),
-		      nation_adjective_for_player(pgiver));
+        notify_player(pdest, NULL, E_DIPLOMACY, ftc_server,
+                      /* TRANS: ... Polish worldmap. */
+                      _("You receive the %s worldmap."),
+                      nation_adjective_for_player(pgiver));
 
-	check_city_workers(pdest); /* See CLAUSE_VISION */
+        worker_refresh_required = TRUE; /* See CLAUSE_VISION */
 	break;
       case CLAUSE_SEAMAP:
 	give_seamap_from_player_to_player(pgiver, pdest);
-	notify_player(pdest, NULL, E_DIPLOMACY,
-		      /* TRANS: ... Polish seamap. */
-		      _("You receive the %s seamap."),
-		      nation_adjective_for_player(pgiver));
+        notify_player(pdest, NULL, E_DIPLOMACY, ftc_server,
+                      /* TRANS: ... Polish seamap. */
+                      _("You receive the %s seamap."),
+                      nation_adjective_for_player(pgiver));
 
-	check_city_workers(pdest); /* See CLAUSE_VISION */
+        worker_refresh_required = TRUE; /* See CLAUSE_VISION */
 	break;
       case CLAUSE_CITY:
 	{
 	  struct city *pcity = game_find_city_by_number(pclause->value);
 
 	  if (!pcity) {
-	    freelog(LOG_NORMAL,
+	    freelog(LOG_ERROR,
 		    "Treaty city id %d not found - skipping clause.",
 		    pclause->value);
 	    break;
 	  }
 
-	  notify_player(pdest, pcity->tile, E_CITY_TRANSFER,
-			   _("You receive city of %s from %s."),
-			   city_name(pcity),
-			   player_name(pgiver));
+          notify_player(pdest, city_tile(pcity), E_CITY_TRANSFER, ftc_server,
+                        _("You receive city of %s from %s."),
+                        city_link(pcity), player_name(pgiver));
 
-	  notify_player(pgiver, pcity->tile, E_CITY_LOST,
-			   _("You give city of %s to %s."),
-			   city_name(pcity),
-			   player_name(pdest));
+          notify_player(pgiver, city_tile(pcity), E_CITY_LOST, ftc_server,
+                        _("You give city of %s to %s."),
+                        city_link(pcity), player_name(pdest));
 
 	  transfer_city(pdest, pcity, -1, TRUE, TRUE, FALSE);
 	  break;
 	}
       case CLAUSE_CEASEFIRE:
-	pgiver->diplstates[pdest->player_no].type=DS_CEASEFIRE;
-	pgiver->diplstates[pdest->player_no].turns_left = TURNS_LEFT;
-	pdest->diplstates[pgiver->player_no].type=DS_CEASEFIRE;
-	pdest->diplstates[pgiver->player_no].turns_left = TURNS_LEFT;
-	notify_player(pgiver, NULL, E_TREATY_CEASEFIRE,
-			 _("You agree on a cease-fire with %s."),
-			 player_name(pdest));
-	notify_player(pdest, NULL, E_TREATY_CEASEFIRE,
-			 _("You agree on a cease-fire with %s."),
-			 player_name(pgiver));
+	pgiver->diplstates[player_index(pdest)].type=DS_CEASEFIRE;
+	pgiver->diplstates[player_index(pdest)].turns_left = TURNS_LEFT;
+	pdest->diplstates[player_index(pgiver)].type=DS_CEASEFIRE;
+	pdest->diplstates[player_index(pgiver)].turns_left = TURNS_LEFT;
+        notify_player(pgiver, NULL, E_TREATY_CEASEFIRE, ftc_server,
+                      _("You agree on a cease-fire with %s."),
+                      player_name(pdest));
+        notify_player(pdest, NULL, E_TREATY_CEASEFIRE, ftc_server,
+                      _("You agree on a cease-fire with %s."),
+                      player_name(pgiver));
 	if (old_diplstate == DS_ALLIANCE) {
 	  update_players_after_alliance_breakup(pgiver, pdest);
 	}
-	check_city_workers(pplayer);
-	check_city_workers(pother);
+
+        worker_refresh_required = TRUE;
 	break;
       case CLAUSE_PEACE:
-	pgiver->diplstates[pdest->player_no].type = DS_ARMISTICE;
-	pdest->diplstates[pgiver->player_no].type = DS_ARMISTICE;
-	pgiver->diplstates[pdest->player_no].turns_left = TURNS_LEFT;
-	pdest->diplstates[pgiver->player_no].turns_left = TURNS_LEFT;
-	pgiver->diplstates[pdest->player_no].max_state = 
-          MAX(DS_PEACE, pgiver->diplstates[pdest->player_no].max_state);
-	pdest->diplstates[pgiver->player_no].max_state = 
-          MAX(DS_PEACE, pdest->diplstates[pgiver->player_no].max_state);
-	notify_player(pgiver, NULL, E_TREATY_PEACE,
-		      /* TRANS: ... the Poles ... Polish territory. */
-		      PL_("You agree on an armistice with the %s. In %d turn, "
-			  "it will become a peace treaty. Move your "
-			  "units out of %s territory.",
-			  "You agree on an armistice with the %s. In %d turns, "
-			  "it will become a peace treaty. Move your "
-			  "units out of %s territory.",
-			  TURNS_LEFT),
-		      nation_plural_for_player(pdest),
-		      TURNS_LEFT,
-		      nation_adjective_for_player(pdest));
-	notify_player(pdest, NULL, E_TREATY_PEACE,
-		      /* TRANS: ... the Poles ... Polish territory. */
-		      PL_("You agree on an armistice with the %s. In %d turn, "
-			  "it will become a peace treaty. Move your "
-			  "units out of %s territory.",
-			  "You agree on an armistice with the %s. In %d turns, "
-			  "it will become a peace treaty. Move your "
-			  "units out of %s territory.",
-			  TURNS_LEFT),
-		      nation_plural_for_player(pgiver),
-		      TURNS_LEFT,
-		      nation_adjective_for_player(pgiver));
+	pgiver->diplstates[player_index(pdest)].type = DS_ARMISTICE;
+	pdest->diplstates[player_index(pgiver)].type = DS_ARMISTICE;
+	pgiver->diplstates[player_index(pdest)].turns_left = TURNS_LEFT;
+	pdest->diplstates[player_index(pgiver)].turns_left = TURNS_LEFT;
+	pgiver->diplstates[player_index(pdest)].max_state = 
+          MAX(DS_PEACE, pgiver->diplstates[player_index(pdest)].max_state);
+	pdest->diplstates[player_index(pgiver)].max_state = 
+          MAX(DS_PEACE, pdest->diplstates[player_index(pgiver)].max_state);
+        notify_player(pgiver, NULL, E_TREATY_PEACE, ftc_server,
+                      /* TRANS: ... the Poles ... Polish territory. */
+                      PL_("You agree on an armistice with the %s. In %d turn, "
+                          "it will become a peace treaty. Move your "
+                          "units out of %s territory.",
+                          "You agree on an armistice with the %s. In %d turns, "
+                          "it will become a peace treaty. Move your "
+                          "units out of %s territory.",
+                          TURNS_LEFT),
+                      nation_plural_for_player(pdest),
+                      TURNS_LEFT,
+                      nation_adjective_for_player(pdest));
+        notify_player(pdest, NULL, E_TREATY_PEACE, ftc_server,
+                      /* TRANS: ... the Poles ... Polish territory. */
+                      PL_("You agree on an armistice with the %s. In %d turn, "
+                          "it will become a peace treaty. Move your "
+                          "units out of %s territory.",
+                          "You agree on an armistice with the %s. In %d turns, "
+                          "it will become a peace treaty. Move your "
+                          "units out of %s territory.",
+                          TURNS_LEFT),
+                      nation_plural_for_player(pgiver),
+                      TURNS_LEFT,
+                      nation_adjective_for_player(pgiver));
 	if (old_diplstate == DS_ALLIANCE) {
 	  update_players_after_alliance_breakup(pgiver, pdest);
 	}
-	check_city_workers(pplayer);
-	check_city_workers(pother);
+
+        worker_refresh_required = TRUE;
 	break;
       case CLAUSE_ALLIANCE:
-	pgiver->diplstates[pdest->player_no].type=DS_ALLIANCE;
-	pdest->diplstates[pgiver->player_no].type=DS_ALLIANCE;
-	pgiver->diplstates[pdest->player_no].max_state = 
-          MAX(DS_ALLIANCE, pgiver->diplstates[pdest->player_no].max_state);
-	pdest->diplstates[pgiver->player_no].max_state = 
-          MAX(DS_ALLIANCE, pdest->diplstates[pgiver->player_no].max_state);
-	notify_player(pgiver, NULL, E_TREATY_ALLIANCE,
-			 _("You agree on an alliance with %s."),
-			 player_name(pdest));
-	notify_player(pdest, NULL, E_TREATY_ALLIANCE,
-			 _("You agree on an alliance with %s."),
-			 player_name(pgiver));
+	pgiver->diplstates[player_index(pdest)].type=DS_ALLIANCE;
+	pdest->diplstates[player_index(pgiver)].type=DS_ALLIANCE;
+	pgiver->diplstates[player_index(pdest)].max_state = 
+          MAX(DS_ALLIANCE, pgiver->diplstates[player_index(pdest)].max_state);
+	pdest->diplstates[player_index(pgiver)].max_state = 
+          MAX(DS_ALLIANCE, pdest->diplstates[player_index(pgiver)].max_state);
+        notify_player(pgiver, NULL, E_TREATY_ALLIANCE, ftc_server,
+                      _("You agree on an alliance with %s."),
+                      player_name(pdest));
+        notify_player(pdest, NULL, E_TREATY_ALLIANCE, ftc_server,
+                      _("You agree on an alliance with %s."),
+                      player_name(pgiver));
 
-	check_city_workers(pplayer);
-	check_city_workers(pother);
+        worker_refresh_required = TRUE;
 	break;
       case CLAUSE_VISION:
 	give_shared_vision(pgiver, pdest);
-	notify_player(pgiver, NULL, E_TREATY_SHARED_VISION,
-			 _("You give shared vision to %s."),
-			 player_name(pdest));
-	notify_player(pdest, NULL, E_TREATY_SHARED_VISION,
-			 _("%s gives you shared vision."),
-			 player_name(pgiver));
+        notify_player(pgiver, NULL, E_TREATY_SHARED_VISION, ftc_server,
+                      _("You give shared vision to %s."),
+                      player_name(pdest));
+        notify_player(pdest, NULL, E_TREATY_SHARED_VISION, ftc_server,
+                      _("%s gives you shared vision."),
+                      player_name(pgiver));
 
         /* Yes, shared vision may let us to _know_ tiles
          * within radius of our own city. */
-	check_city_workers(pdest);
+        worker_refresh_required = TRUE;
 	break;
       case CLAUSE_LAST:
         freelog(LOG_ERROR, "Received bad clause type");
@@ -537,6 +566,18 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
       }
 
     } clause_list_iterate_end;
+
+    /* In theory, we would need refresh only receiving party of
+     * CLAUSE_MAP, CLAUSE_SEAMAP and CLAUSE_VISION clauses.
+     * It's quite unlikely that there is such a clause going one
+     * way but no clauses affecting both parties or going other
+     * way. */
+    if (worker_refresh_required) {
+      city_map_update_all_cities_for_player(pplayer);
+      city_map_update_all_cities_for_player(pother);
+      sync_cities();
+    }
+
   cleanup:
     treaty_list_unlink(treaties, ptreaty);
     clear_treaty(ptreaty);
@@ -552,7 +593,7 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
 void establish_embassy(struct player *pplayer, struct player *aplayer)
 {
   /* Establish the embassy. */
-  BV_SET(pplayer->embassy, player_index(aplayer));
+  BV_SET(pplayer->real_embassy, player_index(aplayer));
   send_player_info(pplayer, pplayer);
   send_player_info(pplayer, aplayer);  /* update player dialog with embassy */
   send_player_info(aplayer, pplayer);  /* INFO_EMBASSY level info */
@@ -586,12 +627,8 @@ void handle_diplomacy_remove_clause_req(struct player *pplayer,
     dlsend_packet_diplomacy_remove_clause(pother->connections,
 					  player_number(pplayer), giver, type,
 					  value);
-    if (pplayer->ai.control) {
-      ai_treaty_evaluate(pplayer, pother, ptreaty);
-    }
-    if (pother->ai.control) {
-      ai_treaty_evaluate(pother, pplayer, ptreaty);
-    }
+    call_treaty_evaluate(pplayer, pother, ptreaty);
+    call_treaty_evaluate(pother, pplayer, ptreaty);
   }
 }
 
@@ -638,12 +675,8 @@ void handle_diplomacy_create_clause_req(struct player *pplayer,
     dlsend_packet_diplomacy_create_clause(pother->connections,
 					  player_number(pplayer), giver, type,
 					  value);
-    if (pplayer->ai.control) {
-      ai_treaty_evaluate(pplayer, pother, ptreaty);
-    }
-    if (pother->ai.control) {
-      ai_treaty_evaluate(pother, pplayer, ptreaty);
-    }
+    call_treaty_evaluate(pplayer, pother, ptreaty);
+    call_treaty_evaluate(pother, pplayer, ptreaty);
   }
 }
 
@@ -659,16 +692,16 @@ static void really_diplomacy_cancel_meeting(struct player *pplayer,
     dlsend_packet_diplomacy_cancel_meeting(pother->connections,
 					   player_number(pplayer),
 					   player_number(pplayer));
-    notify_player(pother, NULL, E_DIPLOMACY,
-		  _("%s canceled the meeting!"), 
-		  player_name(pplayer));
+    notify_player(pother, NULL, E_DIPLOMACY, ftc_server,
+                  _("%s canceled the meeting!"), 
+                  player_name(pplayer));
     /* Need to send to pplayer too, for multi-connects: */
     dlsend_packet_diplomacy_cancel_meeting(pplayer->connections,
 					   player_number(pother),
 					   player_number(pplayer));
-    notify_player(pplayer, NULL, E_DIPLOMACY,
-		  _("Meeting with %s canceled."), 
-		  player_name(pother));
+    notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
+                  _("Meeting with %s canceled."), 
+                  player_name(pother));
     treaty_list_unlink(treaties, ptreaty);
     clear_treaty(ptreaty);
     free(ptreaty);
@@ -708,8 +741,8 @@ void handle_diplomacy_init_meeting_req(struct player *pplayer,
 
   if (get_player_bonus(pplayer, EFT_NO_DIPLOMACY)
       || get_player_bonus(pother, EFT_NO_DIPLOMACY)) {
-    notify_player(pplayer, NULL, E_DIPLOMACY,
-		  _("Your diplomatic envoy was decapitated!"));
+    notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
+                  _("Your diplomatic envoy was decapitated!"));
     return;
   }
 
@@ -735,7 +768,7 @@ void handle_diplomacy_init_meeting_req(struct player *pplayer,
 **************************************************************************/
 void send_diplomatic_meetings(struct connection *dest)
 {
-  struct player *pplayer = dest->player;
+  struct player *pplayer = dest->playing;
 
   if (!pplayer) {
     return;
