@@ -18,70 +18,44 @@
 #include <assert.h>
 #include <stdio.h> /* for remove() */ 
 
+/* utility */
 #include "capability.h"
-#include "events.h"
 #include "fcintl.h"
-#include "game.h"
-#include "improvement.h"
 #include "log.h"
 #include "mem.h"
-#include "movement.h"
-#include "packets.h"
 #include "rand.h"
 #include "registry.h"
 #include "shared.h"
 #include "support.h"
 
+/* common */
+#include "events.h"
+#include "game.h"
+#include "improvement.h"
+#include "movement.h"
+#include "packets.h"
+
+/* server */
 #include "connecthand.h"
+#include "ggzserver.h"
 #include "maphand.h"
+#include "notify.h"
 #include "plrhand.h"
+#include "srv_main.h"
 #include "unittools.h"
 
 #include "gamehand.h"
-#include "srv_main.h"
 
 #define CHALLENGE_ROOT "challenge"
 
 
 /****************************************************************************
-  Initialize the game.id variable to a random string of characters.
+  Get unit_type for given role character
 ****************************************************************************/
-static void init_game_id(void)
+struct unit_type *crole_to_unit_type(char crole,struct player *pplayer)
 {
-  static const char chars[] =
-    "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
-  int i;
-
-  for (i = 0; i < ARRAY_SIZE(game.id) - 1; i++) {
-    game.id[i] = chars[myrand(sizeof(chars) - 1)];
-  }
-  game.id[i] = '\0';
-}
-
-/****************************************************************************
-  Place a starting unit for the player.
-****************************************************************************/
-static void place_starting_unit(struct tile *ptile, struct player *pplayer,
-				char crole)
-{
-  struct unit_type *utype;
-  enum unit_flag_id role;
-
-  assert(!is_non_allied_unit_tile(ptile, pplayer));
-
-  /* For scenarios or dispersion, huts may coincide with player starts (in 
-   * other cases, huts are avoided as start positions).  Remove any such hut,
-   * and make sure to tell the client, since we may have already sent this
-   * tile (with the hut) earlier: */
-  if (tile_has_special(ptile, S_HUT)) {
-    tile_clear_special(ptile, S_HUT);
-    update_tile_knowledge(ptile);
-    freelog(LOG_VERBOSE, "Removed hut on start position for %s",
-	    player_name(pplayer));
-  }
-
-  /* Expose visible area. */
-  map_show_circle(pplayer, ptile, game.info.init_vis_radius_sq);
+  struct unit_type *utype = NULL;
+  enum unit_role_id role;
 
   switch(crole) {
   case 'c':
@@ -116,31 +90,83 @@ static void place_starting_unit(struct tile *ptile, struct player *pplayer,
     break;
   default: 
     assert(FALSE);
-    return;
+    return NULL;
   }
 
   /* Create the unit of an appropriate type, if it exists */
   if (num_role_units(role) > 0) {
-    utype = first_role_unit_for_player(pplayer, role);
+    if (pplayer != NULL) {
+      utype = first_role_unit_for_player(pplayer, role);
+    }
     if (utype == NULL) {
       utype = get_role_unit(role, 0);
     }
+  }
 
+  return utype;
+}
+
+/****************************************************************************
+  Place a starting unit for the player. Returns tile where unit was really
+  placed.
+****************************************************************************/
+static struct tile *place_starting_unit(struct tile *starttile,
+                                        struct player *pplayer,
+                                        char crole)
+{
+  struct tile *ptile = NULL;
+  struct unit_type *utype = crole_to_unit_type(crole, pplayer);
+
+  if (utype != NULL) {
+    iterate_outward(starttile, map.xsize + map.ysize, itertile) {
+      if (!is_non_allied_unit_tile(itertile, pplayer)
+          && is_native_tile(utype, itertile)) {
+        ptile = itertile;
+        break;
+      }
+    } iterate_outward_end;
+  }
+
+  if (ptile == NULL) {
+    /* No place where unit may exist. */
+    return NULL;
+  }
+
+  assert(!is_non_allied_unit_tile(ptile, pplayer));
+
+  /* For scenarios or dispersion, huts may coincide with player starts (in 
+   * other cases, huts are avoided as start positions).  Remove any such hut,
+   * and make sure to tell the client, since we may have already sent this
+   * tile (with the hut) earlier: */
+  if (tile_has_special(ptile, S_HUT)) {
+    tile_clear_special(ptile, S_HUT);
+    update_tile_knowledge(ptile);
+    freelog(LOG_VERBOSE, "Removed hut on start position for %s",
+	    player_name(pplayer));
+  }
+
+  /* Expose visible area. */
+  map_show_circle(pplayer, ptile, game.info.init_vis_radius_sq);
+
+  if (utype != NULL) {
     /* We cannot currently handle sea units as start units.
      * TODO: remove this code block when we can. */
-    if (utype->move_type == SEA_MOVING) {
+    if (utype_move_type(utype) == SEA_MOVING) {
       freelog(LOG_ERROR, "Sea moving start units are not yet supported, "
                            "%s not created.",
                          utype_rule_name(utype));
-      notify_player(pplayer, NULL, E_BAD_COMMAND,
-		    _("Sea moving start units are not yet supported. "
-		      "Nobody gets %s."),
-		    utype_name_translation(utype));
-      return;
+      notify_player(pplayer, NULL, E_BAD_COMMAND, ftc_server,
+                    _("Sea moving start units are not yet supported. "
+                      "Nobody gets %s."),
+                    utype_name_translation(utype));
+      return NULL;
     }
 
     (void) create_unit(pplayer, ptile, utype, FALSE, 0, 0);
+    return ptile;
   }
+
+  return NULL;
 }
 
 /****************************************************************************
@@ -158,8 +184,8 @@ static struct tile *find_dispersed_position(struct player *pplayer,
     y = p->tile->y + myrand(2 * game.info.dispersion + 1)
         - game.info.dispersion;
   } while (!((ptile = map_pos_to_tile(x, y))
-             && tile_get_continent(p->tile) == tile_get_continent(ptile)
-             && !is_ocean(tile_get_terrain(ptile))
+             && tile_continent(p->tile) == tile_continent(ptile)
+             && !is_ocean_tile(ptile)
              && !is_non_allied_unit_tile(ptile, pplayer)));
 
   return ptile;
@@ -171,43 +197,42 @@ static struct tile *find_dispersed_position(struct player *pplayer,
 void init_new_game(void)
 {
   const int NO_START_POS = -1;
-  int start_pos[game.info.nplayers];
-  bool pos_used[map.num_start_positions];
+  int start_pos[player_slot_count()];
+  int placed_units[player_slot_count()];
+  bool pos_used[map.server.num_start_positions];
   int i, num_used = 0;
 
-  init_game_id();
+  randomize_base64url_string(server.game_identifier,
+                             sizeof(server.game_identifier));
 
   /* Shuffle starting positions around so that they match up with the
    * desired players. */
 
   /* First set up some data fields. */
   freelog(LOG_VERBOSE, "Placing players at start positions.");
-  for (i = 0; i < map.num_start_positions; i++) {
-    struct nation_type *n = map.start_positions[i].nation;
+  for (i = 0; i < map.server.num_start_positions; i++) {
+    struct nation_type *n = map.server.start_positions[i].nation;
 
     pos_used[i] = FALSE;
     freelog(LOG_VERBOSE, "%3d : (%2d,%2d) : \"%s\" (%d)",
-	    i,
-	    map.start_positions[i].tile->x,
-	    map.start_positions[i].tile->y,
-	    n ? nation_rule_name(n) : "",
-	    n ? n->index : -1);
+	    i, TILE_XY(map.server.start_positions[i].tile),
+	    n ? nation_rule_name(n) : "", n ? nation_number(n) : -1);
   }
   players_iterate(pplayer) {
-    start_pos[pplayer->player_no] = NO_START_POS;
+    start_pos[player_index(pplayer)] = NO_START_POS;
   } players_iterate_end;
 
   /* Second, assign a nation to a start position for that nation. */
   freelog(LOG_VERBOSE, "Assigning matching nations.");
   players_iterate(pplayer) {
-    for (i = 0; i < map.num_start_positions; i++) {
+    for (i = 0; i < map.server.num_start_positions; i++) {
       assert(pplayer->nation != NO_NATION_SELECTED);
-      if (pplayer->nation == map.start_positions[i].nation) {
+      if (pplayer->nation == map.server.start_positions[i].nation) {
 	freelog(LOG_VERBOSE, "Start_pos %d matches player %d (%s).",
 		i,
-		pplayer->player_no,
+		player_number(pplayer),
 		nation_rule_name(nation_of_player(pplayer)));
-	start_pos[pplayer->player_no] = i;
+	start_pos[player_index(pplayer)] = i;
 	pos_used[i] = TRUE;
 	num_used++;
       }
@@ -217,18 +242,18 @@ void init_new_game(void)
   /* Third, assign players randomly to the remaining start positions. */
   freelog(LOG_VERBOSE, "Assigning random nations.");
   players_iterate(pplayer) {
-    if (start_pos[pplayer->player_no] == NO_START_POS) {
-      int which = myrand(map.num_start_positions - num_used);
+    if (start_pos[player_index(pplayer)] == NO_START_POS) {
+      int which = myrand(map.server.num_start_positions - num_used);
 
-      for (i = 0; i < map.num_start_positions; i++) {
+      for (i = 0; i < map.server.num_start_positions; i++) {
 	if (!pos_used[i]) {
 	  if (which == 0) {
 	    freelog(LOG_VERBOSE,
 		    "Randomly assigning player %d (%s) to pos %d.",
-		    pplayer->player_no,
+		    player_number(pplayer),
 		    nation_rule_name(nation_of_player(pplayer)),
 		    i);
-	    start_pos[pplayer->player_no] = i;
+	    start_pos[player_index(pplayer)] = i;
 	    pos_used[i] = TRUE;
 	    num_used++;
 	    break;
@@ -237,16 +262,21 @@ void init_new_game(void)
 	}
       }
     }
-    assert(start_pos[pplayer->player_no] != NO_START_POS);
+    assert(start_pos[player_index(pplayer)] != NO_START_POS);
   } players_iterate_end;
 
   /* Loop over all players, creating their initial units... */
   players_iterate(pplayer) {
     struct start_position pos
-      = map.start_positions[start_pos[pplayer->player_no]];
+      = map.server.start_positions[start_pos[player_index(pplayer)]];
 
     /* Place the first unit. */
-    place_starting_unit(pos.tile, pplayer, game.info.start_units[0]);
+    if (place_starting_unit(pos.tile, pplayer,
+                            game.info.start_units[0]) != NULL) {
+      placed_units[player_index(pplayer)] = 1;
+    } else {
+      placed_units[player_index(pplayer)] = 0;
+    }
   } players_iterate_end;
 
   /* Place all other units. */
@@ -255,14 +285,17 @@ void init_new_game(void)
     struct tile *ptile;
     struct nation_type *nation = nation_of_player(pplayer);
     struct start_position p
-      = map.start_positions[start_pos[pplayer->player_no]];
+      = map.server.start_positions[start_pos[player_index(pplayer)]];
 
     /* Place global start units */
     for (i = 1; i < strlen(game.info.start_units); i++) {
       ptile = find_dispersed_position(pplayer, &p);
 
       /* Create the unit of an appropriate type. */
-      place_starting_unit(ptile, pplayer, game.info.start_units[i]);
+      if (place_starting_unit(ptile, pplayer,
+                              game.info.start_units[i]) != NULL) {
+        placed_units[player_index(pplayer)]++;
+      }
     }
 
     /* Place nation specific start units (not role based!) */
@@ -270,21 +303,19 @@ void init_new_game(void)
     while (nation->init_units[i] != NULL && i < MAX_NUM_UNIT_LIST) {
       ptile = find_dispersed_position(pplayer, &p);
       create_unit(pplayer, ptile, nation->init_units[i], FALSE, 0, 0);
+      placed_units[player_index(pplayer)]++;
       i++;
     }
   } players_iterate_end;
 
-  shuffle_players();
-}
+  players_iterate(pplayer) {
+    if (placed_units[player_index(pplayer)] == 0) {
+      /* No units at all for some player! */
+      die(_("No units placed for %s!"), player_name(pplayer));
+    }
+  } players_iterate_end;
 
-/**************************************************************************
-  This is called once at the start of each phase to alert the clients to
-  the new phase.  game.info.phase should be incremented before calling it.
-**************************************************************************/
-void send_start_phase_to_clients(void)
-{
-  /* This function is so simple it could probably be dropped... */
-  dlsend_packet_start_phase(game.est_connections, game.info.phase);
+  shuffle_players();
 }
 
 /**************************************************************************
@@ -294,33 +325,20 @@ void send_start_phase_to_clients(void)
 void send_year_to_clients(int year)
 {
   struct packet_new_year apacket;
-  int i;
   
-  for(i=0; i<game.info.nplayers; i++) {
-    struct player *pplayer = &game.players[i];
+  players_iterate(pplayer) {
     pplayer->nturns_idle++;
-  }
+  } players_iterate_end;
 
   apacket.year = year;
   apacket.turn = game.info.turn;
   lsend_packet_new_year(game.est_connections, &apacket);
 
   /* Hmm, clients could add this themselves based on above packet? */
-  notify_conn(game.est_connections, NULL, E_NEXT_YEAR, _("Year: %s"),
-		 textyear(year));
+  notify_conn(game.est_connections, NULL, E_NEXT_YEAR, ftc_any,
+              _("Year: %s"), textyear(year));
 }
 
-
-/**************************************************************************
-  Send specified state; should be a CLIENT_GAME_*_STATE ?
-  (But note client also changes state from other events.)
-**************************************************************************/
-void send_game_state(struct conn_list *dest, int state)
-{
-  dlsend_packet_game_state(dest, state);
-}
-
-
 /**************************************************************************
   Send game_info packet; some server options and various stuff...
   dest==NULL means game.est_connections
@@ -341,24 +359,40 @@ void send_game_info(struct conn_list *dest)
 
   /* the following values are computed every
      time a packet_game_info packet is created */
-  if (game.info.timeout > 0 && game.phase_timer) {
+  if (game.info.timeout > 0 && game.server.phase_timer) {
     /* Sometimes this function is called before the phase_timer is
      * initialized.  In that case we want to send the dummy value. */
-    ginfo.seconds_to_phasedone
-      = game.info.seconds_to_phasedone - read_timer_seconds(game.phase_timer);
+    ginfo.seconds_to_phasedone = game.info.seconds_to_phasedone
+        - read_timer_seconds(game.server.phase_timer);
   } else {
     /* unused but at least initialized */
     ginfo.seconds_to_phasedone = -1.0;
   }
 
   conn_list_iterate(dest, pconn) {
-    /* ? fixme: check for non-players: */
-    ginfo.player_idx = (pconn->player ? pconn->player->player_no : -1);
     send_packet_game_info(pconn, &ginfo);
   }
   conn_list_iterate_end;
 }
 
+/**************************************************************************
+  Send current scenario info. dest NULL causes send to everyone
+**************************************************************************/
+void send_scenario_info(struct conn_list *dest)
+{
+  struct packet_scenario_info sinfo;
+
+  if (!dest) {
+    dest = game.est_connections;
+  }
+
+  sinfo = game.scenario;
+
+  conn_list_iterate(dest, pconn) {
+    send_packet_scenario_info(pconn, &sinfo);
+  } conn_list_iterate_end;
+}
+
 /**************************************************************************
   adjusts game.info.timeout based on various server options
 
@@ -372,41 +406,42 @@ void send_game_info(struct conn_list *dest)
 int update_timeout(void)
 {
   /* if there's no timer or we're doing autogame, do nothing */
-  if (game.info.timeout < 1 || game.timeoutint == 0) {
+  if (game.info.timeout < 1 || game.server.timeoutint == 0) {
     return game.info.timeout;
   }
 
-  if (game.timeoutcounter >= game.timeoutint) {
-    game.info.timeout += game.timeoutinc;
-    game.timeoutinc *= game.timeoutincmult;
+  if (game.server.timeoutcounter >= game.server.timeoutint) {
+    game.info.timeout += game.server.timeoutinc;
+    game.server.timeoutinc *= game.server.timeoutincmult;
 
-    game.timeoutcounter = 1;
-    game.timeoutint += game.timeoutintinc;
+    game.server.timeoutcounter = 1;
+    game.server.timeoutint += game.server.timeoutintinc;
 
     if (game.info.timeout > GAME_MAX_TIMEOUT) {
-      notify_conn(game.est_connections, NULL, E_SETTING,
-		     _("The turn timeout has exceeded its maximum value, "
-		       "fixing at its maximum"));
+      notify_conn(game.est_connections, NULL, E_SETTING, ftc_server,
+                  _("The turn timeout has exceeded its maximum value, "
+                    "fixing at its maximum."));
       freelog(LOG_DEBUG, "game.info.timeout exceeded maximum value");
       game.info.timeout = GAME_MAX_TIMEOUT;
-      game.timeoutint = 0;
-      game.timeoutinc = 0;
+      game.server.timeoutint = 0;
+      game.server.timeoutinc = 0;
     } else if (game.info.timeout < 0) {
-      notify_conn(game.est_connections, NULL, E_SETTING,
-		     _("The turn timeout is smaller than zero, "
-		       "fixing at zero."));
+      notify_conn(game.est_connections, NULL, E_SETTING, ftc_server,
+                  _("The turn timeout is smaller than zero, "
+                    "fixing at zero."));
       freelog(LOG_DEBUG, "game.info.timeout less than zero");
       game.info.timeout = 0;
     }
   } else {
-    game.timeoutcounter++;
+    game.server.timeoutcounter++;
   }
 
   freelog(LOG_DEBUG, "timeout=%d, inc=%d incmult=%d\n   "
 	  "int=%d, intinc=%d, turns till next=%d",
-	  game.info.timeout, game.timeoutinc, game.timeoutincmult,
-	  game.timeoutint, game.timeoutintinc,
-	  game.timeoutint - game.timeoutcounter);
+	  game.info.timeout, game.server.timeoutinc,
+          game.server.timeoutincmult, game.server.timeoutint,
+          game.server.timeoutintinc,
+          game.server.timeoutint - game.server.timeoutcounter);
 
   return game.info.timeout;
 }
@@ -421,9 +456,9 @@ int update_timeout(void)
 **************************************************************************/
 void increase_timeout_because_unit_moved(void)
 {
-  if (game.info.timeout > 0 && game.timeoutaddenemymove > 0) {
-    double maxsec = (read_timer_seconds(game.phase_timer)
-		     + (double)game.timeoutaddenemymove);
+  if (game.info.timeout > 0 && game.server.timeoutaddenemymove > 0) {
+    double maxsec = (read_timer_seconds(game.server.phase_timer)
+		     + (double) game.server.timeoutaddenemymove);
 
     if (maxsec > game.info.seconds_to_phasedone) {
       game.info.seconds_to_phasedone = maxsec;
@@ -506,24 +541,26 @@ opens a file specified by the packet and compares the packet values with
 the file values. Sends an answer to the client once it's done.
 **************************************************************************/
 void handle_single_want_hack_req(struct connection *pc,
-                                 struct packet_single_want_hack_req *packet)
+    				 struct packet_single_want_hack_req *packet)
 {
   struct section_file file;
   char *token = NULL;
   bool you_have_hack = FALSE;
 
-  if (section_file_load_nodup(&file, get_challenge_fullname(pc))) {
-    token = secfile_lookup_str_default(&file, NULL, "challenge.token");
-    you_have_hack = (token && strcmp(token, packet->token) == 0);
-    section_file_free(&file);
-  }
+  if (!with_ggz) {
+    if (section_file_load_nodup(&file, get_challenge_fullname(pc))) {
+      token = secfile_lookup_str_default(&file, NULL, "challenge.token");
+      you_have_hack = (token && strcmp(token, packet->token) == 0);
+      section_file_free(&file);
+    }
 
-  if (!token) {
-    freelog(LOG_DEBUG, "Failed to read authentication token");
+    if (!token) {
+      freelog(LOG_DEBUG, "Failed to read authentication token");
+    }
   }
 
   if (you_have_hack) {
-    pc->access_level = ALLOW_HACK;
+    conn_set_access(pc, ALLOW_HACK, TRUE);
   }
 
   dsend_packet_single_want_hack_reply(pc, you_have_hack);
