@@ -14,6 +14,8 @@
 #include <config.h>
 #endif
 
+#include <math.h> /* sqrt */
+
 #include "map.h"
 #include "log.h"
 
@@ -32,8 +34,10 @@ int ice_base_colatitude = 0 ;
 int map_colatitude(const struct tile *ptile)
 {
   double x, y;
-  
-  if (map.alltemperate) {
+
+  RETURN_VAL_IF_FAIL(ptile != NULL, MAX_COLATITUDE / 2);
+
+  if (map.server.alltemperate) {
     /* An all-temperate map has "average" temperature everywhere.
      *
      * TODO: perhaps there should be a random temperature variation. */
@@ -120,14 +124,14 @@ int map_colatitude(const struct tile *ptile)
    *
    * ....
    * :\ N
-   * : \ 
+   * : \
    * :S \
    *
    * Now flip it along the X direction to get this:
    *
    * ....
    * :N /
-   * : / 
+   * : /
    * :/ S
    */
   x = 1.0 - x;
@@ -137,7 +141,7 @@ int map_colatitude(const struct tile *ptile)
    *
    * .....
    * :P /
-   * : / 
+   * : /
    * :/
    *
    * where P is the polar regions and / is the equator. */
@@ -149,11 +153,11 @@ int map_colatitude(const struct tile *ptile)
   /* This projection makes poles with a shape of a quarter-circle along
    * "P" and the equator as a straight line along "/".
    *
-   * This is explained more fully at
-   * http://bugs.freeciv.org/Ticket/Display.html?id=8624. */
-  return MAX_COLATITUDE * (1.5 * (x * x * y + x * y * y) 
-		     - 0.5 * (x * x * x + y * y * y) 
-		     + 1.5 * (x * x + y * y));
+   * This is explained more fully in RT 8624; the discussion can be found at
+   * http://thread.gmane.org/gmane.games.freeciv.devel/42648 */
+  return MAX_COLATITUDE * (1.5 * (x * x * y + x * y * y)
+                           - 0.5 * (x * x * x + y * y * y)
+                           + 1.5 * (x * x + y * y));
 }
 
 /****************************************************************************
@@ -215,15 +219,16 @@ static void set_sizes(double size, int Xratio, int Yratio)
 
   /* If the ratio is too big for some topology the simplest way to avoid
    * this error is to set the maximum size smaller for all topologies! */
-  if (map.size > size + 0.9) {
+  if (map.server.size > size + 0.9) {
     /* Warning when size is set uselessly big */ 
     freelog(LOG_ERROR,
 	    "Requested size of %d is too big for this topology.",
-	    map.size);
+	    map.server.size);
   }
   freelog(LOG_VERBOSE,
 	  "Creating a map of size %d x %d = %d tiles (%d requested).",
-	  map.xsize, map.ysize, map.xsize * map.ysize, map.size * 1000);
+	  map.xsize, map.ysize, map.xsize * map.ysize,
+          map.server.size * 1000);
 }
 
 /*
@@ -243,6 +248,8 @@ static void set_sizes(double size, int Xratio, int Yratio)
 ***************************************************************************/
 void generator_init_topology(bool autosize)
 {
+  int sqsize = get_sqsize();
+
   /* The default server behavior is to generate xsize/ysize from the
    * "size" server option.  Others may want to set xsize/ysize directly. */
   if (autosize) {
@@ -255,47 +262,33 @@ void generator_init_topology(bool autosize)
     assert(TF_WRAPX == 0x1 && TF_WRAPY == 0x2);
 
     /* Set map.xsize and map.ysize based on map.size. */
-    set_sizes(map.size, default_ratios[id][0], default_ratios[id][1]);
+    set_sizes(map.server.size, default_ratios[id][0], default_ratios[id][1]);
   }
 
   /* initialize the ICE_BASE_LEVEL */
 
-  /*
-   * this is the base value for the isle poles 
-   */ 
-  ice_base_colatitude =  COLD_LEVEL /  3 ;
-
-  /*
-   *if maps has strip like poles we get smaller poles 
+  /* if maps has strip like poles we get smaller poles
    * (less playables than island poles)
-   *  5% for little maps; 2% for big ones, if map.temperature == 50 
-   * exept if separate poles is set
-   */
-  if (!topo_has_flag(TF_WRAPX) || !topo_has_flag(TF_WRAPY)) {
-    int sqsize = get_sqsize();
-
-    if (map.separatepoles) {
-      /* with separatepoles option strip poles are useless */
-      ice_base_colatitude =
-	  (MAX(0, 100 * COLD_LEVEL / 3 - 1 *  MAX_COLATITUDE) 
-	   + 1 *  MAX_COLATITUDE * sqsize) / (100 * sqsize);
-      /* correction for single pole 
-       * TODO uncomment it when generator 5 was well tuned 
-       *      sometime it can put too many land near pole 
-
-      if (topo_has_flag(TF_WRAPX) == topo_has_flag(TF_WRAPY)) {
-	ice_base_colatitude /= 2;
-      }
-
-      */
-    } else {
-      /* any way strip poles are not so playable has isle poles */
-      ice_base_colatitude =
-	  (MAX(0, 100 * COLD_LEVEL / 3 - 2 *  MAX_COLATITUDE) 
-	   + 2 *  MAX_COLATITUDE * sqsize) / (100 * sqsize);
-    }
+   *   5% for little maps
+   *   2% for big ones, if map.server.temperature == 50
+   * exept if separate poles is set */
+  if (map.server.separatepoles) {
+    /* with separatepoles option strip poles are useless */
+    ice_base_colatitude =
+        (MAX(0, 100 * COLD_LEVEL / 3 - 1 *  MAX_COLATITUDE)
+         + 1 *  MAX_COLATITUDE * sqsize) / (100 * sqsize);
+  } else {
+    /* any way strip poles are not so playable as isle poles */
+    ice_base_colatitude =
+        (MAX(0, 100 * COLD_LEVEL / 3 - 2 * MAX_COLATITUDE)
+         + 2 * MAX_COLATITUDE * sqsize) / (100 * sqsize);
   }
- 
+
+  /* correction for single pole (Flat Earth) */
+  if (!topo_has_flag(TF_WRAPX) && !topo_has_flag(TF_WRAPY)) {
+    ice_base_colatitude /= 2;
+  }
+
   map_init_topology(TRUE);
 }
 
