@@ -44,6 +44,7 @@
 #include "connection.h"
 #include "dataio.h"
 #include "fcintl.h"
+#include "game.h"
 #include "log.h"
 #include "mem.h"
 #include "netintf.h"
@@ -51,14 +52,16 @@
 #include "timing.h"
 #include "version.h"
 
+/* server */
 #include "console.h"
+#include "plrhand.h"
 #include "srv_main.h"
 
 #include "meta.h"
 
 static bool server_is_open = FALSE;
 
-static union my_sockaddr meta_addr;
+static union fc_sockaddr meta_addr;
 static char  metaname[MAX_LEN_ADDR];
 static int   metaport;
 static char *metaserver_path;
@@ -111,8 +114,8 @@ const char *get_meta_message_string(void)
 *************************************************************************/
 const char *get_user_meta_message_string(void)
 {
-  if (game.meta_info.user_message_set) {
-    return game.meta_info.user_message;
+  if (game.server.meta_info.user_message_set) {
+    return game.server.meta_info.user_message;
   }
 
   return NULL;
@@ -163,13 +166,13 @@ void set_meta_message_string(const char *string)
 void set_user_meta_message_string(const char *string)
 {
   if (string != NULL && string[0] != '\0') {
-    sz_strlcpy(game.meta_info.user_message, string);
-    game.meta_info.user_message_set = TRUE;
+    sz_strlcpy(game.server.meta_info.user_message, string);
+    game.server.meta_info.user_message_set = TRUE;
     set_meta_message_string(string);
   } else {
     /* Remove user meta message. We will use automatic messages instead */
-    game.meta_info.user_message[0] = '\0';
-    game.meta_info.user_message_set = FALSE;
+    game.server.meta_info.user_message[0] = '\0';
+    game.server.meta_info.user_message_set = FALSE;
     set_meta_message_string(default_meta_message_string());    
   }
 }
@@ -211,17 +214,18 @@ static bool send_to_metaserver(enum meta_flag flag)
     return FALSE;
   }
 
-  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
+  if ((sock = socket(meta_addr.saddr.sa_family, SOCK_STREAM, 0)) == -1) {
     freelog(LOG_ERROR, "Metaserver: can't open stream socket: %s",
-	    mystrerror());
+	    fc_strerror(fc_get_errno()));
     metaserver_failed();
     return FALSE;
   }
 
-  if (my_connect(sock, (struct sockaddr *) &meta_addr, sizeof(meta_addr)) == -1) {
-    freelog(LOG_ERROR, "Metaserver: connect failed: %s", mystrerror());
+  if (fc_connect(sock, &meta_addr.saddr,
+                 sockaddr_size(&meta_addr)) == -1) {
+    freelog(LOG_ERROR, "Metaserver: connect failed: %s", fc_strerror(fc_get_errno()));
     metaserver_failed();
-    my_closesocket(sock);
+    fc_closesocket(sock);
     return FALSE;
   }
 
@@ -241,7 +245,9 @@ static bool send_to_metaserver(enum meta_flag flag)
   }
 
   /* get hostname */
-  if (my_gethostname(host, sizeof(host)) != 0) {
+  if (srvarg.metaserver_name[0] != '\0') {
+    sz_strlcpy(host, srvarg.metaserver_name);
+  } else if (my_gethostname(host, sizeof(host)) != 0) {
     sz_strlcpy(host, "unknown");
   }
 
@@ -252,26 +258,26 @@ static bool send_to_metaserver(enum meta_flag flag)
     mystrlcpy(s, "bye=1&", rest);
     s = end_of_strn(s, &rest);
   } else {
-    my_snprintf(s, rest, "version=%s&", my_url_encode(VERSION_STRING));
+    my_snprintf(s, rest, "version=%s&", fc_url_encode(VERSION_STRING));
     s = end_of_strn(s, &rest);
 
     my_snprintf(s, rest, "patches=%s&", 
-                my_url_encode(get_meta_patches_string()));
+                fc_url_encode(get_meta_patches_string()));
     s = end_of_strn(s, &rest);
 
-    my_snprintf(s, rest, "capability=%s&", my_url_encode(our_capability));
+    my_snprintf(s, rest, "capability=%s&", fc_url_encode(our_capability));
     s = end_of_strn(s, &rest);
 
     my_snprintf(s, rest, "serverid=%s&",
-                my_url_encode(srvarg.serverid));
+                fc_url_encode(srvarg.serverid));
     s = end_of_strn(s, &rest);
 
     my_snprintf(s, rest, "message=%s&",
-                my_url_encode(get_meta_message_string()));
+                fc_url_encode(get_meta_message_string()));
     s = end_of_strn(s, &rest);
 
     /* NOTE: send info for ALL players or none at all. */
-    if (get_num_human_and_ai_players() == 0) {
+    if (normal_player_count() == 0) {
       mystrlcpy(s, "dropplrs=1&", rest);
       s = end_of_strn(s, &rest);
     } else {
@@ -286,43 +292,45 @@ static bool send_to_metaserver(enum meta_flag flag)
           sz_strlcpy(type, "Dead");
         } else if (is_barbarian(plr)) {
           sz_strlcpy(type, "Barbarian");
-        } else if (plr->ai.control) {
+        } else if (plr->ai_data.control) {
           sz_strlcpy(type, "A.I.");
         } else {
           sz_strlcpy(type, "Human");
         }
 
-        my_snprintf(s, rest, "plu[]=%s&", my_url_encode(plr->username));
+        my_snprintf(s, rest, "plu[]=%s&", fc_url_encode(plr->username));
         s = end_of_strn(s, &rest);
 
         my_snprintf(s, rest, "plt[]=%s&", type);
         s = end_of_strn(s, &rest);
 
-        my_snprintf(s, rest, "pll[]=%s&", my_url_encode(player_name(plr)));
+        my_snprintf(s, rest, "pll[]=%s&", fc_url_encode(player_name(plr)));
         s = end_of_strn(s, &rest);
 
         my_snprintf(s, rest, "pln[]=%s&",
-                    my_url_encode(plr->nation != NO_NATION_SELECTED 
+                    fc_url_encode(plr->nation != NO_NATION_SELECTED 
                                   ? nation_plural_for_player(plr)
                                   : "none"));
         s = end_of_strn(s, &rest);
 
         my_snprintf(s, rest, "plh[]=%s&",
-                    pconn ? my_url_encode(pconn->addr) : "");
+                    pconn ? fc_url_encode(pconn->addr) : "");
         s = end_of_strn(s, &rest);
 
         /* is this player available to take?
          * TODO: there's some duplication here with 
          * stdinhand.c:is_allowed_to_take() */
-        if (is_barbarian(plr) && !strchr(game.allow_take, 'b')) {
+        if (is_barbarian(plr) && !strchr(game.server.allow_take, 'b')) {
           is_player_available = FALSE;
-        } else if (!plr->is_alive && !strchr(game.allow_take, 'd')) {
+        } else if (!plr->is_alive && !strchr(game.server.allow_take, 'd')) {
           is_player_available = FALSE;
-        } else if (plr->ai.control
-            && !strchr(game.allow_take, (game.info.is_new_game ? 'A' : 'a'))) {
+        } else if (plr->ai_data.control
+                   && !strchr(game.server.allow_take,
+                              (game.info.is_new_game ? 'A' : 'a'))) {
           is_player_available = FALSE;
-        } else if (!plr->ai.control
-            && !strchr(game.allow_take, (game.info.is_new_game ? 'H' : 'h'))) {
+        } else if (!plr->ai_data.control
+                   && !strchr(game.server.allow_take,
+                              (game.info.is_new_game ? 'H' : 'h'))) {
           is_player_available = FALSE;
         }
 
@@ -343,39 +351,39 @@ static bool send_to_metaserver(enum meta_flag flag)
     /* send some variables: should be listed in inverted order
      * FIXME: these should be input from the settings array */
     my_snprintf(s, rest, "vn[]=%s&vv[]=%d&",
-                my_url_encode("timeout"), game.info.timeout);
+                fc_url_encode("timeout"), game.info.timeout);
     s = end_of_strn(s, &rest);
 
     my_snprintf(s, rest, "vn[]=%s&vv[]=%d&",
-                my_url_encode("year"), game.info.year);
+                fc_url_encode("year"), game.info.year);
     s = end_of_strn(s, &rest);
 
     my_snprintf(s, rest, "vn[]=%s&vv[]=%d&",
-                my_url_encode("turn"), game.info.turn);
+                fc_url_encode("turn"), game.info.turn);
     s = end_of_strn(s, &rest);
 
     my_snprintf(s, rest, "vn[]=%s&vv[]=%d&",
-                my_url_encode("endyear"), game.info.end_year);
+                fc_url_encode("endturn"), game.info.end_turn);
     s = end_of_strn(s, &rest);
 
     my_snprintf(s, rest, "vn[]=%s&vv[]=%d&",
-                my_url_encode("mirules.nplayers"), game.info.min_players);
+                fc_url_encode("minplayers"), game.info.min_players);
     s = end_of_strn(s, &rest);
 
     my_snprintf(s, rest, "vn[]=%s&vv[]=%d&",
-                my_url_encode("maxplayers"), game.info.max_players);
+                fc_url_encode("maxplayers"), game.info.max_players);
     s = end_of_strn(s, &rest);
 
     my_snprintf(s, rest, "vn[]=%s&vv[]=%s&",
-                my_url_encode("allowtake"), game.allow_take);
+                fc_url_encode("allowtake"), game.server.allow_take);
     s = end_of_strn(s, &rest);
 
     my_snprintf(s, rest, "vn[]=%s&vv[]=%d&",
-                my_url_encode("generator"), map.generator);
+                fc_url_encode("generator"), map.server.generator);
     s = end_of_strn(s, &rest);
 
     my_snprintf(s, rest, "vn[]=%s&vv[]=%d&",
-                my_url_encode("size"), map.size);
+                fc_url_encode("size"), map.server.size);
     s = end_of_strn(s, &rest);
   }
 
@@ -393,9 +401,9 @@ static bool send_to_metaserver(enum meta_flag flag)
     str
   );
 
-  my_writesocket(sock, msg, n);
+  fc_writesocket(sock, msg, n);
 
-  my_closesocket(sock);
+  fc_closesocket(sock);
 
   return TRUE;
 }
@@ -411,7 +419,7 @@ void server_close_meta(void)
 /*************************************************************************
  lookup the correct address for the metaserver.
 *************************************************************************/
-void server_open_meta(void)
+bool server_open_meta(void)
 {
   const char *path;
  
@@ -420,17 +428,17 @@ void server_open_meta(void)
     metaserver_path = NULL;
   }
   
-  if (!(path = my_lookup_httpd(metaname, &metaport, srvarg.metaserver_addr))) {
-    return;
+  if (!(path = fc_lookup_httpd(metaname, &metaport, srvarg.metaserver_addr))) {
+    return FALSE;
   }
   
   metaserver_path = mystrdup(path);
 
-  if (!net_lookup_service(metaname, metaport, &meta_addr)) {
-    freelog(LOG_ERROR, _("Metaserver: bad address: [%s:%d]."),
+  if (!net_lookup_service(metaname, metaport, &meta_addr, FALSE)) {
+    freelog(LOG_ERROR, _("Metaserver: bad address: <%s %d>."),
             metaname, metaport);
     metaserver_failed();
-    return;
+    return FALSE;
   }
 
   if (meta_patches[0] == '\0') {
@@ -441,6 +449,8 @@ void server_open_meta(void)
   }
 
   server_is_open = TRUE;
+
+  return TRUE;
 }
 
 /**************************************************************************
