@@ -38,6 +38,7 @@
 #include "citytools.h"
 #include "report.h"
 #include "score.h"
+#include "srv_main.h"
 
 static void page_conn_etype(struct conn_list *dest, const char *caption,
 			    const char *headline, const char *lines,
@@ -53,22 +54,35 @@ enum historian_type {
 #define HISTORIAN_LAST 		HISTORIAN_LARGEST
 
 static const char *historian_message[]={
-    N_("%s report on the RICHEST Civilizations in the World."),
-    N_("%s report on the most ADVANCED Civilizations in the World."),
-    N_("%s report on the most MILITARIZED Civilizations in the World."),
-    N_("%s report on the HAPPIEST Civilizations in the World."),
-    N_("%s report on the LARGEST Civilizations in the World.")
+    /* TRANS: year <name> reports ... */
+    N_("%s %s reports on the RICHEST Civilizations in the World."),
+    /* TRANS: year <name> reports ... */
+    N_("%s %s reports on the most ADVANCED Civilizations in the World."),
+    /* TRANS: year <name> reports ... */
+    N_("%s %s reports on the most MILITARIZED Civilizations in the World."),
+    /* TRANS: year <name> reports ... */
+    N_("%s %s reports on the HAPPIEST Civilizations in the World."),
+    /* TRANS: year <name> reports ... */
+    N_("%s %s reports on the LARGEST Civilizations in the World.")
 };
 
 static const char *historian_name[]={
-    N_("Herodotus'"),
-    N_("Thucydides'"),
-    N_("Pliny the Elder's"),
-    N_("Livy's"),
-    N_("Toynbee's"),
-    N_("Gibbon's"),
-    N_("Ssu-ma Ch'ien's"),
-    N_("Pan Ku's")
+    /* TRANS: [year] <name> [reports ...] */
+    N_("Herodotus"),
+    /* TRANS: [year] <name> [reports ...] */
+    N_("Thucydides"),
+    /* TRANS: [year] <name> [reports ...] */
+    N_("Pliny the Elder"),
+    /* TRANS: [year] <name> [reports ...] */
+    N_("Livy"),
+    /* TRANS: [year] <name> [reports ...] */
+    N_("Toynbee"),
+    /* TRANS: [year] <name> [reports ...] */
+    N_("Gibbon"),
+    /* TRANS: [year] <name> [reports ...] */
+    N_("Ssu-ma Ch'ien"),
+    /* TRANS: [year] <name> [reports ...] */
+    N_("Pan Ku")
 };
 
 static const char scorelog_magic[] = "#FREECIV SCORELOG2 ";
@@ -136,6 +150,40 @@ static struct dem_col {
   char key;
 } coltable[] = {{'q'}, {'r'}, {'b'}}; /* Corresponds to dem_flag enum */
 
+/* prime number of entries makes for better scaling */
+static const char *ranking[] = {
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Supreme %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Magnificent %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Great %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Glorious %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Excellent %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Eminent %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Distinguished %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Average %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Mediocre %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Ordinary %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Pathetic %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Useless %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Valueless %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Worthless %s"),
+  /* TRANS: <#>: The <ranking> Poles */
+  N_("%2d: The Wretched %s"),
+};
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -145,15 +193,6 @@ static int secompare(const void *a, const void *b)
 	  ((const struct player_score_entry *)a)->value);
 }
 
-static const char *greatness[MAX_NUM_PLAYERS] = {
-  N_("Magnificent"),  N_("Glorious"), N_("Great"), N_("Decent"),
-  N_("Mediocre"), N_("Hilarious"), N_("Worthless"), N_("Pathetic"),
-  N_("Useless"), "Useless", "Useless", "Useless", "Useless", "Useless",
-  "Useless", "Useless", "Useless", "Useless", "Useless", "Useless",
-  "Useless", "Useless", "Useless", "Useless", "Useless", "Useless",
-  "Useless", "Useless", "Useless", "Useless"
-};
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -194,17 +233,23 @@ static void historian_generic(enum historian_type which_news)
   qsort(size, j, sizeof(size[0]), secompare);
   buffer[0] = '\0';
   for (i = 0; i < j; i++) {
-    if (i == 0 || size[i].value < size[i - 1].value) {
-      rank = i;
+    if (i > 0 && size[i].value < size[i - 1].value) {
+      /* since i < j, only top entry reigns Supreme */
+      rank = ((i * ARRAY_SIZE(ranking)) / j) + 1;
+    }
+    if (rank >= ARRAY_SIZE(ranking)) {
+      /* clamp to final entry */
+      rank = ARRAY_SIZE(ranking) - 1;
     }
     cat_snprintf(buffer, sizeof(buffer),
-		 _("%2d: The %s %s\n"),
-		 rank + 1,
-		 _(greatness[rank]),
+		 _(ranking[rank]),
+		 i + 1,
 		 nation_plural_for_player(size[i].player));
+    mystrlcat(buffer, "\n", sizeof(buffer));
   }
   my_snprintf(title, sizeof(title), _(historian_message[which_news]),
-    _(historian_name[myrand(ARRAY_SIZE(historian_name))]));
+              textyear(game.info.year),
+              _(historian_name[myrand(ARRAY_SIZE(historian_name))]));
   page_conn_etype(game.est_connections, _("Historian Publishes!"),
 		  title, buffer, E_BROADCAST_REPORT);
 }
@@ -216,11 +261,11 @@ static int nr_wonders(struct city *pcity)
 {
   int result = 0;
 
-  built_impr_iterate(pcity, i) {
+  city_built_iterate(pcity, i) {
     if (is_great_wonder(i)) {
       result++;
     }
-  } built_impr_iterate_end;
+  } city_built_iterate_end;
 
   return result;
 }
@@ -294,27 +339,28 @@ void report_wonders_of_the_world(struct conn_list *dest)
 
   buffer[0] = '\0';
 
-  impr_type_iterate(i) {
+  improvement_iterate(i) {
     if (is_great_wonder(i)) {
       struct city *pcity = find_city_from_great_wonder(i);
 
       if (pcity) {
 	cat_snprintf(buffer, sizeof(buffer), _("%s in %s (%s)\n"),
-		     get_impr_name_ex(pcity, i),
+		     city_improvement_name_translation(pcity, i),
 		     city_name(pcity),
 		     nation_adjective_for_player(city_owner(pcity)));
-      } else if (great_wonder_was_built(i)) {
+      } else if (great_wonder_is_destroyed(i)) {
 	cat_snprintf(buffer, sizeof(buffer), _("%s has been DESTROYED\n"),
 		     improvement_name_translation(i));
       }
     }
-  } impr_type_iterate_end;
+  } improvement_iterate_end;
 
-  impr_type_iterate(i) {
+  improvement_iterate(i) {
     if (is_great_wonder(i)) {
       players_iterate(pplayer) {
 	city_list_iterate(pplayer->cities, pcity) {
-	  if (pcity->production.value == i && !pcity->production.is_unit) {
+	  if (VUT_IMPROVEMENT == pcity->production.kind
+	   && pcity->production.value.building == i) {
 	    cat_snprintf(buffer, sizeof(buffer),
 			 _("(building %s in %s (%s))\n"),
 			 improvement_name_translation(i),
@@ -324,7 +370,7 @@ void report_wonders_of_the_world(struct conn_list *dest)
 	} city_list_iterate_end;
       } players_iterate_end;
     }
-  } impr_type_iterate_end;
+  } improvement_iterate_end;
 
   page_conn(dest, _("Traveler's Report:"),
 	    _("Wonders of the World"), buffer);
@@ -510,7 +556,7 @@ static int get_specialists(struct player *pplayer)
 
 static int get_gov(struct player *pplayer)
 {
-  return government_of_player(pplayer)->index;
+  return government_number(government_of_player(pplayer));
 }
 
 static int get_corruption(struct player *pplayer)
@@ -524,6 +570,11 @@ static int get_corruption(struct player *pplayer)
   return result;
 }
 
+static int get_total_score(struct player *pplayer)
+{
+  return pplayer->score.game;
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -544,7 +595,8 @@ static const char *value_units(int val, const char *uni)
 **************************************************************************/
 static const char *area_to_text(int value)
 {
-  return value_units(value, _(" sq. mi."));
+  /* TRANS: abbreviation of "square miles" */
+  return value_units(value, PL_(" sq. mi.", " sq. mi.", value));
 }
 
 static const char *percent_to_text(int value)
@@ -554,17 +606,20 @@ static const char *percent_to_text(int value)
 
 static const char *production_to_text(int value)
 {
-  return value_units(MAX(0, value), _(" M tons"));
+  int clip = MAX(0, value);
+  /* TRANS: "M tons" = million tons, so always plural */
+  return value_units(clip, PL_(" M tons", " M tons", clip));
 }
 
 static const char *economics_to_text(int value)
 {
-  return value_units(value, _(" M goods"));
+  /* TRANS: "M goods" = million goods, so always plural */
+  return value_units(value, PL_(" M goods", " M goods", value));
 }
 
 static const char *science_to_text(int value)
 {
-  return value_units(value, _(" bulbs"));
+  return value_units(value, PL_(" bulb", " bulbs", value));
 }
 
 static const char *mil_service_to_text(int value)
@@ -714,18 +769,18 @@ bool is_valid_demography(const char *demography,
 *************************************************************************/
 void report_demographics(struct connection *pconn)
 {
-  struct player *pplayer = pconn->player;
   char civbuf[1024];
   char buffer[4096];
   unsigned int i;
   bool anyrows;
   bv_cols selcols;
   int numcols = 0;
+  struct player *pplayer = pconn->playing;
 
   BV_CLR_ALL(selcols);
   assert(ARRAY_SIZE(coltable) == DEM_COL_LAST);
   for (i = 0; i < DEM_COL_LAST; i++) {
-    if (strchr(game.demography, coltable[i].key)) {
+    if (strchr(game.server.demography, coltable[i].key)) {
       BV_SET(selcols, i);
       numcols++;
     }
@@ -733,7 +788,7 @@ void report_demographics(struct connection *pconn)
 
   anyrows = FALSE;
   for (i = 0; i < ARRAY_SIZE(rowtable); i++) {
-    if (strchr(game.demography, rowtable[i].key)) {
+    if (strchr(game.server.demography, rowtable[i].key)) {
       anyrows = TRUE;
       break;
     }
@@ -754,7 +809,7 @@ void report_demographics(struct connection *pconn)
 
   buffer[0] = '\0';
   for (i = 0; i < ARRAY_SIZE(rowtable); i++) {
-    if (strchr(game.demography, rowtable[i].key)) {
+    if (strchr(game.server.demography, rowtable[i].key)) {
       const char *name = _(rowtable[i].name);
 
       cat_snprintf(buffer, sizeof(buffer), "%s", name);
@@ -813,10 +868,10 @@ static bool scan_score_log(FILE * fp, int *last_turn, char *id,
 	freelog(LOG_ERROR, "Multiple ID entries!");
 	return FALSE;
       }
-      mystrlcpy(id, line + strlen("id "), MAX_ID_LEN);
-      if (strcmp(id, game.id) != 0) {
+      mystrlcpy(id, line + strlen("id "), MAX_LEN_GAME_IDENTIFIER);
+      if (strcmp(id, server.game_identifier) != 0) {
 	freelog(LOG_ERROR, "IDs don't match! game='%s' scorelog='%s'",
-		game.id, id);
+		server.game_identifier, id);
 	return FALSE;
       }
     }
@@ -878,9 +933,9 @@ static bool scan_score_log(FILE * fp, int *last_turn, char *id,
 }
 
 /**************************************************************************
-Create a log file of the civilizations so you can see what was happening.
+  Create a log file of the civilizations so you can see what was happening.
 **************************************************************************/
-static void log_civ_score(void)
+void log_civ_score(void)
 {
   static const char logname[] = "civscore.log";
   static FILE *fp = NULL;
@@ -924,12 +979,18 @@ static void log_civ_score(void)
     {"unhappypop",      get_unhappypop},
     {"specialists",     get_specialists},
     {"gov",             get_gov},
-    {"corruption",      get_corruption} /* new 1.11.5 tags end here */
+    {"corruption",      get_corruption}, /* new 1.11.5 tags end here */
+
+    {"score",           get_total_score} /* New 2.1.10 tag */
   };
 
   enum { SL_CREATE, SL_APPEND, SL_UNSPEC } oper = SL_UNSPEC;
   int i;
-  char id[MAX_ID_LEN];
+  char id[MAX_LEN_GAME_IDENTIFIER];
+
+  if (!game.server.scorelog) {
+    return;
+  }
 
   if (!player_name_ptrs[0]) {
     int i;
@@ -948,7 +1009,7 @@ static void log_civ_score(void)
     if (game.info.year == GAME_START_YEAR) {
       oper = SL_CREATE;
     } else {
-      fp = fopen(logname, "r");
+      fp = fc_fopen(logname, "r");
       if (!fp) {
 	oper = SL_CREATE;
       } else {
@@ -964,7 +1025,7 @@ static void log_civ_score(void)
 
     switch (oper) {
     case SL_CREATE:
-      fp = fopen(logname, "w");
+      fp = fc_fopen(logname, "w");
       if (!fp) {
 	freelog(LOG_ERROR, "Can't open scorelog file for creation!");
 	goto log_civ_score_disable;
@@ -976,13 +1037,13 @@ static void log_civ_score(void)
 	      "# <http://svn.gna.org/viewcvs/freeciv/trunk/doc/README.scorelog?view=auto>.\n"
 	      "\n");
 
-      fprintf(fp, "id %s\n", game.id);
+      fprintf(fp, "id %s\n", server.game_identifier);
       for (i = 0; i<ARRAY_SIZE(score_tags); i++) {
 	fprintf(fp, "tag %d %s\n", i, score_tags[i].name);
       }
       break;
     case SL_APPEND:
-      fp = fopen(logname, "a");
+      fp = fc_fopen(logname, "a");
       if (!fp) {
 	freelog(LOG_ERROR, "Can't open scorelog file for appending!");
 	goto log_civ_score_disable;
@@ -1060,33 +1121,34 @@ log_civ_score_disable:
 **************************************************************************/
 void make_history_report(void)
 {
-  if (game.scorelog) {
-    log_civ_score();
-  }
-
   if (player_count() == 1) {
     return;
   }
 
-  if (game.scoreturn > game.info.turn) {
+  if (game.server.scoreturn > game.info.turn) {
     return;
   }
 
-  game.scoreturn = game.info.turn + GAME_DEFAULT_SCORETURN
-                 + myrand(GAME_DEFAULT_SCORETURN);
+  game.server.scoreturn = (game.info.turn + GAME_DEFAULT_SCORETURN
+                           + myrand(GAME_DEFAULT_SCORETURN));
 
-  historian_generic(game.scoreturn % HISTORIAN_LAST);
+  historian_generic(game.server.scoreturn % HISTORIAN_LAST);
 }
 
 /**************************************************************************
   Inform clients about player scores and statistics when the game ends.
+  Called only from server/srv_main.c srv_scores()
 **************************************************************************/
-void report_final_scores(void)
+void report_final_scores(struct conn_list *dest)
 {
   int i, j = 0;
   struct player_score_entry size[player_count()];
   struct packet_endgame_report packet;
 
+  if (!dest) {
+    dest = game.est_connections;
+  }
+
   players_iterate(pplayer) {
     if (GOOD_PLAYER(pplayer)) {
       size[j].value = pplayer->score.game;
@@ -1105,7 +1167,7 @@ void report_final_scores(void)
     packet.bnp[i] = get_economics(size[i].player); 
     packet.mfg[i] = get_production(size[i].player); 
     packet.cities[i] = get_cities(size[i].player); 
-    packet.techs[i] = get_techs(size[i].player) - 1; 
+    packet.techs[i] = get_techs(size[i].player);
     packet.mil_service[i] = get_mil_service(size[i].player); 
     packet.wonders[i] = get_wonders(size[i].player); 
     packet.research[i] = get_research(size[i].player); 
@@ -1115,7 +1177,7 @@ void report_final_scores(void)
     packet.spaceship[i] = get_spaceship(size[i].player); 
   }  
 
-  lsend_packet_endgame_report(game.est_connections, &packet);
+  lsend_packet_endgame_report(dest, &packet);
 }	
 
 /**************************************************************************
