@@ -21,6 +21,7 @@ struct conn_list;
 void page_conn(struct conn_list *dest, const char *caption, const char *headline,
 	       const char *lines);
 
+void log_civ_score(void);
 void make_history_report(void);
 void report_wonders_of_the_world(struct conn_list *dest);
 void report_top_five_cities(struct conn_list *dest);
@@ -28,8 +29,6 @@ bool is_valid_demography(const char *demography,
                          struct connection *caller,
                          const char **error_message);
 void report_demographics(struct connection *pconn);
-void report_final_scores(void);
-
-/* See also report_server_options() in stdinhand.h */
+void report_final_scores(struct conn_list *dest);
 
 #endif  /* FC__REPORT_H */
