@@ -18,18 +18,23 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "improvement.h"
 #include "log.h"
-#include "map.h"
 #include "mem.h"
-#include "player.h"
-#include "srv_main.h"
 #include "shared.h"
-#include "support.h"
-#include "terrain.h"
-#include "score.h"
-#include "unit.h"
 
+#include "game.h"
+#include "improvement.h"
+#include "map.h"
+#include "player.h"
+#include "specialist.h"
+#include "unit.h"
+#include "unitlist.h"
+
+#include "score.h"
+#include "srv_main.h"
+
+static int get_civ_score(const struct player *pplayer);
+static int get_spaceship_score(const struct player *pplayer);
 
 /**************************************************************************
   Allocates, fills and returns a land area claim map.
@@ -38,18 +43,10 @@
 
 #define USER_AREA_MULT 1000
 
-struct claim_cell {
-  int when;
-  int whom;
-  int know;
-  int cities;
-};
-
 struct claim_map {
-  struct claim_cell *claims;
-  int *player_landarea;
-  int *player_owndarea;
-  struct tile **edges;
+  struct {
+    int landarea, settledarea;
+  } player[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
 };
 
 /**************************************************************************
@@ -102,7 +99,7 @@ static char when_char(int when)
 
 /**************************************************************************
   Prints the landarea map to stdout (a debugging tool).
-**************************************************************************/
+*******************************************o*******************************/
 static void print_landarea_map(struct claim_map *pcmap, int turn)
 {
   int p;
@@ -114,7 +111,7 @@ static void print_landarea_map(struct claim_map *pcmap, int turn)
   if (turn == 0) {
     printf("Player Info...\n");
 
-    for (p = 0; p < game.nplayers; p++) {
+    for (p = 0; p < player_count(); p++) {
       printf(".know (%d)\n  ", p);
       WRITE_MAP_DATA("%c",
 		     TEST_BIT(pcmap->claims[map_pos_to_index(x, y)].know,
@@ -141,200 +138,62 @@ static void print_landarea_map(struct claim_map *pcmap, int turn)
 
 #endif
 
-static int no_owner = MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS;
-
-/**************************************************************************
-  allocate and clear claim map; determine city radii
-**************************************************************************/
-static void build_landarea_map_new(struct claim_map *pcmap)
+/****************************************************************************
+  Count landarea, settled area, and claims map for all players.
+****************************************************************************/
+static void build_landarea_map(struct claim_map *pcmap)
 {
-  int nbytes;
+  bv_player claims[MAP_INDEX_SIZE];
 
-  nbytes = map.xsize * map.ysize * sizeof(struct claim_cell);
-  pcmap->claims = fc_malloc(nbytes);
-  memset(pcmap->claims, 0, nbytes);
-
-  nbytes = game.nplayers * sizeof(int);
-  pcmap->player_landarea = fc_malloc(nbytes);
-  memset(pcmap->player_landarea, 0, nbytes);
-
-  nbytes = game.nplayers * sizeof(int);
-  pcmap->player_owndarea = fc_malloc(nbytes);
-  memset(pcmap->player_owndarea, 0, nbytes);
-
-  nbytes = 2 * map.xsize * map.ysize * sizeof(*pcmap->edges);
-  pcmap->edges = fc_malloc(nbytes);
+  memset(claims, 0, sizeof(claims));
+  memset(pcmap, 0, sizeof(*pcmap));
 
+  /* First calculate claims: which tiles are owned by each player. */
   players_iterate(pplayer) {
     city_list_iterate(pplayer->cities, pcity) {
-      map_city_radius_iterate(pcity->tile, tile1) {
-	pcmap->claims[tile1->index].cities |= (1u << pcity->owner);
-      } map_city_radius_iterate_end;
+      struct tile *pcenter = city_tile(pcity);
+
+      city_tile_iterate(pcenter, tile1) {
+	BV_SET(claims[tile_index(tile1)], player_index(city_owner(pcity)));
+      } city_tile_iterate_end;
     } city_list_iterate_end;
   } players_iterate_end;
-}
-
-/**************************************************************************
-  0th turn: install starting points, counting their tiles
-**************************************************************************/
-static void build_landarea_map_turn_0(struct claim_map *pcmap)
-{
-  int turn, owner;
-  struct tile **nextedge;
-  struct claim_cell *pclaim;
-
-  turn = 0;
-  nextedge = pcmap->edges;
 
   whole_map_iterate(ptile) {
-    int i = ptile->index;
+    struct player *owner = NULL;
+    bv_player *pclaim = &claims[tile_index(ptile)];
 
-    pclaim = &pcmap->claims[i];
-    ptile = &map.tiles[i];
-
-    if (is_ocean(ptile->terrain)) {
-      /* pclaim->when = 0; */
-      pclaim->whom = no_owner;
-      /* pclaim->know = 0; */
-    } else if (ptile->city) {
-      owner = ptile->city->owner;
-      pclaim->when = turn + 1;
-      pclaim->whom = owner;
-      *nextedge = ptile;
-      nextedge++;
-      pcmap->player_landarea[owner]++;
-      pcmap->player_owndarea[owner]++;
-      pclaim->know = ptile->known;
-    } else if (ptile->worked) {
-      owner = ptile->worked->owner;
-      pclaim->when = turn + 1;
-      pclaim->whom = owner;
-      *nextedge = ptile;
-      nextedge++;
-      pcmap->player_landarea[owner]++;
-      pcmap->player_owndarea[owner]++;
-      pclaim->know = ptile->known;
-    } else if (unit_list_size(&ptile->units) > 0) {
-      owner = (unit_list_get(&ptile->units, 0))->owner;
-      pclaim->when = turn + 1;
-      pclaim->whom = owner;
-      *nextedge = ptile;
-      nextedge++;
-      pcmap->player_landarea[owner]++;
-      if (TEST_BIT(pclaim->cities, owner)) {
-	pcmap->player_owndarea[owner]++;
+    if (is_ocean_tile(ptile)) {
+      /* Nothing. */
+    } else if (NULL != tile_city(ptile)) {
+      owner = city_owner(tile_city(ptile));
+      pcmap->player[player_index(owner)].settledarea++;
+    } else if (NULL != tile_worked(ptile)) {
+      owner = city_owner(tile_worked(ptile));
+      pcmap->player[player_index(owner)].settledarea++;
+    } else if (unit_list_size(ptile->units) > 0) {
+      /* Because of allied stacking these calculations are a bit off. */
+      owner = unit_owner(unit_list_get(ptile->units, 0));
+      if (BV_ISSET(*pclaim, player_index(owner))) {
+	pcmap->player[player_index(owner)].settledarea++;
       }
-      pclaim->know = ptile->known;
-    } else {
-      /* pclaim->when = 0; */
-      pclaim->whom = no_owner;
-      pclaim->know = ptile->known;
+    }
+
+    if (game.info.borders > 0) {
+      /* If borders are enabled, use owner information directly from the
+       * map.  Otherwise use the calculations above. */
+      owner = tile_owner(ptile);
+    }
+    if (owner) {
+      pcmap->player[player_index(owner)].landarea++;
     }
   } whole_map_iterate_end;
 
-  *nextedge = NULL;
-
 #if LAND_AREA_DEBUG >= 2
   print_landarea_map(pcmap, turn);
 #endif
 }
 
-/**************************************************************************
-  expand outwards evenly from each starting point, counting tiles
-**************************************************************************/
-static void build_landarea_map_expand(struct claim_map *pcmap)
-{
-  struct tile **midedge;
-  int turn, accum, other;
-  struct tile **thisedge;
-  struct tile **nextedge;
-
-  midedge = &pcmap->edges[map.xsize * map.ysize];
-
-  for (accum = 1, turn = 1; accum > 0; turn++) {
-    thisedge = ((turn & 0x1) == 1) ? pcmap->edges : midedge;
-    nextedge = ((turn & 0x1) == 1) ? midedge : pcmap->edges;
-
-    for (accum = 0; *thisedge; thisedge++) {
-      struct tile *ptile = *thisedge;
-      int i = ptile->index;
-      int owner = pcmap->claims[i].whom;
-
-      if (owner != no_owner) {
-	adjc_iterate(ptile, tile1) {
-	  int j = tile1->index;
-	  struct claim_cell *pclaim = &pcmap->claims[j];
-
-	  if (TEST_BIT(pclaim->know, owner)) {
-	    if (pclaim->when == 0) {
-	      pclaim->when = turn + 1;
-	      pclaim->whom = owner;
-	      *nextedge = tile1;
-	      nextedge++;
-	      pcmap->player_landarea[owner]++;
-	      if (TEST_BIT(pclaim->cities, owner)) {
-		pcmap->player_owndarea[owner]++;
-	      }
-	      accum++;
-	    } else if (pclaim->when == turn + 1
-		       && pclaim->whom != no_owner
-		       && pclaim->whom != owner) {
-	      other = pclaim->whom;
-	      if (TEST_BIT(pclaim->cities, other)) {
-		pcmap->player_owndarea[other]--;
-	      }
-	      pcmap->player_landarea[other]--;
-	      pclaim->whom = no_owner;
-	      accum--;
-	    }
-	  }
-	} adjc_iterate_end;
-      }
-    }
-
-    *nextedge = NULL;
-
-#if LAND_AREA_DEBUG >= 2
-    print_landarea_map(pcmap, turn);
-#endif
-  }
-}
-
-/**************************************************************************
-  this just calls the three worker routines
-**************************************************************************/
-static void build_landarea_map(struct claim_map *pcmap)
-{
-  build_landarea_map_new(pcmap);
-  build_landarea_map_turn_0(pcmap);
-  build_landarea_map_expand(pcmap);
-}
-
-/**************************************************************************
-  Frees and NULLs an allocated claim map.
-**************************************************************************/
-static void free_landarea_map(struct claim_map *pcmap)
-{
-  if (pcmap) {
-    if (pcmap->claims) {
-      free(pcmap->claims);
-      pcmap->claims = NULL;
-    }
-    if (pcmap->player_landarea) {
-      free(pcmap->player_landarea);
-      pcmap->player_landarea = NULL;
-    }
-    if (pcmap->player_owndarea) {
-      free(pcmap->player_owndarea);
-      pcmap->player_owndarea = NULL;
-    }
-    if (pcmap->edges) {
-      free(pcmap->edges);
-      pcmap->edges = NULL;
-    }
-  }
-}
-
 /**************************************************************************
   Returns the given player's land and settled areas from a claim map.
 **************************************************************************/
@@ -345,18 +204,18 @@ static void get_player_landarea(struct claim_map *pcmap,
 {
   if (pcmap && pplayer) {
 #if LAND_AREA_DEBUG >= 1
-    printf("%-14s", pplayer->name);
+    printf("%-14s", player_name(pplayer));
 #endif
-    if (return_landarea && pcmap->player_landarea) {
-      *return_landarea =
-	USER_AREA_MULT * pcmap->player_landarea[pplayer->player_no];
+    if (return_landarea) {
+      *return_landarea
+	= USER_AREA_MULT * pcmap->player[player_index(pplayer)].landarea;
 #if LAND_AREA_DEBUG >= 1
       printf(" l=%d", *return_landarea / USER_AREA_MULT);
 #endif
     }
-    if (return_settledarea && pcmap->player_owndarea) {
-      *return_settledarea =
-	USER_AREA_MULT * pcmap->player_owndarea[pplayer->player_no];
+    if (return_settledarea) {
+      *return_settledarea
+	= USER_AREA_MULT * pcmap->player[player_index(pplayer)].settledarea;
 #if LAND_AREA_DEBUG >= 1
       printf(" s=%d", *return_settledarea / USER_AREA_MULT);
 #endif
@@ -374,15 +233,15 @@ void calc_civ_score(struct player *pplayer)
 {
   struct city *pcity;
   int landarea = 0, settledarea = 0;
-  static struct claim_map cmap = { NULL, NULL, NULL,NULL };
+  static struct claim_map cmap;
 
   pplayer->score.happy = 0;
   pplayer->score.content = 0;
   pplayer->score.unhappy = 0;
   pplayer->score.angry = 0;
-  pplayer->score.taxmen = 0;
-  pplayer->score.scientists = 0;
-  pplayer->score.elvis = 0;
+  specialist_type_iterate(sp) {
+    pplayer->score.specialists[sp] = 0;
+  } specialist_type_iterate_end;
   pplayer->score.wonders = 0;
   pplayer->score.techs = 0;
   pplayer->score.techout = 0;
@@ -398,50 +257,43 @@ void calc_civ_score(struct player *pplayer)
   pplayer->score.spaceship = 0;
 
   if (is_barbarian(pplayer)) {
-    if (pplayer->player_no == game.nplayers - 1) {
-      free_landarea_map(&cmap);
-    }
     return;
   }
 
   city_list_iterate(pplayer->cities, pcity) {
     int bonus;
 
-    pplayer->score.happy += pcity->ppl_happy[4];
-    pplayer->score.content += pcity->ppl_content[4];
-    pplayer->score.unhappy += pcity->ppl_unhappy[4];
-    pplayer->score.angry += pcity->ppl_angry[4];
-    pplayer->score.taxmen += pcity->specialists[SP_TAXMAN];
-    pplayer->score.scientists += pcity->specialists[SP_SCIENTIST];
-    pplayer->score.elvis += pcity->specialists[SP_ELVIS];
+    pplayer->score.happy += pcity->feel[CITIZEN_HAPPY][FEELING_FINAL];
+    pplayer->score.content += pcity->feel[CITIZEN_CONTENT][FEELING_FINAL];
+    pplayer->score.unhappy += pcity->feel[CITIZEN_UNHAPPY][FEELING_FINAL];
+    pplayer->score.angry += pcity->feel[CITIZEN_ANGRY][FEELING_FINAL];
+    specialist_type_iterate(sp) {
+      pplayer->score.specialists[sp] += pcity->specialists[sp];
+    } specialist_type_iterate_end;
     pplayer->score.population += city_population(pcity);
     pplayer->score.cities++;
     pplayer->score.pollution += pcity->pollution;
-    pplayer->score.techout += pcity->science_total;
-    pplayer->score.bnp += pcity->trade_prod;
-    pplayer->score.mfg += pcity->shield_surplus;
+    pplayer->score.techout += pcity->prod[O_SCIENCE];
+    pplayer->score.bnp += pcity->surplus[O_TRADE];
+    pplayer->score.mfg += pcity->surplus[O_SHIELD];
 
-    bonus = CLIP(0, get_city_bonus(pcity, EFT_SCIENCE_BONUS), 100);
+    bonus = get_final_city_output_bonus(pcity, O_SCIENCE) - 100;
+    bonus = CLIP(0, bonus, 100);
     pplayer->score.literacy += (city_population(pcity) * bonus) / 100;
   } city_list_iterate_end;
 
-  if (pplayer->player_no == 0) {
-    free_landarea_map(&cmap);
-    build_landarea_map(&cmap);
-  }
+  build_landarea_map(&cmap);
+
   get_player_landarea(&cmap, pplayer, &landarea, &settledarea);
   pplayer->score.landarea = landarea;
   pplayer->score.settledarea = settledarea;
-  if (pplayer->player_no == game.nplayers - 1) {
-    free_landarea_map(&cmap);
-  }
 
-  tech_type_iterate(i) {
-    if (get_invention(pplayer, i)==TECH_KNOWN) {
+  advance_index_iterate(A_FIRST, i) {
+    if (player_invention_state(pplayer, i) == TECH_KNOWN) {
       pplayer->score.techs++;
     }
-  } tech_type_iterate_end;
-  pplayer->score.techs += pplayer->future_tech * 5 / 2;
+  } advance_index_iterate_end;
+  pplayer->score.techs += get_player_research(pplayer)->future_tech * 5 / 2;
   
   unit_list_iterate(pplayer->units, punit) {
     if (is_military_unit(punit)) {
@@ -449,33 +301,45 @@ void calc_civ_score(struct player *pplayer)
     }
   } unit_list_iterate_end
 
-  impr_type_iterate(i) {
-    if (is_wonder(i)
-	&& (pcity = find_city_by_id(game.global_wonders[i]))
+  improvement_iterate(i) {
+    if (is_great_wonder(i)
+	&& (pcity = find_city_from_great_wonder(i))
 	&& player_owns_city(pplayer, pcity)) {
       pplayer->score.wonders++;
     }
-  } impr_type_iterate_end;
+  } improvement_iterate_end;
 
-  /* How much should a spaceship be worth?
-   * This gives 100 points per 10,000 citizens. */
-  if (pplayer->spaceship.state == SSHIP_ARRIVED) {
-    pplayer->score.spaceship += (int)(100 * pplayer->spaceship.habitation
-				      * pplayer->spaceship.success_rate);
-  }
+  pplayer->score.spaceship = pplayer->spaceship.state;
+
+  pplayer->score.game = get_civ_score(pplayer);
 }
 
 /**************************************************************************
   Return the civilization score (a numerical value) for the player.
 **************************************************************************/
-int get_civ_score(const struct player *pplayer)
+static int get_civ_score(const struct player *pplayer)
 {
   /* We used to count pplayer->score.happy here too, but this is too easily
-   * manipulated by players at the endyear. */
+   * manipulated by players at the endrturn. */
   return (total_player_citizens(pplayer)
 	  + pplayer->score.techs * 2
 	  + pplayer->score.wonders * 5
-	  + pplayer->score.spaceship);
+	  + get_spaceship_score(pplayer));
+}
+
+/**************************************************************************
+  Return the spaceship score
+**************************************************************************/
+static int get_spaceship_score(const struct player *pplayer)
+{
+  if (pplayer->score.spaceship == SSHIP_ARRIVED) {
+    /* How much should a spaceship be worth?
+     * This gives 100 points per 10,000 citizens. */
+    return (int)(100 * pplayer->spaceship.habitation
+		 * pplayer->spaceship.success_rate);
+  } else {
+    return 0;
+  }
 }
 
 /**************************************************************************
@@ -483,13 +347,16 @@ int get_civ_score(const struct player *pplayer)
 **************************************************************************/
 int total_player_citizens(const struct player *pplayer)
 {
-  return (pplayer->score.happy
-	  + pplayer->score.content
-	  + pplayer->score.unhappy
-	  + pplayer->score.angry
-	  + pplayer->score.scientists
-	  + pplayer->score.elvis
-	  + pplayer->score.taxmen);
+  int count = (pplayer->score.happy
+	       + pplayer->score.content
+	       + pplayer->score.unhappy
+	       + pplayer->score.angry);
+
+  specialist_type_iterate(sp) {
+    count += pplayer->score.specialists[sp];
+  } specialist_type_iterate_end;
+
+  return count;
 }
 
 /**************************************************************************
@@ -526,7 +393,7 @@ void save_ppm(void)
 
   /* put this file in the same place we put savegames */
   my_snprintf(filename, sizeof(filename),
-              "%s%+05d.int.ppm", game.save_name, game.year);
+              "%s%+05d.int.ppm", game.server.save_name, game.info.year);
 
   /* Ensure the saves directory exists. */
   make_dir(srvarg.saves_pathname);
@@ -538,24 +405,25 @@ void save_ppm(void)
   sz_strlcat(tmpname, filename);
   sz_strlcpy(filename, tmpname);
 
-  fp = fopen(filename, "w");
+  fp = fc_fopen(filename, "w");
 
   if (!fp) {
     freelog(LOG_ERROR, "couldn't open file ppm save: %s\n", filename);
     return;
   }
 
-  fprintf(fp, "P3\n# version:2\n# gameid: %s\n", game.id);
+  fprintf(fp, "P3\n# version:2\n# gameid: %s\n", server.game_identifier);
   fprintf(fp, "# An intermediate map from saved Freeciv game %s%+05d\n",
-          game.save_name, game.year);
+          game.server.save_name, game.info.year);
 
-
-  for (i = 0; i < game.nplayers; i++) {
-    struct player *pplayer = get_player(i);
+  players_iterate(pplayer) {
     fprintf(fp, "# playerno:%d:color:#%02x%02x%02x:name:\"%s\"\n", 
-            pplayer->player_no, col[i][0], col[i][1], col[i][2],
-            pplayer->name);
-  }
+            player_number(pplayer),
+            col[player_index(pplayer)][0],
+            col[player_index(pplayer)][1],
+            col[player_index(pplayer)][2],
+            player_name(pplayer));
+  } players_iterate_end;
 
   fprintf(fp, "%d %d\n", map.xsize, map.ysize);
   fprintf(fp, "255\n");
@@ -566,11 +434,11 @@ void save_ppm(void)
        int *color;
 
        /* color for cities first, then units, then land */
-       if (ptile->city) {
-         color = col[city_owner(ptile->city)->player_no];
-       } else if (unit_list_size(&ptile->units) > 0) {
-         color = col[unit_owner(unit_list_get(&ptile->units, 0))->player_no];
-       } else if (is_ocean(ptile->terrain)) {
+       if (tile_city(ptile)) {
+         color = col[player_index(city_owner(tile_city(ptile)))];
+       } else if (unit_list_size(ptile->units) > 0) {
+         color = col[player_index(unit_owner(unit_list_get(ptile->units, 0)))];
+       } else if (is_ocean_tile(ptile)) {
          color = watercol;
        } else {
          color = landcol;
@@ -582,3 +450,113 @@ void save_ppm(void)
 
   fclose(fp);
 }
+
+/**************************************************************************
+  At the end of a game, figure the winners and losers of the game and
+  output to a suitable place.
+
+  The definition of winners and losers: a winner is one who is alive at the
+  end of the game and has not surrendered, or in the case of a team game, 
+  is alive or a teammate is alive and has not surrendered. A loser is
+  surrendered or dead. Exception: the winner of the spacerace and his 
+  teammates will win of course.
+
+  Barbarians do not count as winners or losers.
+
+  Games ending by endturn results in a draw, we don't rank in that case.
+**************************************************************************/
+void rank_users(void)
+{
+  FILE *fp;
+  int i;
+  enum victory_state { VS_NONE, VS_LOSER, VS_WINNER };
+  enum victory_state plr_state[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
+  struct player *spacerace_winner = NULL;
+
+  /* game ending via endturn results in a draw. We don't rank. */
+  if (game.info.turn > game.info.end_turn) {
+    return;
+  }
+
+  /* don't output ranking info if we haven't enabled it via cmdline */
+  if (!srvarg.ranklog_filename) {
+    return;
+  }
+
+  fp = fc_fopen(srvarg.ranklog_filename,"w");
+
+  /* don't fail silently, at least print an error */
+  if (!fp) {
+    freelog(LOG_ERROR, "couldn't open ranking log file: \"%s\"",
+            srvarg.ranklog_filename);
+    return;
+  }
+
+  /* initialize plr_state */
+  for (i = 0; i < MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS; i++) {
+    plr_state[i] = VS_NONE;
+  }
+
+  /* do we have a spacerace winner? */
+  players_iterate(pplayer) {
+    if (pplayer->spaceship.state == SSHIP_ARRIVED) {
+      spacerace_winner = pplayer;
+      break;
+    }
+  } players_iterate_end;
+
+  /* make this easy: if we have a spacerace winner, then treat all others
+   * who are still alive as surrendered */
+  if (spacerace_winner) {
+    players_iterate(pplayer) {
+      if (pplayer != spacerace_winner) {
+        pplayer->surrendered = TRUE;
+      }
+    } players_iterate_end;
+  }
+
+  /* first pass: locate those alive who haven't surrendered, set them to win;
+   * barbarians won't count, and everybody else is a loser for now. */
+  players_iterate(pplayer) {
+    if (is_barbarian(pplayer)) {
+      plr_state[player_index(pplayer)] = VS_NONE;
+    } else if (pplayer->is_alive && !pplayer->surrendered) {
+      plr_state[player_index(pplayer)] = VS_WINNER;
+    } else {
+      plr_state[player_index(pplayer)] = VS_LOSER;
+    }
+  } players_iterate_end;
+
+  /* second pass: find the teammates of those winners, they win too. */
+  players_iterate(pplayer) {
+    if (plr_state[player_index(pplayer)] == VS_WINNER) {
+      players_iterate(aplayer) {
+        if (aplayer->team == pplayer->team) {
+          plr_state[player_index(aplayer)] = VS_WINNER;
+        }
+      } players_iterate_end;
+    }
+  } players_iterate_end;
+
+  /* write out ranking information to file */
+  fprintf(fp, "turns: %d\n", game.info.turn);
+  fprintf(fp, "winners: ");
+  players_iterate(pplayer) {
+    if (plr_state[player_index(pplayer)] == VS_WINNER) {
+      fprintf(fp, "%s (%s,%s), ", pplayer->ranked_username,
+                                  player_name(pplayer),
+                                  pplayer->username);
+    }
+  } players_iterate_end;
+  fprintf(fp, "\nlosers: ");
+  players_iterate(pplayer) {
+    if (plr_state[player_index(pplayer)] == VS_LOSER) {
+      fprintf(fp, "%s (%s,%s), ", pplayer->ranked_username,
+                                  player_name(pplayer),
+                                  pplayer->username);
+    }
+  } players_iterate_end;
+  fprintf(fp, "\n");
+
+  fclose(fp);
+}
