@@ -17,9 +17,10 @@
 
 void calc_civ_score(struct player *pplayer);
 
-int get_civ_score(const struct player *pplayer);
 int total_player_citizens(const struct player *pplayer);
 
 void save_ppm(void);
 
+void rank_users(void);
+
 #endif /* FC__SCORE_H */
