@@ -66,6 +66,7 @@
 #include "dataio.h"
 #include "events.h"
 #include "fcintl.h"
+#include "game.h"
 #include "log.h"
 #include "mem.h"
 #include "netintf.h"
@@ -82,6 +83,7 @@
 #include "plrhand.h"
 #include "srv_main.h"
 #include "stdinhand.h"
+#include "voting.h"
 
 #include "sernet.h"
 
@@ -126,7 +128,6 @@ static int server_accept_connection(int sockfd);
 static void start_processing_request(struct connection *pconn,
 				     int request_id);
 static void finish_processing_request(struct connection *pconn);
-static void ping_connection(struct connection *pconn);
 static void send_ping_times_to_all(void);
 
 static void get_lanserver_announcement(void);
@@ -203,11 +204,12 @@ void close_connection(struct connection *pconn)
     return;
   }
 
+  cancel_connection_votes(pconn);
+
   if (pconn->server.ping_timers != NULL) {
     timer_list_iterate(pconn->server.ping_timers, timer) {
       free_timer(timer);
     } timer_list_iterate_end;
-    timer_list_unlink_all(pconn->server.ping_timers);
     timer_list_free(pconn->server.ping_timers);
     pconn->server.ping_timers = NULL;
   }
@@ -216,9 +218,11 @@ void close_connection(struct connection *pconn)
   conn_list_unlink(game.all_connections, pconn);
   conn_list_unlink(game.est_connections, pconn);
 
-  pconn->player = NULL;
+  pconn->playing = NULL;
   pconn->access_level = ALLOW_NONE;
   connection_common_close(pconn);
+
+  send_updated_vote_totals(NULL);
 }
 
 /*****************************************************************************
@@ -233,7 +237,6 @@ void close_connections_and_socket(void)
     if(connections[i].used) {
       close_connection(&connections[i]);
     }
-    conn_list_unlink_all(connections[i].self);
     conn_list_free(connections[i].self);
   }
 
@@ -241,8 +244,8 @@ void close_connections_and_socket(void)
   conn_list_free(game.all_connections);
   conn_list_free(game.est_connections);
 
-  my_closesocket(sock);
-  my_closesocket(socklan);
+  fc_closesocket(sock);
+  fc_closesocket(socklan);
 
 #ifdef HAVE_LIBREADLINE
   if (history_file) {
@@ -254,7 +257,7 @@ void close_connections_and_socket(void)
   send_server_info_to_metaserver(META_GOODBYE);
   server_close_meta();
 
-  my_shutdown_network();
+  fc_shutdown_network();
 }
 
 /*****************************************************************************
@@ -331,7 +334,7 @@ void flush_packets(void)
       return;
     }
 
-    if(my_select(max_desc+1, NULL, &writefs, &exceptfs, &tv)<=0) {
+    if(fc_select(max_desc+1, NULL, &writefs, &exceptfs, &tv)<=0) {
       return;
     }
 
@@ -495,9 +498,9 @@ enum server_events server_sniff_all_input(void)
       if (connections && conn_list_size(game.est_connections) == 0) {
 	if (last_noplayers != 0) {
 	  if (time(NULL) > last_noplayers + srvarg.quitidle) {
-	    save_game_auto("Lost all connections");
-	    set_meta_message_string("restarting for lack of players");
-	    freelog(LOG_NORMAL, "%s", get_meta_message_string());
+	    save_game_auto("Lost all connections", "quitidle");
+	    set_meta_message_string(N_("restarting for lack of players"));
+	    freelog(LOG_NORMAL, "%s", Q_(get_meta_message_string()));
 	    (void) send_server_info_to_metaserver(META_INFO);
 
             set_server_state(S_S_OVER);
@@ -516,14 +519,13 @@ enum server_events server_sniff_all_input(void)
             connections = FALSE;
 	  }
 	} else {
-          char buf[256];
 	  last_noplayers = time(NULL);
-	  
-	  my_snprintf(buf, sizeof(buf),
-		      "restarting in %d seconds for lack of players",
-		      srvarg.quitidle);
-          set_meta_message_string((const char *)buf);
-	  freelog(LOG_NORMAL, "%s", get_meta_message_string());
+
+	  freelog(LOG_NORMAL,
+		  _("restarting in %d seconds for lack of players"),
+		  srvarg.quitidle);
+
+          set_meta_message_string(N_("restarting soon for lack of players"));
 	  (void) send_server_info_to_metaserver(META_INFO);
 	}
       } else {
@@ -532,7 +534,7 @@ enum server_events server_sniff_all_input(void)
     }
 
     /* Pinging around for statistics */
-    if (time(NULL) > (game.last_ping + game.info.pingtime)) {
+    if (time(NULL) > (game.server.last_ping + game.info.pingtime)) {
       /* send data about the previous run */
       send_ping_times_to_all();
 
@@ -551,11 +553,11 @@ enum server_events server_sniff_all_input(void)
 		    conn_description(pconn));
 	    close_socket_callback(pconn);
 	  }
-	} else {
-	  ping_connection(pconn);
-	}
+        } else {
+          connection_ping(pconn);
+        }
       } conn_list_iterate_end;
-      game.last_ping = time(NULL);
+      game.server.last_ping = time(NULL);
     }
 
     /* if we've waited long enough after a failure, respond to the client */
@@ -566,7 +568,7 @@ enum server_events server_sniff_all_input(void)
     } conn_list_iterate_end
 
     /* Don't wait if timeout == -1 (i.e. on auto games) */
-    if (S_S_INITIAL != server_state() && game.info.timeout == -1) {
+    if (S_S_RUNNING == server_state() && game.info.timeout == -1) {
       (void) send_server_info_to_metaserver(META_REFRESH);
       return S_E_END_OF_TURN_TIMEOUT;
     }
@@ -613,13 +615,13 @@ enum server_events server_sniff_all_input(void)
     }
     con_prompt_off();		/* output doesn't generate a new prompt */
 
-    if (my_select(max_desc + 1, &readfs, &writefs, &exceptfs, &tv) == 0) {
+    if (fc_select(max_desc + 1, &readfs, &writefs, &exceptfs, &tv) == 0) {
       /* timeout */
       (void) send_server_info_to_metaserver(META_REFRESH);
       if (game.info.timeout > 0
 	  && S_S_RUNNING == server_state()
-	  && game.phase_timer
-	  && (read_timer_seconds(game.phase_timer)
+	  && game.server.phase_timer
+	  && (read_timer_seconds(game.server.phase_timer)
 	      > game.info.seconds_to_phasedone)) {
 	con_prompt_off();
 	return S_E_END_OF_TURN_TIMEOUT;
@@ -782,8 +784,9 @@ enum server_events server_sniff_all_input(void)
 
   if (game.info.timeout > 0
       && S_S_RUNNING == server_state()
-      && game.phase_timer
-      && read_timer_seconds(game.phase_timer) > game.info.seconds_to_phasedone) {
+      && game.server.phase_timer
+      && (read_timer_seconds(game.server.phase_timer)
+          > game.info.seconds_to_phasedone)) {
     return S_E_END_OF_TURN_TIMEOUT;
   }
   return S_E_OTHERWISE;
@@ -828,24 +831,48 @@ static int server_accept_connection(int sockfd)
   socklen_t fromlen;
 
   int new_sock;
-  union my_sockaddr fromend;
+  union fc_sockaddr fromend;
+  bool nameinfo = FALSE;
+#ifdef IPV6_SUPPORT
+  char host[NI_MAXHOST], service[NI_MAXSERV];
+  char dst[INET6_ADDRSTRLEN];
+#else  /* IPv6 support */
   struct hostent *from;
+  const char *host = NULL;
+  const char *dst;
+#endif /* IPv6 support */
 
   fromlen = sizeof(fromend);
 
-  if ((new_sock = accept(sockfd, &fromend.sockaddr, &fromlen)) == -1) {
-    freelog(LOG_ERROR, "accept failed: %s", mystrerror());
+  if ((new_sock = accept(sockfd, &fromend.saddr, &fromlen)) == -1) {
+    freelog(LOG_ERROR, "accept failed: %s", fc_strerror(fc_get_errno()));
     return -1;
   }
 
+#ifdef IPV6_SUPPORT
+  if (!getnameinfo(&fromend.saddr, fromlen, host, NI_MAXHOST,
+                   service, NI_MAXSERV, NI_NUMERICSERV)) {
+    nameinfo = TRUE;
+  }
+  if (fromend.saddr.sa_family == AF_INET6) {
+    inet_ntop(AF_INET6, &fromend.saddr_in6.sin6_addr,
+              dst, sizeof(dst));
+  } else {
+    inet_ntop(AF_INET, &fromend.saddr_in4.sin_addr, dst, sizeof(dst));
+  }
+#else  /* IPv6 support */
   from =
-      gethostbyaddr((char *) &fromend.sockaddr_in.sin_addr,
-		    sizeof(fromend.sockaddr_in.sin_addr), AF_INET);
+    gethostbyaddr((char *) &fromend.saddr_in4.sin_addr,
+                  sizeof(fromend.saddr_in4.sin_addr), AF_INET);
+  if (from) {
+    host = from->h_name;
+    nameinfo = TRUE;
+  }
+  dst = inet_ntoa(fromend.saddr_in4.sin_addr);
+#endif /* IPv6 support */
 
   return server_make_connection(new_sock,
-				(from ? from->h_name
-				 : inet_ntoa(fromend.sockaddr_in.sin_addr)),
-				inet_ntoa(fromend.sockaddr_in.sin_addr));
+				(nameinfo ? host : dst), dst);
 }
 
 /********************************************************************
@@ -858,7 +885,7 @@ int server_make_connection(int new_sock, const char *client_addr, const char *cl
 {
   int i;
 
-  my_nonblock(new_sock);
+  fc_nonblock(new_sock);
 
   for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
     struct connection *pconn = &connections[i];
@@ -866,20 +893,18 @@ int server_make_connection(int new_sock, const char *client_addr, const char *cl
       connection_common_init(pconn);
       pconn->sock = new_sock;
       pconn->observer = FALSE;
-      pconn->player = NULL;
+      pconn->playing = NULL;
       pconn->capability[0] = '\0';
       pconn->access_level = access_level_for_next_connection();
       pconn->delayed_disconnect = FALSE;
       pconn->notify_of_writable_data = NULL;
-      pconn->is_server = TRUE;
       pconn->server.currently_processed_request_id = 0;
       pconn->server.last_request_id_seen = 0;
       pconn->server.auth_tries = 0;
       pconn->server.auth_settime = 0;
       pconn->server.status = AS_NOT_ESTABLISHED;
-      pconn->server.ping_timers =
-	  fc_malloc(sizeof(*pconn->server.ping_timers));
       pconn->server.ping_timers = timer_list_new();
+      pconn->server.granted_access_level = pconn->access_level;
       pconn->ping_time = -1.0;
       pconn->incoming_packet_notify = NULL;
       pconn->outgoing_packet_notify = NULL;
@@ -892,7 +917,6 @@ int server_make_connection(int new_sock, const char *client_addr, const char *cl
   
       freelog(LOG_VERBOSE, "connection (%s) from %s (%s)", 
               pconn->username, pconn->addr, pconn->server.ipaddr);
-      ping_connection(pconn);
       return 0;
     }
   }
@@ -908,77 +932,160 @@ int server_make_connection(int new_sock, const char *client_addr, const char *cl
 int server_open_socket(void)
 {
   /* setup socket address */
-  union my_sockaddr src;
-  union my_sockaddr addr;
-  struct ip_mreq mreq;
+  union fc_sockaddr src;
+  union fc_sockaddr addr;
+  struct ip_mreq mreq4;
   const char *group;
   int opt;
+  int lan_family;
 
-  /* Create socket for client connections. */
-  if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
-    die("socket failed: %s", mystrerror());
-  }
+#ifdef IPV6_SUPPORT
+  struct ipv6_mreq mreq6;
+#endif
 
-  opt=1; 
-  if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
-		(char *)&opt, sizeof(opt)) == -1) {
-    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", mystrerror());
-  }
-
-  if (!net_lookup_service(srvarg.bind_addr, srvarg.port, &src)) {
-    freelog(LOG_ERROR, _("Server: bad address: [%s:%d]."),
+  if (!net_lookup_service(srvarg.bind_addr, srvarg.port, &src, FALSE)) {
+    freelog(LOG_FATAL, _("Server: bad address: <%s:%d>."),
 	    srvarg.bind_addr, srvarg.port);
     exit(EXIT_FAILURE);
   }
 
-  if(bind(sock, &src.sockaddr, sizeof (src)) == -1) {
-    freelog(LOG_FATAL, "bind failed: %s", mystrerror());
+  /* Create socket for client connections. */
+  if((sock = socket(src.saddr.sa_family, SOCK_STREAM, 0)) == -1) {
+    fc_errno error = fc_get_errno();
+
+#ifdef IPV6_SUPPORT
+#ifdef EAFNOSUPPORT
+    bool still_error = TRUE;
+
+    if (error == EAFNOSUPPORT && src.saddr.sa_family == AF_INET6 && srvarg.bind_addr == NULL) {
+      /* Let's try IPv4 socket instead */
+      freelog(LOG_NORMAL, _("Cannot open IPv6 socket, trying IPv4 instead"));
+
+      if (!net_lookup_service(NULL, srvarg.port, &src, TRUE)) {
+	freelog(LOG_FATAL, _("IPv4 service lookup failed <%d>."),
+		srvarg.port);
+	exit(EXIT_FAILURE);
+      }
+
+      /* Create socket for client connections. */
+      if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
+	fc_errno error2 = fc_get_errno();
+	freelog(LOG_ERROR, "Even IPv4 socket failed: %s", fc_strerror(error2));
+      } else {
+	still_error = FALSE;
+      }
+    }
+
+    if (still_error)
+#endif /* EAFNOSUPPORT */
+#endif /* IPv6 support */
+    {
+      die("socket failed: %s", fc_strerror(error));
+    }
+  }
+
+  opt = 1;
+  if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
+		(char *)&opt, sizeof(opt)) == -1) {
+    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", fc_strerror(fc_get_errno()));
+  }
+
+  if(bind(sock, &src.saddr, sockaddr_size(&src)) == -1) {
+    freelog(LOG_FATAL, "Server bind failed: %s", fc_strerror(fc_get_errno()));
+    sockaddr_debug(&src);
     exit(EXIT_FAILURE);
   }
 
   if(listen(sock, MAX_NUM_CONNECTIONS) == -1) {
-    freelog(LOG_FATAL, "listen failed: %s", mystrerror());
+    freelog(LOG_FATAL, "listen failed: %s", fc_strerror(fc_get_errno()));
     exit(EXIT_FAILURE);
   }
 
+  if (srvarg.announce == ANNOUNCE_NONE) {
+    return 0;
+  }
+
+#ifdef IPV6_SUPPORT
+  if (srvarg.announce == ANNOUNCE_IPV6) {
+    lan_family = AF_INET6;
+  } else
+#endif /* IPV6 used */
+  {
+    lan_family = AF_INET;
+  }
+
   /* Create socket for server LAN announcements */
-  if ((socklan = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
-     freelog(LOG_ERROR, "socket failed: %s", mystrerror());
+  if ((socklan = socket(lan_family, SOCK_DGRAM, 0)) < 0) {
+    freelog(LOG_ERROR, "socket failed: %s", fc_strerror(fc_get_errno()));
   }
 
   if (setsockopt(socklan, SOL_SOCKET, SO_REUSEADDR,
                  (char *)&opt, sizeof(opt)) == -1) {
-    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", mystrerror());
+    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", fc_strerror(fc_get_errno()));
   }
 
-  my_nonblock(socklan);
+  fc_nonblock(socklan);
 
-  group = get_multicast_group();
+  group = get_multicast_group(srvarg.announce == ANNOUNCE_IPV6);
 
   memset(&addr, 0, sizeof(addr));
-  addr.sockaddr_in.sin_family = AF_INET;
-  addr.sockaddr_in.sin_addr.s_addr = htonl(INADDR_ANY);
-  addr.sockaddr_in.sin_port = htons(SERVER_LAN_PORT);
 
-  if (bind(socklan, &addr.sockaddr, sizeof(addr)) < 0) {
-    freelog(LOG_ERROR, "bind failed: %s", mystrerror());
+  addr.saddr.sa_family = lan_family;
+
+#ifdef IPV6_SUPPORT
+  if (addr.saddr.sa_family == AF_INET6) {
+    addr.saddr_in6.sin6_family = AF_INET6;
+    addr.saddr_in6.sin6_port = htons(SERVER_LAN_PORT);
+    addr.saddr_in6.sin6_addr = in6addr_any;
+  } else
+#endif /* IPv6 support */
+  {
+    addr.saddr_in4.sin_family = AF_INET;
+    addr.saddr_in4.sin_port = htons(SERVER_LAN_PORT);
+    addr.saddr_in4.sin_addr.s_addr = htonl(INADDR_ANY);
   }
 
-  mreq.imr_multiaddr.s_addr = inet_addr(group);
-  mreq.imr_interface.s_addr = htonl(INADDR_ANY);
+  if (bind(socklan, &addr.saddr, sockaddr_size(&addr)) < 0) {
+    freelog(LOG_ERROR, "Lan bind failed: %s", fc_strerror(fc_get_errno()));
+  }
 
-  if (setsockopt(socklan, IPPROTO_IP, IP_ADD_MEMBERSHIP,
-                 (const char*)&mreq, sizeof(mreq)) < 0) {
-    freelog(LOG_ERROR, "setsockopt failed: %s", mystrerror());
+#ifndef IPV6_SUPPORT
+  {
+#ifdef HAVE_INET_ATON
+    inet_aton(group, &mreq4.imr_multiaddr);
+#else  /* HEVE_INET_ATON */
+    mreq4.imr_multiaddr.s_addr = inet_addr(group);
+#endif /* HAVE_INET_ATON */
+#else  /* IPv6 support */
+  if (addr.saddr.sa_family == AF_INET6) {
+    inet_pton(AF_INET6, group, &mreq6.ipv6mr_multiaddr.s6_addr);
+    mreq6.ipv6mr_interface = 0; /* TODO: Interface selection */
+    if (setsockopt(socklan, IPPROTO_IPV6, FC_IPV6_ADD_MEMBERSHIP,
+                   (const char*)&mreq6, sizeof(mreq6)) < 0) {
+      freelog(LOG_ERROR, "FC_IPV6_ADD_MEMBERSHIP (%s) failed: %s",
+              group, fc_strerror(fc_get_errno()));
+    }
+  } else {
+    inet_pton(AF_INET, group, &mreq4.imr_multiaddr.s_addr);
+#endif /* IPv6 support */
+    mreq4.imr_interface.s_addr = htonl(INADDR_ANY);
+
+    if (setsockopt(socklan, IPPROTO_IP, IP_ADD_MEMBERSHIP,
+                   (const char*)&mreq4, sizeof(mreq4)) < 0) {
+      freelog(LOG_ERROR, "IP_ADD_MEMBERSHIP (%s) failed: %s",
+              group, fc_strerror(fc_get_errno()));
+    }
   }
 
   close_socket_set_callback(close_socket_callback);
+
   return 0;
 }
 
 
 /********************************************************************
-...
+  Initialize connection related stuff. Attention: Logging is not
+  available within this functions!
 ********************************************************************/
 void init_connections(void)
 {
@@ -1033,15 +1140,15 @@ static void finish_processing_request(struct connection *pconn)
 }
 
 /**************************************************************************
-...
+  Ping a connection.
 **************************************************************************/
-static void ping_connection(struct connection *pconn)
+void connection_ping(struct connection *pconn)
 {
   freelog(LOG_DEBUG, "sending ping to %s (open=%d)",
-	  conn_description(pconn),
-	  timer_list_size(pconn->server.ping_timers));
+          conn_description(pconn),
+          timer_list_size(pconn->server.ping_timers));
   timer_list_append(pconn->server.ping_timers,
-			 new_timer_start(TIMER_USER, TIMER_ACTIVE));
+                    new_timer_start(TIMER_USER, TIMER_ACTIVE));
   send_packet_conn_ping(pconn);
 }
 
@@ -1053,7 +1160,7 @@ void handle_conn_pong(struct connection *pconn)
   struct timer *timer;
 
   if (timer_list_size(pconn->server.ping_timers) == 0) {
-    freelog(LOG_NORMAL, "got unexpected pong from %s",
+    freelog(LOG_ERROR, "got unexpected pong from %s",
 	    conn_description(pconn));
     return;
   }
@@ -1113,9 +1220,9 @@ static void get_lanserver_announcement(void)
   tv.tv_sec = 0;
   tv.tv_usec = 0;
 
-  while (my_select(socklan + 1, &readfs, NULL, &exceptfs, &tv) == -1) {
+  while (fc_select(socklan + 1, &readfs, NULL, &exceptfs, &tv) == -1) {
     if (errno != EINTR) {
-      freelog(LOG_ERROR, "select failed: %s", mystrerror());
+      freelog(LOG_ERROR, "select failed: %s", fc_strerror(fc_get_errno()));
       return;
     }
     /* EINTR can happen sometimes, especially when compiling with -pg.
@@ -1154,7 +1261,7 @@ static void send_lanserver_response(void)
   char players[256];
   char status[256];
   struct data_out dout;
-  union my_sockaddr addr;
+  union fc_sockaddr addr;
   int socksend, setting = 1;
   const char *group;
   size_t size;
@@ -1164,16 +1271,16 @@ static void send_lanserver_response(void)
 
   /* Create a socket to broadcast to client. */
   if ((socksend = socket(AF_INET,SOCK_DGRAM, 0)) < 0) {
-    freelog(LOG_ERROR, "socket failed: %s", mystrerror());
+    freelog(LOG_ERROR, "socket failed: %s", fc_strerror(fc_get_errno()));
     return;
   }
 
   /* Set the UDP Multicast group IP address of the packet. */
-  group = get_multicast_group();
+  group = get_multicast_group(srvarg.announce == ANNOUNCE_IPV6);
   memset(&addr, 0, sizeof(addr));
-  addr.sockaddr_in.sin_family = AF_INET;
-  addr.sockaddr_in.sin_addr.s_addr = inet_addr(group);
-  addr.sockaddr_in.sin_port = htons(SERVER_LAN_PORT + 1);
+  addr.saddr_in4.sin_family = AF_INET;
+  addr.saddr_in4.sin_addr.s_addr = inet_addr(group);
+  addr.saddr_in4.sin_port = htons(SERVER_LAN_PORT + 1);
 
 /* this setsockopt call fails on Windows 98, so we stick with the default
  * value of 1 on Windows, which should be fine in most cases */
@@ -1182,14 +1289,14 @@ static void send_lanserver_response(void)
   ttl = SERVER_LAN_TTL;
   if (setsockopt(socksend, IPPROTO_IP, IP_MULTICAST_TTL, 
                  (const char*)&ttl, sizeof(ttl))) {
-    freelog(LOG_ERROR, "setsockopt failed: %s", mystrerror());
+    freelog(LOG_ERROR, "setsockopt failed: %s", fc_strerror(fc_get_errno()));
     return;
   }
 #endif
 
   if (setsockopt(socksend, SOL_SOCKET, SO_BROADCAST, 
                  (const char*)&setting, sizeof(setting))) {
-    freelog(LOG_ERROR, "setsockopt failed: %s", mystrerror());
+    freelog(LOG_ERROR, "setsockopt failed: %s", fc_strerror(fc_get_errno()));
     return;
   }
 
@@ -1203,20 +1310,24 @@ static void send_lanserver_response(void)
 
   switch (server_state()) {
   case S_S_INITIAL:
-    my_snprintf(status, sizeof(status), "Pregame");
+    /* TRANS: Game state for local server */
+    my_snprintf(status, sizeof(status), _("Pregame"));
     break;
   case S_S_RUNNING:
-    my_snprintf(status, sizeof(status), "Running");
+    /* TRANS: Game state for local server */
+    my_snprintf(status, sizeof(status), _("Running"));
     break;
   case S_S_OVER:
-    my_snprintf(status, sizeof(status), "Game over");
+    /* TRANS: Game state for local server */
+    my_snprintf(status, sizeof(status), _("Game over"));
     break;
   case S_S_GENERATING_WAITING:
-    my_snprintf(status, sizeof(status), "Waiting");
+    /* TRANS: Game state for local server */
+    my_snprintf(status, sizeof(status), _("Waiting"));
   }
 
    my_snprintf(players, sizeof(players), "%d",
-               get_num_human_and_ai_players());
+               normal_player_count());
    my_snprintf(port, sizeof(port), "%d",
               srvarg.port );
 
@@ -1231,11 +1342,12 @@ static void send_lanserver_response(void)
   size = dio_output_used(&dout);
 
   /* Sending packet to client with the information gathered above. */
-  if (sendto(socksend, buffer,  size, 0, &addr.sockaddr,
-      sizeof(addr)) < 0) {
-    freelog(LOG_ERROR, "sendto failed: %s", mystrerror());
+  if (sendto(socksend, buffer,  size, 0, &addr.saddr,
+      sockaddr_size(&addr)) < 0) {
+    freelog(LOG_ERROR, "landserver response sendto failed: %s",
+	    fc_strerror(fc_get_errno()));
     return;
   }
 
-  my_closesocket(socksend);
+  fc_closesocket(socksend);
 }
