@@ -14,28 +14,30 @@
 #define FC__UNITTOOLS_H
 
 #include "fc_types.h"
-#include "packets.h"		/* enum unit_info_use */
-#include "unit.h"
 
 #include "gotohand.h"
+#include "packets.h"		/* enum unit_info_use */
+#include "unitlist.h"
 
 /* battle related */
-int find_a_unit_type(int role, int role_tech);
+struct unit_type *find_a_unit_type(enum unit_role_id role,
+				   enum unit_role_id role_tech);
 bool maybe_make_veteran(struct unit *punit);
+void notify_unit_experience(struct unit *punit);
 void unit_versus_unit(struct unit *attacker, struct unit *defender,
 		      bool bombard);
 
 /* move check related */
+bool is_unit_being_refueled(const struct unit *punit);
 bool is_airunit_refuel_point(struct tile *ptile, struct player *pplayer,
-			     Unit_Type_id type, bool unit_is_on_tile);
+			     const struct unit_type *punittype,
+			     bool unit_is_on_carrier);
 
 /* turn update related */
 void player_restore_units(struct player *pplayer);
 void update_unit_activities(struct player *pplayer);
 
 /* various */
-char *get_location_str_in(struct player *pplayer, struct tile *ptile);
-char *get_location_str_at(struct player *pplayer, struct tile *ptile);
 enum goto_move_restriction get_activity_move_restriction(enum unit_activity activity);
 void make_partisans(struct city *pcity);
 bool enemies_at(struct unit *punit, struct tile *ptile);
@@ -44,22 +46,28 @@ bool teleport_unit_to_city(struct unit *punit, struct city *pcity, int move_cost
 void resolve_unit_stacks(struct player *pplayer, struct player *aplayer,
                          bool verbose);
 void remove_allied_visibility(struct player* pplayer, struct player* aplayer);
-int get_watchtower_vision(struct unit *punit);
-bool unit_profits_of_watchtower(struct unit *punit);
-void pay_for_units(struct player *pplayer, struct city *pcity);
+int get_unit_vision_at(struct unit *punit, struct tile *ptile,
+		       enum vision_layer vlayer);
+void unit_refresh_vision(struct unit *punit);
+void unit_list_refresh_vision(struct unit_list *punitlist);
 void bounce_unit(struct unit *punit, bool verbose);
+void unit_activity_assign_target(struct unit *punit,
+                                 enum unit_activity *activity,
+                                 enum tile_special_type *target,
+                                 Base_type_id *base);
 
 /* creation/deletion/upgrading */
-void upgrade_unit(struct unit *punit, Unit_Type_id to_unit, bool has_to_pay);
-struct unit *create_unit(struct player *pplayer, struct tile *ptile, Unit_Type_id type,
+void upgrade_unit(struct unit *punit, struct unit_type *to_unit,
+		  bool has_to_pay);
+struct unit *create_unit(struct player *pplayer, struct tile *ptile,
+			 struct unit_type *punittype,
 			 int veteran_level, int homecity_id, int moves_left);
 struct unit *create_unit_full(struct player *pplayer, struct tile *ptile,
-			      Unit_Type_id type, int veteran_level,
+			      struct unit_type *punittype, int veteran_level,
 			      int homecity_id, int moves_left, int hp_left,
 			      struct unit *ptrans);
 void wipe_unit(struct unit *punit);
-void wipe_unit_spec_safe(struct unit *punit, bool wipe_cargo);
-void kill_unit(struct unit *pkiller, struct unit *punit);
+void kill_unit(struct unit *pkiller, struct unit *punit, bool vet);
 
 /* sending to client */
 void package_unit(struct unit *punit, struct packet_unit_info *packet);
@@ -75,8 +83,8 @@ void unit_goes_out_of_sight(struct player *pplayer, struct unit *punit);
 
 /* doing a unit activity */
 void do_nuclear_explosion(struct player *pplayer, struct tile *ptile);
-bool try_move_unit(struct unit *punit, struct tile *ptile); 
 bool do_airline(struct unit *punit, struct city *city2);
+void do_explore(struct unit *punit);
 bool do_paradrop(struct unit *punit, struct tile *ptile);
 void load_unit_onto_transporter(struct unit *punit, struct unit *ptrans);
 void unload_unit_from_transporter(struct unit *punit);
