@@ -23,21 +23,24 @@
 
 #include "widget_p.h"
 
+#include "fcintl.h"
 #include "log.h"
 #include "mem.h"
 
 /* REMOVE ME */
 #include "timing.h"
 
+#define FLUSH_RECTS TRUE       /* flush dirty rectangles or whole screen */
+
 #define DEBUG_UPDATES  0
 #define DUMP_UPDATES   0
 #define DUMP_WINDOWS   0
 #define DEBUG_PAINT_ALL	FALSE
 
 struct sw_widget *root_window;
-static struct widget_list windows_back_to_front;
-static struct widget_list windows_front_to_back;
-struct widget_list deferred_destroyed_widgets;
+static struct widget_list *windows_back_to_front;
+static struct widget_list *windows_front_to_back;
+struct widget_list *deferred_destroyed_widgets;
 static bool dump_screen = FALSE;
 
 #define TITLE_PADDING 3
@@ -49,14 +52,13 @@ static void draw_extra_background(struct sw_widget *widget,
 				  const struct ct_rect *region)
 {
   if (widget->data.window.canvas_background) {
-    struct ct_size size = { region->width,
-			    region->height
-    };
-    struct ct_point pos={region->x,region->y};
+   struct ct_size size = { region->width,
+			    region->height };
+    struct ct_point pos = { region->x, region->y };
 
-    be_copy_osda_to_osda(get_osda(widget),
+    be_copy_osda_to_osda(sw_widget_get_osda(widget),
 			 widget->data.window.canvas_background,
-			 &size, &pos, &pos, 0);
+			 &size, &pos, &pos);
   }
 }
 
@@ -71,13 +73,13 @@ static void draw(struct sw_widget *widget)
 
     rect.height = widget->data.window.title->size.height + 2 * TITLE_PADDING;
 
-    be_draw_region(widget->data.window.target, BE_OPAQUE, &rect,
+    be_draw_region(widget->data.window.target, &rect,
 		   widget->data.window.title->background);
-    be_draw_rectangle(get_osda(widget), BE_OPAQUE, &rect, 1,
+    be_draw_rectangle(sw_widget_get_osda(widget), &rect, 1,
 		      widget->data.window.title->foreground);
-    pos.x = widget->inner_bounds.x+TITLE_PADDING;
-    pos.y = widget->inner_bounds.y+TITLE_PADDING;
-    be_draw_string(get_osda(widget), BE_OPAQUE, &pos,
+    pos.x = widget->inner_bounds.x + TITLE_PADDING;
+    pos.y = widget->inner_bounds.y + TITLE_PADDING;
+    be_draw_string(sw_widget_get_osda(widget), &pos,
 		   widget->data.window.title);
   }
 }
@@ -93,7 +95,8 @@ static void drag_start(struct sw_widget *widget,
     widget->data.window.pos_at_drag_start.x = widget->outer_bounds.x;
     widget->data.window.pos_at_drag_start.y = widget->outer_bounds.y;
 
-    sw_widget_set_border_color(widget, be_get_color(255, 255, 0));
+    sw_widget_set_border_color(widget, 
+                               be_get_color(255, 255, 0, MAX_OPACITY));
   } else if (widget->data.window.user_drag_start) {
     widget->data.window.user_drag_start(widget, mouse, button);
   }
@@ -131,7 +134,8 @@ static void drag_move(struct sw_widget *widget,
 static void drag_end(struct sw_widget *widget, enum be_mouse_button button)
 {
   if (button == BE_MB_LEFT) {
-    sw_widget_set_border_color(widget, be_get_color(255, 255, 255));
+    sw_widget_set_border_color(widget, 
+                               be_get_color(255, 255, 255, MAX_OPACITY));
   } else if (widget->data.window.user_drag_end) {
     widget->data.window.user_drag_end(widget, button);
   }
@@ -152,8 +156,8 @@ static void destroy(struct sw_widget *widget)
     ct_string_destroy(widget->data.window.title);
   }
 
-  widget_list_unlink(&windows_front_to_back, widget);
-  widget_list_unlink(&windows_back_to_front, widget);
+  widget_list_unlink(windows_front_to_back, widget);
+  widget_list_unlink(windows_back_to_front, widget);
 }
 
 /*************************************************************************
@@ -187,8 +191,8 @@ static int my_sort_front_to_back(const void *p1, const void *p2)
 *************************************************************************/
 static void sort(void)
 {
-  widget_list_sort(&windows_front_to_back, my_sort_front_to_back);
-  widget_list_sort(&windows_back_to_front, my_sort_back_to_front);
+  widget_list_sort(windows_front_to_back, my_sort_front_to_back);
+  widget_list_sort(windows_back_to_front, my_sort_back_to_front);
 }
 
 static void sw_window_set_depth(struct sw_widget *widget, int depth)
@@ -202,8 +206,7 @@ static void sw_window_set_depth(struct sw_widget *widget, int depth)
 *************************************************************************/
 struct sw_widget *sw_window_create(struct sw_widget *parent, int width,
 				   int height, struct ct_string *title,
-				   int transparency, bool has_border,
-				   int depth)
+				   bool has_border, int depth)
 {
   struct sw_widget *result = create_widget(parent, WT_WINDOW);
   int border_width = has_border ? BORDER_WIDTH : 0;
@@ -231,11 +234,11 @@ struct sw_widget *sw_window_create(struct sw_widget *parent, int width,
 
   result->can_be_dragged[BE_MB_LEFT] = TRUE;
 
-  widget_list_init(&result->data.window.children);
-  widget_list_insert_back(&windows_front_to_back, result);
-  widget_list_insert_back(&windows_back_to_front, result);
+  result->data.window.children = widget_list_new();
+  widget_list_append(windows_front_to_back, result);
+  widget_list_append(windows_back_to_front, result);
 
-  region_list_init(&result->data.window.to_flush);
+  result->data.window.to_flush = region_list_new();
 
   result->inner_bounds.x = border_width;
   result->inner_bounds.y = border_width;
@@ -248,7 +251,6 @@ struct sw_widget *sw_window_create(struct sw_widget *parent, int width,
   result->data.window.target =
       be_create_osda(2 * border_width + width, 2 * border_width + height);
   result->data.window.title = title;
-  result->data.window.transparency = (transparency * MAX_TRANSPARENCY) / 100;
   result->data.window.shown = TRUE;
   result->data.window.list = NULL;
 
@@ -261,7 +263,8 @@ struct sw_widget *sw_window_create(struct sw_widget *parent, int width,
   }
 
   if (has_border) {
-    sw_widget_set_border_color(result, be_get_color(255, 255, 255));
+    sw_widget_set_border_color(result, 
+                               be_get_color(255, 255, 255, MAX_OPACITY));
   }
 
   sw_window_set_depth(result, depth);
@@ -280,19 +283,20 @@ struct sw_widget *sw_create_root_window(void)
   struct sw_widget *result;
 
   assert(!root_window);
-  widget_list_init(&windows_front_to_back);
-  widget_list_init(&windows_back_to_front);
-  widget_list_init(&deferred_destroyed_widgets);
+  windows_front_to_back = widget_list_new();
+  windows_back_to_front = widget_list_new();
+  deferred_destroyed_widgets = widget_list_new();
 
   be_screen_get_size(&size);
 
   whole_osda = be_create_osda(size.width, size.height);
 
   result =
-      sw_window_create(NULL, size.width, size.height, NULL, 0, FALSE,
+      sw_window_create(NULL, size.width, size.height, NULL, FALSE,
 		       DEPTH_MIN + 1);
   sw_widget_set_position(result, 0, 0);
-  sw_widget_set_background_color(result, be_get_color(22, 44, 88));
+  sw_widget_set_background_color(result, 
+                                 be_get_color(22, 44, 88, MAX_OPACITY));
   sw_window_set_draggable(result, FALSE);
 
   root_window = result;
@@ -311,7 +315,7 @@ void sw_window_add(struct sw_widget *window, struct sw_widget *widget)
     assert(pwidget != widget);
   } widget_list_iterate_end;
 
-  widget_list_insert(&window->data.window.children, widget);
+  widget_list_prepend(window->data.window.children, widget);
   widget->parent = window;
 }
 
@@ -320,8 +324,8 @@ void sw_window_add(struct sw_widget *window, struct sw_widget *widget)
 *************************************************************************/
 void remove_all_from_window(struct sw_widget *widget)
 {
-  while (widget_list_size(&widget->data.window.children) > 0) {
-    sw_window_remove(widget_list_get(&widget->data.window.children, 0));
+  while (widget_list_size(widget->data.window.children) > 0) {
+    sw_window_remove(widget_list_get(widget->data.window.children, 0));
   }
 }
 
@@ -342,7 +346,7 @@ void sw_window_remove(struct sw_widget *widget)
   } widget_list_iterate_end;
 
   assert(found == 1);
-  widget_list_unlink(&old_parent->data.window.children, widget);
+  widget_list_unlink(old_parent->data.window.children, widget);
   widget->parent = NULL;
 }
 
@@ -367,8 +371,15 @@ static void flush_one_window(struct sw_widget *widget,
   src_pos.y = screen_rect->y - widget->outer_bounds.y;
 
   be_copy_osda_to_osda(whole_osda, widget->data.window.target, &size,
-		       &dest_pos, &src_pos,
-		       widget->data.window.transparency);
+		       &dest_pos, &src_pos);
+}
+
+/*************************************************************************
+  ...
+*************************************************************************/
+void flush_rect_to_screen(const struct ct_rect *rect)
+{
+  be_copy_osda_to_screen(whole_osda, rect);  
 }
 
 /*************************************************************************
@@ -376,17 +387,15 @@ static void flush_one_window(struct sw_widget *widget,
 *************************************************************************/
 void flush_all_to_screen(void)
 {
-  be_copy_osda_to_screen(whole_osda);
+  be_copy_osda_to_screen(whole_osda, NULL);
 
   if (dump_screen) {
-    static int counter = -1;
+    static int counter = 0;
     char b[100];
 
-    counter++;
+    my_snprintf(b, sizeof(b), "fc_%05d.ppm", counter++);
+    freelog(LOG_NORMAL, _("Making screenshot %s"), b);
 
-    freelog(LOG_NORMAL, "flush to screen %d",counter);
-
-    sprintf(b, "screen-%04d.ppm", counter);
     be_write_osda_to_file(whole_osda, b);
   }
 }
@@ -423,10 +432,10 @@ static void draw_background_region(struct sw_widget *widget,
     size.width = rect->width;
     size.height = rect->height;
 
-    be_draw_sprite(get_osda(widget), BE_OPAQUE, widget->background_sprite,
+    be_draw_sprite(sw_widget_get_osda(widget), widget->background_sprite,
 		   &size, &pos, &pos);
   } else if (widget->has_background_color) {
-    be_draw_region(get_osda(widget), BE_ALPHA, rect,
+    be_draw_region(sw_widget_get_osda(widget), rect,
 		   widget->background_color);
   } else {
     if (widget->parent && widget->type != WT_WINDOW) {
@@ -440,8 +449,6 @@ static void draw_background_region(struct sw_widget *widget,
 *************************************************************************/
 static void draw_background(struct sw_widget *widget)
 {
-  be_set_transparent(get_osda(widget), &widget->inner_bounds);
-
   draw_background_region(widget, &widget->inner_bounds);
 
   if (widget->has_border_color) {
@@ -452,7 +459,7 @@ static void draw_background(struct sw_widget *widget)
       rect.y = 0;
     }
 
-    be_draw_rectangle(get_osda(widget), BE_OPAQUE, &rect, BORDER_WIDTH,
+    be_draw_rectangle(sw_widget_get_osda(widget), &rect, BORDER_WIDTH,
 		      widget->border_color);
   }
 }
@@ -477,7 +484,7 @@ static void add_flush_region(struct sw_widget *widget, const struct ct_rect
     }
   } region_list_iterate_end;
   
-  region_list_insert(&window->data.window.to_flush, ct_rect_clone(region));
+  region_list_prepend(window->data.window.to_flush, ct_rect_clone(region));
 }
 
 /*************************************************************************
@@ -488,7 +495,6 @@ static void draw_tooltip(struct sw_widget *widget)
   struct ct_point pos;
   struct ct_rect rect;
   const int PADDING = 5;
-  enum be_draw_type draw_type = BE_OPAQUE;
   struct osda *osda;
   int i, extra = 2 * PADDING + widget->tooltip->shadow;
 
@@ -505,9 +511,9 @@ static void draw_tooltip(struct sw_widget *widget)
   pos.y = rect.y + PADDING;
 
   if (widget->type == WT_WINDOW) {
-    osda = get_osda(widget->parent);
+    osda = sw_widget_get_osda(widget->parent);
   } else {
-    osda = get_osda(widget);
+    osda = sw_widget_get_osda(widget);
   }
 
   for (i = 1; i < widget->tooltip->shadow; i++) {
@@ -515,13 +521,67 @@ static void draw_tooltip(struct sw_widget *widget)
 
     rect2.x += i;
     rect2.y += i;
-    be_draw_region(osda, draw_type, &rect2, widget->tooltip->shadow_color);
+    be_draw_region(osda, &rect2, widget->tooltip->shadow_color);
   }
 
-  be_draw_region(osda, draw_type, &rect, widget->tooltip->text->background);
-  be_draw_string(osda, draw_type, &pos, widget->tooltip->text);
-  be_draw_rectangle(osda, draw_type, &rect, 1,
-		    widget->tooltip->text->foreground);
+  be_draw_region(osda, &rect, widget->tooltip->text->background);
+  be_draw_string(osda, &pos, widget->tooltip->text);
+  be_draw_rectangle(osda, &rect, 1, widget->tooltip->text->foreground);
+}
+
+/*************************************************************************
+  ...
+*************************************************************************/
+void untooltip(struct sw_widget *widget)
+{
+  struct ct_point pos;
+  struct ct_rect rect;
+  const int PADDING = 5;
+  struct osda *osda;
+  int i, extra;
+
+  if (widget->tooltip && widget->tooltip_callback_id != 0) {
+    sw_remove_timeout(widget->tooltip_callback_id);
+    widget->tooltip_callback_id = 0;
+  }
+
+  if (!widget->tooltip || (widget->tooltip && !widget->tooltip_shown)) {
+    return;
+  }
+  widget->tooltip_shown = FALSE;
+  
+  extra = 2 * PADDING + widget->tooltip->shadow;
+  
+  rect.width = widget->tooltip->text->size.width + extra;
+  rect.height = widget->tooltip->text->size.height + extra;
+
+  rect.x =
+      MAX(0,
+	  widget->outer_bounds.x + (widget->outer_bounds.width -
+				    rect.width) / 2);
+  rect.y = MAX(0,widget->outer_bounds.y - rect.height - PADDING);
+
+  pos.x = rect.x + PADDING;
+  pos.y = rect.y + PADDING;
+
+  if (widget->type == WT_WINDOW) {
+    osda = sw_widget_get_osda(widget->parent);
+  } else {
+    osda = sw_widget_get_osda(widget);
+  }
+
+  for (i = 1; i < widget->tooltip->shadow; i++) {
+    struct ct_rect rect2 = rect;
+
+    rect2.x += i;
+    rect2.y += i;
+    be_draw_region(osda, &rect2, be_get_color(0, 0, 0, MIN_OPACITY));
+  }
+
+  be_draw_region(osda, &rect, be_get_color(0, 0, 0, MIN_OPACITY));
+  be_draw_rectangle(osda, &rect, 1, be_get_color(0, 0, 0, MIN_OPACITY));  
+  
+  parent_needs_paint(widget);
 }
 
 /*************************************************************************
@@ -578,54 +638,39 @@ void update_window(struct sw_widget *widget)
   }
 }
 
-/*************************************************************************
-  ...
-*************************************************************************/
-static bool is_opaque(struct sw_widget *widget)
-{
-  if (widget->type == WT_WINDOW) {
-    if (widget->data.window.transparency > 0) {
-      return FALSE;
-    }
-    if (widget->data.window.canvas_background) {
-      return TRUE;
-    }
-  }
-  return FALSE;
-}
-
 /*************************************************************************
   ...
 *************************************************************************/
 static void merge_regions(struct region_list *list)
 {
-  struct region_list tmp, *orig, *copy;
+  struct region_list *tmp, *orig, *copy;
+
+  tmp = region_list_new();
 
   orig = list;
-  copy = &tmp;
-  region_list_init(copy);
+  copy = tmp;
 
-  region_list_iterate(*orig, region) {
+  region_list_iterate(orig, region) {
     if (ct_rect_in_rect_list(region, copy)) {
-	//printf("####### 1\n");
       free(region);
     } else {
-      region_list_insert(copy, region);
+      region_list_prepend(copy, region);
     }
   } region_list_iterate_end;
 
-  orig = &tmp;
+  orig = tmp;
   copy = list;
-  region_list_init(copy);
+  region_list_clear(copy);
 
-  region_list_iterate(*orig, region) {
+  region_list_iterate(orig, region) {
     if (ct_rect_in_rect_list(region, copy)) {
-	//printf("####### 2\n");
       free(region);
     } else {
-      region_list_insert(copy, region);
+      region_list_prepend(copy, region);
     }
   } region_list_iterate_end;
+
+  region_list_free(tmp);
 }
 
 /*************************************************************************
@@ -638,25 +683,23 @@ void sw_paint_all(void)
   struct timer *timer2 = new_timer(TIMER_USER, TIMER_ACTIVE);
   struct timer *timer3 = new_timer(TIMER_USER, TIMER_ACTIVE);
   struct timer *timer4 = new_timer(TIMER_USER, TIMER_ACTIVE);
-  struct region_list normalized_regions;
+  struct region_list *normalized_regions;
   static int call = -1;
 #if DUMP_UPDATES
   char filename[100];
   int region_nr;
 #endif
 
+  call++;
+
   handle_destroyed_widgets();
 
-  call++;
-  if(DEBUG_PAINT_ALL)
-  printf("%%%%%%%% starting sw_paint_all %d\n",call);
-
-  region_list_init(&normalized_regions);
+  normalized_regions = region_list_new();
 
   start_timer(timer1);
   widget_list_iterate(windows_back_to_front, widget) {
     update_window(widget);
-    regions += region_list_size(&widget->data.window.to_flush);
+    regions += region_list_size(widget->data.window.to_flush);
   } widget_list_iterate_end;
   stop_timer(timer1);
 
@@ -664,7 +707,9 @@ void sw_paint_all(void)
     return;
   }
 
-  if(DEBUG_PAINT_ALL)  printf("%%%%%%%% updated windows; %d regions have to be flushed\n",regions);
+  if (DEBUG_PAINT_ALL) {
+    printf("updated windows; %d regions have to be flushed\n", regions);
+  }
 
   if (DUMP_WINDOWS) {
     widget_list_iterate(windows_back_to_front, widget) {
@@ -677,38 +722,53 @@ void sw_paint_all(void)
     } widget_list_iterate_end;
   }
 
-  start_timer(timer2);
+  if (DEBUG_PAINT_ALL) {
+    start_timer(timer2);
+  }
   widget_list_iterate(windows_back_to_front, widget) {
+#if 0
     if (DEBUG_PAINT_ALL) {
-      printf("window=%s d=%d is_opaque=%d\n",
-	     ct_rect_to_string(&widget->outer_bounds),
-	     widget->data.window.depth, is_opaque(widget));
       region_list_iterate(widget->data.window.to_flush, region) {
 	printf("  region=%s\n", ct_rect_to_string(region));
       } region_list_iterate_end;
     }
+#endif
 
     region_list_iterate(widget->data.window.to_flush, region) {
       region->x += widget->outer_bounds.x;
       region->y += widget->outer_bounds.y;
-      region_list_unlink(&widget->data.window.to_flush, region);
-      region_list_insert(&normalized_regions, region);
+      region_list_unlink(widget->data.window.to_flush, region);
+      region_list_prepend(normalized_regions, region);
     } region_list_iterate_end;
   } widget_list_iterate_end;
-  stop_timer(timer2);
+  if (DEBUG_PAINT_ALL) {
+    stop_timer(timer2);
+  }
 
+#if 0
   if (DEBUG_PAINT_ALL) {
     printf("  normalized_regions\n");
     region_list_iterate(normalized_regions, region) {
       printf("    region=%s\n", ct_rect_to_string(region));
     } region_list_iterate_end;
   }
+#endif
 
-  merge_regions(&normalized_regions);
+  merge_regions(normalized_regions);
 
-  if(DEBUG_PAINT_ALL)
-  printf("%%%%%%%% starting flushing of %d regions\n",
-	 region_list_size(&normalized_regions));
+#if 0
+  if (DEBUG_PAINT_ALL) {
+    printf("  merged normalized_regions\n");
+    region_list_iterate(normalized_regions, region) {
+      printf("    region=%s\n", ct_rect_to_string(region));
+    } region_list_iterate_end;
+  }
+#endif
+
+  if(DEBUG_PAINT_ALL) {
+    printf("starting flushing of %d regions\n",
+           region_list_size(normalized_regions));
+  }
 
 #if DUMP_UPDATES
   sprintf(filename,"whole-c%03d-r000-before.ppm",call);
@@ -716,7 +776,9 @@ void sw_paint_all(void)
   region_nr = 1;
 #endif
 
-  start_timer(timer3);
+  if (DEBUG_PAINT_ALL) {
+    start_timer(timer3);
+  }
   region_list_iterate(normalized_regions, region) {
     int window_nr = 0;
 
@@ -746,32 +808,47 @@ void sw_paint_all(void)
 		region_nr, window_nr);
 	be_write_osda_to_file(whole_osda,filename);
 #endif
-      }else {
-	if(DEBUG_UPDATES)
+      } else {
+	if (DEBUG_UPDATES) {
 	  printf("    disjunkt\n");
+        }
       }
       window_nr++;
     } widget_list_iterate_end;
-    region_list_unlink(&normalized_regions, region);
+    
+#if FLUSH_RECTS
+    flush_rect_to_screen(region);
+#endif
+    
+    region_list_unlink(normalized_regions, region);
     free(region);
 #if DUMP_UPDATES
     region_nr++;
 #endif
   } region_list_iterate_end;
-  stop_timer(timer3);
+  if (DEBUG_PAINT_ALL) {
+    stop_timer(timer3);
+  }
 #if DUMP_UPDATES
   sprintf(filename,"whole-c%03d-r999-after.ppm",call);
   be_write_osda_to_file(whole_osda,filename);
 #endif
 
-  start_timer(timer4);
+#if !FLUSH_RECTS
+  if (DEBUG_PAINT_ALL) {
+    start_timer(timer4);
+  }
   flush_all_to_screen();
-  stop_timer(timer4);
-
-  if(0)
-  printf("PAINT-ALL: update=%fs normalize=%fs flushs=%fs flush-all=%fs\n",
-	 read_timer_seconds(timer1), read_timer_seconds(timer2),
-	 read_timer_seconds(timer3), read_timer_seconds(timer4));
+  if (DEBUG_PAINT_ALL) {
+    stop_timer(timer4);
+  }
+#endif
+  
+  if (DEBUG_PAINT_ALL) {
+    printf("PAINT-ALL: update=%fs normalize=%fs flushs=%fs flush-all=%fs\n",
+           read_timer_seconds(timer1), read_timer_seconds(timer2),
+	   read_timer_seconds(timer3), read_timer_seconds(timer4));
+  }
 }
 
 /*************************************************************************
@@ -802,15 +879,13 @@ struct sw_widget *search_widget(const struct ct_point *pos,
   widget_list_iterate(windows_front_to_back, pwidget) {
     if (pwidget->data.window.shown && pwidget->accepts_events[event_type]
 	&& ct_point_in_rect(pos, &pwidget->outer_bounds)) {
-	bool is_transparent;
+      bool is_transparent;
       tmp = *pos;
       tmp.x -= pwidget->outer_bounds.x;
       tmp.y -= pwidget->outer_bounds.y;
 
-      is_transparent =
+      is_transparent = 
 	  be_is_transparent_pixel(pwidget->data.window.target, &tmp);
-      /*printf("%d %d | %d -> %d\n", tmp.x, tmp.y,
-	pwidget->data.window.depth, is_transparent);*/
       if (!is_transparent) {
 	window = pwidget;
 	break;
@@ -843,7 +918,7 @@ struct sw_widget *sw_window_create_by_clone(struct sw_widget *widget,
 {
   struct sw_widget *result =
       sw_window_create(widget, widget->outer_bounds.width,
-		       widget->outer_bounds.height, NULL, 0, FALSE, depth);
+		       widget->outer_bounds.height, NULL, FALSE, depth);
 
   result->can_be_dragged[BE_MB_LEFT] = widget->can_be_dragged;
   return result;
@@ -892,22 +967,27 @@ void sw_window_set_mouse_press_notify(struct sw_widget *widget,
 /*************************************************************************
   ...
 *************************************************************************/
-void sw_window_canvas_background_region_needs_repaint(struct sw_widget
-						      *widget, const struct ct_rect
-						      *region)
+void sw_window_canvas_background_region_needs_repaint(
+                  struct sw_widget *widget, const struct ct_rect *region)
 {
-#if 0
-  struct ct_size size = { region->width,
-    region->height
-  };
-  struct ct_point pos = { region->x, region->y };
+  struct ct_rect region2;
+  struct ct_size size;
+  struct ct_point pos;
 
-  be_copy_osda_to_osda(get_osda(widget),
+  if (region) {
+    region2 = *region;
+  } else {
+    sw_widget_get_bounds(widget, &region2);
+  }
+
+  size = (struct ct_size){ region2.width, region2.height };
+  pos = (struct ct_point){ region2.x, region2.y };
+  
+  be_copy_osda_to_osda(sw_widget_get_osda(widget),
 		       widget->data.window.canvas_background,
-		       &size, &pos, &pos, 0);
-#endif
-  //add_flush_region(widget, region);
-  widget_needs_paint(widget);
+		       &size, &pos, &pos);
+
+  add_flush_region(widget, &region2);
 }
 
 /*************************************************************************
@@ -957,13 +1037,11 @@ bool deliver_key(const struct be_key *key)
       continue;
     }
     if (pwidget->key && pwidget->key(pwidget, key,pwidget->key_data)) {
-      // printf("deliver key %s to window %p\n", ct_key_format(key), pwidget);
       return TRUE;
     }
 
     widget_list_iterate(pwidget->data.window.children, pwidget2) {
       if (pwidget2->key && pwidget2->key(pwidget2, key, pwidget2->key_data)) {
-        // printf("deliver key %s to widget %p\n", ct_key_format(key), pwidget2);
 	return TRUE;
       }
     } widget_list_iterate_end;
