@@ -130,8 +130,16 @@ struct hash_table {
   unsigned int num_entries;	/* does not included deleted entries */
   unsigned int num_deleted;
   bool frozen;			/* do not auto-resize when set */
+  bool no_shrink;		/* do not auto-shrink when set */
 };
 
+struct hash_iter {
+  struct iterator vtable;
+  const struct hash_bucket *b, *end;
+};
+
+#define HASH_ITER(p) ((struct hash_iter *)(p))
+
 /* Calculate hash value given hash_table ptr and key: */
 #define HASH_VAL(h,k) (((h)->fval)((k), ((h)->num_buckets)))
 
@@ -158,28 +166,7 @@ static void zero_htable(struct hash_table *h)
   h->free_data_func = NULL;
   h->num_buckets = h->num_entries = h->num_deleted = 0;
   h->frozen = FALSE;
-}
-
-
-/**************************************************************************
-  A supplied hash function where key is pointer to int.  
-  Prefers table sizes that are prime numbers.
-**************************************************************************/
-unsigned int hash_fval_int(const void *vkey, unsigned int num_buckets)
-{
-  const unsigned int key = (unsigned int) *(const int*)vkey;
-  return (key % num_buckets);
-}
-
-/**************************************************************************
-  A supplied function for comparison of pointers to int:
-**************************************************************************/
-int hash_fcmp_int(const void *vkey1, const void *vkey2)
-{
-  const int *key1 = vkey1, *key2 = vkey2;
-
-  /* avoid overflow issues: */
-  return (*key1 < *key2) ? -1 : (*key1 > *key2) ? 1 : 0;
+  h->no_shrink = FALSE;
 }
 
 
@@ -286,6 +273,9 @@ static unsigned int calc_appropriate_nbuckets(unsigned int num_entries)
   Allows to specify functions to free the memory allocated for the key and
   user-data that get called when removing the bucket from the hash table or
   changing key/user-data values.
+
+  NB: Be sure to check the "copy constructor" hash_copy() if you change
+  this function significantly.
 **************************************************************************/
 static struct hash_table *hash_new_nbuckets(hash_val_fn_t fval,
 					    hash_cmp_fn_t fcmp,
@@ -438,6 +428,9 @@ static void hash_maybe_resize(struct hash_table *h, bool expandingp)
   if (h->frozen) {
     return;
   }
+  if (!expandingp && h->no_shrink) {
+    return;
+  }
   num_used = h->num_entries + h->num_deleted;
   if (expandingp) {
     limit = FULL_RATIO * h->num_buckets;
@@ -663,9 +656,16 @@ void hash_delete_all_entries(struct hash_table *h)
 {
   unsigned int bucket_nr;
 
-  /* Modeled after hash_key_by_number and hash_delete_entry. */
-  for (bucket_nr = 0; bucket_nr < h->num_buckets; bucket_nr++) {
-    hash_delete_bucket(h, &h->buckets[bucket_nr], NULL, NULL);
+  if (h->free_key_func == NULL && h->free_data_func == NULL) {
+    memset(h->buckets, 0, sizeof(struct hash_bucket) * h->num_buckets);
+    h->num_entries = 0;
+    h->num_deleted = 0;
+    h->frozen = FALSE;
+  } else {
+    /* Modeled after hash_key_by_number and hash_delete_entry. */
+    for (bucket_nr = 0; bucket_nr < h->num_buckets; bucket_nr++) {
+      hash_delete_bucket(h, &h->buckets[bucket_nr], NULL, NULL);
+    }
   }
   hash_maybe_shrink(h);
 }
@@ -771,3 +771,197 @@ const void *hash_value_by_number(const struct hash_table *h,
 {
   return hash_lookup_data(h, hash_key_by_number(h, entry_number));
 }
+
+/**************************************************************************
+  Prevent or allow the hash table automatically shrinking. Returns
+  the old value of the setting.
+**************************************************************************/
+bool hash_set_no_shrink(struct hash_table *h, bool no_shrink)
+{
+  bool old = h->no_shrink;
+  h->no_shrink = no_shrink;
+  return old;
+}
+
+/**************************************************************************
+  "Sizeof" function implementation for generic_iterate hash iterators.
+**************************************************************************/
+size_t hash_iter_sizeof(void)
+{
+  return sizeof(struct hash_iter);
+}
+
+/**************************************************************************
+  Helper function for hash (key, value) pair iteration.
+**************************************************************************/
+void *hash_iter_get_key(const struct iterator *hash_iter)
+{
+  struct hash_iter *it = HASH_ITER(hash_iter);
+  return (void *) it->b->key;
+}
+
+/**************************************************************************
+  Helper function for hash (key, value) pair iteration.
+**************************************************************************/
+void *hash_iter_get_value(const struct iterator *hash_iter)
+{
+  struct hash_iter *it = HASH_ITER(hash_iter);
+  return (void *) it->b->data;
+}
+
+/**************************************************************************
+  Iterator interface 'next' function implementation.
+**************************************************************************/
+static void hash_iter_next(struct iterator *iter)
+{
+  struct hash_iter *it = HASH_ITER(iter);
+  do {
+    it->b++;
+  } while (it->b < it->end && it->b->used != BUCKET_USED);
+}
+
+/**************************************************************************
+  Iterator interface 'get' function implementation. This just returns the
+  iterator itself, so you would need to use hash_iter_get_key/value to
+  get the actual keys and values.
+**************************************************************************/
+static void *hash_iter_get(const struct iterator *iter)
+{
+  return (void *) iter;
+}
+
+/**************************************************************************
+  Iterator interface 'valid' function implementation.
+**************************************************************************/
+static bool hash_iter_valid(const struct iterator *iter)
+{
+  struct hash_iter *it = HASH_ITER(iter);
+  return it->b < it->end;
+}
+
+/**************************************************************************
+  Returns an iterator that iterates over both keys and values of the hash
+  table. NB: iterator_get() returns an iterator pointer, so use the helper
+  functions hash_iter_get_{key,value} to access the key and value.
+**************************************************************************/
+struct iterator *hash_iter_init(struct hash_iter *it,
+                                const struct hash_table *h)
+{
+  if (!h) {
+    return invalid_iter_init(ITERATOR(it));
+  }
+
+  it->vtable.next = hash_iter_next;
+  it->vtable.get = hash_iter_get;
+  it->vtable.valid = hash_iter_valid;
+  it->b = h->buckets - 1;
+  it->end = h->buckets + h->num_buckets;
+
+  /* Seek to the first used bucket. */
+  hash_iter_next(ITERATOR(it));
+
+  return ITERATOR(it);
+}
+
+/**************************************************************************
+  Returns an iterator over the hash table's keys.
+**************************************************************************/
+struct iterator *hash_key_iter_init(struct hash_iter *it,
+                                    const struct hash_table *h)
+{
+  struct iterator *ret;
+
+  if (!h) {
+    return invalid_iter_init(ITERATOR(it));
+  }
+
+  ret = hash_iter_init(it, h);
+  it->vtable.get = hash_iter_get_key;
+  return ret;
+}
+
+/**************************************************************************
+  Returns an iterator over the hash table's values.
+**************************************************************************/
+struct iterator *hash_value_iter_init(struct hash_iter *it,
+                                      const struct hash_table *h)
+{
+  struct iterator *ret;
+
+  if (!h) {
+    return invalid_iter_init(ITERATOR(it));
+  }
+
+  ret = hash_iter_init(it, h);
+  it->vtable.get = hash_iter_get_value;
+  return ret;
+}
+
+/**************************************************************************
+  Returns a newly allocated mostly deep copy of the given hash table.
+
+  NB: This just copies the key and value pointers (it does NOT make a deep
+  copy of those). So using this function with allocated keys/values or with
+  a hash_free_fn_t should be avoided, unless you know what you are doing.
+**************************************************************************/
+struct hash_table *hash_copy(const struct hash_table *other)
+{
+  struct hash_table *h;
+  size_t size;
+
+  if (!other) {
+    return NULL;
+  }
+
+  h = fc_malloc(sizeof(*h));
+
+  /* Copy fields. */
+  *h = *other;
+
+  /* But make fresh buckets. */
+  size = h->num_buckets * sizeof(struct hash_bucket);
+  h->buckets = fc_malloc(size);
+
+  /* NB: Shallow copy. */
+  memcpy(h->buckets, other->buckets, size);
+
+  return h;
+}
+
+/**************************************************************************
+  Returns TRUE if the two hash tables are equal. If the hash tables do
+  not have the same number of elements, or have different hashing, compare
+  or free functions, they will be "unequal". Otherwise this function will
+  compare key and value pointers.
+**************************************************************************/
+bool hash_equal(const struct hash_table *a, const struct hash_table *b)
+{
+  struct hash_iter hita, hitb;
+  struct iterator *ita, *itb;
+
+  if (a == b) {
+    return TRUE;
+  }
+  if (!a || !b) {
+    return FALSE;
+  }
+  if (a->fval != b->fval || a->fcmp != b->fcmp
+      || a->free_key_func != b->free_key_func
+      || a->free_data_func != b->free_data_func
+      || hash_num_entries(a) != hash_num_entries(b)) {
+    return FALSE;
+  }
+
+  ita = hash_iter_init(&hita, a);
+  itb = hash_iter_init(&hitb, b);
+  while (iterator_valid(ita) && iterator_valid(itb)) {
+    if (hash_iter_get_key(ita) != hash_iter_get_key(itb)
+        || hash_iter_get_value(ita) != hash_iter_get_value(itb)) {
+      return FALSE;
+    }
+    iterator_next(ita);
+    iterator_next(itb);
+  }
+
+  return TRUE;
+}
