@@ -358,8 +358,7 @@ static bool check_include(struct inputfile *inf)
     len = strlen(include_prefix);
   }
   assert_sanity(inf);
-  if (inf->at_eof || inf->in_string || inf->cur_line.n <= len
-      || inf->cur_line_pos > 0) {
+  if (inf->in_string || inf->cur_line.n <= len || inf->cur_line_pos > 0) {
     return FALSE;
   }
   if (strncmp(inf->cur_line.str, include_prefix, len)!=0) {
@@ -469,28 +468,12 @@ static bool read_a_line(struct inputfile *inf)
     if (!ret) {
       /* fgets failed */
       inf->at_eof = TRUE;
-      if (pos != 0) {
-	inf_warn(inf, "missing newline at EOF, or failed read");
-	/* treat as simple EOF, ignoring last line: */
-	pos = 0;
-      }
-      line->str[0] = '\0';
-      line->n = 0;
       if (inf->in_string) {
-	/* Note: Don't allow multi-line strings to cross "include"
-	   boundaries */
-	inf_log(inf, LOG_ERROR, "Multi-line string went to end-of-file");
+        /* Note: Don't allow multi-line strings to cross "include"
+         * boundaries */
+        inf_log(inf, LOG_ERROR, "Multi-line string went to end-of-file");
         return FALSE;
       }
-      if (inf->included_from) {
-	/* Pop the include, and get next line from file above instead. */
-	struct inputfile *inc = inf->included_from;
-	inf_close_partial(inf);
-	*inf = *inc;		/* so the user pointer in still valid
-				   (and inf pointers in calling functions) */
-	free(inc);
-	return read_a_line(inf);
-      }
       break;
     }
     
@@ -516,7 +499,23 @@ static bool read_a_line(struct inputfile *inf)
   if (check_include(inf)) {
     return read_a_line(inf);
   }
-  return (!inf->at_eof);
+
+  if (inf->at_eof) {
+    line->str[0] = '\0';
+    line->n = 0;
+    if (inf->included_from) {
+      /* Pop the include, and get next line from file above instead. */
+      struct inputfile *inc = inf->included_from;
+      inf_close_partial(inf);
+      *inf = *inc;    /* so the user pointer in still valid
+                       * (and inf pointers in calling functions) */
+      free(inc);
+      return read_a_line(inf);
+    }
+    return FALSE;
+  } else {
+    return TRUE;
+  }
 }
 
 /********************************************************************** 
@@ -577,9 +576,7 @@ static void inf_warn(struct inputfile *inf, const char *message)
 /********************************************************************** 
   ...
 ***********************************************************************/
-static const char *get_token(struct inputfile *inf,
-			     enum inf_token_type type,
-			     bool required)
+const char *inf_token(struct inputfile *inf, enum inf_token_type type)
 {
   const char *c;
   const char *name;
@@ -592,7 +589,7 @@ static const char *get_token(struct inputfile *inf,
   func = tok_tab[type].func;
   
   if (!func) {
-    freelog(LOG_NORMAL, "token type %d (%s) not supported yet", type, name);
+    freelog(LOG_ERROR, "token type %d (%s) not supported yet", type, name);
     c = NULL;
   } else {
     if (!have_line(inf))
@@ -603,33 +600,11 @@ static const char *get_token(struct inputfile *inf,
       c = func(inf);
     }
   }
-  if (c) {
-    if (INF_DEBUG_FOUND) {
-      freelog(LOG_DEBUG, "inputfile: found %s '%s'", name, inf->token.str);
-    }
-  } else {
-    if (!required) {
-      if (INF_DEBUG_NOT_FOUND) {
-	freelog(LOG_DEBUG, "inputfile: did not find %s", name);
-      }
-    } else {
-      /* should be varargs... */
-      freelog(LOG_ERROR, "Did not find required token: %s", name);
-      return NULL;
-    }
+  if (c && INF_DEBUG_FOUND) {
+    freelog(LOG_DEBUG, "inputfile: found %s '%s'", name, inf->token.str);
   }
   return c;
 }
-  
-const char *inf_token(struct inputfile *inf, enum inf_token_type type)
-{
-  return get_token(inf, type, FALSE);
-}
-
-const char *inf_token_required(struct inputfile *inf, enum inf_token_type type)
-{
-  return get_token(inf, type, TRUE);
-}
 
 /********************************************************************** 
   Read as many tokens of specified type as possible, discarding
@@ -785,6 +760,7 @@ static const char *get_token_value(struct inputfile *inf)
   char *c, *start;
   char trailing;
   bool has_i18n_marking = FALSE;
+  char border_character = '\"';
   
   assert(have_line(inf));
 
@@ -829,8 +805,13 @@ static const char *get_token_value(struct inputfile *inf)
       return NULL;
   }
 
-  if (!(*c == '\"'))
+  border_character = *c;
+
+  if (border_character != '\"'
+      && border_character != '\''
+      && border_character != '$') {
     return NULL;
+  }
 
   /* From here, we know we have a string, we just have to find the
      trailing (un-escaped) double-quote.  We read in extra lines if
@@ -858,7 +839,7 @@ static const char *get_token_value(struct inputfile *inf)
   for(;;) {
     int pos;
     
-    while(*c != '\0' && *c != '\"') {
+    while(*c != '\0' && *c != border_character) {
       /* skip over escaped chars, including backslash-doublequote,
 	 and backslash-backslash: */
       if (*c == '\\' && *(c+1) != '\0') {  
@@ -866,8 +847,11 @@ static const char *get_token_value(struct inputfile *inf)
       }
       c++;
     }
-    if (*c == '\"') 
-      break;      /* found end of string */
+
+    if (*c == border_character) {
+      /* Found end of string */
+      break;
+    }
 
     /* Accumulate to partial string and try more lines;
      * note partial->n must be _exactly_ the right size, so we
