@@ -154,6 +154,7 @@
 #include <string.h>
 
 #include "astring.h"
+#include "fcintl.h"
 #include "genlist.h"
 #include "hash.h"
 #include "inputfile.h"
@@ -186,6 +187,7 @@ struct entry {
   int dim;			/* vector's size */
   int  used;			/* number of times entry looked up */
   char *comment;                /* comment, may be NULL */
+  bool escaped;                 /* " or $. Usually TRUE */
 };
 
 /* create a 'struct entry_list' and related functions: */
@@ -199,7 +201,7 @@ struct entry {
 
 struct section {
   char *name;
-  struct entry_list entries;
+  struct entry_list *entries;
 };
 
 /* create a 'struct section_list' and related functions: */
@@ -226,8 +228,9 @@ static void secfilehash_check(struct section_file *file);
 static void secfilehash_insert(struct section_file *file,
 			       char *key, struct entry *data);
 
-static char *minstrdup(struct sbuffer *sb, const char *str);
-static char *moutstr(char *str);
+static char *minstrdup(struct sbuffer *sb, const char *str,
+                       bool full_escapes);
+static char *moutstr(char *str, bool full_escapes);
 
 static struct entry*
 section_file_lookup_internal(struct section_file *my_section_file,  
@@ -236,6 +239,14 @@ static struct entry*
 section_file_insert_internal(struct section_file *my_section_file, 
 			     char *fullpath);
 
+/* Only need use these explicitly on section_files constructed,
+   not for ones generated by section_file_load:
+*/
+static bool secfilehash_hashash(struct section_file *file);
+static void secfilehash_build(struct section_file *file,
+			      bool allow_duplicates);
+static void secfilehash_free(struct section_file *file);
+
 /**************************************************************************
   Return the filename the sectionfile was loaded as, or "(anonymous)"
   if this sectionfile was created rather than loaded from file.
@@ -257,8 +268,8 @@ const char *secfile_filename(const struct section_file *file)
 void section_file_init(struct section_file *file)
 {
   file->filename = NULL;
-  file->sections = fc_malloc(sizeof(struct section_list));
-  section_list_init(file->sections);
+  file->sections = section_list_new();
+  file->hash_sections = hash_new(hash_fval_string, hash_fcmp_string);
   file->num_entries = 0;
   file->hashd = NULL;
   file->sb = sbuf_new();
@@ -272,16 +283,16 @@ void section_file_free(struct section_file *file)
   /* all the real data is stored in the sbuffer;
      just free the list meta-data:
   */
-  section_list_iterate(*file->sections, psection) {
-    entry_list_unlink_all(&psection->entries);
-  }
-  section_list_iterate_end;
-  
-  section_list_unlink_all(file->sections);
-  
-  free(file->sections);
+  section_list_iterate(file->sections, psection) {
+    entry_list_free(psection->entries);
+  } section_list_iterate_end;
+  section_list_free(file->sections);
   file->sections = NULL;
 
+  /* free the sections hash data: */
+  hash_free(file->hash_sections);
+  file->hash_sections = NULL;
+
   /* free the hash data: */
   if(secfilehash_hashash(file)) {
     secfilehash_free(file);
@@ -297,6 +308,26 @@ void section_file_free(struct section_file *file)
   file->filename = NULL;
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+static struct section*
+section_file_append_section(struct section_file *sf, const char *sec_name)
+{
+  struct section *psection;
+
+  psection = sbuf_malloc(sf->sb, sizeof(*psection));
+  psection->name = sbuf_strdup(sf->sb, sec_name);
+  psection->entries = entry_list_new();
+  section_list_append(sf->sections, psection);
+
+  if (!hash_insert(sf->hash_sections, psection->name, psection)) {
+    freelog(LOG_ERROR, "Section \"%s\" name collision", sec_name);
+  }
+
+  return psection;
+}
+
 /**************************************************************************
   Print log messages for any entries in the file which have
   not been looked up -- ie, unused or unrecognised entries.
@@ -308,7 +339,7 @@ void section_file_check_unused(struct section_file *file, const char *filename)
 {
   int any = 0;
 
-  section_list_iterate(*file->sections, psection) {
+  section_list_iterate(file->sections, psection) {
     entry_list_iterate(psection->entries, pentry) {
       if (pentry->used == 0) {
 	if (any == 0 && filename) {
@@ -324,6 +355,21 @@ void section_file_check_unused(struct section_file *file, const char *filename)
   section_list_iterate_end;
 }
 
+/**************************************************************************
+  Initialize the entry struct and set to default (empty) values.
+**************************************************************************/
+static void entry_init(struct entry *pentry)
+{
+  pentry->name = NULL;
+  pentry->svalue = NULL;
+  pentry->ivalue = 0;
+  pentry->used = 0;
+  pentry->dim = 0;
+  pentry->vec_values = NULL;
+  pentry->comment = NULL;
+  pentry->escaped = TRUE;
+}
+
 /**************************************************************************
   Return a new entry struct, allocated from sb, with given name,
   and where tok is a "value" return token from inputfile.
@@ -335,16 +381,18 @@ static struct entry *new_entry(struct sbuffer *sb, const char *name,
   struct entry *pentry;
 
   pentry = sbuf_malloc(sb, sizeof(struct entry));
+  entry_init(pentry);
   pentry->name = sbuf_strdup(sb, name);
-  pentry->comment = NULL;
-  if (tok[0] == '\"') {
-    pentry->svalue = minstrdup(sb, tok+1);
-    pentry->ivalue = 0;
+  if (tok[0] != '-' && !my_isdigit(tok[0])) {
+    /* It is not integer, but string with some border character. */
+    pentry->escaped = (tok[0] != '$'); /* Border character '$' means no escapes */
+
+    /* minstrdup() starting after border character */
+    pentry->svalue = minstrdup(sb, tok+1, pentry->escaped);
     if (SECF_DEBUG_ENTRIES) {
       freelog(LOG_DEBUG, "entry %s '%s'", name, pentry->svalue);
     }
   } else {
-    pentry->svalue = NULL;
     if (sscanf(tok, "%d", &pentry->ivalue) != 1) {
       freelog(LOG_ERROR, "'%s' isn't an integer", tok);
     }
@@ -352,7 +400,6 @@ static struct entry *new_entry(struct sbuffer *sb, const char *name,
       freelog(LOG_DEBUG, "entry %s %d", name, pentry->ivalue);
     }
   }
-  pentry->used = FALSE;
   return pentry;
 }
 
@@ -362,20 +409,7 @@ static struct entry *new_entry(struct sbuffer *sb, const char *name,
 static struct section *find_section_by_name(struct section_file *sf,
 					    const char *name)
 {
-  /*
-   * Search backwards since on insertion the requested section is most
-   * likely at the end (on lookup it doesn't matter).
-   *
-   * Nonetheless this is slow if there are lots of sections.  We could have
-   * a hash on section names to speed it up.
-   */
-  section_list_iterate_rev(*sf->sections, psection) {
-    if (strcmp(psection->name, name) == 0) {
-      return psection;
-    }
-  } section_list_iterate_rev_end;
-
-  return NULL;
+  return hash_lookup_data(sf->hash_sections, name);
 }	
 
 /**************************************************************************
@@ -384,7 +418,8 @@ static struct section *find_section_by_name(struct section_file *sf,
 static bool section_file_read_dup(struct section_file *sf,
       	      	      	      	  const char *filename,
       	      	      	      	  struct inputfile *inf,
-				  bool allow_duplicates)
+				  bool allow_duplicates,
+				  const char *requested_section)
 {
   struct section *psection = NULL;
   struct entry *pentry;
@@ -396,10 +431,8 @@ static bool section_file_read_dup(struct section_file *sf,
   struct astring base_name = ASTRING_INIT;    /* for table or single entry */
   struct astring entry_name = ASTRING_INIT;
   struct astring_vector columns;    /* astrings for column headings */
+  bool found_my_section = FALSE;
 
-  if (!inf) {
-    return FALSE;
-  }
   section_file_init(sf);
   if (filename) {
     sf->filename = mystrdup(filename);
@@ -424,6 +457,16 @@ static bool section_file_read_dup(struct section_file *sf,
     }
     tok = inf_token(inf, INF_TOK_SECTION_NAME);
     if (tok) {
+      if (found_my_section) {
+	/* This shortcut will stop any further loading after the requested
+	 * section has been loaded (i.e., at the start of a new section).
+	 * This is needed to make the behavior useful, since the whole
+	 * purpose is to short-cut further loading of the file.  However
+	 * normally a section may be split up, and that will no longer
+	 * work here because it will be short-cut. */
+	inf_log(inf, LOG_DEBUG, "found requested section; finishing");
+	return TRUE;
+      }
       if (table_state) {
 	inf_log(inf, LOG_ERROR, "new section during table");
         return FALSE;
@@ -435,24 +478,37 @@ static bool section_file_read_dup(struct section_file *sf,
       */
       psection = find_section_by_name(sf, tok);
       if (!psection) {
-	psection = sbuf_malloc(sb, sizeof(struct section));
-	psection->name = sbuf_strdup(sb, tok);
-	entry_list_init(&psection->entries);
-	section_list_insert_back(sf->sections, psection);
+	if (!requested_section || strcmp(tok, requested_section) == 0) {
+	  psection = section_file_append_section(sf, tok);
+	  if (requested_section) {
+	    found_my_section = TRUE;
+	  }
+	}
+      }
+      if (!inf_token(inf, INF_TOK_EOL)) {
+        inf_log(inf, LOG_ERROR, "expected end of line");
+        return FALSE;
       }
-      (void) inf_token_required(inf, INF_TOK_EOL);
       continue;
     }
+#if 0
     if (!psection) {
+      /* This used to be an error.  However there's no reason it shouldn't
+       * be allowed, and it breaks the rest of the requested_section code.
+       * It's been defined out but may be of use sometime. */
       inf_log(inf, LOG_ERROR, "data before first section");
       return FALSE;
     }
+#endif
     if (inf_token(inf, INF_TOK_TABLE_END)) {
       if (!table_state) {
 	inf_log(inf, LOG_ERROR, "misplaced \"}\"");
         return FALSE;
       }
-      (void) inf_token_required(inf, INF_TOK_EOL);
+      if (!inf_token(inf, INF_TOK_EOL)) {
+        inf_log(inf, LOG_ERROR, "expected end of line");
+        return FALSE;
+      }
       table_state = FALSE;
       continue;
     }
@@ -463,7 +519,8 @@ static bool section_file_read_dup(struct section_file *sf,
 
 	i++;
 	inf_discard_tokens(inf, INF_TOK_EOL);  	/* allow newlines */
-	if (!(tok = inf_token_required(inf, INF_TOK_VALUE))) {
+	if (!(tok = inf_token(inf, INF_TOK_VALUE))) {
+          inf_log(inf, LOG_ERROR, "expected value");
           return FALSE;
         }
 
@@ -479,17 +536,25 @@ static bool section_file_read_dup(struct section_file *sf,
 		      columns.p[num_columns - 1].str,
 		      (int) (i - num_columns + 1));
 	}
-	pentry = new_entry(sb, entry_name.str, tok);
-	entry_list_insert_back(&psection->entries, pentry);
-	sf->num_entries++;
+	if (psection) {
+	  /* Load this entry (if psection == NULL the entry is silently
+	   * skipped). */
+	  pentry = new_entry(sb, entry_name.str, tok);
+	  entry_list_append(psection->entries, pentry);
+	  sf->num_entries++;
+	}
       } while(inf_token(inf, INF_TOK_COMMA));
-      
-      (void) inf_token_required(inf, INF_TOK_EOL);
+
+      if (!inf_token(inf, INF_TOK_EOL)) {
+        inf_log(inf, LOG_ERROR, "expected end of line");
+        return FALSE;
+      }
       table_lineno++;
       continue;
     }
     
-    if (!(tok = inf_token_required(inf, INF_TOK_ENTRY_NAME))) {
+    if (!(tok = inf_token(inf, INF_TOK_ENTRY_NAME))) {
+      inf_log(inf, LOG_ERROR, "expected entry name");
       return FALSE;
     }
 
@@ -504,12 +569,13 @@ static bool section_file_read_dup(struct section_file *sf,
       do {
 	i++;
 	inf_discard_tokens(inf, INF_TOK_EOL);  	/* allow newlines */
-	if (!(tok = inf_token_required(inf, INF_TOK_VALUE))) {
+	if (!(tok = inf_token(inf, INF_TOK_VALUE))) {
+          inf_log(inf, LOG_ERROR, "expected value");
           return FALSE;
         }
 	if( tok[0] != '\"' ) {
 	  inf_log(inf, LOG_ERROR, "table column header non-string");
-          return FALSE;
+         return FALSE;
 	}
 	{ 	/* expand columns: */
 	  int j, n_prev;
@@ -526,8 +592,11 @@ static bool section_file_read_dup(struct section_file *sf,
 	strcpy(columns.p[i].str, tok+1);
 	
       } while(inf_token(inf, INF_TOK_COMMA));
-      
-      (void) inf_token_required(inf, INF_TOK_EOL);
+
+      if (!inf_token(inf, INF_TOK_EOL)) {
+        inf_log(inf, LOG_ERROR, "expected end of line");
+        return FALSE;
+      }
       table_state = TRUE;
       table_lineno=0;
       continue;
@@ -537,7 +606,8 @@ static bool section_file_read_dup(struct section_file *sf,
     do {
       i++;
       inf_discard_tokens(inf, INF_TOK_EOL);  	/* allow newlines */
-      if (!(tok = inf_token_required(inf, INF_TOK_VALUE))) {
+      if (!(tok = inf_token(inf, INF_TOK_VALUE))) {
+        inf_log(inf, LOG_ERROR, "expected value");
         return FALSE;
       }
       if (i==0) {
@@ -548,10 +618,17 @@ static bool section_file_read_dup(struct section_file *sf,
 		    "%s,%d", base_name.str, i);
 	pentry = new_entry(sb, entry_name.str, tok);
       }
-      entry_list_insert_back(&psection->entries, pentry);
-      sf->num_entries++;
+      if (psection) {
+	/* Load this entry (if psection == NULL the entry is silently
+	 * skipped). */
+	entry_list_append(psection->entries, pentry);
+	sf->num_entries++;
+      }
     } while(inf_token(inf, INF_TOK_COMMA));
-    (void) inf_token_required(inf, INF_TOK_EOL);
+    if (!inf_token(inf, INF_TOK_EOL)) {
+      inf_log(inf, LOG_ERROR, "expected end of line");
+      return FALSE;
+    }
   }
   
   if (table_state) {
@@ -562,9 +639,6 @@ static bool section_file_read_dup(struct section_file *sf,
     }
     exit(EXIT_FAILURE);
   }
-
-  inf_close(inf);
-  inf = NULL;
   
   astr_free(&base_name);
   astr_free(&entry_name);
@@ -583,14 +657,36 @@ static bool section_file_read_dup(struct section_file *sf,
 **************************************************************************/
 bool section_file_load(struct section_file *my_section_file,
 		      const char *filename)
+{
+  return section_file_load_section(my_section_file, filename, NULL);
+}
+
+/**************************************************************************
+  Like section_file_load, but this function will only load one "part" of
+  the section file.  For instance if you pass in "tutorial", then it will
+  only load the [tutorial] section.
+
+  Passing in NULL will give identical results to section_file_load.
+**************************************************************************/
+bool section_file_load_section(struct section_file *my_section_file,
+			       const char *filename, const char *part)
 {
   char real_filename[1024];
   struct inputfile *inf;
+  bool success;
 
   interpret_tilde(real_filename, sizeof(real_filename), filename);
   inf = inf_from_file(real_filename, datafilename);
 
-  return section_file_read_dup(my_section_file, real_filename, inf, TRUE);
+  if (!inf) {
+    return FALSE;
+  }
+  success = section_file_read_dup(my_section_file, real_filename,
+                                  inf, TRUE, part);
+
+  inf_close(inf);
+
+  return success;
 }
 
 /**************************************************************************
@@ -601,11 +697,20 @@ bool section_file_load_nodup(struct section_file *my_section_file,
 {
   char real_filename[1024];
   struct inputfile *inf;
+  bool success;
 
   interpret_tilde(real_filename, sizeof(real_filename), filename);
   inf = inf_from_file(real_filename, datafilename);
 
-  return section_file_read_dup(my_section_file, real_filename, inf, FALSE);
+  if (!inf) {
+    return FALSE;
+  }
+  success = section_file_read_dup(my_section_file, real_filename,
+			       inf, FALSE, NULL);
+
+  inf_close(inf);
+
+  return success;
 }
 
 /**************************************************************************
@@ -614,9 +719,17 @@ bool section_file_load_nodup(struct section_file *my_section_file,
 bool section_file_load_from_stream(struct section_file *my_section_file,
 				   fz_FILE * stream)
 {
+  bool success;
   struct inputfile *inf = inf_from_stream(stream, datafilename);
 
-  return section_file_read_dup(my_section_file, NULL, inf, TRUE);
+  if (!inf) {
+    return FALSE;
+  }
+  success = section_file_read_dup(my_section_file, NULL, inf, TRUE, NULL);
+
+  inf_close(inf);
+
+  return success;
 }
 
 /**************************************************************************
@@ -640,29 +753,31 @@ bool section_file_load_from_stream(struct section_file *my_section_file,
 **************************************************************************/
 bool section_file_save(struct section_file *my_section_file,
                        const char *filename,
-		       int compression_level)
+                       int compression_level,
+                       enum fz_method compression_method)
 {
   char real_filename[1024];
   fz_FILE *fs;
-  struct genlist_link *ent_iter, *save_iter, *col_iter;
+  const struct genlist_link *ent_iter, *save_iter, *col_iter;
   struct entry *pentry, *col_pentry;
   int i;
   
   interpret_tilde(real_filename, sizeof(real_filename), filename);
-  fs = fz_from_file(real_filename, "w", FZ_ZLIB, compression_level);
+  fs = fz_from_file(real_filename, "w", compression_method, compression_level);
 
-  if (!fs)
+  if (!fs) {
     return FALSE;
+  }
 
-  section_list_iterate(*my_section_file->sections, psection) {
+  section_list_iterate(my_section_file->sections, psection) {
     fz_fprintf(fs, "\n[%s]\n", psection->name);
 
     /* Following doesn't use entry_list_iterate() because we want to do
      * tricky things with the iterators...
      */
-    for(ent_iter = psection->entries.list.head_link;
-	(pentry = ITERATOR_PTR(ent_iter));
-	ITERATOR_NEXT(ent_iter)) {
+    for (ent_iter = genlist_head(entry_list_base(psection->entries));
+         ent_iter && (pentry = genlist_link_data(ent_iter));
+         ent_iter = genlist_link_next(ent_iter)) {
 
       /* Tables: break out of this loop if this is a non-table
        * entry (pentry and ent_iter unchanged) or after table (pentry
@@ -706,7 +821,8 @@ bool section_file_save(struct section_file *my_section_file,
 	/* write the column names, and calculate ncol: */
 	ncol = 0;
 	col_iter = save_iter;
-	for( ; (col_pentry = ITERATOR_PTR(col_iter)); ITERATOR_NEXT(col_iter)) {
+        for(; (col_pentry = genlist_link_data(col_iter));
+            col_iter = genlist_link_next(col_iter)) {
 	  if(strncmp(col_pentry->name, first, offset) != 0)
 	    break;
 	  fz_fprintf(fs, "%c\"%s\"", (ncol==0?' ':','), col_pentry->name+offset);
@@ -723,8 +839,8 @@ bool section_file_save(struct section_file *my_section_file,
 	for(;;) {
 	  char expect[128];	/* pentry->name we're expecting */
 
-	  pentry = ITERATOR_PTR(ent_iter);
-	  col_pentry = ITERATOR_PTR(col_iter);
+	  pentry = genlist_link_data(ent_iter);
+	  col_pentry = genlist_link_data(col_iter);
 
 	  my_snprintf(expect, sizeof(expect), "%s%d.%s",
 		      base, irow, col_pentry->name+offset);
@@ -743,13 +859,15 @@ bool section_file_save(struct section_file *my_section_file,
 	       * format without an error message. */
 	      freelog(LOG_ERROR,
 		      "In file %s, there is no entry in the registry for \n"
-		      "%s (or the entries are out of order. This means a \n"
+		      "%s.%s (or the entries are out of order. This means a \n"
 		      "less efficient non-tabular format will be used. To\n"
 		      "avoid this make sure all rows of a table are filled\n"
-		      "out with an entry for every column.  This is surely\n"
-		      "a bug so if you're reading this message, report it\n"
-		      "to bugs@freeciv.org.",
-		      real_filename, expect);
+		      "out with an entry for every column.",
+		      real_filename, psection->name, expect);
+	      freelog(LOG_ERROR,
+                      /* TRANS: No full stop after the URL, could cause confusion. */
+                      _("Please report this message at %s"),
+		      BUG_URL);
 	      fz_fprintf(fs, "\n");
 	    }
 	    fz_fprintf(fs, "}\n");
@@ -758,13 +876,13 @@ bool section_file_save(struct section_file *my_section_file,
 
 	  if(icol>0)
 	    fz_fprintf(fs, ",");
-	  if(pentry->svalue) 
-	    fz_fprintf(fs, "\"%s\"", moutstr(pentry->svalue));
+	  if(pentry->svalue)
+	    fz_fprintf(fs, "%s", moutstr(pentry->svalue, pentry->escaped));
 	  else
 	    fz_fprintf(fs, "%d", pentry->ivalue);
-	  
-	  ITERATOR_NEXT(ent_iter);
-	  ITERATOR_NEXT(col_iter);
+
+          ent_iter = genlist_link_next(ent_iter);
+          col_iter = genlist_link_next(col_iter);
 	  
 	  icol++;
 	  if(icol==ncol) {
@@ -779,13 +897,13 @@ bool section_file_save(struct section_file *my_section_file,
       if(!pentry) break;
       
       if (pentry->vec_values) {
-        fz_fprintf(fs, "%s=\"%s\"", pentry->name,
-	           moutstr(pentry->vec_values[0]));
+        fz_fprintf(fs, "%s=%s", pentry->name,
+	           moutstr(pentry->vec_values[0], pentry->escaped));
 	for (i = 1; i < pentry->dim; i++) {
-	  fz_fprintf(fs, ", \"%s\"", moutstr(pentry->vec_values[i]));
+	  fz_fprintf(fs, ", %s", moutstr(pentry->vec_values[i], pentry->escaped));
 	}
       } else if (pentry->svalue) {
-	fz_fprintf(fs, "%s=\"%s\"", pentry->name, moutstr(pentry->svalue));
+	fz_fprintf(fs, "%s=%s", pentry->name, moutstr(pentry->svalue, pentry->escaped));
       } else {
 	fz_fprintf(fs, "%s=%d", pentry->name, pentry->ivalue);
       }
@@ -799,7 +917,7 @@ bool section_file_save(struct section_file *my_section_file,
   }
   section_list_iterate_end;
   
-  (void) moutstr(NULL);		/* free internal buffer */
+  (void) moutstr(NULL, TRUE);		/* free internal buffer */
 
   if (fz_ferror(fs) != 0) {
     freelog(LOG_ERROR, "Error before closing %s: %s", real_filename,
@@ -891,10 +1009,6 @@ void secfile_insert_int(struct section_file *my_section_file,
   pentry=section_file_insert_internal(my_section_file, buf);
 
   pentry->ivalue=val;
-  pentry->svalue = NULL;
-  pentry->vec_values = NULL;
-  pentry->dim = 0;
-  pentry->comment = NULL;
 }
 
 /**************************************************************************
@@ -915,9 +1029,6 @@ void secfile_insert_int_comment(struct section_file *my_section_file,
   pentry = section_file_insert_internal(my_section_file, buf);
 
   pentry->ivalue = val;
-  pentry->svalue = NULL;
-  pentry->vec_values = NULL;
-  pentry->dim = 0;
   pentry->comment = sbuf_strdup(my_section_file->sb, comment);
 }
 
@@ -944,10 +1055,6 @@ void secfile_insert_bool(struct section_file *my_section_file,
   }
 
   pentry->ivalue = val ? 1 : 0;
-  pentry->svalue = NULL;
-  pentry->vec_values = NULL;
-  pentry->dim = 0;
-  pentry->comment = NULL;
 }
 
 /**************************************************************************
@@ -966,11 +1073,29 @@ void secfile_insert_str(struct section_file *my_section_file,
 
   pentry = section_file_insert_internal(my_section_file, buf);
   pentry->svalue = sbuf_strdup(my_section_file->sb, sval);
-  pentry->vec_values = NULL;
-  pentry->dim = 0;
-  pentry->comment = NULL;
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void secfile_insert_str_noescape(struct section_file *my_section_file,
+			const char *sval, const char *path, ...)
+{
+  struct entry *pentry;
+  char buf[MAX_LEN_BUFFER];
+  va_list ap;
+
+  va_start(ap, path);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
+  va_end(ap);
+
+  pentry = section_file_insert_internal(my_section_file, buf);
+  pentry->svalue = sbuf_strdup(my_section_file->sb, sval);
+  pentry->escaped = FALSE;
+}
+
+
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -988,8 +1113,6 @@ void secfile_insert_str_comment(struct section_file *my_section_file,
 
   pentry = section_file_insert_internal(my_section_file, buf);
   pentry->svalue = sbuf_strdup(my_section_file->sb, sval);
-  pentry->vec_values = NULL;
-  pentry->dim = 0;
   pentry->comment = sbuf_strdup(my_section_file->sb, comment);
 }
 
@@ -1024,9 +1147,6 @@ void secfile_insert_str_vec(struct section_file *my_section_file,
   for (i = 0; i < dim; i++) {
     pentry->vec_values[i] = sbuf_strdup(my_section_file->sb, values[i]);
   }
-				 
-  pentry->svalue = NULL;
-  pentry->comment = NULL;
 }
 
 /**************************************************************************
@@ -1064,7 +1184,7 @@ int secfile_lookup_int(struct section_file *my_section_file,
   entry does not exist.  If the entry exists as a string, then die.
 **************************************************************************/
 int secfile_lookup_int_default(struct section_file *my_section_file,
-			       int def, const char *path, ...)
+                               int def, const char *path, ...)
 {
   struct entry *pentry;
   char buf[MAX_LEN_BUFFER];
@@ -1077,14 +1197,55 @@ int secfile_lookup_int_default(struct section_file *my_section_file,
   if(!(pentry=section_file_lookup_internal(my_section_file, buf))) {
     return def;
   }
-  if(pentry->svalue) {
-    freelog(LOG_FATAL, "sectionfile %s contains a '%s', but string not integer",
-	    secfile_filename(my_section_file), buf);
-    exit(EXIT_FAILURE);
-  }
+
+  RETURN_VAL_IF_FAIL_MSG(pentry->svalue == NULL, def, "sectionfile %s "
+                         "contains a '%s', but string not integer",
+                         secfile_filename(my_section_file), buf);
+
   return pentry->ivalue;
 }
 
+/**************************************************************************
+  As secfile_lookup_int_default(), but also check the range [min/max].
+**************************************************************************/
+int secfile_lookup_int_default_min_max(error_func_t error_handle,
+                                       struct section_file *my_section_file,
+                                       int def, int minval, int maxval,
+                                       const char *path, ...)
+{
+  assert(error_handle != NULL);
+
+  char buf[MAX_LEN_BUFFER];
+  va_list ap;
+  int ival;
+
+  va_start(ap, path);
+  my_vsnprintf(buf, sizeof(buf), path, ap);
+  va_end(ap);
+
+  ival = secfile_lookup_int_default(my_section_file, def, "%s", buf);
+
+  if(ival < minval) {
+    error_handle(LOG_ERROR, "sectionfile %s: '%s' should be in the "
+                            "interval [%d, %d] but is %d; using the "
+                            "minimal value.",
+                 secfile_filename(my_section_file), buf, minval, maxval,
+                 ival);
+    ival = minval;
+  }
+
+  if(ival > maxval) {
+    error_handle(LOG_ERROR, "sectionfile %s: '%s' should be in the "
+                            "interval [%d, %d] but is %d; using the "
+                            "maximal value.",
+                 secfile_filename(my_section_file), buf, minval, maxval,
+                 ival);
+    ival = maxval;
+  }
+
+  return ival;
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -1126,7 +1287,7 @@ bool secfile_lookup_bool(struct section_file *my_section_file,
   entry does not exist.  If the entry exists as a string, then die.
 **************************************************************************/
 bool secfile_lookup_bool_default(struct section_file *my_section_file,
-				 bool def, const char *path, ...)
+                                 bool def, const char *path, ...)
 {
   struct entry *pentry;
   char buf[MAX_LEN_BUFFER];
@@ -1139,18 +1300,17 @@ bool secfile_lookup_bool_default(struct section_file *my_section_file,
   if(!(pentry=section_file_lookup_internal(my_section_file, buf))) {
     return def;
   }
-  if(pentry->svalue) {
-    freelog(LOG_FATAL, "sectionfile %s contains a '%s', but string not integer",
-	    secfile_filename(my_section_file), buf);
-    exit(EXIT_FAILURE);
-  }
+
+  RETURN_VAL_IF_FAIL_MSG(pentry->svalue == NULL, def, "sectionfile %s "
+                         "contains a '%s', but string not integer",
+                         secfile_filename(my_section_file), buf);
 
   if (pentry->ivalue != 0 && pentry->ivalue != 1) {
     freelog(LOG_ERROR, "Value read for key %s isn't boolean: %d", buf,
-	    pentry->ivalue);
+            pentry->ivalue);
     pentry->ivalue = 1;
   }
-  
+
   return pentry->ivalue != 0;
 }
 
@@ -1159,7 +1319,7 @@ bool secfile_lookup_bool_default(struct section_file *my_section_file,
   entry does not exist.  If the entry exists as an int, then die.
 **************************************************************************/
 char *secfile_lookup_str_default(struct section_file *my_section_file, 
-				 const char *def, const char *path, ...)
+                                 const char *def, const char *path, ...)
 {
   struct entry *pentry;
   char buf[MAX_LEN_BUFFER];
@@ -1173,12 +1333,10 @@ char *secfile_lookup_str_default(struct section_file *my_section_file,
     return (char *) def;
   }
 
-  if(!pentry->svalue) {
-    freelog(LOG_FATAL, "sectionfile %s contains a '%s', but integer not string",
-	    secfile_filename(my_section_file), buf);
-    exit(EXIT_FAILURE);
-  }
-  
+  RETURN_VAL_IF_FAIL_MSG(pentry->svalue != NULL, (char *)def, "sectionfile "
+                         "%s contains a '%s', but integer not string",
+                         secfile_filename(my_section_file), buf);
+
   return pentry->svalue;
 }
 
@@ -1257,7 +1415,6 @@ section_file_lookup_internal(struct section_file *my_section_file,
   return NULL;
 }
 
-
 /**************************************************************************
  The caller should ensure that "fullpath" should not refer to an entry
  which already exists in "my_section_file".  (Actually, in some cases
@@ -1299,19 +1456,18 @@ section_file_insert_internal(struct section_file *my_section_file,
      * the section, to avoid O(N^2) behaviour.
      */
     pentry = sbuf_malloc(sb, sizeof(struct entry));
+    entry_init(pentry);
     pentry->name = sbuf_strdup(sb, ent_name);
-    entry_list_insert_back(&psection->entries, pentry);
+    entry_list_append(psection->entries, pentry);
     return pentry;
   }
 
-  psection = sbuf_malloc(sb, sizeof(struct section));
-  psection->name = sbuf_strdup(sb, sec_name);
-  entry_list_init(&psection->entries);
-  section_list_insert_back(my_section_file->sections, psection);
-  
+  psection = section_file_append_section(my_section_file, sec_name);
+
   pentry = sbuf_malloc(sb, sizeof(struct entry));
+  entry_init(pentry);
   pentry->name = sbuf_strdup(sb, ent_name);
-  entry_list_insert_back(&psection->entries, pentry);
+  entry_list_append(psection->entries, pentry);
 
   return pentry;
 }
@@ -1320,7 +1476,7 @@ section_file_insert_internal(struct section_file *my_section_file,
 /**************************************************************************
  Return 0 if the section_file has not been setup for hashing.
 **************************************************************************/
-bool secfilehash_hashash(struct section_file *file)
+static bool secfilehash_hashash(struct section_file *file)
 {
   return (file->hashd && hash_num_buckets(file->hashd->htbl) != 0);
 }
@@ -1378,7 +1534,7 @@ static void secfilehash_insert(struct section_file *file,
  all entries must have unique names; in this case for duplicates
  the hash ref will be to the _last_ entry.
 **************************************************************************/
-void secfilehash_build(struct section_file *file, bool allow_duplicates)
+static void secfilehash_build(struct section_file *file, bool allow_duplicates)
 {
   struct hash_data *hashd;
   char buf[256];
@@ -1391,7 +1547,7 @@ void secfilehash_build(struct section_file *file, bool allow_duplicates)
   hashd->allow_duplicates = allow_duplicates;
   hashd->num_duplicates = 0;
   
-  section_list_iterate(*file->sections, psection) {
+  section_list_iterate(file->sections, psection) {
     entry_list_iterate(psection->entries, pentry) {
       my_snprintf(buf, sizeof(buf), "%s.%s", psection->name, pentry->name);
       secfilehash_insert(file, buf, pentry);
@@ -1410,7 +1566,7 @@ void secfilehash_build(struct section_file *file, bool allow_duplicates)
 /**************************************************************************
  Free the memory allocated for the hash table.
 **************************************************************************/
-void secfilehash_free(struct section_file *file)
+static void secfilehash_free(struct section_file *file)
 {
   secfilehash_check(file);
   hash_free(file->hashd->htbl);
@@ -1458,7 +1614,7 @@ int *secfile_lookup_int_vec(struct section_file *my_section_file,
   my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
-  *dimen = secfile_lookup_vec_dimen(my_section_file, buf);
+  *dimen = secfile_lookup_vec_dimen(my_section_file, "%s", buf);
   if (*dimen == 0) {
     return NULL;
   }
@@ -1490,7 +1646,7 @@ char **secfile_lookup_str_vec(struct section_file *my_section_file,
   my_vsnprintf(buf, sizeof(buf), path, ap);
   va_end(ap);
 
-  *dimen = secfile_lookup_vec_dimen(my_section_file, buf);
+  *dimen = secfile_lookup_vec_dimen(my_section_file, "%s", buf);
   if (*dimen == 0) {
     return NULL;
   }
@@ -1502,28 +1658,28 @@ char **secfile_lookup_str_vec(struct section_file *my_section_file,
 }
 
 /***************************************************************
- Copies a string and does '\n' -> newline translation.
- Other '\c' sequences (any character 'c') are just passed
- through with the '\' removed (eg, includes '\\', '\"')
- Backslash followed by a genuine newline removes the newline.
+  Copies a string. Backslash followed by a genuine newline always
+  removes the newline.
+  If full_escapes is TRUE:
+    - '\n' -> newline translation.
+    - Other '\c' sequences (any character 'c') are just passed
+      through with the '\' removed (eg, includes '\\', '\"')
 ***************************************************************/
-static char *minstrdup(struct sbuffer *sb, const char *str)
+static char *minstrdup(struct sbuffer *sb, const char *str,
+                       bool full_escapes)
 {
   char *dest = sbuf_malloc(sb, strlen(str)+1);
   char *d2=dest;
   if(dest) {
     while (*str != '\0') {
-      if (*str=='\\') {
+      if (*str == '\\' && *(str+1) == '\n') {
+        /* Escape followed by newline. Skip both */
+        str += 2;
+      } else if (full_escapes && *str=='\\') {
 	str++;
-	if (*str=='\\') {
-	  *dest++='\\';
-	  str++;
-	} else if (*str=='n') {
+        if (*str=='n') {
 	  *dest++='\n';
 	  str++;
-	} else if (*str=='\n') {
-	  /* skip */
-	  str++;
 	}
       } else {
 	*dest++=*str++;
@@ -1541,11 +1697,12 @@ static char *minstrdup(struct sbuffer *sb, const char *str)
  string at a time) with str escaped the opposite of minstrdup.
  Specifically, any newline, backslash, or double quote is
  escaped with a backslash.
+ Adds appropriate delimiters: "" if escaped, $$ unescaped.
  The internal buffer is grown as necessary, and not normally
  freed (since this will be called frequently.)  A call with
  str=NULL frees the buffer and does nothing else (returns NULL).
 ***************************************************************/
-static char *moutstr(char *str)
+static char *moutstr(char *str, bool full_escapes)
 {
   static char *buf = NULL;
   static int nalloc = 0;
@@ -1561,10 +1718,12 @@ static char *moutstr(char *str)
     return NULL;
   }
   
-  len = strlen(str)+1;
-  for(c=str; *c != '\0'; c++) {
-    if (*c == '\n' || *c == '\\' || *c == '\"') {
-      len++;
+  len = strlen(str)+3;
+  if (full_escapes) {
+    for(c=str; *c != '\0'; c++) {
+      if (*c == '\n' || *c == '\\' || *c == '\"') {
+        len++;
+      }
     }
   }
   if (len > nalloc) {
@@ -1573,8 +1732,9 @@ static char *moutstr(char *str)
   }
   
   dest = buf;
+  *dest++ = (full_escapes ? '\"' : '$');
   while(*str != '\0') {
-    if (*str == '\n' || *str == '\\' || *str == '\"') {
+    if (full_escapes && (*str == '\n' || *str == '\\' || *str == '\"')) {
       *dest++ = '\\';
       if (*str == '\n') {
 	*dest++ = 'n';
@@ -1586,6 +1746,7 @@ static char *moutstr(char *str)
       *dest++ = *str++;
     }
   }
+  *dest++ = (full_escapes ? '\"' : '$');
   *dest = '\0';
   return buf;
 }
@@ -1611,7 +1772,7 @@ char **secfile_get_secnames_prefix(struct section_file *my_section_file,
 
   /* count 'em: */
   i = 0;
-  section_list_iterate(*my_section_file->sections, psection) {
+  section_list_iterate(my_section_file->sections, psection) {
     if (strncmp(psection->name, prefix, len) == 0) {
       i++;
     }
@@ -1626,7 +1787,7 @@ char **secfile_get_secnames_prefix(struct section_file *my_section_file,
   ret = fc_malloc((*num)*sizeof(char*));
 
   i = 0;
-  section_list_iterate(*my_section_file->sections, psection) {
+  section_list_iterate(my_section_file->sections, psection) {
     if (strncmp(psection->name, prefix, len) == 0) {
       ret[i++] = psection->name;
     }
@@ -1658,7 +1819,7 @@ char **secfile_get_section_entries(struct section_file *my_section_file,
     return NULL;
   }
 
-  *num = entry_list_size(&psection->entries);
+  *num = entry_list_size(psection->entries);
 
   if (*num == 0) {
     return NULL;
@@ -1673,3 +1834,19 @@ char **secfile_get_section_entries(struct section_file *my_section_file,
 
   return ret;
 }
+
+/****************************************************************************
+  Returns TRUE if the given section exists in the secfile.
+****************************************************************************/
+bool secfile_has_section(const struct section_file *sf,
+                         const char *section_name_fmt, ...)
+{
+  char name[MAX_LEN_BUFFER];
+  va_list ap;
+
+  va_start(ap, section_name_fmt);
+  my_vsnprintf(name, sizeof(name), section_name_fmt, ap);
+  va_end(ap);
+
+  return hash_key_exists(sf->hash_sections, name);
+}
