@@ -71,6 +71,11 @@
                           "~/.freeciv"
 #endif
 
+/* environment */
+#ifndef FREECIV_PATH
+#define FREECIV_PATH "FREECIV_PATH"
+#endif
+
 /* Both of these are stored in the local encoding.  The grouping_sep must
  * be converted to the internal encoding when it's used. */
 static char *grouping = NULL;
@@ -244,7 +249,7 @@ static size_t my_strcspn(const char *s, const char *reject)
  num_tokens are extracted.
 
  The user has the responsiblity to free the memory allocated by
- **tokens.
+ **tokens using free_tokens().
 ***************************************************************/
 int get_tokens(const char *str, char **tokens, size_t num_tokens,
 	       const char *delimiterset)
@@ -290,6 +295,19 @@ int get_tokens(const char *str, char **tokens, size_t num_tokens,
   return token;
 }
 
+/***************************************************************
+  Frees a set of tokens created by get_tokens().
+***************************************************************/
+void free_tokens(char **tokens, size_t ntokens)
+{
+  size_t i;
+  for (i = 0; i < ntokens; i++) {
+    if (tokens[i]) {
+      free(tokens[i]);
+    }
+  }
+}
+
 /***************************************************************
   Returns a statically allocated string containing a nicely-formatted
   version of the given number according to the user's locale.  (Only
@@ -490,20 +508,6 @@ void randomize_base64url_string(char *s, size_t n)
   s[i] = '\0';
 }
 
-/***************************************************************
-  Produce a statically allocated textual representation of the given
-  year.
-***************************************************************/
-const char *textyear(int year)
-{
-  static char y[32];
-  if (year<0) 
-    my_snprintf(y, sizeof(y), _("%d BC"), -year);
-  else
-    my_snprintf(y, sizeof(y), _("%d AD"), year);
-  return y;
-}
-
 /**************************************************************************
   Compares two strings, in the collating order of the current locale,
   given pointers to the two strings (i.e., given "char *"s).
@@ -734,7 +738,7 @@ int cat_snprintf(char *str, size_t n, const char *format, ...)
   va_start(ap, format);
   ret = my_vsnprintf(str+len, n-len, format, ap);
   va_end(ap);
-  return (int) (ret + len);
+  return (-1 == ret ? -1 : ret + len);
 }
 
 /***************************************************************************
@@ -788,13 +792,25 @@ char *user_home_dir(void)
       
       if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, CSIDL_APPDATA, &pidl))) {
         
-        home_dir = fc_malloc(PATH_MAX);
+        char *home_dir_in_local_encoding = fc_malloc(PATH_MAX);
         
-        if (!SUCCEEDED(SHGetPathFromIDList(pidl, home_dir))) {
-          free(home_dir);
-          home_dir = NULL;
-          freelog(LOG_ERROR,
-            "Could not find home directory (SHGetPathFromIDList() failed)");
+        if (SUCCEEDED(SHGetPathFromIDList(pidl, home_dir_in_local_encoding))) {
+        	/* convert to internal encoding */
+        	home_dir = local_to_internal_string_malloc(home_dir_in_local_encoding);
+        	free(home_dir_in_local_encoding);
+        	
+        	/* replace backslashes with forward slashes */
+        	char *c;
+        	for (c = home_dir; *c != 0; c++) {
+        		if (*c == '\\') {
+        			*c = '/';
+        		}
+        	}
+        } else {
+            free(home_dir_in_local_encoding);
+            home_dir = NULL;
+            freelog(LOG_ERROR,
+              "Could not find home directory (SHGetPathFromIDList() failed)");
         }
         
         SHGetMalloc(&pMalloc);
@@ -814,6 +830,7 @@ char *user_home_dir(void)
     }
     init = TRUE;
   }
+
   return home_dir;
 #endif
 }
@@ -917,12 +934,14 @@ const char **get_data_dirs(int *num_dirs)
     return dirs;
   }
 
-  path = getenv("FREECIV_PATH");
+  path = getenv(FREECIV_PATH);
   if (!path) {
     path = DEFAULT_DATA_PATH;
   } else if (*path == '\0') {
-    freelog(LOG_ERROR, _("FREECIV_PATH is set but empty; "
-			 "using default path instead."));
+    freelog(LOG_ERROR,
+            /* TRANS: <FREECIV_PATH> configuration error */
+            _("\"%s\" is set but empty; using default \"%s\" instead."),
+            FREECIV_PATH, DEFAULT_DATA_PATH);
     path = DEFAULT_DATA_PATH;
   }
   assert(path != NULL);
@@ -1019,14 +1038,15 @@ char **datafilelist(const char* suffix)
     struct dirent *entry;
 
     /* Open the directory for reading. */
-    dir = opendir(dirs[dir_num]);
+    dir = fc_opendir(dirs[dir_num]);
     if (!dir) {
       if (errno == ENOENT) {
 	freelog(LOG_VERBOSE, "Skipping non-existing data directory %s.",
 		dirs[dir_num]);
       } else {
-	freelog(LOG_ERROR, _("Could not read data directory %s: %s."),
-		dirs[dir_num], mystrerror());
+	/* TRANS: "...: <externally translated error string>."*/
+        freelog(LOG_ERROR, _("Could not read data directory %s: %s."),
+		dirs[dir_num], fc_strerror(fc_get_errno()));
       }
       continue;
     }
@@ -1130,7 +1150,7 @@ char *datafilename(const char *filename)
     
     astr_minsize(&realfile, len);
     my_snprintf(realfile.str, len, "%s/%s", dirs[i], filename);
-    if (stat(realfile.str, &buf) == 0) {
+    if (fc_stat(realfile.str, &buf) == 0) {
       return realfile.str;
     }
   }
@@ -1144,22 +1164,18 @@ char *datafilename(const char *filename)
 /**************************************************************************
   Compare modification times.
 **************************************************************************/
-static int compare_file_mtime_ptrs(const void *a, const void *b)
+static int compare_file_mtime_ptrs(const struct datafile *const *ppa,
+                                   const struct datafile * const *ppb)
 {
-  struct datafile * const *ppa = a;
-  struct datafile * const *ppb = b;
-
   return ((*ppa)->mtime < (*ppb)->mtime);
 }
 
 /**************************************************************************
   Compare names.
 **************************************************************************/
-static int compare_file_name_ptrs(const void *a, const void *b)
+static int compare_file_name_ptrs(const struct datafile *const *ppa,
+                                  const struct datafile * const *ppb)
 {
-  struct datafile * const *ppa = a;
-  struct datafile * const *ppb = b;
-
   return compare_strings((*ppa)->name, (*ppb)->name);
 }
 
@@ -1195,7 +1211,7 @@ struct datafile_list *datafilelist_infix(const char *subpath,
     }
 
     /* Open the directory for reading. */
-    dir = opendir(path);
+    dir = fc_opendir(path);
     if (!dir) {
       continue;
     }
@@ -1216,7 +1232,7 @@ struct datafile_list *datafilelist_infix(const char *subpath,
 	fullname = fc_malloc(len);
 	my_snprintf(fullname, len, "%s/%s", path, filename);
 
-	if (stat(fullname, &buf) == 0) {
+	if (fc_stat(fullname, &buf) == 0) {
 	  file = fc_malloc(sizeof(*file));
 
 	  /* Clip the suffix. */
@@ -1276,17 +1292,21 @@ char *datafilename_required(const char *filename)
   if (dname) {
     return dname;
   } else {
-    freelog(LOG_ERROR, _("The data path may be set via"
-			 " the environment variable FREECIV_PATH."));
-    freelog(LOG_ERROR, _("Current data path is: \"%s\""), datafilename(NULL));
+    freelog(LOG_ERROR,
+            /* TRANS: <FREECIV_PATH> configuration error */
+            _("The data path may be set via the \"%s\" environment variable."),
+            FREECIV_PATH);
+    freelog(LOG_ERROR,
+            _("Current data path is: \"%s\""),
+            datafilename(NULL));
     freelog(LOG_FATAL,
-		 _("The \"%s\" file is required ... aborting!"), filename);
+            _("The \"%s\" file is required ... aborting!"), filename);
     exit(EXIT_FAILURE);
   }
 }
 
 /***************************************************************************
-  Language environmental variable.
+  Language environmental variable (with emulation).
 ***************************************************************************/
 char *get_langname(void)
 {
@@ -1385,7 +1405,7 @@ char *get_langname(void)
 ***************************************************************************/
 void init_nls(void)
 {
-  /* 
+  /*
    * Setup the cached locale numeric formatting information. Defaults
    * are as appropriate for the US.
    */
@@ -1422,8 +1442,9 @@ void init_nls(void)
 
     if (lc->grouping[0] == '\0') {
       /* This actually indicates no grouping at all. */
-      static char m = CHAR_MAX;
-      grouping = &m;
+      char *m = malloc(sizeof(char));
+      *m = CHAR_MAX;
+      grouping = m;
     } else {
       size_t len;
       for (len = 0;
@@ -1441,6 +1462,17 @@ void init_nls(void)
 #endif
 }
 
+/***************************************************************************
+  Free memory allocated by Native Language Support
+***************************************************************************/
+void free_nls(void)
+{
+  free(grouping);
+  grouping = NULL;
+  free(grouping_sep);
+  grouping_sep = NULL;
+}
+
 /***************************************************************************
   If we have root privileges, die with an error.
   (Eg, for security reasons.)
@@ -1486,6 +1518,21 @@ const char *m_pre_description(enum m_pre_result result)
   return descriptions[result];
 }
 
+/***************************************************************************
+  See match_prefix_full().
+***************************************************************************/
+enum m_pre_result match_prefix(m_pre_accessor_fn_t accessor_fn,
+                               size_t n_names,
+                               size_t max_len_name,
+                               m_pre_strncmp_fn_t cmp_fn,
+                               m_strlen_fn_t len_fn,
+                               const char *prefix,
+                               int *ind_result)
+{
+  return match_prefix_full(accessor_fn, n_names, max_len_name, cmp_fn,
+                           len_fn, prefix, ind_result, NULL, 0, NULL);
+}
+
 /***************************************************************************
   Given n names, with maximum length max_len_name, accessed by
   accessor_fn(0) to accessor_fn(n-1), look for matching prefix
@@ -1493,14 +1540,20 @@ const char *m_pre_description(enum m_pre_result result)
   Returns type of match or fail, and for return <= M_PRE_AMBIGUOUS
   sets *ind_result with matching index (or for ambiguous, first match).
   If max_len_name==0, treat as no maximum.
+  If the int array 'matches' is non-NULL, up to 'max_matches' ambiguous
+  matching names indices will be inserted into it. If 'pnum_matches' is
+  non-NULL, it will be set to the number of indices inserted into 'matches'.
 ***************************************************************************/
-enum m_pre_result match_prefix(m_pre_accessor_fn_t accessor_fn,
-			       size_t n_names,
-			       size_t max_len_name,
-			       m_pre_strncmp_fn_t cmp_fn,
-                               m_strlen_fn_t len_fn,
-			       const char *prefix,
-			       int *ind_result)
+enum m_pre_result match_prefix_full(m_pre_accessor_fn_t accessor_fn,
+                                    size_t n_names,
+                                    size_t max_len_name,
+                                    m_pre_strncmp_fn_t cmp_fn,
+                                    m_strlen_fn_t len_fn,
+                                    const char *prefix,
+                                    int *ind_result,
+                                    int *matches,
+                                    int max_matches,
+                                    int *pnum_matches)
 {
   int i, len, nmatches;
 
@@ -1527,6 +1580,9 @@ enum m_pre_result match_prefix(m_pre_accessor_fn_t accessor_fn,
       if (nmatches==0) {
 	*ind_result = i;	/* first match */
       }
+      if (matches != NULL && nmatches < max_matches) {
+        matches[nmatches] = i;
+      }
       nmatches++;
     }
   }
@@ -1534,6 +1590,9 @@ enum m_pre_result match_prefix(m_pre_accessor_fn_t accessor_fn,
   if (nmatches == 1) {
     return M_PRE_ONLY;
   } else if (nmatches > 1) {
+    if (pnum_matches != NULL) {
+      *pnum_matches = MIN(max_matches, nmatches);
+    }
     return M_PRE_AMBIGUOUS;
   } else {
     return M_PRE_FAIL;
@@ -1584,18 +1643,29 @@ bool bv_are_equal(const unsigned char *vec1, const unsigned char *vec2,
   servers on the LAN, as specified by $FREECIV_MULTICAST_GROUP.
   Gets value once, and then caches result.
 ***************************************************************************/
-char *get_multicast_group(void)
+char *get_multicast_group(bool ipv6_prefered)
 {
   static bool init = FALSE;
   static char *group = NULL;
-  static char *default_multicast_group = "225.1.1.1";
-  
+  static char *default_multicast_group_ipv4 = "225.1.1.1";
+#ifdef IPV6_SUPPORT
+  /* TODO: Get useful group (this is node local) */
+  static char *default_multicast_group_ipv6 = "FF31::8000:15B4";
+#endif
+
   if (!init) {
     char *env = getenv("FREECIV_MULTICAST_GROUP");
     if (env) {
       group = mystrdup(env);	        
     } else {
-      group = mystrdup(default_multicast_group);
+#ifdef IPV6_SUPPORT
+      if (ipv6_prefered) {
+        group = mystrdup(default_multicast_group_ipv6);
+      } else
+#endif /* IPv6 support */
+      {
+        group = mystrdup(default_multicast_group_ipv4);
+      }
     }
     init = TRUE;
   }
@@ -1645,6 +1715,23 @@ char *interpret_tilde_alloc(const char* filename)
   }
 }
 
+/**************************************************************************
+  Return a pointer to the start of the file basename in filepath.
+  If the string contains no dir separator, it is returned itself.
+**************************************************************************/
+char *skip_to_basename(char *filepath)
+{
+  int j;
+  assert(filepath != NULL);
+
+  for (j = strlen(filepath); j >= 0; j--) {
+    if (filepath[j] == '/') {
+      return &filepath[j+1];
+    }
+  }
+  return filepath;
+}
+
 /**************************************************************************
   If the directory "pathname" does not exist, recursively create all
   directories until it does.
@@ -1664,11 +1751,13 @@ bool make_dir(const char *pathname)
     }
 
 #ifdef WIN32_NATIVE
-    mkdir(path);
+    char *path_in_local_encoding = internal_to_local_string_malloc(path);
+    _mkdir(path_in_local_encoding);
+    free(path_in_local_encoding);
 #else
     mkdir(path, 0755);
 #endif
-      
+
     if (dir) {
       *dir = '/';
       dir++;
@@ -1761,3 +1850,214 @@ char scanin(char **buf, char *delimiters, char *dest, int size)
 
   return found;
 }
+
+/************************************************************************
+  Randomize the elements of an array using the Fisher-Yates shuffle.
+
+  see: http://benpfaff.org/writings/clc/shuffle.html
+************************************************************************/
+void array_shuffle(int *array, int n)
+{
+  if (n > 1 && array != NULL) {
+    int i, j, t;
+    for (i = 0; i < n - 1; i++) {
+      j = i + myrand(n - i);
+      t = array[j];
+      array[j] = array[i];
+      array[i] = t;
+    }
+  }
+}
+
+/****************************************************************************
+  Print a string with a custom format. sequences is a pointer to an array of
+  sequences, probably defined with CF_*_SEQ(). sequences_num is the number of
+  the sequences, or -1 in the case the array is terminated with CF_END.
+
+  Example:
+  static const struct cf_sequence sequences[] = {
+    CF_INT_SEQ('y', 2010)
+  };
+  char buf[256];
+
+  fc_vsnprintcf(buf, sizeof(buf), "%y %+06y", sequences, 1);
+  // This will print "2010 +02010" into buf.
+****************************************************************************/
+int fc_vsnprintcf(char *buf, size_t buf_len, const char *format,
+                  const struct cf_sequence *sequences, size_t sequences_num)
+{
+  const struct cf_sequence *pseq;
+  char cformat[32];
+  const char *f = format;
+  char *const max = buf + buf_len - 1;
+  char *b = buf, *c;
+  const char *const cmax = cformat + sizeof(cformat - 2);
+  int i, j;
+
+  if ((size_t) -1 == sequences_num) {
+    /* Find the number of sequences. */
+    sequences_num = 0;
+    for (pseq = sequences; CF_LAST != pseq->type; pseq++) {
+      sequences_num++;
+    }
+  }
+
+  while ('\0' != *f) {
+    if ('%' == *f) {
+      /* Sequence. */
+
+      f++;
+      if ('%' == *f) {
+        /* Double '%'. */
+        *b++ = '%';
+        f++;
+        continue;
+      }
+
+      /* Make format. */
+      c = cformat;
+      *c++ = '%';
+      for (; !my_isalpha(*f) && '\0' != *f && '%' != *f && cmax > c; f++) {
+        *c++ = *f;
+      }
+
+      if (!my_isalpha(*f)) {
+        /* Beginning of a new sequence, end of the format, or too long
+         * sequence. */
+        *c = '\0';
+        j = my_snprintf(b, max - b + 1, "%s", cformat);
+        if (-1 == j) {
+          return -1;
+        }
+        b += j;
+        continue;
+      }
+
+      for (i = 0, pseq = sequences; i < sequences_num; i++, pseq++) {
+        if (pseq->letter == *f) {
+          j = -2;
+          switch (pseq->type) {
+          case CF_BOOLEAN:
+            *c++ = 's';
+            *c = '\0';
+            j = my_snprintf(b, max - b + 1, cformat,
+                            pseq->bool_value ? "TRUE" : "FALSE");
+            break;
+          case CF_TRANS_BOOLEAN:
+            *c++ = 's';
+            *c = '\0';
+            j = my_snprintf(b, max - b + 1, cformat,
+                            pseq->bool_value ? _("TRUE") : _("FALSE"));
+            break;
+          case CF_CHARACTER:
+            *c++ = 'c';
+            *c = '\0';
+            j = my_snprintf(b, max - b + 1, cformat, pseq->char_value);
+            break;
+          case CF_INTEGER:
+            *c++ = 'd';
+            *c = '\0';
+            j = my_snprintf(b, max - b + 1, cformat, pseq->int_value);
+            break;
+          case CF_HEXA:
+            *c++ = 'x';
+            *c = '\0';
+            j = my_snprintf(b, max - b + 1, cformat, pseq->int_value);
+            break;
+          case CF_FLOAT:
+            *c++ = 'f';
+            *c = '\0';
+            j = my_snprintf(b, max - b + 1, cformat, pseq->float_value);
+            break;
+          case CF_POINTER:
+            *c++ = 'p';
+            *c = '\0';
+            j = my_snprintf(b, max - b + 1, cformat, pseq->ptr_value);
+            break;
+          case CF_STRING:
+            *c++ = 's';
+            *c = '\0';
+            j = my_snprintf(b, max - b + 1, cformat, pseq->str_value);
+            break;
+          case CF_LAST:
+            break;
+          };
+          if (-2 == j) {
+            freelog(LOG_ERROR, "Error: unsupported sequence type: %d.",
+                    pseq->type);
+            break;
+          }
+          if (-1 == j) {
+            /* Full! */
+            return -1;
+          }
+          f++;
+          b += j;
+          break;
+        }
+      }
+      if (i >= sequences_num) {
+        /* Format not supported. */
+        *c = '\0';
+        j = my_snprintf(b, max - b + 1, "%s%c", cformat, *f);
+        if (-1 == j) {
+          return -1;
+        }
+        f++;
+        b += j;
+      }
+    } else {
+      /* Not a sequence. */
+      *b++ = *f++;
+    }
+    if (max <= b) {
+      /* Too long. */
+      *max = '\0';
+      return -1;
+    }
+  }
+  *b = '\0';
+  return b - buf;
+}
+
+/****************************************************************************
+  Print a string with a custom format. The additional arguments are a suite
+  of cf_*_seq() finished by cf_end(). This return the number of printed
+  characters (excluding the last '\0') or -1 if the buffer is full.
+
+  Example:
+  char buf[256];
+
+  fc_snprintcf(buf, sizeof(buf), "%y %+06y",
+               cf_int_seq('y', 2010), cf_end());
+  // This will print "2010 +02010" into buf.
+****************************************************************************/
+int fc_snprintcf(char *buf, size_t buf_len, const char *format, ...)
+{
+  struct cf_sequence sequences[16];
+  size_t sequences_num = 0;
+  va_list args;
+
+  /* Collect sequence array. */
+  va_start(args, format);
+  do {
+    sequences[sequences_num] = va_arg(args, struct cf_sequence);
+    if (CF_LAST == sequences[sequences_num].type) {
+      break;
+    } else {
+      sequences_num++;
+    }
+  } while (ARRAY_SIZE(sequences) > sequences_num);
+
+  if (ARRAY_SIZE(sequences) <= sequences_num
+      && CF_LAST != va_arg(args, struct cf_sequence).type) {
+    freelog(LOG_ERROR, "Too many custom sequences. Maybe did you forget "
+            "cf_end() at the end of the arguments?");
+    buf[0] = '\0';
+    va_end(args);
+    return -1;
+  }
+  va_end(args);
+
+  return fc_vsnprintcf(buf, buf_len, format, sequences, sequences_num);
+}
